Package: TRONCO
Version: 2.43.0
Date: 2025-09-10
Title: TRONCO, an R package for TRanslational ONCOlogy
Authors@R: c(person("Marco", "Antoniotti", role=c("ctb")), 
    person("Giulio", "Caravagna", role=c("aut"), email="gcaravagna@units.it"),
    person("Luca", "De Sano", role=c("cre","aut"), email="luca.desano@gmail.com",
                comment = c(ORCID = "0000-0002-9618-3774")), 
    person("Alex", "Graudenzi", role=c("aut"), email="alex.graudenzi@unimib.it"),
    person("Giancarlo", "Mauri", role=c("ctb")), 
    person("Bud", "Mishra", role=c("ctb")), 
    person("Daniele", "Ramazzotti", role=c("aut"),email="daniele.ramazzotti@unimib.it",
                comment = c(ORCID = "0000-0002-6087-2666")))
Depends: R (>= 4.1.0),
Imports: bnlearn, Rgraphviz, gtools, parallel, foreach, doParallel,
        iterators, RColorBrewer, circlize, igraph, grid, gridExtra,
        xtable, gtable, scales, R.matlab, grDevices, graphics, stats,
        utils, methods
Suggests: BiocGenerics, BiocStyle, testthat, knitr, rWikiPathways,
        magick
Name: An R package for the inference of cancer progression models from
        heterogeneous genomic data
Description: 
    The TRONCO (TRanslational ONCOlogy) R package collects algorithms to infer
    progression models via the approach of Suppes-Bayes Causal Network, both 
    from an ensemble of tumors (cross-sectional samples) and within an individual 
    patient (multi-region or single-cell samples). The package provides parallel 
    implementation of algorithms that process binary matrices where each row 
    represents a tumor sample and each column a single-nucleotide or a structural 
    variant driving the  progression; a 0/1 value models the absence/presence of 
    that alteration in the sample. The tool can import data from plain, MAF or 
    GISTIC format files, and can fetch it from the cBioPortal for cancer genomics. 
    Functions for  data manipulation and visualization are provided, as well as 
    functions to import/export such data to other bioinformatics  tools for, e.g, 
    clustering or detection of mutually exclusive alterations. Inferred models can be 
    visualized and tested for their confidence via bootstrap and cross-validation.
    TRONCO is used for  the implementation of the Pipeline for Cancer Inference (PICNIC). 
Encoding: UTF-8
License: GPL-3
URL: https://sites.google.com/site/troncopackage/
BugReports: https://github.com/BIMIB-DISCo/TRONCO
biocViews: BiomedicalInformatics, Bayesian, GraphAndNetwork,
        SomaticMutation, NetworkInference, Network, Clustering,
        DataImport, SingleCell, ImmunoOncology
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/TRONCO
git_branch: devel
git_last_commit: 7420d7e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 23:31:21 UTC; biocbuild
Author: Marco Antoniotti [ctb],
  Giulio Caravagna [aut],
  Luca De Sano [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9618-3774>),
  Alex Graudenzi [aut],
  Giancarlo Mauri [ctb],
  Bud Mishra [ctb],
  Daniele Ramazzotti [aut] (ORCID:
    <https://orcid.org/0000-0002-6087-2666>)
Maintainer: Luca De Sano <luca.desano@gmail.com>

# Generated by roxygen2: do not edit by hand

export(AND)
export(OR)
export(TCGA.map.clinical.data)
export(TCGA.multiple.samples)
export(TCGA.remove.multiple.samples)
export(TCGA.shorten.barcodes)
export(XOR)
export(annotate.description)
export(annotate.stages)
export(as.adj.matrix)
export(as.alterations)
export(as.bootstrap.scores)
export(as.colors)
export(as.conditional.probs)
export(as.confidence)
export(as.description)
export(as.events)
export(as.events.in.patterns)
export(as.events.in.sample)
export(as.gene)
export(as.genes)
export(as.genes.in.patterns)
export(as.genotypes)
export(as.hypotheses)
export(as.joint.probs)
export(as.kfold.eloss)
export(as.kfold.posterr)
export(as.kfold.prederr)
export(as.marginal.probs)
export(as.models)
export(as.parameters)
export(as.pathway)
export(as.patterns)
export(as.samples)
export(as.selective.advantage.relations)
export(as.stages)
export(as.types)
export(as.types.in.patterns)
export(change.color)
export(consolidate.data)
export(delete.event)
export(delete.gene)
export(delete.hypothesis)
export(delete.model)
export(delete.pattern)
export(delete.samples)
export(delete.type)
export(duplicates)
export(ebind)
export(enforce.numeric)
export(enforce.string)
export(events.selection)
export(export.graphml)
export(export.mutex)
export(export.nbs.input)
export(extract.MAF.HuGO.Entrez.map)
export(genes.table.report)
export(has.duplicates)
export(has.model)
export(has.stages)
export(hypothesis.add)
export(hypothesis.add.group)
export(hypothesis.add.homologous)
export(import.GISTIC)
export(import.MAF)
export(import.genotypes)
export(import.model)
export(import.mutex.groups)
export(intersect.datasets)
export(is.compliant)
export(join.events)
export(join.types)
export(keysToNames)
export(nameToKey)
export(nevents)
export(ngenes)
export(nhypotheses)
export(npatterns)
export(nsamples)
export(ntypes)
export(oncoprint)
export(oncoprint.cbio)
export(order.frequency)
export(pathway.visualization)
export(pheatmap)
export(rank.recurrents)
export(rename.gene)
export(rename.type)
export(samples.selection)
export(sbind)
export(ssplit)
export(trim)
export(tronco.bootstrap)
export(tronco.caprese)
export(tronco.capri)
export(tronco.chowliu)
export(tronco.edmonds)
export(tronco.gabow)
export(tronco.kfold.eloss)
export(tronco.kfold.posterr)
export(tronco.kfold.prederr)
export(tronco.pattern.plot)
export(tronco.plot)
export(tronco.prim)
export(view)
export(which.samples)
importFrom(R.matlab,writeMat)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rgraphviz,edgeNames)
importFrom(Rgraphviz,plot)
importFrom(bnlearn,"amat<-")
importFrom(bnlearn,amat)
importFrom(bnlearn,bn.cv)
importFrom(bnlearn,chow.liu)
importFrom(bnlearn,empty.graph)
importFrom(bnlearn,hc)
importFrom(bnlearn,score)
importFrom(bnlearn,set.arc)
importFrom(bnlearn,tabu)
importFrom(circlize,chordDiagram)
importFrom(circlize,circos.clear)
importFrom(circlize,circos.par)
importFrom(circlize,circos.text)
importFrom(circlize,circos.trackPlotRegion)
importFrom(circlize,get.cell.meta.data)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(grDevices,bmp)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.copy2pdf)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.set)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,tiff)
importFrom(graphics,barplot)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,locator)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,strwidth)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,grid.pretty)
importFrom(grid,grobTree)
importFrom(grid,polylineGrob)
importFrom(grid,rectGrob)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,unit.c)
importFrom(grid,viewport)
importFrom(gridExtra,grid.arrange)
importFrom(gridExtra,grid.table)
importFrom(gtable,gtable)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_height)
importFrom(gtable,gtable_width)
importFrom(gtools,permutations)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,clusters)
importFrom(igraph,edge)
importFrom(igraph,get.adjacency)
importFrom(igraph,get.edgelist)
importFrom(igraph,get.shortest.paths)
importFrom(igraph,graph.adjacency)
importFrom(igraph,graph.union)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,igraph.from.graphNEL)
importFrom(igraph,igraph.to.graphNEL)
importFrom(igraph,is.dag)
importFrom(igraph,minimum.spanning.tree)
importFrom(igraph,set.edge.attribute)
importFrom(igraph,set.graph.attribute)
importFrom(igraph,set.vertex.attribute)
importFrom(igraph,unfold.tree)
importFrom(igraph,write.graph)
importFrom(iterators,icount)
importFrom(methods,getPackageName)
importFrom(methods,is)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(scales,brewer_pal)
importFrom(scales,dscale)
importFrom(scales,hue_pal)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,logLik)
importFrom(stats,phyper)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,wilcox.test)
importFrom(utils,combn)
importFrom(utils,count.fields)
importFrom(utils,flush.console)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(xtable,xtable)

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

#' Plots a progression model from a recostructed dataset. For details and examples
#' regarding the visualization of an inferred model,
#' we refer to the Vignette Section 7.
#'
#' @title tronco.plot
#'
#' @examples
#' data(test_model)
#' tronco.plot(test_model)
#'
#' @param x A reconstructed model (the output of the inference
#' by a tronco function)
#' @param models A vector containing the names of the
#' algorithms used (caprese, capri_bic, etc)
#' @param fontsize For node names. Default NA for
#' automatic rescaling
#' @param height Proportion node height - node width.
#' Default height 2
#' @param width Proportion node height - node width.
#' Default width 2
#' @param height.logic Height of logical nodes.
#' Defaul 1
#' @param pf Should I print Prima Facie?
#' Default False
#' @param disconnected Should I print disconnected
#' nodes? Default False
#' @param scale.nodes Node scaling coefficient
#' (based on node frequency). Default NA (autoscale)
#' @param title Title of the plot. Default as.description(x)
#' @param confidence Should I add confidence
#' informations? No if NA
#' @param p.min p-value cutoff. Default automatic
#' @param legend Should I visualise the legend?
#' @param legend.cex CEX value for legend. Default 1.0
#' @param edge.cex CEX value for edge labels. Default 1.0
#' @param label.edge.size Size of edge labels.
#' Default NA for automatic rescaling
#' @param expand Should I expand hypotheses? Default TRUE
#' @param genes Visualise only genes in this list.
#' Default NULL, visualise all.
#' @param relations.filter Filter relations to dispaly
#' according to this functions. Default NA
#' @param edge.color Edge color. Default 'black'
#' @param pathways.color RColorBrewer colorser
#' for patways. Default 'Set1'.
#' @param file String containing filename for PDF output.
#' If NA no PDF output will be provided
#' @param legend.pos Legend position. Default 'bottom',
#' @param pathways A vector containing pathways information
#' as described in as.patterns()
#' @param lwd Edge base lwd. Default 3
#' @param samples.annotation = List of samples to search
#' for events in model
#' @param export.igraph If TRUE export the generated igraph
#' object
#' @param create.new.dev If TRUE create a new graphical device 
#' when calling trono.plot. Set this to FALSE, e.g., if you do not 
#' wish to create a new device when executing the command with 
#' export.igraph = TRUE
#' @param ... Additional arguments for RGraphviz
#' plot function
#' @return Information about the reconstructed model
#' @export tronco.plot
#' @importFrom RColorBrewer brewer.pal.info brewer.pal
#' @importFrom igraph graph.adjacency get.adjacency graph.union edge
#' @importFrom igraph V V<- igraph.to.graphNEL igraph.from.graphNEL
#' @importFrom Rgraphviz edgeNames plot
#' @importFrom graphics locator
#' @importFrom grDevices dev.copy2pdf
#'
tronco.plot <- function(x,
                        models = names(x$model),
                        fontsize = NA,
                        height = 2,
                        width = 3,
                        height.logic = 1,
                        pf = FALSE,
                        disconnected = FALSE,
                        scale.nodes = NA,
                        title = as.description(x),
                        confidence = NA,
                        p.min = 0.05,
                        legend = TRUE,
                        legend.cex = 1.0,
                        edge.cex = 1.0,
                        label.edge.size = NA,
                        expand = TRUE,
                        genes = NULL,
                        relations.filter = NA,
                        edge.color = 'black',
                        pathways.color = 'Set1',
                        file = NA,
                        # print to pdf,
                        legend.pos = 'bottom',
                        pathways = NULL,
                        lwd = 3,
                        samples.annotation = NA,
                        export.igraph = FALSE,
                        create.new.dev = TRUE,
                        ...) {
  is.compliant(x)
  is.model(x)
  
  ## Checks if reconstruction exists.
  
  logical_op = list("AND", "OR", "NOT", "XOR", "*", "UPAND", "UPOR", "UPXOR")
  
  
  if (length(models) > 2) {
    stop("Too many regularizators (max is 2)", call. = FALSE)
  }
  
  if (!all(models %in% names(x$model))) {
    stop(paste(
      paste(models, collapse = ' '),
      "not in reconstructed models. Use: ",
      paste(names(x$model), collapse = ' ')
    ),
    call. = FALSE)
    
  }
  
  if (!all(is.na(samples.annotation)) && !is.null(pathways))
    stop('Select either to annotate pathways or a sample.')
  
  ## Annotate samples.
  
  if (!all(is.na(samples.annotation))) {
    if (!all(samples.annotation %in% as.samples(x))) {
      stop('Sample(s) to annotate are not in the dataset -- see as.samples.')
    }
    
    if (npatterns(x) > 0) {
      nopatt.data = delete.model(x)
      nopatt.data = delete.type(nopatt.data, 'Pattern')
    } else {
      nopatt.data = x
    }
    
    sample.events = as.events.in.sample(nopatt.data, samples.annotation)
    sample.events = unique(sample.events[, 'event'])
    
    cat(
      'Annotating sample',
      samples.annotation,
      'with color red. Annotated genes:',
      paste(sample.events, collapse = ', '),
      '\n'
    )
    
    pathways = list(sample.events)
    names(pathways) = paste(samples.annotation, collapse = ', ')
    if (nchar(names(pathways)) > 15)
      names(pathways) = paste0(substr(names(pathways), 1, 15), '...')
    
    pathways.color = 'red'
  }
  
  sec = ifelse(length(models) == 2, TRUE, FALSE)
  
  if (sec && !models[2] %in% names(x$model)) {
    stop(paste(models[2], "not in model"), call. = FALSE)
    
  }
  
  ## Models objects.
  
  primary = as.models(x, models = models[1])[[1]]
  if (sec)
    secondary = as.models(x, models = models[2])[[1]]
  
  ## USARE getters adj.matrix.
  
  if (sec && !all(
    rownames(primary$adj.matrix$adj.matrix.fit)
    %in% rownames(secondary$adj.matrix$adj.matrix.fit)
  )) {
    stop(
      "primary and secondary must have the same adj.matrix! See: the function tronco.bootstrap.",
      call. = FALSE
    )
  }
  
  ## Get the adjacency matrix - this could have been donw with
  ## getters.
  
  adj.matrix = primary$adj.matrix
  if (sec)
    adj.matrix = secondary$adj.matrix
  c_matrix = adj.matrix$adj.matrix.fit
  
  if (is.function(relations.filter)) {
    cat(
      '*** Filtering relations according to function "relations.filter", visualizing:\n'
    )
    adj.matrix = as.adj.matrix(x, models = models)
    sel.relation = as.selective.advantage.relations(x, models = models)
    
    ## Select only relations which get TRUE by "relations.filter".
    
    sel.relation = lapply(sel.relation,
                          function(z) {
                            ## Apply can not be used - implicit coercion to char is crap
                            ## z[ apply(z, 1, relations.filter), ]
                            mask = rep(TRUE, nrow(z))
                            for (i in 1:nrow(z))
                              mask[i] = relations.filter(z[i, ])
                            return(z[mask, , drop = FALSE])
                          })
    
    sel.relation = get(models[2], sel.relation)
    
    c_matrix.names = rownames(c_matrix)
    c_matrix = matrix(0, nrow = nrow(c_matrix), ncol = ncol(c_matrix))
    rownames(c_matrix) = c_matrix.names
    colnames(c_matrix) = c_matrix.names
    
    cat(paste0(
      'Selected relations: ',
      nrow(sel.relation),
      ' [out of ',
      nrow(as.selective.advantage.relations(x,
                                            models = models)[[2]]),
      ']\n'
    ))
    
    if (nrow(sel.relation) > 0) {
      for (i in 1:nrow(sel.relation)) {
        c_matrix[nameToKey(x, sel.relation[i, 'SELECTS']),
                 nameToKey(x, sel.relation[i, 'SELECTED'])] = 1
      }
    }
    
    
  }
  
  ## Get the probabilities.
  
  probabilities = primary$probabilities
  if (sec) {
    probabilities = secondary$probabilities
  }
  marginal_p = probabilities$probabilities.observed$marginal.probs
  
  ## If prima facie change the adj matrix.
  if (pf) {
    c_matrix = adj.matrix$adj.matrix.pf
  }
  
  if (all(c_matrix == FALSE)
      ||
      (sec && all(primary$adj.matrix$adj.matrix.fit == FALSE))) {
    warning('No edge in adjacency matrix! Nothing to show here.')
    return(NULL)
  }
  
  ## Get algorithm parameters.
  
  parameters = x$parameters
  
  ## Get hypotheses.
  
  hypotheses = x$hypotheses
  hstruct = NULL
  if (!is.null(hypotheses) && !all(is.na(hypotheses))) {
    hstruct = hypotheses$hstructure
  }
  
  ## Get event from genes list.
  
  events = NULL
  if (is.vector(genes)) {
    events =
      unlist(lapply(genes,
                    function(x) {
                      names(which(as.events(x)[, 'event'] == x))
                    }))
  }
  
  cat('*** Expanding hypotheses syntax as graph nodes:')
  
  ## Expand hypotheses.
  
  expansion =
    hypotheses.expansion(c_matrix,
                         hstruct,
                         expand,
                         events)
  hypo_mat = expansion[[1]]
  hypos_new_name = expansion[[2]]
  
  cat('\n*** Rendering graphics\n')
  
  ## Remove disconnected nodes.
  
  if (!disconnected) {
    cat('Nodes with no incoming/outgoing edges will not be displayed.\n')
    del = which(rowSums(hypo_mat) + colSums(hypo_mat) == 0)
    w = !(rownames(hypo_mat) %in% names(del))
    hypo_mat = hypo_mat[w,]
    hypo_mat = hypo_mat[, w]
  }
  
  attrs = list(node = list())
  
  hypo_graph = graph.adjacency(hypo_mat)
  
  v_names = gsub("_.*$", "", V(hypo_graph)$name)
  if (!expand) {
    v_names = gsub("^[*]_(.+)", "*", V(hypo_graph)$name)
  }
  new_name = list()
  
  for (v in v_names) {
    if (v %in% rownames(x$annotations)) {
      n = x$annotations[v, "event"]
      new_name = append(new_name, n)
    } else {
      new_name = append(new_name, v)
    }
  }
  
  V(hypo_graph)$label = new_name
  
  graph = igraph::igraph.to.graphNEL(hypo_graph)
  
  node_names = V(hypo_graph)$name
  
  nAttrs = list()
  
  nAttrs$label = V(hypo_graph)$label
  names(nAttrs$label) = node_names
  
  ## Set a default color.
  
  nAttrs$fillcolor =  rep('White', length(node_names))
  names(nAttrs$fillcolor) = node_names
  
  ## Set fontsize.
  
  if (!is.na(fontsize)) {
    nAttrs$fontsize = rep(fontsize, length(node_names))
    names(nAttrs$fontsize) = node_names
  }
  
  ## Set node shape.
  
  nAttrs$shape = rep('ellipse', length(node_names))
  names(nAttrs$shape) = node_names
  
  ## Set node height.
  
  nAttrs$height = rep(height, length(node_names))
  names(nAttrs$height) = node_names
  
  ## Set node width.
  
  nAttrs$width = rep(width, length(node_names))
  names(nAttrs$width) = node_names
  
  short.label = nAttrs$label
  names(short.label) = names(nAttrs$label)
  
  ## Set gene cases
  
  nAttrs$cases = rep(0, length(node_names))
  names(nAttrs$cases) = node_names
  
  ## Set sumGenotypes
  
  nAttrs$sum.genotypes = rep(0, length(node_names))
  names(nAttrs$sum.genotypes) = node_names
  
  if (!is.na(scale.nodes)) {
    ## Foreach node.
    
    min_p = min(marginal_p)
    max_p = max(marginal_p)
    
    for (node in node_names) {
      prefix = gsub("_.*$", "", node)
      if (!(prefix %in% logical_op)) {
        ## Scaling ANDRE CITROLO.
        
        increase_coeff = scale.nodes + (marginal_p[node,] - min_p) / (max_p - min_p)
        nAttrs$width[node] = nAttrs$width[node] * increase_coeff
        nAttrs$height[node] = nAttrs$height[node] * increase_coeff
        nAttrs$cases[node] = round(marginal_p[node, ] * 100, 0)
        nAttrs$sum.genotypes[node] = sum(as.genotypes(x)[, node])
        nAttrs$label[node] =
          paste0(nAttrs$label[node],
                 '\\\n',
                 nAttrs$cases[node],
                 '%',
                 ' (',
                 nAttrs$sum.genotypes[node],
                 ')')
      }
    }
  }
  
  ## Use colors defined in
  ## tronco$types.
  
  w =
    unlist(lapply(names(nAttrs$fillcolor),
                  function(w) {
                    if (w %in% rownames(x$annotations)) {
                      x$types[x$annotations[w, 'type'], 'color']
                    } else
                      'White'
                  }))
  nAttrs$fillcolor[] = w
  
  legend_logic = NULL
  
  ## Set color, size form and shape each logic nodes (if hypos
  ## expansion actived)
  
  node.type = 'box'
  if (expand) {
    w = unlist(nAttrs$label[names(nAttrs$fillcolor)]) == 'OR'
    if (any(w)) {
      legend_logic['Exclusivity (soft)'] = 'orange'
    }
    nAttrs$fillcolor[which(w)] = 'orange'
    nAttrs$label[which(w)] = ''
    nAttrs$shape[which(w)] = node.type
    nAttrs$height[which(w)] = height.logic
    nAttrs$width[which(w)] = height.logic
    
    w = unlist(nAttrs$label[names(nAttrs$fillcolor)]) == 'AND'
    if (any(w)) {
      legend_logic['Co-occurence'] = 'darkgreen'
    }
    nAttrs$fillcolor[which(w)] = 'darkgreen'
    nAttrs$label[which(w)] = ''
    nAttrs$shape[which(w)] = node.type
    nAttrs$height[which(w)] = height.logic
    nAttrs$width[which(w)] = height.logic
    
    w = unlist(nAttrs$label[names(nAttrs$fillcolor)]) == 'XOR'
    if (any(w)) {
      legend_logic['Exclusivity (hard)'] = 'red'
    }
    nAttrs$fillcolor[which(w)] = 'red'
    nAttrs$label[which(w)] = ''
    nAttrs$shape[which(w)] = node.type
    nAttrs$height[which(w)] = height.logic
    nAttrs$width[which(w)] = height.logic
    
    w = unlist(nAttrs$label[names(nAttrs$fillcolor)]) == 'UPOR'
    if (any(w)) {
      legend_logic['Exclusivity (soft)'] = 'orange'
    }
    nAttrs$fillcolor[which(w)] = 'orange'
    nAttrs$label[which(w)] = ''
    nAttrs$shape[which(w)] = node.type
    nAttrs$height[which(w)] = height.logic
    nAttrs$width[which(w)] = height.logic
    
    w = unlist(nAttrs$label[names(nAttrs$fillcolor)]) == 'UPAND'
    if (any(w)) {
      legend_logic['Co-occurence'] = 'lightgreen'
    }
    nAttrs$fillcolor[which(w)] = 'lightgreen'
    nAttrs$label[which(w)] = ''
    nAttrs$shape[which(w)] = node.type
    nAttrs$height[which(w)] = height.logic
    nAttrs$width[which(w)] = height.logic
    
    w = unlist(nAttrs$label[names(nAttrs$fillcolor)]) == 'UPXOR'
    if (any(w)) {
      legend_logic['Exclusivity (hard)'] = 'red'
    }
    nAttrs$fillcolor[which(w)] = 'red'
    nAttrs$label[which(w)] = ''
    nAttrs$shape[which(w)] = node.type
    nAttrs$height[which(w)] = height.logic
    nAttrs$width[which(w)] = height.logic
    
  }
  
  w = unlist(nAttrs$label[names(nAttrs$fillcolor)]) == '*'
  if (any(w)) {
    legend_logic['Co-occurence'] = 'darkgreen'
  }
  nAttrs$fillcolor[which(w)] = 'darkgreen'
  nAttrs$label[which(w)] = ''
  nAttrs$shape[which(w)] = node.type
  nAttrs$height[which(w)] = height.logic
  nAttrs$width[which(w)] = height.logic
  
  
  ## Node border to black.
  
  nAttrs$color = rep("black", length(node_names))
  names(nAttrs$color) = node_names
  
  nAttrs$fontcolor = rep("black", length(node_names))
  names(nAttrs$fontcolor) = node_names
  
  nAttrs$lwd = rep(1, length(node_names))
  names(nAttrs$lwd) = node_names
  
  ## Set node border based on pathways information.
  
  legend_pathways = NULL
  
  if (!is.null(pathways)) {
    cat('Annotating nodes with pathway information. \n')
    
    if (length(pathways.color) == 1
        && pathways.color %in% rownames(brewer.pal.info)) {
      cat('Annotating pathways with RColorBrewer color palette',
          pathways.color,
          '.\n')
      n = length(names(pathways))
      if (n < 3) {
        n = 3
      }
      cols = brewer.pal(n = n, name = pathways.color)
    } else {
      if (length(pathways.color) != length(names(pathways)))
        stop(
          'You did not provide enough colors to annotate',
          length(names(pathways)),
          'pathways.
          Either set pathways.color to a valid RColorBrewer palette or provide the explicit correct number of colors.'
        )
      cols = pathways.color
    }
    
    names(cols) = names(pathways)
    names(nAttrs$col) = node_names
    
    for (path in names(pathways)) {
      n = short.label[which(short.label %in% pathways[[path]])]
      nAttrs$color[unlist(names(n))] = cols[[path]]
      nAttrs$fontcolor[unlist(names(n))] = cols[[path]]
      
      nAttrs$lwd[unlist(names(n))] = 4
      
      if (length(n) > 0) {
        legend_pathways[path] = cols[[path]]
      }
    }
  }
  
  ## Edges properties.
  
  edge_names = Rgraphviz::edgeNames(graph)
  eAttrs = list()
  
  ## If confidences are given create a column for each confidence
  if (any(!is.na(confidence))) {
    eAttrs$confidences = matrix('', length(edge_names), length(confidence))
    colnames(eAttrs$confidences) = confidence
    rownames(eAttrs$confidences) = edge_names
    names(eAttrs$confidences) = edge_names
  }
  
  ## Set temporary edge shape.
  
  eAttrs$lty = rep("solid", length(edge_names))
  names(eAttrs$lty) = edge_names
  
  ## Set temporary fontocolor.
  
  eAttrs$fontcolor = rep("darkblue", length(edge_names))
  names(eAttrs$fontcolor) = edge_names
  
  ## Set edge thikness based on prob.
  
  eAttrs$lwd = rep(1, length(edge_names))
  names(eAttrs$lwd) = edge_names
  
  ## Set edge name based on prob.
  
  eAttrs$label = rep('', length(edge_names))
  names(eAttrs$label) = edge_names
  
  ## Set fontsize to label.edge.size (default)
  
  if (!is.na(label.edge.size)) {
    eAttrs$fontsize = rep(label.edge.size, length(edge_names))
    names(eAttrs$fontsize) = edge_names
  } else if (!is.na(fontsize)) {
    label.edge.size = fontsize / 2
    cat('Set automatic fontsize for edge labels: ',
        label.edge.size,
        '\n')
    eAttrs$fontsize = rep(label.edge.size, length(edge_names))
    names(eAttrs$fontsize) = edge_names
  }
  
  ## Set edge color to black (default).
  
  eAttrs$color = rep(ifelse(sec, 'darkgrey', edge.color), length(edge_names))
  names(eAttrs$color) = edge_names
  
  ## Set edge arrowsize to 1 (default).
  
  eAttrs$arrowsize = rep(1 * edge.cex, length(edge_names))
  names(eAttrs$arrowsize) = edge_names
  
  ## Record logic edge.
  
  eAttrs$logic = rep(FALSE, length(edge_names))
  names(eAttrs$logic) = edge_names
  
  pval.names = c('hg', 'pr', 'tp')
  boot.names = c('npb', 'pb', 'sb')
  
  edge_label = function(value, conf, edge, model, pvalue) {
    ret = list()
    
    if (conf %in% boot.names) {
      ret$lwd = 1 + (value * 3)
    } else {
      ret$lwd = eAttrs$lwd[edge]
    }
    
    if (c %in% c('posterr', 'prederr')) {
      value = mean(unlist(value))
      
      if (value < 0.01) {
        tmp = "< 0.01"
      } else {
        tmp = round(value, 2)
      }
      ret$label = paste0(eAttrs$label[edge], tmp)
      ret$confidence = tmp
    } else {
      if (value < 0.01) {
        tmp = "< 0.01"
      } else {
        tmp = round(value, 2)
      }
      ret$label = paste0(eAttrs$label[edge], tmp)
      ret$confidence = tmp
    }
    
    
    # insert here edges visualization rules
    
    if (c == 'hg' && value > pvalue) {
      ret$fontcolor = 'red'
      ret$label = paste0(ret$label, ' *')
    }
    else if (c %in% c('pr', 'tp')
             && model != 'caprese'
             && value > pvalue) {
      ret$fontcolor = 'red'
      ret$label = paste0(ret$label, ' *')
    } else {
      ret$fontcolor = eAttrs$fontcolor[edge]
    }
    ret$label = paste0(ret$label, '\\\n')
    return(ret)
  }
  
  if (any(!is.na(confidence))) {
    cat('Adding confidence information: ')
    conf = as.confidence(x, confidence, models)
    cat(paste(paste(confidence, collapse = ', '), '\n'))
    
    
    for (c in confidence) {
      if (c == 'eloss') {
        next
        cat('skip eloss \n')
      }
      
      if (c == 'posterr') {
        conf_sel = as.kfold.posterr(x, models = models, table = TRUE)
      } else if (c == 'prederr') {
        conf_sel = as.kfold.prederr(x, models = models, table = TRUE)
      } else {
        conf_sel = get(c, as.confidence(x, c))
      }
      
      for (e in edge_names) {
        # configure conf_from and conf_to
        
        edge = unlist(strsplit(e, '~'))
        
        from = edge[1]
        to = edge[2]
        
        if (is.logic.node.up(from) ||
            is.logic.node.down(to)) {
          next
        }
        
        
        if (from %in% names(hypos_new_name)) {
          conf_from = hypos_new_name[[from]]
        } else {
          conf_from = from
        }
        if (to %in% names(hypos_new_name)) {
          conf_to = hypos_new_name[[to]]
        } else {
          conf_to = to
        }
        
        conf_p = conf_sel
        mod = ''
        
        if (!c %in% pval.names) {
          if (sec &&
              primary$adj.matrix$adj.matrix.fit[conf_from, conf_to] == 0) {
            conf_p = get(models[[2]], conf_sel)
            mod = models[[2]]
          } else {
            conf_p = get(models[[1]], conf_sel)
            mod = models[[1]]
          }
        }
        
        if (c == 'prederr') {
          value = conf_p[[conf_to]]
        } else {
          value = conf_p[conf_from, conf_to]
        }
        
        if (c != 'prederr' &&
            !(conf_from %in% rownames(conf_p) &&
              conf_to %in% colnames(conf_p))) {
          next
        } else if (c == 'prederr' &&
                   !(conf_to %in% names(conf_p))) {
          next
        }
        
        edge_info = edge_label(value, c, e, mod, p.min)
        eAttrs$label[e] = edge_info$label
        eAttrs$lwd[e] = edge_info$lwd
        eAttrs$fontcolor[e] = edge_info$fontcolor
        if (any(!is.na(confidence))) {
          eAttrs$confidences[e, c] =  edge_info$confidence
        }
      }
    }
  }
  cat('RGraphviz object prepared.\n')
  
  ## Remove arrows from logic node.
  
  for (e in edge_names) {
    edge = unlist(strsplit(e, '~'))
    from = edge[1]
    to = edge[2]
    
    if (is.logic.node.down(to)) {
      eAttrs$logic[e] = TRUE
      eAttrs$arrowsize[e] = 0
      
      if (substr(to, start = 1, stop = 2) == 'OR')
        eAttrs$color[e] = 'orange'
      if (substr(to, start = 1, stop = 3) == 'XOR')
        eAttrs$color[e] = 'red'
      if (substr(to, start = 1, stop = 3) == 'AND')
        eAttrs$color[e] = 'darkgreen'
      
      eAttrs$lty[e] = 'dashed'
      
      nAttrs$shape[to] = 'circle'
    }
    
    if (is.logic.node.up(from)) {
      eAttrs$logic[e] = TRUE
      eAttrs$arrowsize[e] = 0
      
      eAttrs$lty[e] = 'dashed'
      
      if (substr(from, start = 1, stop = 4) == 'UPOR')
        eAttrs$color[e] = 'orange'
      if (substr(from, start = 1, stop = 5) == 'UPXOR')
        eAttrs$color[e] = 'red'
      if (substr(from, start = 1, stop = 5) == 'UPAND')
        eAttrs$color[e] = 'darkgreen'
      
      
    } else if (substr(from, start = 1, stop = 1) == '*') {
      eAttrs$logic[e] = TRUE
      eAttrs$arrowsize[e] = 0
      eAttrs$color[e] = 'black'
    }
  }
  
  if (pf) {
    cat('*** Add prima facie edges: ')
    ## For each edge...
    
    bic = adj.matrix$adj.matrix.bic
    
    for (e in edge_names) {
      edge = unlist(strsplit(e, '~'))
      from = edge[1]
      old_name = hypos_new_name[[from]]
      if (!is.null(old_name)) {
        from = old_name
      }
      to = edge[2]
      if (substr(to, start = 1, stop = 1) == '*') {
        to = substr(to, start = 3, stop = nchar(to))
      }
      
      ## ...checks if edge is present in BIC
      ## Check if edge in BIC (valid only if not logic edge) and
      ## 'to' is not a fake and
      
      if ((from %in% rownames(bic))
          && (to %in% colnames(bic))
          && !eAttrs$logic[e]
          && bic[from, to] == 0) {
        eAttrs$color[e] = 'red'
      } else {
        ## No PF
      }
    }
    cat('done')
  }
  
  if (sec) {
    pri.adj = primary$adj.matrix$adj.matrix.fit
    for (from in rownames(pri.adj)) {
      for (to in colnames(pri.adj)) {
        from.alt.name = from
        to.alt.name = to
        if (from %in% hypos_new_name) {
          matching_nodes = names(which(hypos_new_name == from))
          for (node in matching_nodes) {
            if (is.logic.node.down(node)) {
              from.alt.name = node
            }
          }
        }
        
        if (to %in% hypos_new_name) {
          matching_nodes = names(which(hypos_new_name == to))
          for (node in matching_nodes) {
            if (is.logic.node.up(node)) {
              to.alt.name = node
            }
          }
        }
        
        if (pri.adj[from, to] == 1) {
          eAttrs$color[paste(from.alt.name, to.alt.name, sep = '~')] = edge.color
        }
        
      }
    }
  }
  
  if(create.new.dev) {
    cat('Plotting graph and adding legends.\n')
    Rgraphviz::plot(
      graph,
      nodeAttrs = nAttrs,
      edgeAttrs = eAttrs,
      main = title,
      ...
    )
    
    ## Adds the legend to the plot.
    
    if (legend) {
      valid_events = colnames(hypo_mat)[which(colnames(hypo_mat) %in% colnames(c_matrix))]
      legend_names = unique(x$annotations[which(rownames(x$annotations) %in% valid_events), 'type'])
      pt_bg = x$types[legend_names, 'color']
      legend_colors = rep('black', length(legend_names))
      pch = rep(21, length(legend_names))
      
      if (length(legend_logic) > 0) {
        pch = c(pch, 0, 0, rep(22, length(legend_logic)))
        legend_names = c(legend_names,
                         ' ',
                         expression(bold('Patterns')),
                         names(legend_logic))
        legend_colors = c(legend_colors, 'white', 'white', rep('black', length(legend_logic)))
        pt_bg = c(pt_bg, 'white', 'white', legend_logic)
      }
      
      if (length(legend_pathways) > 0) {
        pch = c(pch, 0, 0, rep(21, length(legend_pathways)))
        legend_names = c(legend_names,
                         ' ',
                         expression(bold('Pathways')),
                         names(legend_pathways))
        pt_bg = c(pt_bg, 'white', 'white', rep('white', length(legend_pathways)))
        legend_colors = c(legend_colors, 'white', 'white', legend_pathways)
      }
      
      if (legend.pos == 'bottom') {
        legend.pos.l = 'bottomleft'
        legend.pos.r = 'bottomright'
      } else if (legend.pos == 'top') {
        legend.pos.l = 'topleft'
        legend.pos.r = 'topright'
      } else {
        legend.pos.l = graphics::locator(1)
        legend.pos.r = graphics::locator(1)
      }
      
      legend(
        legend.pos.r,
        legend = legend_names,
        title = expression(bold('Events type')),
        bty = 'n',
        cex = legend.cex,
        pt.cex = 1.5 * legend.cex,
        pch = pch,
        col = legend_colors,
        pt.bg = pt_bg
      )
      
      ## Add thickness legend.
      valid_names = node_names
      
      if (!disconnected) {
        del = which(rowSums(hypo_mat) + colSums(hypo_mat) == 0)
        w = !(rownames(hypo_mat) %in% names(del))
        valid_names = rownames(hypo_mat[w,])
      }
      
      if (expand) {
        valid_names =
          valid_names[unlist(lapply(valid_names,
                                    function(x) {
                                      !is.logic.node(x)
                                    }))]
      }
      
      valid_names = grep('^[*]_(.+)$',
                         valid_names,
                         value = TRUE,
                         invert = TRUE)
      
      
      text = ""
      stat.pch = 0
      pt.bg = "white"
      col = "white"
      eloss = FALSE
      if ('eloss' %in% confidence) {
        eloss = TRUE
      }
      confidence = confidence[confidence != 'eloss']
      if (any(!is.na(confidence))) {
        text =
          c(
            expression(bold('Edge confidence')),
            lapply(confidence,
                   function(x) {
                     if (x == "hg")
                       return("Hypergeometric test")
                     if (x == "tp")
                       return("Temporal Priority")
                     if (x == "pr")
                       return("Probability Raising")
                     if (x == "pb")
                       return("Parametric Bootstrap")
                     if (x == "sb")
                       return("Statistical Bootstrap")
                     if (x == "npb")
                       return("Non Parametric Bootstrap")
                     if (x == "prederr")
                       return("Prediction Error")
                     if (x == "posterr")
                       return("Posterior Classification Error")
                     
                   }),
            paste("p-value cutoff <", p.min)
          )
        
        stat.pch = c(0, rep(18, length(confidence)), 0)
        pt.bg = c('white', rep('white', length(confidence)), 'white')
        col = c('white', rep('black', length(confidence)), 'white')
      }
      
      if ('Pattern' %in% as.types(x)) {
        y = delete.model(x)
        y = delete.type(y, 'Pattern')
      } else
        y = x
      
      text =
        c(
          text,
          ' ',
          expression(bold('Sample size')),
          paste0('n = ', nsamples(x), ', m = ', nevents(x)),
          paste0('|G| = ', ngenes(y), ', |P| = ', npatterns(x))
        )
      
      stat.pch = c(stat.pch, rep(0, 2), rep(20, 2), rep(0, 2))
      pt.bg = c(pt.bg, rep('white', 2), rep('black', 2), rep('white', 2))
      col = c(col, rep('white', 2), rep('white', 2), rep('white', 2))
      
      mods = NULL
      
      if (eloss) {
        for (model in models) {
          mods_label = gsub('_', ' ', model)
          if (!is.null(x$kfold) &&
              !is.null(get(model, x$kfold)$eloss)) {
            mods_label = paste(mods_label, '- eloss:', round(mean(get(
              model, x$kfold
            )$eloss), 5))
          }
          mods = c(mods, mods_label)
        }
      } else {
        for (model in models) {
          mods_label = gsub('_', ' ', model)
          mods = c(mods, mods_label)
        }
      }
      text =
        c(text, '\n',
          expression(bold('Algorithm:')),
          paste0(mods))
      
      stat.pch = c(stat.pch, 20)
      pt.bg = c(pt.bg, 'black')
      col = c(col, 'black')
      
      if (length(models) > 1) {
        stat.pch = c(stat.pch, 20)
        pt.bg = c(pt.bg, 'darkgrey')
        col = c(col, 'darkgrey')
      }
      
      
      legend(
        legend.pos.l,
        legend = text,
        title = "",
        bty = 'n',
        box.lty = 3,
        box.lwd = .3,
        pch = stat.pch,
        pt.cex = 1.5  * legend.cex,
        ncol = 1,
        pt.bg = pt.bg,
        cex = legend.cex,
        col = col
      )
    }
    
    if (!is.na(file)) {
      cat('Saving visualized device to file:', file)
      dev.copy2pdf(file = file)
    }
    cat('\n')
  }
  
  
  if (export.igraph) {
    output = list()
    output$graph = igraph::igraph.from.graphNEL(graph)
    output$nodes = nAttrs
    output$edges = eAttrs
    output$description = title
    output$models = models
    return(output)
  }
}


#### end of file -- tronco.plot.R

data(maf)
muts = import.MAF(maf, silent = TRUE)
stages = list()
stages$stage = c('A', 'B', 'C')
names(stages$stage) = as.samples(muts)
stages = as.data.frame(stages)
muts_stages = annotate.stages(muts, stages=stages)

data(test_model_kfold)
data(test_model)
data(test_dataset)

context("oncoprint")

test_that("oncoprint accept correct data", {
    expect_output(oncoprint(muts), "Oncoprint")
})

context("tronco.plot")

test_that("tronco.plot accept correct data", {
    expect_output(tronco.plot(test_model), "Plotting")
    expect_output(tronco.plot(test_model, samples.annotation = 'patient 1'), "sample")
    expect_output(tronco.plot(test_model, pf = TRUE), "Plotting")
    expect_output(tronco.plot(test_model, expand = FALSE), "Plotting")
    expect_output(tronco.plot(test_model, fontsize = 20, scale.nodes = 0.3), "Plotting")
    expect_output(tronco.plot(test_model_kfold, conf = c('tp', 'pr', 'hg', 'eloss', 'prederr', 'posterr', 'npb')), "confidence")
    expect_output(tronco.plot(test_model, export.igraph = TRUE), "Plotting")
    expect_error(tronco.plot(test_dataset))
    expect_error(tronco.plot(test_model, models = c('a', 'b', 'c')))
    expect_error(tronco.plot(test_model, models = 'banana'))
    expect_error(tronco.plot(test_model, models = c('capri_bic', 'banana')))
    expect_error(tronco.plot(test_model, samples.annotation = c('s1', 's2'), pathways = TRUE))
    expect_error(tronco.plot(test_model, samples.annotation = c('s1', 's2')))
})

## ----include=FALSE------------------------------------------------------------
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)

## -----------------------------------------------------------------------------
view(aCML)

## -----------------------------------------------------------------------------
as.genotypes(aCML)[1:10,5:10]

## -----------------------------------------------------------------------------
as.events(aCML)[1:5, ]
as.events.in.sample(aCML, sample = 'patient 2')

## -----------------------------------------------------------------------------
as.genes(aCML)[1:8]

## -----------------------------------------------------------------------------
as.types(aCML)
as.colors(aCML)

## -----------------------------------------------------------------------------
head(as.gene(aCML, genes='SETBP1'))

## -----------------------------------------------------------------------------
as.samples(aCML)[1:10]

## -----------------------------------------------------------------------------
which.samples(aCML, gene='TET2', type='Nonsense point')

## -----------------------------------------------------------------------------
dataset = as.alterations(aCML)

## -----------------------------------------------------------------------------
view(dataset)

## -----------------------------------------------------------------------------
ngenes(aCML)
nevents(aCML)
nsamples(aCML)
ntypes(aCML)
npatterns(aCML)

## ----fig.width=6, fig.height=5, fig.cap="This plot gives a graphical visualization of the events that are in the dataset -- with a color per event type. It sorts samples to enhance exclusivity patterns among the events"----
oncoprint(aCML)

## ----fig.width=5, fig.height=5, fig.cap="This plot gives a graphical visualization of the events that are in the dataset -- with a color per event type. It it clusters samples/events"----
oncoprint(aCML, 
    legend = FALSE, 
    samples.cluster = TRUE, 
    gene.annot = list(one = list('NRAS', 'SETBP1'), two = list('EZH2', 'TET2')),
    gene.annot.color = 'Set2',
    genes.cluster = TRUE)

## -----------------------------------------------------------------------------
stages = c(rep('stage 1', 32), rep('stage 2', 32))
stages = as.matrix(stages)
rownames(stages) = as.samples(aCML)
dataset = annotate.stages(aCML, stages = stages)
has.stages(aCML)
head(as.stages(dataset))

## -----------------------------------------------------------------------------
head(as.stages(dataset))

## ----fig.width=6, fig.height=5------------------------------------------------
oncoprint(dataset, legend = FALSE)

## ----fig.width=6, fig.height=5, fig.cap="Example \texttt{oncoprint} output for aCML data with randomly annotated stages, in left, and samples clustered by group assignment in right -- for simplicity the group variable is again the stage annotation."----
oncoprint(dataset, group.samples = as.stages(dataset))

## -----------------------------------------------------------------------------
pathway = as.pathway(aCML,
    pathway.genes = c('SETBP1', 'EZH2', 'WT1'),
    pathway.name = 'MyPATHWAY',
    pathway.color = 'red',
    aggregate.pathway = FALSE)

## ----onco-pathway, fig.width=6.5, fig.height=2, fig.cap="Oncoprint output of a custom pathway called MyPATHWAY involving genes SETBP1, EZH2 and WT1; the genotype of each event is shown."----
oncoprint(pathway, title = 'Custom pathway',  font.row = 8, cellheight = 15, cellwidth = 4)

## ----fig.width=6.5, fig.height=1.8, fig.cap="Oncoprint output of a custom pair of pathways, with events shown"----
pathway.visualization(aCML, 
    pathways=list(P1 = c('TET2', 'IRAK4'),  P2=c('SETBP1', 'KIT')),        
    aggregate.pathways=FALSE,
    font.row = 8)

## ----fig.width=6.5, fig.height=1, fig.cap="Oncoprint output of a custom pair of pathways, with events hidden"----
pathway.visualization(aCML, 
    pathways=list(P1 = c('TET2', 'IRAK4'),  P2=c('SETBP1', 'KIT')),
    aggregate.pathways = TRUE,
    font.row = 8)

## ----eval=FALSE---------------------------------------------------------------
# library(rWikiPathways)
# # quotes inside query to require both terms
# my.pathways <- findPathwaysByText('SETBP1 EZH2 TET2 IRAK4 SETBP1 KIT')
# human.filter <- lapply(my.pathways, function(x) x$species == "Homo sapiens")
# my.hs.pathways <- my.pathways[unlist(human.filter)]
# # collect pathways idenifiers
# my.wpids <- sapply(my.hs.pathways, function(x) x$id)
# 
# pw.title<-my.hs.pathways[[1]]$name
# pw.genes<-getXrefList(my.wpids[1],"H")

## ----wikipathways, eval=FALSE-------------------------------------------------
# browseURL(getPathwayInfo(my.wpids[1])[2])
# browseURL(getPathwayInfo(my.wpids[2])[2])
# browseURL(getPathwayInfo(my.wpids[3])[2])

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' select a subset of the input genotypes 'x'. Selection can be done 
#' by frequency and gene symbols.
#' @title events.selection
#'
#' @examples
#' data(test_dataset)
#' dataset = events.selection(test_dataset, 0.3)
#'
#' @param x A TRONCO compliant dataset.
#' @param filter.freq [0,1] value which constriants the minimum frequence of selected events
#' @param filter.in.names gene symbols which will be included
#' @param filter.out.names gene symbols which will NOT be included
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO compliant dataset.
#' @export events.selection
#' 
events.selection <- function(x,
                             filter.freq = NA,
                             filter.in.names = NA,
                             filter.out.names = NA,
                             silent = FALSE) {

    is.compliant(x, err.fun='events.selection: input')
    dataset = x$genotypes

    if (!silent) {
        cat('*** Events selection: #events = ',
            nevents(x),
            ', #types = ',
            ntypes(x))

        cat(' Filters freq|in|out = {', 
            !is.na(filter.freq), ', ',
            !any(is.na(filter.in.names)), ', ',
            !any(is.na(filter.out.names)), '}')
    }


    if (all(is.na(filter.out.names)) && all(is.na(filter.in.names)) && is.na(filter.freq)) {
        return(x)
    }

    valid = rep(FALSE, ncol(x$genotypes))

    if (!is.na(filter.freq)) {

        if (!silent) {
            cat('\nMinimum event frequency: ',
                filter.freq,
                ' (',
                round(nsamples(x) * filter.freq, 0),
                ' alterations out of ',
                nsamples(x),
                ' samples).\n')
        }
        x = enforce.numeric(x)


        #flush.console()
        #pb = txtProgressBar(1, nevents(x), style = 3)
        
        for (i in 1:nevents(x)) {   
            #setTxtProgressBar(pb, i)
            mut.freq = sum(x$genotypes[,i])/nsamples(x)
            valid[i] = mut.freq > filter.freq
            if (!silent) {
                cat('.')
            }
        }
        
        #close(pb)

        if (!silent) {
            cat('\nSelected ',
                nrow(as.events(x)[valid, ]),
                ' events.\n')
        }
    }

    if (!any(is.na(filter.in.names))) {
        shown = min(5, length(filter.in.names))

        if (!silent) {
            cat('\n[filter.in] Genes hold: ', 
                paste(filter.in.names[1:shown], collapse=', '),
                ' ... ')
        }

        colnames = which(x$annotations[,2] %in% filter.in.names,
            arr.ind = TRUE)

        k = unique(x$annotations[which(x$annotations[ ,'event'] %in% filter.in.names, 
                                    arr.ind = TRUE),
                                    'event'])

        if (!silent) {
            cat(' [', length(k), '/', length(filter.in.names), ' found].')
        }

        valid[colnames] = TRUE
    }

    if (!any(is.na(filter.out.names))) {
        shown = min(5, length(filter.out.names))

        if (!silent) {
            cat('\n[filter.out] Genes dropped: ', 
                paste(filter.out.names[1:shown], collapse=', '),
                ' ... ')
        }

        colnames = which(x$annotations[,2] %in% filter.out.names,
            arr.ind = TRUE)

        if (!silent) {
            cat(' [',
                length(colnames),
                '/',
                length(filter.out.names),
                ' found].')
        }
        valid[colnames] = FALSE
    }

    y = list()
    y$genotypes = x$genotypes[, valid, drop = FALSE]  

    y$annotations = as.matrix(x$annotations[valid, , drop = FALSE])
    colnames(y$annotations) = c('type', 'event')
    rownames(y$annotations) = colnames(y$genotypes)

    y$types = as.matrix(x$types[unique(y$annotations[,1]), 1])
    colnames(y$types) = c('color')
    rownames(y$types) = unique(y$annotations[,1])

    if (!is.null(x$stages)) y$stages=x$stages
    is.compliant(x, err.fun='events.selection: output')

    if (!silent) {
        cat('\nSelected ', nevents(y), ' events, returning.\n')
    }

    return(y)
}


#' Return the first n recurrent events
#' @title rank.recurrents
#'
#' @examples
#' data(test_dataset)
#' dataset = rank.recurrents(test_dataset, 10)
#'
#' @param x A TRONCO compliant dataset.
#' @param n The number of events to rank 
#' @return the first n recurrent events
#' @export rank.recurrents
#' 
rank.recurrents <- function(x, n) {
    is.compliant(x)
    x = enforce.numeric(x)    

    if(n <= 0) {
        stop('Rank value (n) should be positive.')
    }

    ## Sum columns
    sums = colSums(x$genotypes)

    ## Get the names of the first n ranked
    sorted = sort(sums, decreasing = TRUE)

    scores = unique(sorted)
    
    l = length(scores)
    if(n >l) {
        warning(paste0('Rank contains ',
                       l,
                       ' unique entries, using n = ',
                       l,
                       ' instead of n = ',
                       n))
    }

    n = min(n, length(scores))
    scores = scores[1:n]

    sorted = sorted[which(sorted >= min(scores))]

    max = names(sorted[which(sorted == max(scores))])
    min = names(sorted[which(sorted == min(scores))])

    cat(paste0('Most recurrent(s): ',
               paste(as.events(x)[max, 'event'], collapse=', '),
               ' (',
               (max(scores)),
               ' hits).\n' ))
    cat(paste0(n, '-th recurrent(s): ',
               paste(as.events(x)[min, 'event'], collapse=', '),
               ' (',
               (min(scores)),
               ' hits).\n' ))

    order = names(sorted)
    genes = as.events(x)[order, 'event']

    return(as.vector(genes))
}

#' Filter a dataset based on selected samples id
#' @title samples.selection
#'
#' @examples
#' data(test_dataset)
#' dataset = samples.selection(test_dataset, c('patient 1', 'patient 2'))
#'
#' @param x A TRONCO compliant dataset.
#' @param samples A list of samples
#' @return A TRONCO compliant dataset.
#' @export samples.selection
#' 
samples.selection <- function(x, samples) {
    is.compliant(x, 'Input:')

    missing = setdiff(samples, as.samples(x))
    if (length(missing) > 0) {
        warning(paste('Missing samples: ', paste(missing, collapse=', ')))
    }

    delete = setdiff(as.samples(x), samples)
    return(delete.samples(x, delete))
}

#### end of file -- selection.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

.onLoad <- function(libname, pkgname) {
	hypotheses.env <- new.env(parent = emptyenv())
	assign('hypotheses.env', hypotheses.env, asNamespace(pkgname))
}

#### end of file -- as.functions.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

#' Create an input file for MUTEX
#' (ref: https://code.google.com/p/mutex/ )
#' @title export,mutex
#'
#' @examples
#' data(crc_gistic)
#' dataset = import.GISTIC(crc_gistic)
#' export.mutex(dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @param filename The name of the file
#' @param filepath The path where to save the file
#' @param label.mutation The event type to use as mutation
#' @param label.amplification The event type to use as amplification (can be a list)
#' @param label.deletion The event type to use as amplification (can be a list)
#' @return A MUTEX example matrix
#' @export export.mutex
#' @importFrom utils write.table
#' 
export.mutex <- function(x, 
                         filename = 'tronco_to_mutex',
                         filepath = './',
                         label.mutation = 'SNV',
                         label.amplification = list('High-level Gain'),
                         label.deletion = list('Homozygous Loss')) {

    is.compliant(x)
    data = x
    alteration =
        list(unlist(label.mutation),
             unlist(label.amplification),
             unlist(label.deletion))

    ## Merge amplification.
    
    if (length(label.amplification) >= 0) {
        amplification = label.amplification[[1]]
    }
    if (length(label.amplification) >= 2) {
        amplification = 'amplification'
        data = join.types(data, label.amplification[[1]], label.amplification[[2]], 'amplification', 'red')
    }
    if (length(label.amplification) > 2) {
        for (label in label.amplification[3:length(label.amplification)]) {
            data = join.types(data, label, 'amplification', 'amplification', 'red')
        }
    }

    ## Merge deletion.
    
    if (length(label.deletion) >= 0) {
        deletion = label.deletion[[1]]
    }
    if (length(label.deletion) >= 2) {
        deletion = 'deletion'
        data = join.types(data, label.deletion[[1]], label.deletion[[2]], 'deletion', 'blue')
    }
    if (length(label.deletion) > 2) {
        for (label in label.deletion[3:length(label.deletion)]) {
            data = join.types(data, label, 'deletion', 'deletion', 'blue')
        }
    }

    ## Merge mutation.
    
    if (length(label.mutation) >= 0) {
        mutation = label.mutation[[1]]
    }
    if (length(label.mutation) >= 2) {
        mutation = 'mutation'
        data = join.types(data, label.mutation[[1]], label.mutation[[2]], 'mutation', 'green')
    }
    if (length(label.mutation) > 2) {
        for (label in label.mutation[3:length(label.mutation)]) {
            data = join.types(data, label, 'mutation', 'mutation', 'green')
        }
    }

    samples = rownames(data$genotypes) 
    genes = unique(data$annotation[,'event'])

    mutex.matrix = matrix(0, nrow = length(genes), ncol = length(samples))
    colnames(mutex.matrix) = samples
    rownames(mutex.matrix) = genes

    ## Legend:
    ## 0: no alteration
    ## 1: mutation
    ## 2: amplification
    ## 4: deletion
    ## 3: 1+2 a+m
    ## 5: 1+4 d+m

    legend = list(1, 2, 4)
    names(legend) = list(mutation, amplification, deletion)
    tronco.matrix = data$genotypes

    for (sample in rownames(tronco.matrix)) {
        for (gene in colnames(tronco.matrix)) {
            type = data$annotations[[gene, 'type']]

            if (type %in% alteration && tronco.matrix[sample, gene] == 1) {
                to.add = legend[[data$annotations[[gene, 'type']]]]
                actual.value = mutex.matrix[data$annotations[[gene, 'event']], sample]
                mutex.matrix[data$annotations[[gene, 'event']], sample] = actual.value + to.add
            }
        }
    }

    ## Reassign value according to mutex notation

    ## Legend:
    ## 0: no alteration
    ## 1: mutation
    ## 2: amplification
    ## 3: deletion
    ## 4: 1+2 a+m
    ## 5: 1+4 d+m

    ## Move a+m to 10
    mutex.matrix[which(mutex.matrix == 3)] = 10

    ## move deletion to 3
    mutex.matrix[which(mutex.matrix == 4)] = 3

    ## move a+m to 4
    mutex.matrix[which(mutex.matrix == 10)] = 4

    mutex.header = append("Symbol", samples)

    filepath = if (grepl("\\/$", filepath)) filepath else paste0(filepath, "/")
    con = paste0(filepath, filename)
    write(mutex.header, file = con, sep = "\t", ncolumns = length(mutex.header))
    write.table(mutex.matrix, con, sep="\t", append = TRUE, col.names = FALSE, quote = FALSE)

    return(mutex.matrix)
}


#' Create a .mat file which can be used with NBS clustering
#' (ref: http://chianti.ucsd.edu/~mhofree/wordpress/?page_id=26)
#' @title export.nbs.input
#'
#' @param x A TRONCO compliant dataset.
#' @param map_hugo_entrez Hugo_Symbol-Entrez_Gene_Id map
#' @param file output file name
#' @importFrom R.matlab writeMat
#' @export export.nbs.input
#' 
export.nbs.input <-function(x, 
                            map_hugo_entrez,
                            file = 'tronco_to_nbs.mat') {

    is.compliant(x);

    cat('*** Exporting for NBS v. 0.2\n')
    cat('Preparing binary input matrix\n')

    ## gene_indiv_mat <- the matrix
    gene_indiv_mat = as.matrix(x$genotypes)

    ## Remove colnames and rownames from gene_indiv_mat.
    
    rownames(gene_indiv_mat) = NULL
    colnames(gene_indiv_mat) = NULL

    cat('Preparing samples IDs \n')

    ## sample_id <- patient id
    sample_id = as.samples(x)

    cat('Preparing genes list (should be Hugo_Symbol) \n')
    
    ## gene_id_symbol <- sorted name of events
    gene_id_symbol = as.genes(x)

    cat('Preparing genes map (should be Hugo_Symbol -> Entrez_Gene_Id) \n')
    
    if (!('Hugo_Symbol' %in% colnames(map_hugo_entrez))) {
        stop('No Hugo_Symbol column in the input map: ', colnames(map_hugo_entrez))
    }
    if (!('Entrez_Gene_Id' %in% colnames(map_hugo_entrez))) {
        stop('No Entrez_Gene_Id column in the input map: ', colnames(map_hugo_entrez))
    }

    gene_id_all =
        mapply(function(x) as.numeric(map_hugo_entrez[[which(map_hugo_entrez[ ,'Hugo_Symbol'] == x),
                                                       'Entrez_Gene_Id']]),
               gene_id_symbol)

    file = if (grepl("\\.mat$", file)) file else paste0(file, ".mat")
    con = paste0(file)

    cat('Writing Matlab file to disk:', file,  ' ..... ' )
    writeMat(con,
             gene_indiv_mat = gene_indiv_mat,
             gene_id_all = gene_id_all,
             sample_id = sample_id,
             gene_id_symbol = gene_id_symbol)
    cat('DONE')
}


#' Create a list of unique Mutex groups for a given fdr cutoff
#' current Mutex version is Jan 8, 2015
#' (ref: https://code.google.com/p/mutex/ )
#'
#' @title import.mutex.groups
#' @param file Mutex results ("ranked-groups.txt" file)
#' @param fdr cutoff for fdr
#' @param display print summary table of extracted groups
#' @export import.mutex.groups
#' @importFrom utils count.fields
#' 
import.mutex.groups <- function(file, fdr=.2, display = TRUE) {
    ## Found somewhere on the web - makes sense.
    
    read.irregular <- function(filenm) {
        fileID <- file(filenm,open="rt")
        nFields <- count.fields(fileID)
        mat <- matrix(nrow=length(nFields),ncol=max(nFields))
        invisible(seek(fileID,where=0,origin="start",rw="read"))
        for (i in 1:nrow(mat) ) {
            mat[i, 1:nFields[i]] = scan(fileID, what = "", nlines = 1, quiet = TRUE)
        }
        close(fileID)
        df = data.frame(mat, stringsAsFactors = FALSE)
        return(df)
    }

    x = read.irregular(file)

    ## Check header.
    
    if (any(x[1,1:3] != c('Score', 'q-val', 'Members')))
        warning('File header does not seem to contain \'Score\', \'q-val\' and \'Members field\'.\n',
                'Are you sure this is a Mutex result file?')

    ## Remove header.
    
    cat(paste('*** Groups extracted - ', (nrow(x) -1), ' total groups.\n', sep=''))
    x = x[-1, , drop = FALSE]     # this is c('Score', 'q-val', 'Members')
    x[, 1] = as.numeric(x[,1])          # fdr
    x[, 2] = as.numeric(x[,2])          # q-value

    ## Remove groups  with low fdr.
    
    res = x[which(x[,1] < fdr), , drop = FALSE] 

    ## Remove duplicated groups (permutations).
    
    res.g = res[, 3:ncol(res)]

    for (i in 1:nrow(res.g))
        res[i,3:ncol(res)] = sort(res.g[i,], na.last = TRUE)   
    res = res[!duplicated((res[ , 3:ncol(res), drop = FALSE])), ] 

    cat(paste('Selected ',
              nrow(res),
              ' unique groups with fdr < ',
              fdr,
              '\n',
              sep = ''))

    ## Create groups.
    
    groups <- function(g) {
        g = g[3:length(g)]
        g = g[!is.na(g)]
        names(g) = NULL
        return(sort(g))
    }

    G = list()
    for (i in 1:nrow(res)) {
        gr = list(groups(res[i, ]))
        names(gr) = paste('MUTEX_GROUP', i, sep='')
        G = append(G,gr)  
    }

    rownames(res) = names(G)
    colnames(res)[1:2] = c('fdr', 'score')

    ## Summary report.
    
    if (display) 
        { 
            print(res)
        }
    return(G)
}


#' Check if there are multiple sample in x, according to TCGA barcodes naming
#' @title TCGA.multiple.samples
#' 
#' @examples
#' data(test_dataset)
#' TCGA.multiple.samples(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return A list of barcodes. NA if no duplicated barcode is found
#' @export TCGA.multiple.samples
#' 
TCGA.multiple.samples <- function(x) {
    is.compliant(x)

    samples = as.samples(x)
    samples.truncated = substring(samples, 0, 12)

    patients = unique(samples.truncated)

    if (length(patients) != nsamples(x)) {
        dup.samples.start = which(duplicated(samples.truncated)) 
        dup.samples.last = which(duplicated(samples.truncated, fromLast = TRUE))

        return(sort(samples[c(dup.samples.start, dup.samples.last)])) 
    } else
        return(NA)
}


#' If there are multiple sample in x, according to TCGA barcodes naming, remove them
#' @title TCGA.remove.multiple.samples
#' 
#' @examples
#' data(test_dataset)
#' TCGA.remove.multiple.samples(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return A TRONCO compliant dataset
#' @export TCGA.remove.multiple.samples
#' 
TCGA.remove.multiple.samples <- function(x) {
    is.compliant(x, err.fun = 'Removing TCGA multiple samples (input)')

    dup = TCGA.multiple.samples(x)
    dup.truncated = substring(dup, 0, 12)
    patients = unique(dup.truncated)

    for (i in 1:length(patients)) {
        patients.samples = which(dup.truncated == patients[i])
        multiple.samples = dup[patients.samples]

        cat('Patient', patients[i], 'with sample aliquotes\n' )
        print(substring(multiple.samples, 14, 29))

        keep = max(multiple.samples)
        discard = multiple.samples[which(multiple.samples != keep)]

        cat('Selecting', keep, '\n')
        x = delete.samples(x, discard)
    }

    is.compliant(x, err.fun = 'Removing TCGA multiple samples (output)')
    return(x)
}


#' Keep only the first 12 character of samples barcode if there are no duplicates
#' @title TCGA.shorten.barcodes
#' 
#' @examples
#' data(test_dataset)
#' TCGA.shorten.barcodes(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return A TRONCO compliant dataset
#' @export TCGA.shorten.barcodes
#' 
TCGA.shorten.barcodes <- function(x) {
    is.compliant(x, err.fun='Shartening TCGA barcodes (input)')

    ## Check if it has duplicated barcodes.
    
    if (!all(is.na(TCGA.multiple.samples(x))))
        stop(paste('This dataset contains multiple samples for some patients - cannot consolidate.',
                   '\n Samples with barcodes indicating multiple patients: \n',
                   paste(TCGA.multiple.samples(x), collapse = '\n'),
                   '.',
                   sep = ''))

    ## Shorten sample barcodes.
    
    rownames(x$genotypes) = substring(rownames(x$genotypes), 0, 12)
    if (has.stages(x)) rownames(x$stages) = rownames(x$genotypes)

    is.compliant(x, err.fun='Shartening TCGA barcodes (output)')
    return(x)    
}


#' Map clinical data from the TCGA format
#' @title  TCGA.map.clinical.data
#'
#' @param file A file with the clinical data
#' @param sep file delimiter
#' @param column.samples Required columns
#' @param column.map Map to the required columns
#' @return a map
#' @export TCGA.map.clinical.data
#' @importFrom utils read.delim
#' 
TCGA.map.clinical.data <- function(file, sep='\t', column.samples, column.map) {

    data =
        read.delim(file = file,
                   sep = sep,
                   header = TRUE,
                   stringsAsFactors = FALSE)

    if (!(column.samples %in% colnames(data))) 
        stop(paste('Cannot find samples column \"',
                   column.samples,
                   '\". Available columns: \n\t',
                   paste(colnames(data), collapse = '\n\t'),
                   sep = ''))

    if (!(column.map %in% colnames(data))) 
        stop(paste('Cannot find required map column \"',
                   column.map,
                   '\". Available columns: \n\t',
                   paste(colnames(data), collapse = '\n\t'),
                   sep = ''))

    map = data.frame(data[ , column.map], row.names = data[ , column.samples])
    colnames(map) = column.map

    return(map)
}


## Internal function

sample.RColorBrewer.colors <- function(palette, ncolors) {
    if (!palette %in% rownames(brewer.pal.info))
        stop('Invalid RColorBrewer palette.')

    pmax.cols = brewer.pal.info[palette, 'maxcolors']

    cols = min(pmax.cols , ncolors)
    cols = ifelse(cols < 3, 3, cols)

    colors = brewer.pal(n=cols, name=palette)
    if (ncolors < 3) colors = colors[1:ncolors]
    else colors =  colorRampPalette(colors)(ncolors)

    return(colors)
}

#' Create a graphML object which can be imported in cytoscape
#' This function is based on the tronco.plot fuction
#' 
#' @title  export.graphml
#' 
#' @examples
#' data(test_model)
#' export.graphml(test_model, file='text.xml', scale.nodes=0.3)
#'
#' @param x A TRONCO compliant dataset
#' @param file Where to save the output
#' @param ... parameters for tronco.plot
#' @export export.graphml
#' @importFrom igraph write.graph V V<- set.vertex.attribute
#' @importFrom igraph set.edge.attribute set.graph.attribute
#' @importFrom grDevices rgb col2rgb
#' @importFrom utils packageVersion
#' 
export.graphml <- function(x, file, ...) {

    is.compliant(x)
    is.model(x)

    plot.output = tronco.plot(x, export.igraph = TRUE, create.new.dev = FALSE, ...)
    graph = plot.output$graph
    nodes = plot.output$nodes
    edges = plot.output$edges
    description = plot.output$description
    models = plot.output$models
    node.names = V(graph)$name
    edge.names = names(edges$label)

    ## Display information about vertex name

    V(graph)$name = V(graph)$label

    ## Prepare and save vertex label

    vertex.label = sapply(node.names, function(node) {
        return(gsub("\\", "", nodes$label[node], fixed = TRUE))
    })
    V(graph)$label = vertex.label

    ## Prepare and save vertex type

    vertex.type = sapply(node.names, function(node) {
        if (is.logic.node(node)) {
            return('')
        }
        return(as.events(x)[node,'type'])
    })
    graph = set.vertex.attribute(graph, 'type', value=vertex.type)
    
    ## Prepare and save vertex bg color

    vertex.fillcolor = sapply(node.names, function(node){
        rgb(t(col2rgb(nodes$fillcolor[node])), maxColorValue = 255)
    })
    graph = set.vertex.attribute(graph, 'fillcolor', value=vertex.fillcolor)

    ## Prepare and save vertex font color

    vertex.fontcolor = sapply(node.names, function(node){
        rgb(t(col2rgb(nodes$fontcolor[node])), maxColorValue = 255)
    })
    graph = set.vertex.attribute(graph, 'fontcolor', value=vertex.fontcolor)

    ## Prepare and save vertex border color

    vertex.bordercolor = sapply(node.names, function(node){
        rgb(t(col2rgb(nodes$color[node])), maxColorValue = 255)
    })
    graph = set.vertex.attribute(graph, 'bordercolor', value=vertex.bordercolor)

    ## Prepare and save vertex shape

    vertex.shape = sapply(node.names, function(node){
        shape = nodes$shape[node]
        if (shape == 'box') {
            return('Rectangle')
        }
        return(shape)
    })
    graph = set.vertex.attribute(graph, 'shape', value=vertex.shape)

    ## Prepare and save vertex width

    vertex.width = sapply(node.names, function(node){
        return(nodes$width[node] * 50)
    })
    graph = set.vertex.attribute(graph, 'width', value=vertex.width)

    ## Prepare and save vertex height

    vertex.height = sapply(node.names, function(node){
        return(nodes$height[node] * 50)
    })
    graph = set.vertex.attribute(graph, 'height', value=vertex.height)

    ## Prepare and save label fontsize

    vertex.fontsize = sapply(node.names, function(node){
        return(nodes$fontsize[node] * 1.5)
    })
    graph = set.vertex.attribute(graph, 'fontsize', value=vertex.fontsize)

    ## Prepare and save border width

    vertex.borderwidth = sapply(node.names, function(node){
        return(nodes$lwd[node])
    })
    graph = set.vertex.attribute(graph, 'borderwidth', value=vertex.borderwidth)
    
    ## Prepare and save the attribute cases
    
    vertex.cases = sapply(node.names, function(node) {
      if (is.logic.node(node)) {
        return(0)
      }else{
        return(nodes$cases[node])
      }
      
    })
    
    graph = set.vertex.attribute(graph, 'cases', value = vertex.cases)
    
    ## Prepare and save the attribute sumGenotypes
    
    vertex.sum.genotypes = sapply(node.names, function(node) {
      if (is.logic.node(node)) {
        return(0)
      }
      return(nodes$sum.genotypes[node])
      
    })
    graph = set.vertex.attribute(graph, 'sum genotypes', value = vertex.sum.genotypes)

    ## Prepare and save vertex shape

    edge.line = sapply(edge.names, function(edge){
        line = edges$lty[edge]
        if (line == 'dashed') {
            return('dash')
        }
        return('solid')
    })
    graph = set.edge.attribute(graph, 'line', value=edge.line)

    ## Prepare and save arrow type

    edge.arrow = sapply(edge.names, function(edge){
        arrow = edges$arrowsize[edge]
        if (arrow == 1) {
            return('True')
        }
        return('False')
    })
    graph = set.edge.attribute(graph, 'arrow', value=edge.arrow)

    ## Prepare and save edge color

    edge.color = sapply(edge.names, function(edge){
        rgb(t(col2rgb(edges$color[edge])), maxColorValue = 255)
    })
    graph = set.edge.attribute(graph, 'color', value=edge.color)
    
    ## Save edge label
    
    graph = set.edge.attribute(graph, 'edgelabel', value = edges$label)
    
    ## Prepare and save label color
    
    edge.fontcolor = sapply(edge.names, function(edge){
      rgb(t(col2rgb(edges$fontcolor[edge])), maxColorValue = 255)
    })
    graph = set.edge.attribute(graph, 'labelcolor', value = edge.fontcolor)
    
    ## Save edge width
    
    graph = set.edge.attribute(graph, 'width', value = edges$lwd)
    
    ## Check if cofidence are given and prepare a column for each confidence
    
    if (length(edges$confidences) > 0) {
      for (c in colnames(edges$confidences)) {
        graph = set.edge.attribute(graph, c, value = edges$confidences[, c])
      }
    }

    ## Prepare and save graph attributes

    graph = set.graph.attribute(graph, 'name', description)
    graph = set.graph.attribute(graph, 
                                'models',
                                paste(x$parameters$algorithm, models, collapse=' - '))
    graph = set.graph.attribute(graph,
                                'informations',
                                paste0('Generated with TRONCO v', packageVersion('TRONCO')))
    eloss_output = tryCatch({
      eloss = as.kfold.eloss(x)
      for(i in 1:nrow(eloss)) {
        currmodel = paste0('eloss ', row.names(eloss)[i])
        currvalue = round(eloss[i,'Mean'], digits = 4)
        graph = set.graph.attribute(graph,
                                    currmodel,
                                    currvalue)
      }
    }, error = function(e) {})

    write.graph(graph, file=file, format='graphml')
}


#### end of file -- external.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


# create a reconstructed model
# param datataset genotypes matrix
# param best.parents result of perform.likelihood.fit.<alg>
# param prima.facie.parents result of get.prima.facie.parents <boot|no boot>
create.model <- function(dataset,
                         best.parents,
                         prima.facie.parents){

    ## Set the structure to save the conditional probabilities of
    ## the reconstructed topology.
    
    parents.pos.fit = array(list(),c(ncol(dataset),1));
    conditional.probs.fit = array(list(),c(ncol(dataset),1));

    ## Compute the conditional probabilities.
    
    for (i in 1:ncol(dataset)) {
        for (j in 1:ncol(dataset)) {
            if (i!=j && best.parents$adj.matrix$adj.matrix.fit[i, j] == 1) {
                parents.pos.fit[j,1] =
                    list(c(unlist(parents.pos.fit[j, 1]), i))
                conditional.probs.fit[j,1] =
                    list(c(unlist(conditional.probs.fit[j, 1]),
                           prima.facie.parents$joint.probs[i, j] /
                               prima.facie.parents$marginal.probs[i]))
            }
        }
    }
    parents.pos.fit[unlist(lapply(parents.pos.fit, is.null))] = list(-1)
    conditional.probs.fit[unlist(lapply(conditional.probs.fit, is.null))] = list(1)

    ## Perform the estimation of the probabilities if requested.
    
    estimated.error.rates.fit =
        list(error.fp = NA,
             error.fn = NA)
    estimated.probabilities.fit =
        list(marginal.probs = NA,
             joint.probs = NA,
             conditional.probs = NA)

    ## Set results for the current regolarizator
    probabilities.observed =
        list(marginal.probs = prima.facie.parents$marginal.probs,
             joint.probs = prima.facie.parents$joint.probs,
             conditional.probs = conditional.probs.fit)
    probabilities.fit =
        list(estimated.marginal.probs = estimated.probabilities.fit$marginal.probs,
             estimated.joint.probs = estimated.probabilities.fit$joint.probs,
             estimated.conditional.probs = estimated.probabilities.fit$conditional.probs)
    probabilities =
        list(probabilities.observed = probabilities.observed,
             probabilities.fit = probabilities.fit)
    parents.pos = parents.pos.fit
    error.rates = estimated.error.rates.fit

    ## Save the results for the model.

    result =
        list(probabilities = probabilities,
             parents.pos = parents.pos,
             error.rates = error.rates,
             adj.matrix = best.parents$adj.matrix)

    return(result)
}



#### end of file -- create.model.R

## ----include=FALSE------------------------------------------------------------
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)

## -----------------------------------------------------------------------------
dataset = rename.gene(aCML, 'TET2', 'new name')
dataset = rename.type(dataset, 'Ins/Del', 'new type')
as.events(dataset, type = 'new type')

## -----------------------------------------------------------------------------
dataset = join.events(aCML, 
    'gene 4',
    'gene 88',
    new.event='test',
    new.type='banana',
    event.color='yellow')

## -----------------------------------------------------------------------------
dataset = join.types(dataset, 'Nonsense point', 'Nonsense Ins/Del')
as.types(dataset)

## -----------------------------------------------------------------------------
dataset = delete.gene(aCML, gene = 'TET2')
dataset = delete.event(dataset, gene = 'ASXL1', type = 'Ins/Del')
dataset = delete.samples(dataset, samples = c('patient 5', 'patient 6'))
dataset = delete.type(dataset, type = 'Missense point')
view(dataset)

## -----------------------------------------------------------------------------
dataset = samples.selection(aCML, samples = as.samples(aCML)[1:3])
view(dataset)

## -----------------------------------------------------------------------------
dataset = events.selection(aCML,  filter.freq = .05, 
    filter.in.names = c('EZH1','EZH2'), 
    filter.out.names = 'SETBP1')

## -----------------------------------------------------------------------------
as.events(dataset)

## ----fig.width=7, fig.height=5.5, fig.cap="Multiple output from oncoprint can be captured as a gtable and composed via grid.arrange (package gridExtra). In this case we show  aCML data on top -- displayed after the as.alterations transformation -- versus a selected subdataset of events with a minimum frequency of 5%, force exclusion of SETBP1 (all events associated), and inclusion of EZH1 and EZH2.", results='hide'----
library(gridExtra)
grid.arrange(
    oncoprint(as.alterations(aCML, new.color = 'brown3'), 
        cellheight = 6, cellwidth = 4, gtable = TRUE,
        silent = TRUE, font.row = 6)$gtable,
    oncoprint(dataset, cellheight = 6, cellwidth = 4,
        gtable = TRUE, silent = TRUE, font.row = 6)$gtable, 
    ncol = 1)

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

# reconstruct the best topology based on probabilistic causation and Edmonds algorithm
# @title edmonds.fit
# @param dataset a dataset describing a progressive phenomenon
# @param regularization regularizators to be used for the likelihood fit
# @param score the score to be used, could be either pointwise mutual information (pmi) or conditional entropy (entropy)
# @param do.boot should I perform bootstrap? Yes if TRUE, no otherwise
# @param nboot integer number (greater than 0) of bootstrap sampling to be performed
# @param pvalue pvalue for the tests (value between 0 and 1)
# @param min.boot minimum number of bootstrapping to be performed
# @param min.stat should I keep bootstrapping untill I have nboot valid values?
# @param boot.seed seed to be used for the sampling
# @param silent should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @param hypotheses hypotheses to be considered in the reconstruction. This should be NA for this algorithms. 
# @return topology: the reconstructed tree topology
#
edmonds.fit <- function(dataset,
                        regularization = "no_reg",
                        score = "pmi",
                        do.boot = TRUE,
                        nboot = 100,
                        pvalue = 0.05,
                        min.boot = 3,
                        min.stat = TRUE,
                        boot.seed = NULL,
                        silent = FALSE,
                        epos,
                        eneg,
                        hypotheses = NA) {

    ## Start the clock to measure the execution time.
    
    ptm = proc.time();

    ## Structure with the set of valid edges
    ## I start from the complete graph, i.e., I have no prior and all
    ## the connections are possibly causal.
    
    adj.matrix = array(1, c(ncol(dataset), ncol(dataset)));
    colnames(adj.matrix) = colnames(dataset);
    rownames(adj.matrix) = colnames(dataset);

    ## The diagonal of the adjacency matrix should not be considered,
    ## i.e., no self cause is allowed
    diag(adj.matrix) = 0;

    ## Consider any hypothesis.
    
    adj.matrix = hypothesis.adj.matrix(hypotheses, adj.matrix);

    ## Check if the dataset is valid
    valid.dataset = check.dataset(dataset, adj.matrix, FALSE, epos, eneg)
    adj.matrix = valid.dataset$adj.matrix;
    invalid.events = valid.dataset$invalid.events;

    ## Should I perform bootstrap? Yes if TRUE, no otherwise
    if (do.boot == TRUE) {
        if (!silent)
            cat('*** Bootstraping selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.do.boot(dataset,
                                            hypotheses,
                                            nboot,
                                            pvalue,
                                            adj.matrix,
                                            min.boot,
                                            min.stat,
                                            boot.seed,
                                            silent,
                                            epos,
                                            eneg);
    } else {
        if (!silent)
            cat('*** Computing selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.no.boot(dataset,
                                            hypotheses,
                                            adj.matrix,
                                            silent,
                                            epos,
                                            eneg);
    }

    ## Add back in any connection invalid for the probability raising theory
    if (length(invalid.events) > 0) {
        # save the correct acyclic matrix
        adj.matrix.cyclic.tp.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp
        adj.matrix.cyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic
        adj.matrix.acyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.acyclic
        for (i in 1:nrow(invalid.events)) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.cyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.acyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
        }
        # if the new cyclic.tp contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp = adj.matrix.cyclic.tp.valid
        }
        # if the new cyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic = adj.matrix.cyclic.valid
        }
        # if the new acyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.acyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.acyclic = adj.matrix.acyclic.valid
        }
    }

    adj.matrix.prima.facie =
        prima.facie.parents$adj.matrix$adj.matrix.acyclic

    ## Perform the likelihood fit with the required strategy.
    
    model = list();
    for (reg in regularization) {
        for(my_score in score) {

            ## Perform the likelihood fit with the chosen regularization
            ## and the chosed score on the prima facie topology.
            
            if (!silent)
                cat('*** Performing likelihood-fit with regularization:', reg, 'and score:', my_score, '.\n')
            best.parents =
                perform.likelihood.fit.edmonds(dataset,
                                       adj.matrix.prima.facie,
                                       regularization = reg,
                                       score = my_score,
                                       marginal.probs = prima.facie.parents$marginal.probs,
                                       joint.probs = prima.facie.parents$joint.probs)
    
            ## Set the structure to save the conditional probabilities of
            ## the reconstructed topology.
    
            reconstructed.model = create.model(dataset,
                best.parents,
                prima.facie.parents)
    
            model.name = paste('edmonds', reg, my_score, sep='_')
            model[[model.name]] = reconstructed.model
            
        }
    }

    ## Set the execution parameters.
    
    parameters =
        list(algorithm = "EDMONDS",
             regularization = regularization,
             score = score,
             do.boot = do.boot,
             nboot = nboot,
             pvalue = pvalue,
             min.boot = min.boot,
             min.stat = min.stat,
             boot.seed = boot.seed,
             silent = silent,
             error.rates = list(epos=epos,eneg=eneg));

    ## Return the results.
    
    topology =
        list(dataset = dataset,
             hypotheses = hypotheses,
             adj.matrix.prima.facie = adj.matrix.prima.facie,
             adj.matrix.prima.facie.cyclic = prima.facie.parents$adj.matrix$adj.matrix.cyclic,
             confidence = prima.facie.parents$pf.confidence,
             model = model,
             parameters = parameters,
             execution.time = (proc.time() - ptm))
    topology = rename.reconstruction.fields(topology, dataset)
    return(topology)
}


# reconstruct the best causal topology by Edmonds algorithm combined with probabilistic causation
# @title perform.likelihood.fit.edmonds
# @param dataset a valid dataset
# @param adj.matrix the adjacency matrix of the prima facie causes
# @param regularization regularization term to be used in the likelihood fit
# @param score the score to be used by edmonds algorithm. Could be either pointwise mutual information (pmi) or conditional entropy (entropy)
# @param command type of search, either hill climbing (hc) or tabu (tabu)
# @return topology: the adjacency matrix of both the prima facie and causal topologies
#
perform.likelihood.fit.edmonds = function(dataset,
                                          adj.matrix,
                                          regularization,
                                          score,
                                          command = "hc",
                                          marginal.probs,
                                          joint.probs){

    data = as.categorical.dataset(dataset)
    adj.matrix.prima.facie = adj.matrix
    
    # adjacency matrix of the topology reconstructed by likelihood fit
    adj.matrix.fit = array(0,c(nrow(adj.matrix),ncol(adj.matrix)))
    rownames(adj.matrix.fit) = colnames(dataset)
    colnames(adj.matrix.fit) = colnames(dataset)
       
    # set at most one parent per node based on mutual information
    for (i in 1:ncol(adj.matrix)) {
            
        # consider the parents of i
        curr_parents = which(adj.matrix[,i] == 1)
        
        # if I have more then one valid parent
        if (length(curr_parents) > 1) {
            
            # find the best parent
            curr_best_parent = -1
            curr_best_score = -1
            for (j in curr_parents) {
                
                # if the event is valid
                if(joint.probs[i,j]>=0) {
                    # compute the chosen score
                    new_score = compute.edmonds.score(joint.probs[i,j],marginal.probs[i],marginal.probs[j],score)
                }
                # else, if the two events are indistinguishable
                else if(joint.probs[i,j]<0) {
                    new_score = Inf
                }
                
                if (new_score > curr_best_score) {
                    curr_best_parent = j
                    curr_best_score = new_score
                }
            }
            
            # set the best parent
            for (j in curr_parents) {
                if (j != curr_best_parent) {
                    adj.matrix[j,i] = 0
                }
            }
        }
    }
    
    # perform the likelihood fit if requested
    adj.matrix.fit = lregfit(data,
        adj.matrix,
        adj.matrix.fit,
        regularization,
        command)
    
    ## Save the results and return them.
    
    adj.matrix =
        list(adj.matrix.pf = adj.matrix.prima.facie,
             adj.matrix.fit = adj.matrix.fit)
    topology = list(adj.matrix = adj.matrix)
    return(topology)

}

# compute either pointwise mutual information (pmi) or conditional entropy (mle) for the edge j --> i
compute.edmonds.score = function( joint.prob.i.j, marginal.prob.i, marginal.prob.j, score ) {
    
    # variable to save the score
    new_score = NULL
    
    # this is the pointwise mutual information
    if(score=="pmi") {
        
        # compute the pointwise mutual information for i and j
        # that is log(P(i,j)/[P(i)*P(j)])
        new_score = log(joint.prob.i.j/(marginal.prob.i*marginal.prob.j))
        if(is.nan(new_score)) {
            new_score = 0
        }
        
    }
    # this is the mutual information
    else if(score=="mi") {
        
        # compute the mutual information for i and j
        new_score = compute.mi.score(joint.prob.i.j,marginal.prob.i,marginal.prob.j)
        
    }
    # this is the conditional entropy of i given j
    else if(score=="entropy") {
        
        # compute the 4 components of the conditional entropy
        h.i.j = joint.prob.i.j * 
                    log(marginal.prob.j/joint.prob.i.j)
        if(is.nan(h.i.j)) {
            h.i.j = 0
        }
        h.i.not.j = (marginal.prob.i - joint.prob.i.j)  * 
                    log((1-marginal.prob.j)/(marginal.prob.i - joint.prob.i.j))
        if(is.nan(h.i.not.j)) {
            h.i.not.j = 0
        }
        h.not.i.j = (marginal.prob.j - joint.prob.i.j) * 
                    log(marginal.prob.j/(marginal.prob.j - joint.prob.i.j))
        if(is.nan(h.not.i.j)) {
            h.not.i.j = 0
        }
        h.not.i.not.j = (1 - marginal.prob.i - marginal.prob.j + joint.prob.i.j) * 
                    log((1-marginal.prob.j)/(1 - marginal.prob.i - marginal.prob.j + joint.prob.i.j))
        if(is.nan(h.not.i.not.j)) {
            h.not.i.not.j = 0
        }
        
        # compute the entropy
        new_score = h.i.j + h.i.not.j + h.not.i.j + h.not.i.not.j
        
        # this is a maximization problem, hence we use the negate of the conditional entropy
        new_score = - new_score
        
    }
    # this is the pointwise mutual information of i and j adjusted for the normalized 
    # pointwise mutual information of i and not j
    else if(score=="cpmi") {
        
        # compute the first part of the score,
        # i.e., the pointwise mutual information for i and j
        # that is log(P(i,j)/[P(i)*P(j)])
        new_score = log(joint.prob.i.j/(marginal.prob.i*marginal.prob.j))
        if(is.nan(new_score)) {
            new_score = 0
        }
        
        # now compute the second part,
        # i.e., the normalized pointwise mutual information for i and not j
        if(new_score>0) {
            
            # compute first the pointwise mutual information for i and not j
            # that is log(P(i,not j)/[P(i)*P(not j)])
            pmi_i_not_j = log((marginal.prob.i-joint.prob.i.j)/(marginal.prob.i*(1-marginal.prob.j)))
            
            # compute the normalization factor for pmi_i_not_j,
            # that is -log(P(i,not j))
            norm_pmi_i_not_j = - log(marginal.prob.i-joint.prob.i.j)
            
            # compute the normalized pointwise mutual information for i and not j
            npmi_i_not_j = pmi_i_not_j/norm_pmi_i_not_j
            # now I correct for any NA (e.g., -Inf/Inf)
            if(is.nan(npmi_i_not_j)) {
                npmi_i_not_j = -1
            }
            
            # now I can correct new_score for npmi_i_not_j
            new_score = new_score * (- npmi_i_not_j)
            
        }
        
    }
    
    return(new_score)
    
}


#### end of file -- edmonds.algorithm.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

#' Add a new hypothesis by creating a new event and adding it to the compliant genotypes
#' @title hypothesis add
#' @param data A TRONCO compliant dataset.
#' @param pattern.label Label of the new hypothesis.
#' @param lifted.pattern Vector to be added to the lifted genotype resolving the pattern related to the new hypothesis 
#' @param pattern.effect Possibile effects for the pattern. 
#' @param pattern.cause Possibile causes for the pattern. 
#' @return A TRONCO compliant object with the added hypothesis
#' @export hypothesis.add
#' @importFrom stats fisher.test
#' @importFrom methods getPackageName is
#'
hypothesis.add <- function(data,
                           pattern.label,
                           lifted.pattern,
                           pattern.effect = "*",
                           pattern.cause = "*") {


    pattern.label = gsub('[[:space:]]+', '_', pattern.label)

    

    is.compliant(data)

    # check if there is a reconstructed model
    if(has.model(data)) {
        stop('This dataset has a reconstructed model and no more hypothesis can be added.')
    }

    genotypes = data$genotypes;
    annotations = data$annotations;

    if (!is.null(data$hypotheses)) {
        hypotheses = data$hypotheses;
    } else {
        hypotheses = NA;
    }
    
    ## The Boolean functions look for a set of variables.
    ## If there are already variables named as the ones
    ## used here, make the backup of them.
    
    do.roll.back.lifting.genotypes = FALSE;
    do.roll.back.lifting.annotations = FALSE;
    do.roll.back.lifting.edges = FALSE;
    do.roll.back.fisher.pvalues = FALSE;

    ## I need a variable to save the genotypes of the
    ## lifted pattern.
    
    ## If there is already a variable named
    ## lifting.genotypes, make the backup of it.

    hypotheses.env = get('hypotheses.env', asNamespace(getPackageName()))
    
    if (exists("lifting.genotypes", hypotheses.env)) {
        roll.back.lifting.genotypes = get('lifting.genotypes', envir = hypotheses.env)
        do.roll.back.lifting.genotypes = TRUE;
    }
    assign("lifting.genotypes", genotypes, envir = hypotheses.env);

    ## I need a variable to save the annotations of the
    ## lifted pattern.
    
    ## if there is already a variable named
    ## lifting.annotations, make the backup of it.
    
    if (exists("lifting.annotations", hypotheses.env)) {
        roll.back.lifting.annotations = get('lifting.annotations', envir = hypotheses.env)
        do.roll.back.lifting.annotations = TRUE;
    }
    assign("lifting.annotations", annotations, envir = hypotheses.env);

    ## I need a variable to save the edges of the lifted
    ## pattern.
    
    ## If there is already a variable named lifting.edges,
    ## make the backup of it.
    
    if (exists("lifting.edges", hypotheses.env)) {
        roll.back.lifting.edges = get('lifting.edges', envir = hypotheses.env)
        do.roll.back.lifting.edges = TRUE
    }
    assign("lifting.edges", NULL, envir = hypotheses.env);

    ## I need a variable to save the pvalues of the lifted
    ## pattern.
    
    ## If there is already a variable named fisher.pvalues,
    ## make the backup of it.
    
    if (exists("fisher.pvalues", hypotheses.env)) {
        roll.back.fisher.pvalues = get('fisher.pvalues', envir = hypotheses.env)
        do.roll.back.fisher.pvalues = TRUE;
    }
    assign("fisher.pvalues", NULL, envir = hypotheses.env);

    ## Save the lifted genotypes and its hypotheses for the
    ## current pattern.
    
    curr_pattern = lifted.pattern$pattern;
    curr_hypotheses = lifted.pattern$hypotheses;
    curr_pvalues = get('fisher.pvalues', envir = hypotheses.env)

    ## Save the edges of the lifted pattern.
    
    hstructure = get('lifting.edges', envir = hypotheses.env)

    ## Roll back to the previous value of the variable
    ## lifting.genotypes if any or remove it.
    
    if (do.roll.back.lifting.genotypes) {
        assign("lifting.genotypes",
               roll.back.lifting.genotypes,
               envir = hypotheses.env);
    } else {
        rm('lifting.genotypes', pos = hypotheses.env);
    }

    ## Roll back to the previous value of the variable
    ## lifting.annotations if any or remove it.
    
    if (do.roll.back.lifting.annotations) {
        assign("lifting.annotations",
               roll.back.lifting.annotations,
               envir = hypotheses.env);
    } else {
        rm('lifting.annotations', pos = hypotheses.env)
    }

    ## Roll back to the previous value of the variable
    ## lifting.edges if any or remove it.
    
    if (do.roll.back.lifting.edges) {
        assign("lifting.edges",
               roll.back.lifting.edges,
               envir = hypotheses.env)
    } else {
        rm('lifting.edges', pos = hypotheses.env)
    }

    ## Roll back to the previous value of the variable
    ## fisher.pvalues if any or remove it.
    
    if (do.roll.back.fisher.pvalues) {
        assign("fisher.pvalues",
               roll.back.fisher.pvalues,
               envir = hypotheses.env)
    } else {
        rm('fisher.pvalues', pos = hypotheses.env)
    }

    ## Set the hypotheses number.
    
    if (!is.na(hypotheses[1])) {
        num.hypotheses = hypotheses$num.hypotheses
    } else {
        num.hypotheses = 0
    }

    ## * is a special pattern.effect which indicates to use all
    ## the events as effects for this pattern.
    
    is.to.all.effects = FALSE;
    if (pattern.effect[[1]][1] == "*") {

        pattern.effect =
            colnames(genotypes)[1:(length(colnames(genotypes)) - num.hypotheses)];

        ## Any event can not be both causes and effects for the
        ## pattern to be well-formed.
        
        pattern.effect =
            list(pattern.effect[-which((pattern.effect %in% unlist(curr_hypotheses$llist)))]);
        is.to.all.effects = TRUE;

        if (length(pattern.effect) == 0) {
            stop(paste("[ERR] Missing list of effects to test or wildcard \'*\'.",
                       sep = ''));
        }
    }

    ## Check the pattern to be well-formed.
    
    all.col.nums = vector();
    if (length(pattern.effect) == 0) {
        stop(paste("[ERR] Missing list of effects or wildcard \'*\'.",
                   sep = ''));
    } else {

        ## Check the effects of the pattern to be well-formed.
        
        for (i in 1:length(pattern.effect)) {

            curr.pattern.effect = pattern.effect[[i]];
            if (is.to.all.effects == FALSE) {
                col.num = -1;
                if (length(curr.pattern.effect) == 1) {
                    event.map =
                        emap(c(curr.pattern.effect, "*"),
                             genotypes,
                             annotations);
                    col.num = event.map$col.num;
                    events.name = event.map$events.name;
                } else if (length(curr.pattern.effect) == 2) {
                    event.map =
                        emap(curr.pattern.effect,
                             genotypes,
                             annotations);
                    col.num = event.map$col.num;
                    events.name = event.map$events.name;
                }
            } else {
                col.num = which(colnames(genotypes) %in% curr.pattern.effect);
                if (length(col.num) == 0) {
                    col.num = -1;
                }
                events.name = curr.pattern.effect;
            }

            ## Check the effect to be a valid event.
            
            if (col.num[1] == -1) {
                stop(paste("[ERR] Unknown gene among effects: \"",
                           curr.pattern.effect,
                           "\".",
                           sep = ''));
            }
            all.col.nums = append(all.col.nums,col.num);

            ## Check the pattern to be well-formed.
            ## If the effect is in the pattern, the pattern is not
            ## well-formed.
            
            if (length(which(unlist(curr_hypotheses$llist) %in%
                             events.name)) > 0) {
                stop(paste("[ERR] Bad formed pattern, event \"",
                           curr.pattern.effect,
                           "\" yields a loop.",
                           sep = ''));
            }
        }
    }

    ## Look for duplicated effects in the pattern.
    
    if (anyDuplicated(all.col.nums) > 0) {
        stop(paste("[ERR] Bad formed pattern, duplicated events ", 
                   paste(pattern.effect[duplicated(pattern.effect)],
                         collapse = ', ',
                         sep = ''),
                   "within effects.",
                   sep = ''));
    }

    ## Check that the we are not duplicating any name by adding
    ## the new pattern.
    
    if (length(which(colnames(genotypes) == pattern.label)) > 0) {
        stop(paste("[ERR] This pattern already exists.", sep = ''));
    }

    ## Add the pattern to the genotypes.
    
    genotypes = cbind(genotypes,curr_pattern);

    ## Check that the pattern is valid according to Suppes'
    ## theory.
    ## Compute the observed and observed joint probabilities.
    
    pair.count <-
        array(0, dim = c(ncol(genotypes), ncol(genotypes)));
    
    ## Compute the probabilities on the genotypes.
    for (i in 1:ncol(genotypes)) {
        for (j in 1:ncol(genotypes)) {
            val1 = genotypes[ , i];
            val2 = genotypes[ , j];
            pair.count[i, j] = (t(val1) %*% val2);
        }
    }
    
    ## Marginal.probs is an array of the observed marginal
    ## probabilities.
    
    marginal.probs <-
        array(as.matrix(diag(pair.count) / nrow(genotypes)),
              dim = c(ncol(genotypes), 1));
    
    ## joint.probs is an array of the observed joint probabilities
    
    joint.probs <- as.matrix(pair.count / nrow(genotypes));
    
    ## Check that the probability of the pattern is in (0,1)
    
    if (marginal.probs[ncol(genotypes)] == 0
        || marginal.probs[ncol(genotypes)] == 1) {
        stop(paste("[ERR] The pattern has marginal probability ",
                   marginal.probs[ncol(genotypes)], 
                   ", which should be in (0, 1).",
                   sep = ''));
    }

    ## Check that the pattern does not duplicate any existing
    ## column.
    
    i = ncol(genotypes);
    for (j in 1:ncol(genotypes)) {

        ## If the edge is valid, i.e., not self cause.
        
        if (i != j) {

            ## If the two considered events are not
            ## distinguishable.
            
            if ((joint.probs[i, j] / marginal.probs[i]) == 1
                && (joint.probs[i, j] / marginal.probs[j]) == 1) {
                stop(paste("[ERR] Pattern duplicates ",
                           paste(as.events(data)[j , ],
                                 collapse = ' ',
                                 sep = ''), 
                           ".",
                           sep = ''));
            }
        }
    }

    ## * is a special pattern.cause which indicates to use all the
    ## events as causes for this pattern.
    
    is.to.all.causes = FALSE;
    if (pattern.cause[[1]][1] == "*") {

        pattern.cause =
            colnames(genotypes)[1:(length(colnames(genotypes)) - num.hypotheses-1)];

        ## Any event can not be both causes and effects for the
        ## pattern to be well-formed.
        
        pattern.cause =
            list(pattern.cause[- which((pattern.cause %in% unlist(curr_hypotheses$llist)))]);
        is.to.all.causes = TRUE;
    }

    ## Check the pattern to be well-formed.
    
    all.col.nums = vector();
    if (length(pattern.cause) > 0) {

        ## Check the causes of the pattern to be well-formed.
        
        for (i in 1:length(pattern.cause)) {

            curr.pattern.cause = pattern.cause[[i]];
            if (is.to.all.causes == FALSE) {
                col.num = -1;
                if (length(curr.pattern.cause) == 1) {
                    event.map =
                        emap(c(curr.pattern.cause, "*"),
                             genotypes,
                             annotations);
                    col.num = event.map$col.num;
                    events.name = event.map$events.name;
                } else if (length(curr.pattern.cause) == 2) {
                    event.map =
                        emap(curr.pattern.cause,
                             genotypes,
                             annotations);
                    col.num = event.map$col.num;
                    events.name = event.map$events.name;
                }
            } else {
                col.num = which(colnames(genotypes) %in% curr.pattern.cause);
                if (length(col.num) == 0) {
                    col.num = -1;
                }
                events.name = curr.pattern.cause;
            }

            ## Check the cause to be a valid event.
            
            if (col.num[1] == -1) {
                stop(paste("[ERR] Unknown gene among causes: \"",
                           curr.pattern.cause,
                           "\".",
                           sep = ''));
            }
            all.col.nums = append(all.col.nums, col.num);

            ## Check the pattern to be well-formed.
            ## If the cause is in the pattern, the pattern is not
            ## well-formed.
            
            if (length(which(unlist(curr_hypotheses$llist) %in% events.name)) > 0) {
                stop(paste("[ERR] Bad formed pattern, event \"",
                           curr.pattern.cause,
                           "\" yields a loop.",
                           sep = ''));
            }
        }
    }

    ## Look for duplicated causes in the pattern.
    
    if (anyDuplicated(all.col.nums)>0) {
        stop(paste("[ERR] Bad formed pattern, duplicated events ", 
                   paste(pattern.cause[duplicated(pattern.cause)],
                         collapse = ', ',
                         sep = ''),
                   "within causes.",
                   sep = ''));
    }

    ## Now I can finally add the hypothesis.
    
    colnames(genotypes)[ncol(genotypes)] = pattern.label;
    if (is.na(hypotheses[1])) {
        hypotheses = list();
    }
    hypotheses$num.hypotheses = num.hypotheses + 1;

    ## Add the new hypothesis in the annotations.
    
    annotations = rbind(data$annotations,c("Pattern", pattern.label));
    rownames(annotations)[nrow(annotations)] = pattern.label;

    ## Add the color of the type "Hypothesis" is not already
    ## defined.
    
    if (any(rownames(data$types) == "Pattern") == FALSE) {
        types = rbind(data$types, 'slateblue');
        rownames(types)[nrow(types)] = "Pattern";
        data$types = types;
    }

    ## Create the list of added hypotheses.
    
    if (length(hypotheses$hlist)==0) {
        hypotheses$hlist = vector();
    }
    
    ## Add the new hypothesis to the list.
    for (i in 1:length(pattern.effect)) {
        curr.pattern.effect = pattern.effect[[i]];
        if (is.to.all.effects == FALSE) {
            if (length(curr.pattern.effect) == 1) {
                event.map =
                    emap(c(curr.pattern.effect, "*"),
                         genotypes,
                         annotations);
                col.num = event.map$col.num;
            } else if (length(curr.pattern.effect) == 2) {
                event.map =
                    emap(curr.pattern.effect,
                         genotypes,
                         annotations);
                col.num = event.map$col.num;
            }
        } else {
            col.num = which(colnames(genotypes) %in% curr.pattern.effect);
            if (length(col.num) == 0) {
                col.num = -1;
            }
        }
        
        for (j in 1:length(col.num)) {
            hypotheses$hlist =
                rbind(hypotheses$hlist,
                      t(c(colnames(genotypes)[ncol(genotypes)],
                          colnames(genotypes)[col.num[j]])));
        }
        if (is.null(colnames(hypotheses$hlist))) {
            colnames(hypotheses$hlist) = c("cause", "effect");
        }
    }

    ## Add the causes of the new hypothesis to the list.
    
    if (length(pattern.cause) > 0) {
        for (i in 1:length(pattern.cause)) {
            curr.pattern.cause = pattern.cause[[i]];
            if (is.to.all.causes==FALSE) {
                if (length(curr.pattern.cause)==1) {
                    event.map =
                        emap(c(curr.pattern.cause, "*"),
                             genotypes,
                             annotations);
                    col.num = event.map$col.num;
                }
                else if (length(curr.pattern.cause)==2) {
                    event.map =
                        emap(curr.pattern.cause,
                             genotypes,
                             annotations);
                    col.num = event.map$col.num;
                }
            } else {
                col.num = which(colnames(genotypes) %in% curr.pattern.cause);
                if (length(col.num) == 0) {
                    col.num = -1;
                }
            }
            
            for (j in 1:length(col.num)) {
                hypotheses$hlist =
                    rbind(hypotheses$hlist,
                          t(c(colnames(genotypes)[col.num[j]],
                              colnames(genotypes)[ncol(genotypes)])));
            }
            
            if (is.null(colnames(hypotheses$hlist))) {
                colnames(hypotheses$hlist) = c("cause", "effect");
            }
        }
    }

    ## Create the list of hypotheses' structures.
    
    if (length(hypotheses$hstructure) == 0) {
        #hypotheses$hstructure = new.env(hash = TRUE, parent = emptyenv());
        hypotheses$hstructure = list()
    }
    hypotheses$hstructure[pattern.label] = list(get.lifted.pattern(hstructure))

    ## Add the atoms of the hypothesis.
    
    if (length(hypotheses$patterns) == 0) {
        hypotheses$patterns = list();
    }
    hypotheses$patterns[pattern.label] = lifted.pattern$hypotheses$llist;

    ## Add the hypotheses of the atoms.
    
    if (length(hypotheses$atoms) == 0) {
        hypotheses$atoms =
            vector(mode = "list",
                   length = (ncol(genotypes) - hypotheses$num.hypotheses));
        names(hypotheses$atoms) =
            colnames(genotypes)[1:(ncol(genotypes) - hypotheses$num.hypotheses)];
    }
    atoms.in.pattern =
        which(names(hypotheses$atoms) %in% unlist(hypotheses$patterns[pattern.label]));
    if (length(atoms.in.pattern) > 0) {
        for (i in 1:length(atoms.in.pattern)) {
            hypotheses$atoms[[atoms.in.pattern[i]]] =
                append(hypotheses$atoms[[atoms.in.pattern[i]]], pattern.label);
        }
    }

    ## Add the fisher pvalues.
    
    if (length(hypotheses$pvalues) == 0) {
        hypotheses$pvalues = vector();
    }
    hypotheses$pvalues = append(hypotheses$pvalues, list(curr_pvalues))
    names(hypotheses$pvalues)[length(hypotheses$pvalues)] = pattern.label;

    ## Return the new (compliant) data structure as result.
    
    data$genotypes = genotypes;
    data$annotations = annotations;
    data$hypotheses = hypotheses;

    return(data);
}


# resolve the ellipsis for the effects

hypothesis.lifted.effects <- function( ... ) {
    return(list(...));
}


#' Add all the hypotheses related to a group of events
#' @title hypothesis add group
#' @param x A TRONCO compliant dataset.
#' @param FUN Type of pattern to be added, e.g., co-occurance, soft or hard exclusivity.
#' @param group Group of events to be considered. 
#' @param pattern.cause Possibile causes for the pattern. 
#' @param pattern.effect Possibile effects for the pattern. 
#' @param dim.min Minimum cardinality of the subgroups to be considered.
#' @param dim.max Maximum cardinality of the subgroups to be considered.
#' @param min.prob Minimum probability associated to each valid group.
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO compliant object with the added hypotheses
#' @export hypothesis.add.group
#' @importFrom utils flush.console combn
#' 
hypothesis.add.group <- function(x, 
                                 FUN, 
                                 group, 
                                 pattern.cause = '*',
                                 pattern.effect = '*',
                                 dim.min = 2,
                                 dim.max = length(group),
                                 min.prob = 0,
                                 silent = FALSE) {

    is.compliant(x)
    # check if there is a reconstructed model
    if (has.model(x)) {
        stop('This dataset has a reconstructed model and no more hypothesis can be added.')
    }

    op = deparse(substitute(FUN))

    effect = paste0("c('", paste(pattern.effect, collapse = "', '"), "')")
    cause = paste0("c('", paste(pattern.cause, collapse = "', '"), "')")

    ngroup = length(group)
    if (ngroup < 2) {
        warning("No hypothesis will be created for groups with less than 2 elements.")
        return(x)
    }

    if (!silent) {
        cat("*** Adding Group Hypotheses\n")
        cat(' Group:', paste(group, collapse = ", ", sep = ""), '\n')
        cat(' Function:', op, '\n')
        cat(' Cause:', paste(pattern.cause, collapse=", "), '; ')
        cat(' Effect:', paste(pattern.effect, collapse=", "), '.\n')
    }

    if (min.prob > 0) {
        if (!silent)  {
            cat('\nFiltering genes within the group with alteration frequency below',
                min.prob,
                '\n')
        }

        temp = events.selection(x, filter.in.names = group, silent = silent)
        temp = as.alterations(temp, silent = silent)
        temp = events.selection(temp, filter.freq = min.prob, silent = silent)

        group = as.genes(temp)
        if (!silent) {
            cat('New group:', paste(group, collapse = ", ", sep = ""), '\n')
        }
    }

    ngroup = length(group)
    if (ngroup < 2) {
        warning("No hypothesis will be created for groups with less than 2 elements.")
        return(x)
    }

    hom.group =
        lapply(group,
               function(g, x) {
                   ## Isn't (nevents(x, genes = g) > 1) enough?
                   if (nevents(x, genes = g) > 1) {
                       TRUE
                   } else { 
                       FALSE
                   }
               },
               x)
    hom.group = group[unlist(hom.group)]

    gene.hom <- function(g, h) {
        if (g %in% h) {
            if (any(rowSums(as.gene(x, genes = g)) > 1) ) return(paste0("OR('", g, "')"))
            else return(paste0("XOR('", g, "')"))
        }
        return(paste0("'", g, "'"))
    }

    max.groupsize = min(dim.max, ngroup)
    min.groupsize = max(2, dim.min)

    if (dim.min > dim.max) {
        stop('ERROR - dim.min > dim.max')
    }
    
    if (min.groupsize > max.groupsize) {
        stop('ERROR - min.groupsize > max.groupsize')
    }

    if (length(hom.group) > 0 && !silent) { 
        cat("Genes with multiple events: ",
            paste(unlist(hom.group),
                  collapse=', ',
                  sep = ''),
            "\n")
    }

    error.summary = data.frame()

    ## Get an analytical pattern... !
    
    tot.patterns = 0
    for (i in min.groupsize:max.groupsize) {
        tot.patterns = tot.patterns + ncol(combn(unlist(group), i))
    }

    ## Create a progress bar.
    if (!silent) {
        cat('Generating ',
            tot.patterns,
            'patterns [size: min =',
            max.groupsize,
            ' -  max =',
            max.groupsize,
            '].\n')
    }

    ## pb <- txtProgressBar(0, tot.patterns, style = 3)
    
    pbPos = 0
    for (i in min.groupsize:max.groupsize) {
        gr = combn(unlist(group), i)

        for (j in 1:ncol(gr)) {
            genes = as.list(gr[, j])

            ## Start the progress bar.
            pbPos = pbPos + 1

            hypo.name = paste(unlist(genes), sep = "_", collapse = "_")
            hypo.genes =
                paste(lapply(genes,
                             function(g, hom.group) {
                                 gene.hom(g, hom.group)
                             },
                             hom.group),
                      collapse = ", ")

            hypo.add = paste0("hypothesis.add(x, ", 
                "pattern.label = '", op, "_", hypo.name, "', ",
                "lifted.pattern = ", op, "(", hypo.genes, "), ",
                "pattern.effect = ", effect, ", ",
                "pattern.cause = ", cause, ")")

            err =
                tryCatch({
                    x = eval(parse(text = hypo.add))
                }, error = function(cond) {

                    m = paste("Error on", hypo.add, ".\n", cond)
                    code = strsplit(as.character(cond), " ")[[1]]
                    idx.errcode = which(code == "[ERR]", arr.ind = TRUE) + 1

                    return(
                        data.frame(
                            pattern = paste(unlist(genes), collapse = ", ", sep = ""), 
                            error = paste(code[idx.errcode:length(code)], collapse = " ")
                        ))

                }, warning = function(cond) {
                    m = paste("Warning on", hypo.add, ".\n", cond)
                    return(genes)
                })
            ## Dummy errors detection.
            
            if (!("genotypes" %in% names(err))) 
                error.summary = rbind(error.summary, err)
        }
    }


    if (nrow(error.summary) > 0) {
        cat(paste(nrow(error.summary),
                  " genes pattern could not be added -- showing errors\n",
                  sep = ""))
        print(error.summary)
    } else if (!silent) {
        cat("Hypothesis created for all possible patterns.\n")
    }
    return(x)
}


#' Add all the hypotheses related to homologou events
#' @title hypothesis.add.homologous
#' @param x A TRONCO compliant dataset.
#' @param pattern.cause Possibile causes for the pattern. 
#' @param pattern.effect Possibile effects for the pattern. 
#' @param genes List of genes to be considered as possible homologous. For these genes, all the types of mutations will be considered functionally equivalent.
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO compliant object with the added hypotheses
#' @export hypothesis.add.homologous
#' 
hypothesis.add.homologous <- function(x, 
                                      pattern.cause = '*',
                                      pattern.effect = '*',
                                      genes = as.genes(x),
                                      silent = FALSE) {

    is.compliant(x)
    # check if there is a reconstructed model
    if(has.model(x)) {
        stop('This dataset has a reconstructed model and no more hypothesis can be added.')
    }

    hom.group =
        lapply(genes,
               function(g, x) {
                   if (nevents(x, genes = g) > 1) {
                       TRUE
                   } else {
                    FALSE
                   }
               },
               x)

    hom.group = genes[unlist(hom.group)]

    if (length(hom.group) == 0) {
        warning("No genes with multiple events.")
        return(x)
    }

    if (!silent) {
        cat("*** Adding hypotheses for Homologous Patterns\n")
        cat(' Genes:', paste(hom.group, collapse = ", ", sep = ""), '\n')
        cat(' Cause:', paste(pattern.cause, collapse=", "), '\n')
        cat(' Effect:', paste(pattern.effect, collapse=", "), '\n')
    }

    effect = paste0("c('", paste(pattern.effect, collapse = "', '"), "')")
    cause = paste0("c('", paste(pattern.cause, collapse = "', '"), "')")

    if (length(hom.group) == 0) {
        warning("No genes with multiple events.")
        return(x)
    }

    ## Create a progress bar.
    
    #pb <- txtProgressBar(0, length(hom.group), style = 3)

    error.summary = data.frame()

    for (i in 1:length(hom.group)) {

        ## Start the progress bar.
        
        #setTxtProgressBar(pb, i)

        if (!silent) {
            cat('.')
        }
                
        ## Check if the joint probability of homologous events is > 0,
        ## if yes the event will be added as 'OR', otherwise 'XOR'.
        
        if ( any(rowSums(as.gene(x, genes = hom.group[[i]])) > 1)) {
            FUN = 'OR'
        } else {
            FUN = 'XOR'       
        }

        hypo.add =
            paste0("hypothesis.add(x, ",
                   "pattern.label = '", FUN, "_", hom.group[[i]], "', ",
                   "lifted.pattern = ", FUN, "('", hom.group[[i]], "'), ",
                   "pattern.cause = ",  cause, ", ",
                   "pattern.effect =", effect, ")")

        err =
            tryCatch({
                x = eval(parse(text = hypo.add))
            }, error = function(cond) {
                m = paste("Error on", hypo.add, ".\n", cond)
                code = strsplit(as.character(cond), " ")[[1]]
                idx.errcode = which(code == "[ERR]", arr.ind = TRUE) + 1

                return(data.frame(pattern =
                                      paste(unlist(hom.group[[i]]),
                                            collapse = ", ",
                                            sep = ""),
                                  error =
                                      paste(code[idx.errcode:length(code)],
                                            collapse = " ")))

            }, warning = function(cond) {
                m = paste("Warning on", hypo.add, ".\n", cond)
                return(genes)
            })
        
        ## Dummy errors detection.

        if (!("genotypes" %in% names(err))) 
            error.summary = rbind(error.summary, err)
    }

    ## Close progress bar.
    #close(pb)
    
    if (nrow(error.summary) > 0) {
        cat(paste(nrow(error.summary),
                  " patterns could not be added -- showing errors\n",
                  sep = ""))
        print(error.summary)
    } else if (!silent) {
        cat("Hypothesis created for all possible gene patterns.\n")
    }

    return(x)
}

# Internal function for hypotheses expansion
# @title hypotheses.expansion
# @param input_matrix A TRONCO adjacency matrix
# @param map hypothesis name - hypothesis adjacency matrix map
# @param expand Should I expand the hypotheses?
# @param skip.disconnected Hide disconnected node
#
hypotheses.expansion <- function(input_matrix, 
                                 map = list(),
                                 expand = TRUE,
                                 skip.disconnected = TRUE
                                 ) {
    
    ## Get node list.
    
    node_list = colnames(input_matrix)

    ## Cut input matrix.
    
    num_hypos = 0
    if (length(map) > 0) {
        num_hypos =
            Reduce(sum,
                   lapply(ls(map),
                          function(x, y) {
                              if (x %in% y)
                                  return(1)
                          },
                          y = node_list))
    }

    margin = length(node_list) - num_hypos
    hypos_new_name = list()

    ## Check if there are hypotheses.
    
    if (num_hypos == 0 || !expand) {
        
        ## If no hypos do nothing...
        
        min_matrix = input_matrix
    } else {
        ## ..else expand them.
        
        min_matrix =
            input_matrix[-(margin + 1):-length(node_list),
                         -(margin + 1):-length(node_list)]

        ## Create graph from matrix.
        
        min_graph = graph.adjacency(min_matrix)

        for (h in ls(map)) {

            if (! h %in% node_list) {
                next
            }

            hypo = map[[h]]

            ## Create graph from hypo.
            
            hypo_graph = graph.adjacency(hypo)

            ## Name of this node.
            
            h_mat <- rowSums(get.adjacency(hypo_graph, sparse=FALSE))

            initial_node <- names(h_mat)[which(h_mat == 0)]
            hypos_new_name[initial_node] = h

            ## Change names in confidence matrix according to
            ## hypotesis.
            
            display.up = FALSE
            if (length(which(input_matrix[, h] == 1)) != 0) {
                display.up = TRUE
            }

            display.down = FALSE
            if (length(which(input_matrix[h, ] == 1)) != 0) {
                display.down = TRUE
            }

            ## Display up hypo and reconnect.
            
            if (display.up) {

                hypo_pre = t(hypo)

                node_names = rownames(hypo_pre)
                node_names =
                    lapply(node_names,
                           function(x) {
                               if (is.logic.node(x)) {
                                   paste0('UP', x)
                               } else {
                                   return(x)
                               }
                           })

                rownames(hypo_pre) = node_names
                colnames(hypo_pre) = node_names

                ## Create graph from hypo.
                
                hypo_graph_pre = graph.adjacency(hypo_pre)

                ## Name of this node.
                
                h_mat_pre = colSums(get.adjacency(hypo_graph_pre, sparse = FALSE))

                final_node = names(h_mat_pre)[which(h_mat == 0)]
                hypos_new_name[final_node] = h

                ## Edge to reconstruct.
                
                h_edge = input_matrix[, h]
                initial_node_up = names(h_edge)[which(h_edge == 1)]

                ### Expand any hipothesis within final nodes.
                # 
                # if (length(map) > 0) {
                #   initial_node_up_hyp = which(initial_node_up%in%names(map))
                #   if (length(initial_node_up_hyp) > 0) {
                #     final_node_hyp_expanded = colnames(map[[initial_node_up[initial_node_up_hyp]]])[1:(length(colnames(map[[initial_node_up[initial_node_up_hyp]]]))-1)]
                #     initial_node_up = initial_node_up[-initial_node_up_hyp]
                #     initial_node_up = c(initial_node_up, final_node_hyp_expanded)
                #   }
                # }
                
                ## Add this graph to main graph.
                
                min_graph = graph.union(min_graph, hypo_graph_pre)

                ## Recreate lost edge.
                
                for (node in initial_node_up) {
                    min_graph = min_graph + edge(node, final_node)
                }
            }

            ## Display down hypo.
            
            if (display.down) {
                
                ## Edge to reconstruct.
                
                h_edge <- input_matrix[h,]
                final_node <- names(h_edge)[which(h_edge == 1)]
                
                ### Expand any hipothesis within final nodes.
                # 
                # if (length(map) > 0) {
                #   final_node_hyp = which(final_node%in%names(map))
                #   if (length(final_node_hyp) > 0) {
                #     final_node_hyp_expanded = colnames(map[[final_node[final_node_hyp]]])[1:(length(colnames(map[[final_node[final_node_hyp]]]))-1)]
                #     final_node = final_node[-final_node_hyp]
                #     final_node = c(final_node, final_node_hyp_expanded)
                #   }
                # }
                
                ## Add this graph to main graph.
                
                min_graph = graph.union(min_graph, hypo_graph)

                ## Recreate lost edge.
                
                for (node in final_node) {
                    min_graph = min_graph + edge(initial_node, node)
                }
                
            }
        }
        min_matrix = get.adjacency(min_graph, sparse = FALSE)
    }
    ## Sort col and row (igraph wants the same order).
    
    min_matrix = min_matrix[,order(colnames(min_matrix))]
    min_matrix = min_matrix[order(rownames(min_matrix)),]
    return(list(min_matrix, hypos_new_name))
}

# Utility function to add the hypotheses
aux.log <- function( genotypes, annotations, function.name, ... ) {

    hypotheses.env = get('hypotheses.env', asNamespace(getPackageName()))

    if (!is.null(genotypes)
        && !is.null(annotations)
        && length(list(...)) > 0) {

        clauses = list(...)
        curr_genotypes = array(0, c(nrow(genotypes), length(clauses)))
        hypotheses = list()
        function.inputs = list()
        fisher.tests = vector()

        for (i in 1:length(clauses)) {

            ## If the clause is given by name, get the column from the
            ## genotypes.
            
            if (typeof(clauses[[i]]) == "character") {

                col.num = -1
                ## If I have the label, get the column in the
                ## genotypes for this event.
                
                if (length(clauses[[i]]) == 1) {
                    event.map = emap(c(clauses[[i]],"*"), genotypes, annotations)
                    col.num = event.map$col.num
                }
                else if (length(clauses[[i]]) == 2) {
                    event.map = emap(clauses[[i]], genotypes, annotations)
                    col.num = event.map$col.num
                }

                if (col.num[1] == -1) {
                    stop(paste("[ERR] No events for gene ",
                               paste(clauses[[i]], collapse = ', ', sep = '')))
                }
                else {
                    curr_genotypes[, i] = genotypes[, col.num[1]]
                    if (length(col.num) > 1) {
                        curr_genotypes =
                            cbind(curr_genotypes,
                                  genotypes[ , col.num[2:length(col.num)]])
                    }

                    if (length(hypotheses$llist) == 0) {
                        hypotheses$llist = list(event.map$events.name)
                    }
                    else {
                        hypotheses$llist = list(c(unlist(hypotheses$llist), event.map$events.name))
                    }
                    
                    for (j in 1:length(event.map$events.name)) {
                        function.name =
                            paste(function.name, "_",
                                  event.map$events.name[j],
                                  sep = "")
                        function.inputs = c(function.inputs, event.map$events.name[j])
                    }
                }
            } else {
                ## Otherwise I already have the column as a vector.
                
                curr_genotypes[,i] = clauses[[i]]$pattern
                
                ## if it is a list
                
                if (length(hypotheses$llist) == 0) {
                    hypotheses$llist = clauses[[i]]$hypotheses$llist
                } else {
                    hypotheses$llist = list(c(unlist(clauses[[i]]$hypotheses$llist),unlist(hypotheses$llist)))
                }
                function.name = paste(function.name, "_", clauses[[i]]$function.name, sep="")
                function.inputs = c(function.inputs, clauses[[i]]$function.name)
            }
        }

        result =
            list(curr_genotypes = curr_genotypes,
                 hypotheses = hypotheses,
                 function.name = function.name,
                 function.inputs = function.inputs,
                 tests = pairwise.fisher.test(curr_genotypes))

        ## Save the new edges
        
        for (k in 1:length(result$function.inputs)) {
            lifting.edges.temp =
                rbind(get('lifting.edges', envir = hypotheses.env),
                      c(result$function.inputs[[k]], result$function.name))
            assign("lifting.edges", lifting.edges.temp, envir = hypotheses.env)
        }
        return(result)
    } else {
        stop("[ERR] Either the genotypes or the pattern not provided! No hypothesis will be created.")
    }
    return(NA)
}


#' AND hypothesis
#' @title AND
#' @param ... Atoms of the co-occurance pattern given either as labels or as partielly lifted vectors.
#' @return Vector to be added to the lifted genotype resolving the co-occurance pattern
#' @export AND
#
AND <- function( ... ) {
    
    ## Look for the variables named lifting.genotypes and
    ## lifting.annotations.

    hypotheses.env = get('hypotheses.env', asNamespace(getPackageName()))
    
    genotypes = get('lifting.genotypes', envir = hypotheses.env)
    annotations = get('lifting.annotations', envir = hypotheses.env)
    fisher.pvalues.temp = get('fisher.pvalues', envir = hypotheses.env)
    if (!is.null(genotypes)
        && !is.null(annotations)
        && length(list(...)) > 0) {
        ## Get the vector of the clauses of the pattern from the
        ## genotypes.
        
        result = aux.log(genotypes,annotations, "AND" ,...)
        curr_genotypes = result$curr_genotypes
        hypotheses = result$hypotheses
        function.name = result$function.name
        function.inputs = result$function.inputs

        ## Save the fisher exact tests.
        if (length(result$tests) > 0) {
            for (i in 1:length(result$tests)) {
                curr.test = result$tests[[i]]
                odds.ratio = curr.test[2]

                if (curr.test[2] <= 0) {
                    curr.pvalue = 1
                } else {
                    curr.pvalue = curr.test[1]
                }

                fisher.pvalues.temp = append(fisher.pvalues.temp, curr.pvalue)
            }
            assign("fisher.pvalues", fisher.pvalues.temp, envir = hypotheses.env)
        }

        ## Evaluate the AND operator.
        
        pattern = rep(0, nrow(genotypes))
        for (i in 1:nrow(genotypes)) {
            pattern[i] = sum(curr_genotypes[i, ])
            if (pattern[i]<ncol(curr_genotypes)) {
                pattern[i] = 0
            } else {
                pattern[i] = 1
            }
        }
        pattern = as.integer(pattern)
        result =
            list(pattern = pattern,
                 hypotheses = hypotheses,
                 function.name = function.name,
                 function.inputs = function.inputs,
                 fisher.pvalues = fisher.pvalues.temp)
        return(result)
    } else {
        stop("[ERR] Either the genotypes or the pattern not provided! No hypothesis will be created.");
    }
    return(NA)
}


#' OR hypothesis
#' @title OR
#' @param ... Atoms of the soft exclusive pattern given either as labels or as partielly lifted vectors.
#' @return Vector to be added to the lifted genotype resolving the soft exclusive pattern
#' @export OR
#
OR <- function( ... ) {
    ## Look for the variables named lifting.genotypes and
    ## lifting.annotations.

    hypotheses.env = get('hypotheses.env', asNamespace(getPackageName()))
    
    genotypes = get('lifting.genotypes', envir = hypotheses.env)
    annotations = get('lifting.annotations', envir = hypotheses.env)
    fisher.pvalues.temp = get('fisher.pvalues', envir = hypotheses.env)
    if (!is.null(genotypes)
        && !is.null(annotations)
        && length(list(...)) > 0) {
        
        ## Get the vector of the clauses of the pattern from the
        ## genotypes.
        
        result = aux.log(genotypes,annotations, "OR", ...)
        curr_genotypes = result$curr_genotypes
        hypotheses = result$hypotheses
        function.name = result$function.name
        function.inputs = result$function.inputs

        ## Save the fisher exact tests.
        
        if (length(result$tests) > 0) {
            for (i in 1:length(result$tests)) {
                curr.test = result$tests[[i]]
                curr.p.value = 1 - curr.test[1]

                fisher.pvalues.temp = append(fisher.pvalues.temp, curr.p.value)
            }
            assign("fisher.pvalues", fisher.pvalues.temp, envir = hypotheses.env)
        }

        ## Evaluate the OR operator.
        
        pattern = rep(0, nrow(genotypes))
        for (i in 1:nrow(genotypes)) {
            pattern[i] = sum(curr_genotypes[i, ])
            if (pattern[i] > 1) {
                pattern[i] = 1
            }
        }
        pattern = as.integer(pattern)
        result =
            list(pattern = pattern,
                 hypotheses = hypotheses,
                 function.name = function.name,
                 function.inputs = function.inputs,
                 fisher.pvalues = fisher.pvalues.temp)
        return(result)
    } else {
        stop("[ERR] Either the genotypes or the pattern not provided! No hypothesis will be created.");
    }
    return(NA)
}

#' XOR hypothesis
#' @title XOR
#' @param ... Atoms of the hard exclusive pattern given either as labels or as partielly lifted vectors.
#' @return Vector to be added to the lifted genotype resolving the hard exclusive pattern
#' @export XOR
#' 
XOR <- function( ... ) {
    ## Look for the variables named lifting.genotypes and
    ## lifting.annotations.
    
    hypotheses.env = get('hypotheses.env', asNamespace(getPackageName()))

    genotypes = get('lifting.genotypes', envir = hypotheses.env)
    annotations = get('lifting.annotations', envir = hypotheses.env)
    fisher.pvalues.temp = get('fisher.pvalues', envir = hypotheses.env)
    if (!is.null(genotypes)
        && !is.null(annotations)
        && length(list(...)) > 0) {
        
        ## Get the vector of the clauses of the pattern from the
        ## genotypes.
        
        result = aux.log(genotypes, annotations, "XOR", ...)
        curr_genotypes = result$curr_genotypes
        hypotheses = result$hypotheses
        function.name = result$function.name
        function.inputs = result$function.inputs

        ## Save the fisher exact tests.
        
        if (length(result$tests)>0) {
            for (i in 1:length(result$tests)) {
                curr.test = result$tests[[i]]
                odds.ratio = curr.test[2]

                if (curr.test[2] >= 0) {
                    curr.pvalue = 1
                }
                else {
                    curr.pvalue = curr.test[1]
                }
                fisher.pvalues.temp = append(fisher.pvalues.temp, curr.pvalue)
            }
            assign("fisher.pvalues", fisher.pvalues.temp, envir = hypotheses.env)
        }

        ## Evaluate the XOR operator.
        
        pattern = rep(0,nrow(genotypes))
        for (i in 1:nrow(genotypes)) {
            pattern[i] = sum(curr_genotypes[i,])
            if (pattern[i]>1) {
                pattern[i] = 0
            }
        }
        pattern = as.integer(pattern)
        result =
            list(pattern = pattern,
                 hypotheses = hypotheses,
                 function.name = function.name,
                 function.inputs = function.inputs,
                 fisher.pvalues = fisher.pvalues.temp)
        return(result)
    } else {
        stop("[ERR] Either the genotypes or the pattern not provided! No hypothesis will be created.");
    }
    return(NA);
}


### Return the position in the genotypes of the event referring to the
### given label.event.

emap <- function(label.event, genotypes, annotations) {
    col.num = -1;
    events.name = "";
    if (!is.null(genotypes) && !is.null(annotations)) {
        if (label.event[2] != "*") {
            curr.events = which(annotations[, "event"] == label.event[1] &
                                    annotations[, "type"] == label.event[2]);
        } else {
            curr.events = which(annotations[, "event"] == label.event[1]);
        }
        if (length(curr.events) > 0) {
            events.name = names(curr.events);
            col.num = which(colnames(genotypes) %in% events.name);
        }
    } else {
        stop("[ERR] A genotypes must be available in order to define any hypothesis!",call.=FALSE);
    }
    results = list(col.num = col.num, events.name = events.name);
    return(results);
}


### Return the adjacency matrix of the pattern given the list of edges
### involving it.

get.lifted.pattern <- function(lifted.edges) {
    ## Structure to save the adjacency matrix.
    
    lifted.adj.matrix =
        array(0,
              c(length(unique(c(lifted.edges[,1],lifted.edges[,2]))),
                length(unique(c(lifted.edges[,1],lifted.edges[,2])))));
    rownames(lifted.adj.matrix) = unique(c(lifted.edges[,1],lifted.edges[,2]));
    colnames(lifted.adj.matrix) = rownames(lifted.adj.matrix);
    
    ## Build the matrix given the lifted.edges.
    
    for (i in 1:nrow(lifted.edges)) {
        lifted.adj.matrix[lifted.edges[i, 1], lifted.edges[i, 2]] = 1;
    }
    return(lifted.adj.matrix);
}


### given the hypotheses and the adj.matrix, return the updated
### adj.matrix

hypothesis.adj.matrix <- function(hypotheses, adj.matrix) {

    if (!is.na(hypotheses[1])) {

        ## Set the invalid entries in the adj.matrix hypotheses can
        ## not be causing other hypotheses.
        
        adj.matrix[(ncol(adj.matrix) - hypotheses$num.hypotheses + 1):ncol(adj.matrix),
                   (ncol(adj.matrix) - hypotheses$num.hypotheses + 1):ncol(adj.matrix)] = 0;

        ## Consider the given hypotheses only against the specified possible effects.

        adj.matrix[(ncol(adj.matrix) - hypotheses$num.hypotheses + 1):ncol(adj.matrix),
                   1:(ncol(adj.matrix) - hypotheses$num.hypotheses)] = 0
        adj.matrix[1:(ncol(adj.matrix) - hypotheses$num.hypotheses),
                   (ncol(adj.matrix) - hypotheses$num.hypotheses + 1):ncol(adj.matrix)] = 0

        ## Set the elements from the hlist.
        
        for (i in 1:nrow(hypotheses$hlist)) {
            cause = which(colnames(adj.matrix) %in% hypotheses$hlist[i, "cause"]);
            effect = which(colnames(adj.matrix) %in% hypotheses$hlist[i, "effect"]);
            if (length(cause)>0 && length(effect)>0) {
                adj.matrix[cause,effect] = 1;
            }
        }
    }
    return(adj.matrix);
}


### performs pairwise exact fisher test.
pairwise.fisher.test <- function(data) {

    ## Structure to save the results.
    
    results = vector();

    if (ncol(data) > 1) {
        for (i in 1:ncol(data)) {
            for (j in i:ncol(data)) {
                if (i != j) {

                    df = data[ , c(i, j)]
                    df_x = data[ , 1]
                    df_y = data[ , 2]

                    ## Lifting xor.
                    df_xor = df_x + df_y
                    df_xor[df_xor > 1] = 0

                    ## Lifting or.
                    df_or = df_x + df_y
                    df_or[df_or > 1] = 1

                    ## Lifting and.
                    df_and = df_x + df_y
                    df_and[ df_and < 2] = 0
                    df_and[ df_and == 2] = 1

                    ## 2x2 contingency table.
                    table.xor =
                        rbind(c(nrow(df) - sum(df_or), sum(df_or - df_y)),
                              c(sum(df_or - df_x), sum(df_and))
                              )

                    ## Fisher 2-sided.
                    test = fisher.test(table.xor)

                    ## Save the results.
                    curr_result = c(test$p.value, log(test$estimate['odds ratio']))
                    results = append(results, list(curr_result))
                }
            }
        }
    }

    return(results)
}


#### end of file -- capri.hypotheses.R

data(test_dataset_no_hypos)
data(test_model)

context("hypothesis.add")

test_that("hypotheses.add is working", {
    expect_equal(length(hypothesis.add(test_dataset_no_hypos,
        'OR ASXL1 EZH2 TET2',
        OR('ASXL1', 'EZH2', 'TET2'))), 5)
    expect_equal(length(hypothesis.add(test_dataset_no_hypos,
        'XOR ASXL1 EZH2 TET2',
        XOR('ASXL1', 'EZH2', 'TET2'))), 5)
    expect_equal(length(hypothesis.add(test_dataset_no_hypos,
        'AND SETBP1 CSF3R',
        AND('SETBP1', 'CSF3R'))), 5)
    expect_equal(length(hypothesis.add(test_dataset_no_hypos,
        'test',
        OR('ASXL1', 'EZH2', 'TET2'),
        pattern.effect='SETBP1',
        pattern.cause='CSF3R')), 5)
    hypos = hypothesis.add(test_dataset_no_hypos,
        'OR ASXL1 EZH2 TET2',
        OR('ASXL1', 'EZH2', 'TET2'))
    expect_error(hypothesis.add(hypos,
        'OR ASXL1 EZH2 TET2 2',
        OR('ASXL1', 'EZH2', 'TET2')))
    expect_error(hypothesis.add(test_dataset_no_hypos,
        'AND ASXL1 EZH2 TET2',
        AND('ASXL1', 'EZH2', 'TET2')))
    expect_error(hypothesis.add(hypos,
        'test2',
        OR('ASXL1', 'XXX', 'TET2')))
    expect_error(hypothesis.add(hypos,
        'test',
        OR('ASXL1', 'EZH2', 'TET2')))
    expect_error(hypothesis.add(test_model,
        'OR ASXL1 EZH2 TET2 2',
        OR('ASXL1', 'EZH2', 'TET2')))
    expect_error(hypothesis.add(test_dataset_no_hypos,
        'test',
        OR('ASXL1', 'EZH2', 'TET2'),
        pattern.effect='TET2',
        pattern.cause='CSF3R'))
    expect_error(hypothesis.add(test_dataset_no_hypos,
        'test',
        OR('ASXL1', 'EZH2', 'TET2'),
        pattern.effect='XXX',
        pattern.cause='CSF3R'))
    expect_error(hypothesis.add(test_dataset_no_hypos,
        'test',
        OR('ASXL1', 'EZH2', 'TET2'),
        pattern.effect='SETBP1',
        pattern.cause='TET2'))
    expect_error(hypothesis.add(test_dataset_no_hypos,
        'test',
        OR('ASXL1', 'EZH2', 'TET2'),
        pattern.effect='SETBP1',
        pattern.cause='XXX'))
})

context("hypothesis.add.group")

test_that("hypothesis.add.group is working", {
    expect_equal(length(hypothesis.add.group(test_dataset_no_hypos,
        OR,
        c('ASXL1', 'EZH2', 'TET2'),
        silent = TRUE)), 5)
    expect_equal(length(hypothesis.add.group(test_dataset_no_hypos,
        OR,
        c('ASXL1', 'EZH2', 'TET2'),
        min.prob = 0.2,
        silent = TRUE)), 5)
    expect_error(hypothesis.add.group(test_model,
        OR,
        c('ASXL1', 'EZH2', 'TET2'),
        silent = TRUE))
    expect_warning(hypothesis.add.group(test_dataset_no_hypos,
        OR,
        c('ASXL1'),
        silent = TRUE))
    expect_error(hypothesis.add.group(test_model,
        OR,
        c('ASXL1', 'EZH2', 'TET2'),
        dim.max = 2,
        dim.min = 1,
        silent = TRUE))
})

context("hypothesis.add.homologous")

test_that("hypothesis.add.homologous is working", {
    expect_equal(length(hypothesis.add.homologous(test_dataset_no_hypos, silent = TRUE)), 5)
    expect_error(hypothesis.add.homologous(test_model))
})

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' Check if 'x' is compliant with TRONCO's input: that is if it has dataframes 
#' x$genotypes, x$annotations, x$types and x$stage (optional)
#' @title is.compliant
#' 
#' @examples
#' data(test_dataset)
#' is.compliant(test_dataset)
#' 
#' @param x A TRONCO compliant dataset.
#' @param err.fun string which identifies the function which called is.compliant
#' @param stage boolean flag to check x$stage datagframe
#' @return on error stops the computation
#' @export is.compliant
#' @importFrom utils head
#' 
is.compliant <- function(x,
                         err.fun = '[ERR]',
                         stage = !(all(is.null(x$stages)) || all(is.na(x$stages)))) {
    ## Check if x is defined.
    
    if (is.null(x) || all(is.na(x)))
        stop(paste(err.fun, ': input \'x\' is null.'))

    ## Check then if x is a list.
    
    if (!is.list(x))
        stop(paste(err.fun, ': input \'x\' is not a list.'))

    ## Check then if x has required fields.
    
    if (is.null(x$genotypes) || any(is.na(x$genotypes)))
        stop(paste(err.fun, ': input \'x\' has no genotypes field.'))   
    else if (!is.matrix(x$genotypes) && !is.data.frame(x$genotypes))
        stop(paste(err.fun, ': attribute genotypes in  \'x\' is not a matrix.'))
        
    colnames(x$annotations) = c('type', 'event')
    colnames(x$types) = c('color')
    
    ## Check if annotations table is well-formed
    
    if (is.null(x$annotations) || any(is.na(x$annotations)))
        stop(paste(err.fun, ': input \'x\' has no annotations field.'))
    else if (!is.matrix(x$annotations) && !is.data.frame(x$annotations))
        stop(paste(err.fun, ': attribute annotations in  \'x\' is not a matrix.'))
    if (!is.character(x$annotations)) {
        stop(paste(err.fun, ': attribute annotations in \'x\' is not a matrix of character.'))
    }

    ## Check if annotations table is well-formed

    if (is.null(x$types) || any(is.na(x$types)))
        stop(paste(err.fun, ': input \'x\' has no types field.'))
    else if (!is.matrix(x$types) && !is.data.frame(x$types))
        stop(paste(err.fun, ': attribute types in  \'x\' is not a matrix.'))
        
    if (stage == TRUE && (is.null(x$stages) || all(is.na(x$stages))))
        stop(paste(err.fun, ': input \'x\' has no stage field.'))
    else if (stage == TRUE && !is.matrix(x$stages) && !is.data.frame(x$stages))
        stop(paste(err.fun, ': attribute stage in  \'x\' is not a matrix.'))    

    ## Annotations sould be present for all genotypes columns.
    
    if (nrow(x$annotations) != ncol(x$genotypes) ) 
        stop(paste(err.fun, ': input \'x\' has less annotations than expected.'))           
    if (!all(colnames(x$genotypes) == rownames(x$annotations))) 
        stop(paste(err.fun, ': input \'x\' has inconsistent annotations.'))         

    ## Types should be defined for every annotation.
    
    if (nrow(x$types) != length(unique(x$annotations[,1]))) {
        message('[ERROR] rownames(x$types):',
                paste(rownames(x$types), collapse = ', ', sep = ''))
        message('[ERROR] Annotated event types:',
                paste(unique(x$annotations[,1]), collapse = ', ', sep = ''))
        
        stop(paste(err.fun, ': input \'x\' has less types than expected.'))         
    }
    
    if (!all(unique(x$annotations[,'type']) %in% rownames(x$types))) {
        stop(paste(err.fun,
                   ': input \'x\' has inconsistent types (', 
                   paste(unique(x$annotations[,'type']), collapse = ','),
                   ' vs ',
                   paste(rownames(x$types), collapse=','),
                   ').', 
                   sep = ''))               
    }

    ## Stage should be defined for every samples.
    
    if (stage == TRUE && nrow(x$stages) != nrow(x$genotypes)) 
        stop(paste(err.fun, ': input \'x\' has less stages than expected.'))            
    if (stage == TRUE && !all(rownames(x$stages) == rownames(x$genotypes))) 
        stop(paste(err.fun, ': input \'x\' has inconsistent stages.'))          
    
    if (stage == TRUE)  colnames(x$stages) = c('stage')
    
    dup = duplicated(x$annotations)
    if (any(dup)) {
        cat("Duplicated events in \'x\': \n")
        print(head(x$annotations[dup, ]))
        stop('Duplicated events.')
    }
}

# Check if x is a valid TRONCO model
# @title is.model
# @param x A TRONCO compliant dataset.
#
is.model <- function(x) {
    if (!'model' %in% names(x))
        stop('Input object is not a TRONCO model.')
}

# Check if y is a valid event list for x
# @title is.events.list
# @param x A TRONCO compliant dataset.
# @param y A TRONCO event list
#
is.events.list <- function(x, y) {
    if (!is.matrix(y))
        stop('Events should be given as matrix - see "as.events".')
    
    if (ncol(y) != 2 || !all(c('type', 'event') %in% colnames(y)))
        stop('Events are missing column "type" (type of event) or "event" (gene symbol) - see "as.events".')
    
    if (!all(rownames(y) %in% colnames(x$genotypes))) 
        stop('Events rownames are not valid keys for genotypes - see "as.events".')
}


#### end of file -- correctness.R

## ----include=FALSE------------------------------------------------------------
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)

## ----fig.show='hide', warning=FALSE, eval=FALSE-------------------------------
# export.graphml(model.boot,
#     file = 'graph.gml',
#     fontsize = 12,
#     scale.nodes = .6,
#     height.logic = 0.25)

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

#' Return a list of samples with specified alteration
#' @title which.samples 
#'
#' @examples
#' data(test_dataset)
#' which.samples(test_dataset, 'TET2', 'ins_del')
#' which.samples(test_dataset, 'TET2', 'ins_del', neg=TRUE)
#'
#' @param x A TRONCO compliant dataset.
#' @param gene A list of gene names
#' @param type A list of types
#' @param neg If FALSE return the list, if TRUE return as.samples() - list
#' @return A list of sample
#' @export which.samples
#' 
which.samples <- function(x, gene, type, neg = FALSE) {
    data = as.gene(x, genes = gene, types = type)
    data = data[data == 1, , drop = FALSE] 
    
    samples = as.samples(x)
    
    if (neg) {
        return(setdiff(samples, rownames(data)))
    } else {
        return(rownames(data))  
    }
}

#### end of file -- which.functions.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


# perform the likelihood fit
lregfit <- function(data,
                    adj.matrix,
                    adj.matrix.fit,
                    regularization,
                    command,
                    restart = 0){

    if (regularization == "no_reg") {
        return(adj.matrix)
    }


    ## Create the blacklist based on the prima facie topology
    ## and the tree-structure assumption
    cont = 0
    parent = -1
    child = -1

    for (i in rownames(adj.matrix)) {
        for (j in colnames(adj.matrix)) {
            if(i != j) {
                if (adj.matrix[i,j] == 0) {
                    # [i,j] refers to causation i --> j
                    cont = cont + 1
                    if (cont == 1) {
                        parent = i
                        child = j
                    } else {
                        parent = c(parent, i)
                        child = c(child, j)
                    }
                }
            }
        }
    }

    ## perform the reconstruction by likelihood fit with regularization
    ## either the hill climbing or the tabu search is used as the 
    ## mathematical optimization technique
    if (cont > 0) {
        blacklist = data.frame(from = parent,to = child)
        if (command == "hc") {
            my.net = hc(data,score = regularization, blacklist = blacklist, restart = restart)
        } else if (command == "tabu") {
            my.net = tabu(data,score = regularization, blacklist = blacklist)
        }
    } else {
        if (command == "hc") {
            my.net = hc(data, score = regularization, restart = restart)
        } else if (command == "tabu") {
            my.net = tabu(data, score = regularization)
        }
    }
    my.arcs = my.net$arcs
    
    ## build the adjacency matrix of the reconstructed topology
    if (length(nrow(my.arcs)) > 0 && nrow(my.arcs) > 0) {
        for (i in 1:nrow(my.arcs)) {
            # [i,j] refers to causation i --> j
            adj.matrix.fit[my.arcs[i,1], my.arcs[i,2]] = 1
        }
    }

    return(adj.matrix.fit)
}

#### end of file -- lregfit.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' oncoPrint : plot a genotype. For details and examples 
#' regarding the visualization through oncoprints, we refer to the Vignette Section 4.4. 
#' 
#' @title oncoprint
#' @param x A TRONCO compliant dataset
#' @param excl.sort Boolean value, if TRUE sorts samples to enhance exclusivity of alterations
#' @param samples.cluster Boolean value, if TRUE clusters samples (columns). Default FALSE
#' @param genes.cluster Boolean value, if TRUE clusters genes (rows). Default FALSE
#' @param file If not NA write to \code{file} the Oncoprint, default is NA (just visualization).
#' @param ann.stage Boolean value to annotate stage classification, default depends on \code{x}
#' @param ann.hits Boolean value to annotate the number of events in each sample, default is TRUE
#' @param stage.color RColorbrewer palette to color stage annotations. Default is 'YlOrRd'
#' @param hits.color RColorbrewer palette to color hits annotations. Default is 'Purples'
#' @param null.color Color for the Oncoprint cells with 0s, default is 'lightgray'
#' @param border.color Border color for the Oncoprint, default is white' (no border)
#' @param text.cex Title and annotations cex, multiplied by font size 7
#' @param font.column If NA, half of font.row is used
#' @param font.row If NA, max(c(15 * exp(-0.02 * nrow(data)), 2)) is used, where data is the data 
#' visualized in the Oncoprint
#' @param title Oncoprint title, default is as.name(x) - see \code{as.name}
#' @param sample.id If TRUE shows samples name (columns). Default is FALSE
#' @param hide.zeroes If TRUE trims data - see \code{trim} - before plot. Default is FALSE 
#' @param legend If TRUE shows a legend for the types of events visualized. Defualt is TRUE
#' @param legend.cex Default 0.5; determines legend size if \code{legend = TRUE}
#' @param cellwidth Default NA, sets autoscale cell width 
#' @param cellheight Default NA, sets autoscale cell height
#' @param group.by.label Sort samples (rows) by event label - usefull when multiple events per gene are
#' available 
#' @param group.samples If this samples -> group map is provided, samples are grouped as of groups
#' and sorted according to the number of mutations per sample - usefull when \code{data} was clustered
#' @param group.by.stage Default FALSE; sort samples by stage.
#' @param gene.annot Genes'groups, e.g. list(RAF=c('KRAS','NRAS'), Wnt=c('APC', 'CTNNB1')). Default is NA.
#' @param gene.annot.color Either a RColorColorbrewer palette name or a set of custom colors matching names(gene.annot)
#' @param show.patterns If TRUE shows also a separate oncoprint for each pattern. Default is FALSE
#' @param annotate.consolidate.events Default is FALSE. If TRUE an annotation for events to consolidate is shown.
#' @param txt.stats By default, shows a summary statistics for shown data (n,m, |G| and |P|)
#' @param gtable If TRUE return the gtable object
#' @param ... other arguments to pass to pheatmap
#' @export oncoprint
#' @importFrom gridExtra grid.arrange
#' @importFrom RColorBrewer brewer.pal brewer.pal.info
#' @importFrom gtable gtable gtable_add_grob gtable_height gtable_width
#' @importFrom grDevices colorRampPalette
#' 
oncoprint <- function(x, 
                      excl.sort = TRUE, 
                      samples.cluster = FALSE, 
                      genes.cluster = FALSE, 
                      file = NA, 
                      ann.stage = has.stages(x), 
                      ann.hits = TRUE, 
                      stage.color = 'YlOrRd', 
                      hits.color = 'Purples',  
                      null.color = 'lightgray', 
                      border.color = 'white', 
                      text.cex = 1.0, 
                      font.column = NA, 
                      font.row = NA, 
                      title = as.description(x),
                      sample.id = FALSE,
                      hide.zeroes = FALSE,
                      legend = TRUE,
                      legend.cex = 0.5,
                      cellwidth = NA, 
                      cellheight = NA,
                      group.by.label = FALSE,
                      group.by.stage = FALSE,
                      group.samples = NA,
                      gene.annot = NA,
                      gene.annot.color = 'Set1',
                      show.patterns = FALSE,
                      annotate.consolidate.events = FALSE,
                      txt.stats =
                          paste(nsamples(x),
                                ' samples\n',
                                nevents(x),
                                ' events\n',
                                ngenes(x),
                                ' genes\n',
                                npatterns(x),
                                ' patterns',
                                sep = ''),
                      gtable = FALSE,
                      ...) {

    font.size = text.cex * 7

    ##  This function sorts a matrix to enhance mutual exclusivity
    
    exclusivity.sort <- function(M) {
        geneOrder <- sort(rowSums(M),
                          decreasing = TRUE,
                          index.return = TRUE)$ix
        scoreCol <- function(x) {
            score <- 0;
            for (i in 1:length(x)) {
                if(x[i]) {
                    score <- score + 2^(length(x)-i);
                }
            }
            return(score);
        }
        scores <- apply(M[geneOrder, , drop = FALSE ], 2, scoreCol);
        sampleOrder <- sort(scores, decreasing=TRUE, index.return=TRUE)$ix
        res = list()
        res$geneOrder = geneOrder
        res$sampleOrder = sampleOrder
        res$M = M[geneOrder, sampleOrder]

        return(res);
    }

    ## Check input data.
    
    cat(paste('*** Oncoprint for "',
              title,
              '"\nwith attributes: stage = ',
              ann.stage,
              ', hits = ',
              ann.hits,
              '\n',
              sep = ''))
    is.compliant(x, 'oncoprint', stage=ann.stage)
    x = enforce.numeric(x)

    ## If hide.zeros trim x
    
    if (hide.zeroes) {
        cat(paste('Trimming the input dataset (hide.zeroes).\n',
                  sep = ''))
        x = trim(x)
    }

    ##  Reverse the heatmap under the assumption that ncol(data) << nrow(data)
    data = t(x$genotypes)
    nc = ncol(data) 
    nr = nrow(data)

    ## Sort data, if required. excl.sort and group.samples are not compatible
    hasGroups = !any(is.na(group.samples))

    if (group.by.stage && !ann.stage)
        stop('Cannot group samples by stage if no stage annotation is provided.')

    if (group.by.stage && excl.sort)
        if (excl.sort && (hasGroups || group.by.stage)) 
            stop('Disable sorting for mutual exclusivity (excl.sort=FALSE) or avoid using grouped samples.')

    ## Exclusivity sort
    if (excl.sort && nevents(x) > 1) {
        cat(paste('Sorting samples ordering to enhance exclusivity patterns.\n',
                  sep = ''))
        sorted.data = exclusivity.sort(data)
        data = sorted.data$M  
    }

    if (group.by.stage) {  
        ord.stages = as.stages(x)[order(as.stages(x)[,1]), , drop = FALSE]
        cat('Grouping samples by stage annotation.\n')

        aux.fun <- function(samp) {
            print(samp)
            sub.data = data[, samp, drop= FALSE]      
            sub.data = sub.data[, order(colSums(sub.data), decreasing = FALSE), drop = FALSE]
            return(sub.data)
        }

        new.data = NULL
        u.stages = sort(unlist(unique(as.stages(x))), na.last = TRUE)

        ## print(str(ord.stages))

        for (i in u.stages) {
            print(ord.stages[which(ord.stages == i), , drop=FALSE])
            new.data = cbind(new.data, aux.fun(rownames(ord.stages[which(ord.stages == i), , drop= FALSE])))
        }
        data = new.data
        data = data[order(rowSums(data), decreasing = TRUE), , drop = FALSE ]
    }

    ## Samples grouping via hasGroups
    if (hasGroups) {
        group.samples[, 1] = as.character(group.samples[,1])
        grn = rownames(group.samples)

        cat(paste('Grouping samples according to input groups (group.samples).\n', sep = ''))
        if (any(is.null(grn)))
            stop('"group.samples" should be matrix with sample names and group assignment.')

        if (!setequal(grn, as.samples(x)))
            stop(paste0('Missing group assignment for samples: ',
                        paste(setdiff(as.samples(x), grn),
                              collapse=', '),
                        '.'))

        ## Order groups by label, and then data (by column)
        order = order(group.samples[,1])
        group.samples = group.samples[order, , drop = FALSE]

        data = data[, rownames(group.samples)]            
        data = data[order(rowSums(data), decreasing = TRUE), , drop = FALSE]    

        groups = unique(group.samples[,1])

        for(i in 1:length(groups)) {
            subdata = data[, group.samples == groups[i], drop = FALSE]
            subdata = subdata[, order(colSums(subdata), decreasing = TRUE), drop = FALSE]
            data[ , group.samples == groups[i]] = subdata
        }
    } 

    ## If group.by.label group events involving the gene symbol
    if (group.by.label) {
        cat(paste('Grouping events by gene label, samples will not be sorted.\n',
                  sep = ''))
        genes = as.genes(x)
        data = data[ order(x$annotations[rownames(data), 'event']), ]
    }

    cn = colnames(data)
    rn = rownames(data)

    ## SAMPLES annotations: hits (total 1s per sample), stage or groups
    samples.annotation = data.frame(row.names = cn,  stringsAsFactors = FALSE)
    nmut = colSums(data)

    if (ann.hits)  samples.annotation$hits = nmut
    if (ann.stage) samples.annotation$stage = as.stages(x)[cn, 1]
    if (hasGroups) samples.annotation$cluster = group.samples[cn, 1]

    ## Color samples annotation
    annotation_colors = NULL

    ## Color hits
    if (ann.hits) {
        hits.gradient = (colorRampPalette(brewer.pal(6, hits.color))) (max(nmut))
        annotation_colors = append(annotation_colors, list(hits=hits.gradient))
    }

    ## Color stage
    if (ann.stage) { 
        cat('Annotating stages with RColorBrewer color palette',
            stage.color,
            '\n')

        different.stages = sort(unique(samples.annotation$stage))

        stage.color.attr = sample.RColorBrewer.colors(stage.color, length(different.stages))    
        stage.color.attr = append(stage.color.attr, "#FFFFFF")      

        samples.annotation[is.na(samples.annotation)] = "none"
        names(stage.color.attr) = append(different.stages, "none")    
        annotation_colors = append(annotation_colors, list(stage=stage.color.attr))
    }

    ## Color groups
    if (hasGroups) {
        ngroups = length(unique(group.samples[,1]))
        cat('Grouping labels:', paste(unique(group.samples[,1]), collapse=', '), '\n')

        group.color.attr = sample.RColorBrewer.colors('Accent', ngroups)
        names(group.color.attr) = unique(group.samples[,1])
        annotation_colors = append(annotation_colors, list(cluster=group.color.attr))
    }

    ## GENES/EVENTS annotations: groups or indistinguishable.
    genes.annotation = NA

    ## Annotate genes groups.
    
    if (!all(is.na(gene.annot))) {
        names = names(gene.annot)

        genes.annotation = data.frame(row.names = rn, stringsAsFactors = FALSE)
        genes.annotation$group = rep("none", nrow(data))

        for (i in 1:length(names)) {
            pathway = names[i]
            genes.pathway = rownames(as.events(x, genes=gene.annot[[names[i]]]))
            genes.annotation[genes.pathway, 'group'] = names[i] 
        }

        if (length(gene.annot.color) == 1
            && gene.annot.color %in% rownames(brewer.pal.info)) {
            cat('Annotating genes with RColorBrewer color palette', gene.annot.color, '.\n')
            gene.annot.color = sample.RColorBrewer.colors(gene.annot.color, length(names))
            gene.annot.color = append(gene.annot.color, "#FFFFFF")      
        } else {
            if (length(gene.annot.color) != length(names)) 
                stop('You did not provide enough colors to annotate',
                     length(names),
                     'genes.\n',
                     'Either set gene.annot.color to a valid RColorBrewer palette or provide the explicit correct number of colors.')

            cat('Annotating pathways with custom colors:',
                paste(gene.annot.color, collapse=', '),
                '\n')
            gene.annot.color = append(gene.annot.color, "#FFFFFF")
        }
        names(gene.annot.color) = append(names, "none")
        gene.annot.color = gene.annot.color[ unique(genes.annotation$group) ]   
        annotation_colors = append(annotation_colors, list(group=gene.annot.color))
    }   

    ## Annotate events to consolidate
    if (annotate.consolidate.events) {
        cat('Annotating events to consolidate - see consolidate.data\n')
        invalid = consolidate.data(x, print = FALSE)

        genes.annotation$consolidate = 'none'

        if (length(invalid$indistinguishable) > 0) {

            genes.annotation[
                             unlist(
                                 lapply(
                                     invalid$indistinguishable, 
                                     function(z) {
                                         return(rownames(unlist(z)))
                                     }
                                 )),
                             'consolidate'] = 'indistinguishable'
        }

        if (length(invalid$zeroes) > 0) {
            genes.annotation[ unlist(invalid$zeroes), 'consolidate'] = 'missing'
        }

        if (length(invalid$ones) > 0) {
            genes.annotation[ unlist(invalid$ones), 'consolidate'] = 'persistent'
        }

        consolidate.colors = c('white', 'brown1', 'darkorange4', 'firebrick4', 'deepskyblue3')
        names(consolidate.colors) = c('none', 'indistinguishable', 'missing', 'persistent')
        consolidate.colors = consolidate.colors[unique(genes.annotation$consolidate)]

        annotation_colors =
            append(annotation_colors,
                   list(consolidate = consolidate.colors)
                   )
    }

    ## Augment gene names with frequencies
    genes.freq = rowSums(data) / nsamples(x)
    gene.names = x$annotations[rownames(data), 2]
    gene.names =
        paste(round(100 * genes.freq, 0),
              '% ',
              gene.names,
              sep = '') # row labels


    ## Augment data to make type-dependent colored plots.
    data.lifting <- function(obj, matrix) {
        types = as.types(obj)
        map.gradient = null.color

        for (i in 1:ntypes(obj)) {
            events = as.events(obj, types = as.types(obj)[i])
            keys = rownames(events)

            if (ntypes(obj) > 1) {
                keys.subset =
                    keys[unlist(lapply(keys,
                                       function(obj, matrix) {

                                           ## Are you sure (obj %in% # matrix)
                                           ## is not sufficient?

                                           if (obj %in% matrix) {
                                               TRUE
                                           } else {
                                               FALSE
                                           }},
                                       rownames(matrix)))]
                sub.data = matrix[keys.subset, , drop = FALSE]

                ## shift 1s to 'i', add color to the map.
                
                idx = which(sub.data == 1)
                if(length(idx) > 0) map.gradient = cbind(map.gradient, as.colors(obj)[i])

                sub.data[idx] = i
                matrix[keys.subset, ] = sub.data 

            } else {
                map.gradient = cbind(map.gradient, as.colors(obj)[i])
            }
        }

        map.gradient = c(null.color, as.colors(x))
        names(map.gradient)[1] = 'none'

        return(list(data=matrix, colors=map.gradient))
    }

    pheat.matrix = data.lifting(x,data)
    map.gradient = pheat.matrix$colors
    data = pheat.matrix$data

    ## Set fontisize col/row.
    if (is.na(font.row)) {
        font.row = max(c(15 * exp(-0.02 * nrow(data)), 2))    
        cat(paste('Setting automatic row font (exponential scaling): ',
                  round(font.row, 1),
                  '\n',
                  sep = ''))
    }
    
    if (is.na(font.column) && sample.id) {
        font.column = font.row/2    
        cat(paste('Setting automatic samples font half of row font: ',
                  round(font.column, 1),
                  '\n',
                  sep = '')) 
    }

    ## Finalizing legends etc.
    legend.labels = c('none', unique(x$annotations[, 1]))    
    legend.labels = legend.labels[1:(max(data) + 1)]

    if (samples.cluster) cat('Clustering samples and showing dendogram.\n')
    if (genes.cluster) cat('Clustering alterations and showing dendogram.\n')

    if (is.null(annotation_colors)) annotation_colors = NA

    if (length(list(...)) > 0) {
        cat('Passing the following parameters to TRONCO pheatmap:\n')
        print(list(...))
    }

    ## Real pheatmap.
    
    if (ncol(samples.annotation) == 0) {
        samples.annotation = NA
    }

    ret =
        pheatmap(data, 
                 scale = "none", 
                 color = map.gradient, 
                 cluster_cols = samples.cluster,
                 cluster_rows = genes.cluster,
                 main = title,
                 fontsize = font.size,
                 fontsize_col = font.column,
                 fontsize_row = font.row,
                 annotation_col = samples.annotation,
                 annotation_row = genes.annotation,
                 annotation_colors = annotation_colors, 
                 border_color = border.color,
                 border = TRUE,
                 margins = c(10,10),
                 cellwidth = cellwidth, 
                 cellheight = cellheight,
                 legend = legend,             
                 legend_breaks = c(0:max(data)),
                 legend_labels = legend.labels,
                 legend.cex = legend.cex,
                 labels_row = gene.names,
                 drop_levels = TRUE,
                 show_colnames = sample.id,
                 filename = file,
                 txt.stats = txt.stats,
                 ...
                 )

    ## Extra patterns   

    patt.table =
        gtable(widths = unit(c(7, 2), "null"),
               heights = unit(c(2, 7), "null"))

    patt.table = list()
    map.gradient = map.gradient[names(map.gradient) != 'Pattern']

    ## print(data)
    
    if (npatterns(x) > 0 && show.patterns) {
        cat('Plotting also', npatterns(x), 'patterns\n')
        patterns = as.patterns(x)

        for (i in 1:length(patterns)) {
            genes.patt = as.events.in.patterns(x, patterns[i]) 
            genes.patt.genos = data[rownames(genes.patt), , drop = FALSE]


            genes.patt.genos.gtable =
                pheatmap(exclusivity.sort(genes.patt.genos)$M,
                         scale = "none",
                         color = map.gradient,
                         cluster_cols = FALSE,
                         cluster_rows = FALSE,
                         main = paste('Pattern:', patterns[i]),
                         fontsize = font.size * .75,
                         fontsize_col = font.column * .75,
                         fontsize_row = font.row * .75,
                         ## annotation_row = genes.annotation,
                         annotation_colors = annotation_colors,
                         annotation_legend  = FALSE,
                         border_color = border.color,
                         border = TRUE,
                         cellwidth = 6,
                         cellheight = 6,
                         legend = FALSE,
                         labels_row = genes.patt[rownames(genes.patt.genos), 'event'],
                         drop_levels = TRUE,
                         show_colnames = FALSE,
                         silent = TRUE,
                         ...)$gtable

            ## patt.table = gtable_add_grob(patt.table, genes.patt.genos.gtable, 2, 1)
            patt.table = append(patt.table, list(genes.patt.genos.gtable))
        }

        str =
            paste('grid.arrange(ret$gtable, arrangeGrob(', 
                  paste('patt.table[[', 1:length(patt.table), ']],', collapse = ''),
                  'ncol = 1), ncol = 1, heights = c(4,',
                  paste(rep(1, length(patt.table) ), collapse=', '),
                  '))')

        str =
            paste('grid.arrange(ret$gtable, arrangeGrob(', 
                  paste('patt.table[[', 1:length(patt.table), ']],', collapse = ''),
                  'ncol = 1), ncol = 1, heights = c(',
                  paste(rep(1, length(patt.table) + 1 ),
                        collapse=', '),
                  '))')

        eval(parse(text = str))
    }


    ##  print(ncol(ret$gtable))
    ##  print(ncol(patt.table))
    
    ## ret$gtable = rbind(ret$gtable, patt.table)
    ## ret$gtable = gtable_add_grob(ret$gtable, patt.table, 1 , 1)
    ## grid.newpage()
    ## grid.draw(ret$gtable)
    if (gtable) {
        return(ret)
    }
}


##### Pathway print
#' Visualise pathways informations
#' @title pathway.visualization
#'
#' @param x A TRONCO complian dataset
#' @param title Plot title
#' @param file To generate a PDF a filename have to be given
#' @param pathways.color A RColorBrewer color palette
#' @param aggregate.pathways Boolean parameter
#' @param pathways Pathways
#' @param ... Additional parameters
#' @return plot information
#' @export pathway.visualization
#' 
pathway.visualization <- function(x, 
                                  title =
                                      paste('Pathways:',
                                            paste(names(pathways), collapse=', ', sep='')),
                                  file = NA,
                                  pathways.color = 'Set2',
                                  aggregate.pathways,
                                  pathways,
                                  ...) { 
    names = names(pathways)

    if (length(pathways.color) == 1
        && pathways.color %in% rownames(brewer.pal.info)) {
        cat('Annotating pathways with RColorBrewer color palette', pathways.color, '.\n')
        n = length(names)
        if (n < 3) {
            n = 3
        }
        pathway.colors = brewer.pal(n = n, name=pathways.color)
    } else { 
        if (length(pathways.color) != length(names)) 
            stop('You did not provide enough colors to annotate ',
                 length(names),
                 ' pathways.\n',
                 'Either set pathways.color to a valid RColorBrewer palette or provide the explicit correct number of colors.')

        cat('Annotating pathways with custom colors', paste(pathways.color, collapse=','), '.\n')
        pathway.colors = pathways.color
    }

    names(pathway.colors) = names

    cat(paste('*** Processing pathways: ',
              paste(names, collapse = ', ', sep = ''),
              '\n',
              sep = ''))

    cat(paste('\n[PATHWAY \"', names[1],'\"] ',
              paste(pathways[[1]], collapse = ', ', sep = ''),
              '\n',
              sep = ''))

    data.pathways =
        as.pathway(x,
                   pathway.genes = pathways[[1]], 
                   pathway.name = names[1],
                   aggregate.pathway = aggregate.pathways)

    data.pathways = change.color(data.pathways, 'Pathway', pathway.colors[1])
    data.pathways = rename.type(data.pathways, 'Pathway', names[1])

    if (length(names) > 1) {  
        for(i in 2:length(pathways)) {
            cat(paste('\n\n[PATHWAY \"', names[i],'\"] ', paste(pathways[[i]], collapse=', ', sep=''), '\n', sep=''))
            pathway = as.pathway(x, pathway.genes=pathways[[i]], pathway.name=names[i], aggregate.pathway = aggregate.pathways)

            pathway = change.color(pathway, 'Pathway', pathway.colors[i])
            pathway = rename.type(pathway, 'Pathway', names[i])
            data.pathways = ebind(data.pathways, pathway)
        }
    }

    ret = oncoprint(trim(data.pathways), title=title, file=file, ...)

    return(ret)
}


#' export input for cbio visualization at http://www.cbioportal.org/public-portal/oncoprinter.jsp
#' @title oncoprint.cbio
#'
#' @examples
#' data(crc_gistic)
#' gistic = import.GISTIC(crc_gistic)
#' oncoprint.cbio(gistic)
#'
#' @param x A TRONCO compliant dataset.
#' @param file name of the file where to save the output
#' @param hom.del type of Homozygous Deletion
#' @param het.loss type of Heterozygous Loss
#' @param gain type of Gain
#' @param amp type of Amplification
#' @return A file containing instruction for the CBio visualization Tool
#' @export oncoprint.cbio
#' 
oncoprint.cbio <- function(x, 
                           file = 'oncoprint-cbio.txt', 
                           hom.del = 'Homozygous Loss',
                           het.loss = 'Heterozygous Loss', 
                           gain = 'Low-level Gain', 
                           amp = 'High-level Gain') {
    is.compliant(x)

    ## r = paste(paste(rownames(x$genotypes)), x$annotations[x$annotations[,''],], 'xxx')

    r = 'Sample\tGene\tAlteration\n'
    for (i in 1:nrow(x$genotypes)) {
        for (j in 1:ncol(x$genotypes)) {
            if (x$genotypes[i,j] == 1)
                {
                    s = rownames(x$genotypes)[i]
                    g = x$annotations[colnames(x$genotypes)[j], 'event']

                    t = x$annotations[colnames(x$genotypes)[j], 'type']

                    t.o = 'xxx'
                    if (t == hom.del) t.o = 'HOMDEL'
                    if (t == het.loss)  t.o = 'HETLOSS'
                    if (t == gain)  t.o = 'GAIN'
                    if (t == amp)  t.o = 'AMP'

                    ## cat(paste( s,  g,  t.o, '\n', sep=' ', collpase=''))
                    r = paste(r, s, '\t', g, '\t', t.o, '\n', sep='', collpase='')
                }
        }
    }
    
    write(r, file)
}


#' Generate PDF and laex tables
#' @title genes.table.report
#'
#' @param x A TRONCO compliant dataset.
#' @param name filename
#' @param dir working directory
#' @param maxrow maximum number of row per page
#' @param font document fontsize
#' @param height table height
#' @param width table width
#' @param fill fill color
#' @param silent A parameter to disable/enable verbose messages.
#' @return LaTEX code
#' @importFrom gridExtra grid.table
#' @importFrom xtable xtable
#' @importFrom grDevices pdf dev.cur dev.off dev.set
#' @export genes.table.report
#' 
genes.table.report <- function(x,
                               name,
                               dir = getwd(),
                               maxrow = 33, 
                               font = 10,
                               height = 11,
                               width = 8.5,
                               fill = "lightblue",
                               silent = FALSE) {
    ## Print table with gridExtra and xtables.
    
    print.table <- function(table,
                            name,
                            dir = getwd(),
                            maxrow,
                            font,
                            height, width,
                            fill) {
        cat('\nPrinting PDF and Latex table to files: \n')
        cat(paste('PDF \t\t', dir, '/', name, '.genes-table.pdf\n', sep=''))
        cat(paste('Latex\t\t', dir, '/', name, '.genes-table.tex\n', sep=''))
        cat('\n')

        ## Output pdf
        ## require(gridExtra)  
        ## require(xtable)  

        cur.dev = dev.cur()

        pdf(file = paste(dir, '/', name, '.genes-table.pdf', sep = ''),
            height = height,
            width = width)

        ## Max rows per page
        
        npages = ceiling(nrow(table)/maxrow); 

        #flush.console()

        #pb = txtProgressBar(1, npages, style = 3);      
        for (i in 1:npages) {
            if (!silent) {
              cat('.')
            }
            #setTxtProgressBar(pb, i)  
            idx = seq(1+((i-1)*maxrow), i*maxrow); 

            if (max(idx) > nrow(table)) idx = idx[idx < nrow(table)]   

            grid.newpage()
            grid.table(table[idx, ],
                       gpar.coretext = gpar(fontsize = font),
                       gpar.corefill = gpar(fill = fill, alpha=0.5, col = NA),
                       h.even.alpha = 0.5)
        } 
        #close(pb)
        if (!silent) {
          cat('\n')
        }

        ## Output latex.
        
        print(xtable(table, digits = 0),
              file = paste(dir, '/', name, '.genes-table.tex', sep = ''),
              type = 'latex')

        dev.off()
        dev.set(which = cur.dev)
    }

    cat(paste('Preparing output table with ', ngenes(x), ' genes ...\n'))
    genes = as.genes(x)
    types = as.types(x)

    data = matrix(0, nrow = ngenes(x), ncol = ntypes(x))

    genes.table = data.frame(data, row.names = genes, stringsAsFactors = FALSE)
    colnames(genes.table) = types

    x = enforce.numeric(x)

    #pb = txtProgressBar(1, ngenes(x), style = 3);
    for (i in 1:ngenes(x)) {
        #setTxtProgressBar(pb, i)  
        if (!silent) {
            cat('.')
        }

        g = as.gene(x, genes=genes[i])

        if (ncol(g) > 0) {
            gg = colSums(apply(g, 2, as.numeric))

            genes.table[rownames(genes.table)[i], colnames(g)] = gg
            genes.table[rownames(genes.table)[i], 'Alterations'] =
                paste( round(sum(gg) / nsamples(x) * 100), '%', sep='')  
            genes.table[rownames(genes.table)[i], 'Frequency'] =
                sum(gg) / nsamples(x)  
        }
    }
                                        ## Close progress bar.
    #close(pb)

    if (!silent) {
        cat('\n')
    }

    genes.table = genes.table[order(genes.table$Frequency, decreasing = TRUE), ]
    genes.table$Frequency = NULL

    print.table(table = genes.table,
                name = name,
                dir = getwd(),
                maxrow = maxrow,
                font = font,
                height = height,
                width = width,
                fill = fill)

    return(genes.table)
}


#' @importFrom RColorBrewer brewer.pal
cluster.sensitivity <- function(cluster.map,
                                  reference,
                                  stages = NA,
                                  file = NA) {

    if (ncol(cluster.map) == 1)
        stop('No clustering stability for a unique clustering map!')


    if (!reference %in% colnames(cluster.map)) 
        stop(paste0('The reference cluster specified is not any of: ', 
                    paste(colnames(cluster.map), collapse = ', '), '.'))

    ref.clust = which(reference == colnames(cluster.map))  
    colnames(cluster.map)[ref.clust] =
        paste(colnames(cluster.map)[ref.clust],' [reference]',sep = '') 

    ## Transpose data.
    ## cluster.map = t(cluster.map)

    ## Sort data according to reference row.
    
    cluster.map =
        cluster.map[sort(cluster.map[, ref.clust ],
                         decreasing = FALSE,
                         index.return = TRUE)$ix, ];

    ## Get unique clustering IDs.
    
    id = apply(cluster.map, 2, unique)
    id = unique(unlist(id))

    print(apply(cluster.map, 2, unique))

    ## Compute the clustering score.
    
    subdata = cluster.map[, -ref.clust]
    refcol = cluster.map[, ref.clust]
    urefcol = unique(refcol)

    cat('Found the following cluster labels:', urefcol, '\n')

    cat('Computing clustering scores ... ')

    score = rep(0, nrow(cluster.map))

    for (i in 1:length(urefcol)) {
        tmp = as.matrix(subdata[which(refcol == i), ]);

        curr.score = 0;
        for (j in 1:ncol(tmp)) {
            curr.cardinality = sort(table(tmp[, j]), decreasing = TRUE)[[1]];
            curr.score =
                curr.score + (nrow(tmp) - curr.cardinality) / nrow(tmp);     
        }
        score[which(refcol == i)] = 1 - (curr.score/nrow(tmp))
    }
    cat('DONE\n')


    ## Create annotations.
    
    cn = rownames(cluster.map)

    annotation =
        data.frame(sensitivity = score,
                   row.names = cn,
                   stringsAsFactors = FALSE)

    if (!all(is.na(stages)))
        annotation$stage = stages[cn,1]

    ## Create colors.
    
    col = brewer.pal(n = length(id), name = 'Set1')

    different.stages = sort(unique(annotation$stage))
    num.stages = length(different.stages)

    stage.color = append(brewer.pal(n = num.stages, name = 'YlOrRd'), '#FFFFFF')
    names(stage.color) = append(levels(as.factor(different.stages)), NA)

    score.color = brewer.pal(n = 3, name = 'Greys')

    ## Annotation colors.
    
    annotation_colors = list(stage=stage.color, sensitivity=score.color)

    ## Settings.
    
    main =
        paste0("Clustering Sensitivity\n Reference : ", 
               reference,
               '\nAgainst : ',
               paste(colnames(subdata), collapse = ', ')) 


    cat('Clustering rows in', nrow(cluster.map), 'clusters.\n')
    order =
        sort(cluster.map[,ref.clust],
             decreasing = FALSE,
             index.return = TRUE) 



    pheatmap(cluster.map,
             scale = "none",
             cluster_cols = FALSE,
             cluster_rows = FALSE,
             color = col,
             main = main,
             fontsize = 6,
             fontsize_col = 8,
             fontsize_row = 2,
             annotation_row = annotation,
             annotation_colors = annotation_colors, 
             border_color = 'lightgray',
             border = TRUE,
             margins = c(10, 10),
             cellwidth = 25,
             cellheight = 2.2,
             ## legend = TRUE,
             legend_breaks = 1:4,
             filename = file,
             gaps_col = 1:3,
             ## display_numbers = T,
             cutree_rows = nrow(cluster.map),
             gaps_row = (match(urefcol, order$x) - 1)
             ## number_format = '%d',                 
             )

    return(cluster.map)
}

lo <- function(rown,
               coln,
               nrow,
               ncol,
               cellheight = NA,
               cellwidth = NA,
               treeheight_col,
               treeheight_row,
               legend,
               annotation_row,
               annotation_col,
               annotation_colors,
               annotation_legend,
               main,
               fontsize,
               fontsize_row,
               fontsize_col,
               gaps_row,
               gaps_col,
               ...) {
    
    ## Get height of colnames and length of rownames.

    if (!is.null(coln[1])) {
        t = c(coln, colnames(annotation_row))
        longest_coln = which.max(strwidth(t, units = 'in'))
        gp = list(fontsize = fontsize_col, ...)
        coln_height =
            unit(1,
                 "grobheight",
                 textGrob(t[longest_coln],
                          rot = 90,
                          gp = do.call(gpar, gp))) + unit(10, "bigpts")
    } else {
        coln_height = unit(5, "bigpts")
    }

    if (!is.null(rown[1])) {
        t = c(rown, colnames(annotation_col))
        longest_rown = which.max(strwidth(t, units = 'in'))
        gp = list(fontsize = fontsize_row, ...)
        rown_width =
            unit(1,
                 "grobwidth",
                 textGrob(t[longest_rown],
                          gp = do.call(gpar, gp))) + unit(10, "bigpts")
    } else {
        rown_width = unit(5, "bigpts")
    }

    gp = list(fontsize = fontsize, ...)
    
    ## Legend position

    if (!is.na(legend[1])) {
        longest_break = which.max(nchar(names(legend)))
        longest_break =
            unit(1.1,
                 "grobwidth",
                 textGrob(as.character(names(legend))[longest_break], gp = do.call(gpar, gp)))
        title_length =
            unit(1.1,
                 "grobwidth",
                 textGrob("Scale", gp = gpar(fontface = "bold", ...)))
        legend_width = unit(12, "bigpts") + longest_break * 1.2
        legend_width = max(title_length, legend_width)
    } else {
        legend_width = unit(0, "bigpts")
    }

    ## Set main title height.
    
    if (is.na(main)){
        main_height = unit(0, "npc")
    } else {
        main_height =
            unit(1.5,
                 "grobheight",
                 textGrob(main, gp = gpar(fontsize = 1.3 * fontsize, ...)))
    }

    ## Column annotations.
    
    textheight = unit(fontsize, "bigpts")

    if (!is.na(annotation_col[[1]][1])) {
        ## Column annotation height.
        annot_col_height =
            ncol(annotation_col) * (textheight + unit(2, "bigpts")) + unit(2, "bigpts")

        ## Width of the correponding legend.
        t = c(as.vector(as.matrix(annotation_col)), colnames(annotation_col)) 
        annot_col_legend_width =
            unit(1.2,
                 "grobwidth",
                 textGrob(t[which.max(nchar(t))], gp = gpar(...))) + unit(12, "bigpts")
        if (!annotation_legend) {
            annot_col_legend_width = unit(0, "npc")
        }
    } else {
        annot_col_height = unit(0, "bigpts")
        annot_col_legend_width = unit(0, "bigpts")
    }

    ## Row annotations.
    if (!is.na(annotation_row[[1]][1])) {
        
        ## Row annotation width.
        annot_row_width =
            ncol(annotation_row) * (textheight + unit(2, "bigpts")) + unit(2, "bigpts")

        ## Width of the correponding legend.
        
        t =
            c(as.vector(as.matrix(annotation_row)),
              colnames(annotation_row)) 
        annot_row_legend_width =
            unit(1.2,
                 "grobwidth",
                 textGrob(t[which.max(nchar(t))], gp = gpar(...))) + unit(12, "bigpts")
        
        if (!annotation_legend) {
            annot_row_legend_width = unit(0, "npc")
        }
    } else {
        annot_row_width = unit(0, "bigpts")
        annot_row_legend_width = unit(0, "bigpts")
    }

    annot_legend_width = max(annot_row_legend_width, annot_col_legend_width)

    ## Tree height.
    
    treeheight_col = unit(treeheight_col, "bigpts") + unit(5, "bigpts")
    treeheight_row = unit(treeheight_row, "bigpts") + unit(5, "bigpts") 

    ## Set cell sizes.
    
    if (is.na(cellwidth)) {
        mat_width =
            unit(1, "npc") - rown_width - legend_width - treeheight_row - annot_row_width - annot_legend_width 
    } else {
        mat_width =
            unit(cellwidth * ncol, "bigpts") + length(gaps_col) * unit(4, "bigpts")
    }

    if (is.na(cellheight)) {
        mat_height = unit(1, "npc") - main_height - coln_height - treeheight_col - annot_col_height
    } else {
        mat_height = unit(cellheight * nrow, "bigpts") + length(gaps_row) * unit(4, "bigpts")
    }    

    ## Produce gtable.
    
    gt =
        gtable(widths =
                   unit.c(treeheight_row,
                          annot_row_width, 
                          mat_width,
                          rown_width,
                          legend_width,
                          annot_legend_width),
               heights =
                   unit.c(main_height,
                          treeheight_col,
                          annot_col_height,
                          mat_height,
                          coln_height),
               vp = viewport(gp = do.call(gpar, gp)))

    ## print(unit.c(main_height, treeheight_col, annot_col_height, mat_height, coln_height))

    cw = convertWidth(mat_width - (length(gaps_col) * unit(4, "bigpts")), "bigpts", valueOnly = TRUE) / ncol
    ch = convertHeight(mat_height - (length(gaps_row) * unit(4, "bigpts")), "bigpts", valueOnly = TRUE) / nrow

    ## Return minimal cell dimension in bigpts to decide if borders
    ## are drawn.
    
    mindim = min(cw, ch)

    res = list(gt = gt, mindim = mindim)

    return(res)
}


find_coordinates <- function(n, gaps, m = 1:n) {
    if (length(gaps) == 0) {
        return(list(coord = unit(m / n, "npc"), size = unit(1 / n, "npc") ))
    }

    if (max(gaps) > n) {
        stop("Gaps do not match with matrix size")
    }

    size = (1 / n) * (unit(1, "npc") - length(gaps) * unit("4", "bigpts"))

    gaps2 = apply(sapply(gaps, function(gap, x){x > gap}, m), 1, sum) 
    coord = m * size + (gaps2 * unit("4", "bigpts"))

    return(list(coord = coord, size = size))
}


draw_dendrogram <- function(hc, gaps, horizontal = TRUE) {
    h = hc$height / max(hc$height) / 1.05
    m = hc$merge
    o = hc$order
    n = length(o)

    m[m > 0] = n + m[m > 0] 
    m[m < 0] = abs(m[m < 0])

    dist = matrix(0, nrow = 2 * n - 1, ncol = 2, dimnames = list(NULL, c("x", "y"))) 
    dist[1:n, 1] = 1 / n / 2 + (1 / n) * (match(1:n, o) - 1)

    for (i in 1:nrow(m)) {
        dist[n + i, 1] = (dist[m[i, 1], 1] + dist[m[i, 2], 1]) / 2
        dist[n + i, 2] = h[i]
    }

    draw_connection <- function(x1, x2, y1, y2, y) {
        res =
            list(x = c(x1, x1, x2, x2),
                 y = c(y1, y, y, y2)
                 )
        return(res)
    }

    x = rep(NA, nrow(m) * 4)
    y = rep(NA, nrow(m) * 4)
    id = rep(1:nrow(m), rep(4, nrow(m)))

    for (i in 1:nrow(m)) {
        c = draw_connection(dist[m[i, 1], 1], dist[m[i, 2], 1], dist[m[i, 1], 2], dist[m[i, 2], 2], h[i])
        k = (i - 1) * 4 + 1
        x[k : (k + 3)] = c$x
        y[k : (k + 3)] = c$y
    }

    x = find_coordinates(n, gaps, x * n)$coord
    y = unit(y, "npc")

    if (!horizontal) {
        a = x
        x = unit(1, "npc") - y
        y = unit(1, "npc") - a
    }
    res = polylineGrob(x = x, y = y, id = id)

    return(res)
}


draw_matrix <- function(matrix,
                        border_color,
                        gaps_rows,
                        gaps_cols,
                        fmat,
                        fontsize_number,
                        number_color) {
        n = nrow(matrix)
        m = ncol(matrix)

        coord_x = find_coordinates(m, gaps_cols)
        coord_y = find_coordinates(n, gaps_rows)

        x = coord_x$coord - 0.5 * coord_x$size
        y = unit(1, "npc") - (coord_y$coord - 0.5 * coord_y$size)

        coord = expand.grid(y = y, x = x)

        res = gList()

        res[["rect"]] =
            rectGrob(x = coord$x,
                     y = coord$y,
                     width = coord_x$size,
                     height = coord_y$size,
                     gp = gpar(fill = matrix, col = border_color))

        if (attr(fmat, "draw")) {
            res[["text"]] =
                textGrob(x = coord$x,
                         y = coord$y,
                         label = fmat,
                         gp = gpar(col = number_color, fontsize = fontsize_number))
        }

        res = gTree(children = res)

        return(res)
    }


draw_colnames <- function(coln, gaps, ...) {
    coord = find_coordinates(length(coln), gaps)
    x = coord$coord - 0.5 * coord$size

    res =
        textGrob(coln,
                 x = x,
                 y = unit(1, "npc") - unit(3, "bigpts"),
                 vjust = 0.5,
                 hjust = 0,
                 rot = 270,
                 gp = gpar(...))

    return(res)
}


draw_rownames <- function(rown, gaps, ...) {
    coord = find_coordinates(length(rown), gaps)
    y = unit(1, "npc") - (coord$coord - 0.5 * coord$size)

    res = textGrob(rown, x = unit(3, "bigpts"), y = y, vjust = 0.5, hjust = 0, gp = gpar(...))

    return(res)
}


draw_legend <- function(color, breaks, txt.stats, legend.cex, legend, ...) {
    height = min(unit(1 * legend.cex, "npc"), unit(150 * legend.cex, "bigpts")) 

    ## print('*****')
    ## print(height)
    ## print('*****')
    
    ## print(breaks)  

    ## print('dar_legend')  
    legend_pos = (legend - min(breaks)) / (max(breaks) - min(breaks))
    legend_pos = height * legend_pos + (unit(1, "npc") - height)

    breaks = (breaks - min(breaks)) / (max(breaks) - min(breaks))
    breaks = height * breaks + (unit(1, "npc") - height)

    ## print(breaks)  
    ## print(legend_pos[length(legend_pos)])
    ## print(breaks[-length(breaks)])  

    h = breaks[-1] - breaks[-length(breaks)]

    ## print('***** br')
    ## print(breaks)
    ## print('*****')

    ## print(breaks[-length(breaks)]+unit(1, "npc"))

    ## print('***** h')
    ## print(h)
    ## print('*****')
    ## h = cellheight

    rect =
        rectGrob(x = 0,
                 y = breaks[-length(breaks)],
                 width = unit(10, "bigpts"),
                 height = h,
                 hjust = 0,
                 vjust = 0,
                 gp = gpar(fill = color, col = "#FFFFFF00"))
    text =
        textGrob(names(legend),
                 x = unit(14, "bigpts"),
                 y = legend_pos,
                 hjust = 0,
                 gp = gpar(...))

    ## y.stats = breaks[-length(breaks)]-unit(.1, "npc")
    ## stats = rectGrob(x = 0, y = y.stats, width = unit(10, "bigpts"), height = h, hjust = 0, vjust = 0, gp = gpar(fill ='black'))

    if (!is.na(txt.stats)) {
        ##    rect = rectGrob(x = 0, 
        ##                    y = breaks[-length(breaks)], 
        ##                    width = unit(10, "bigpts"), height = h, hjust = 0, vjust = 0, gp = gpar(fill = color, col = "red"))

        crlf = strsplit(txt.stats, split = "\\n")[[1]]
        h = length(crlf) / 6

        stats =
            textGrob(txt.stats,
                     x = unit(2, "bigpts"),
                     y = legend_pos[1] - unit(2, "cm"),
                     hjust = 0,
                     gp = gpar(fontface='bold'))

        res = grobTree(rect, text, stats)
    } else
        res = grobTree(rect, text)
    
    return(res)
}


convert_annotations <- function(annotation, annotation_colors) {
    new = annotation
    for (i in 1:ncol(annotation)) {
        a = annotation[, i]
        b = annotation_colors[[colnames(annotation)[i]]]
        if (is.character(a) | is.factor(a)) {
            a = as.character(a)
            if (length(setdiff(a, names(b))) > 0) {
                stop(sprintf("Factor levels on variable %s do not match with annotation_colors",
                             colnames(annotation)[i]))
            }
            new[, i] = b[a]
        }
        else{
            a = cut(a, breaks = 100)
            new[, i] = colorRampPalette(b)(100)[a]
        }
    }
    return(as.matrix(new))
}


draw_annotations <- function(converted_annotations,
                             border_color,
                             gaps, fontsize,
                             horizontal) {
    n = ncol(converted_annotations)
    m = nrow(converted_annotations)

    coord_x = find_coordinates(m, gaps)

    x = coord_x$coord - 0.5 * coord_x$size

    ## y = cumsum(rep(fontsize, n)) - 4 + cumsum(rep(2, n))
    y = cumsum(rep(fontsize, n)) + cumsum(rep(2, n)) - fontsize / 2 + 1 
    y = unit(y, "bigpts")

    if (horizontal) {
        coord = expand.grid(x = x, y = y)
        res =
            rectGrob(x = coord$x,
                     y = coord$y,
                     width = coord_x$size,
                     height = unit(fontsize, "bigpts"),
                     gp = gpar(fill = converted_annotations, col = border_color))
    } else {
        a = x
        x = unit(1, "npc") - y
        y = unit(1, "npc") - a

        coord = expand.grid(y = y, x = x)
        res =
            rectGrob(x = coord$x,
                     y = coord$y,
                     width = unit(fontsize, "bigpts"),
                     height = coord_x$size,
                     gp = gpar(fill = converted_annotations, col = border_color))
    }
    return(res)
}

draw_annotation_names = function(annotations, fontsize, horizontal) {
    n = ncol(annotations)

    x = unit(3, "bigpts")

    y = cumsum(rep(fontsize, n)) + cumsum(rep(2, n)) - fontsize / 2 + 1 
    y = unit(y, "bigpts")

    if (horizontal) {
        res = textGrob(colnames(annotations), x = x, y = y, hjust = 0, gp = gpar(fontsize = fontsize, fontface = 2))
    } else {
        a = x
        x = unit(1, "npc") - y
        y = unit(1, "npc") - a

        res = textGrob(colnames(annotations),
            x = x,
            y = y,
            vjust = 0.5,
            hjust = 0,
            rot = 270,
            gp = gpar(fontsize = fontsize, fontface = 2))
    }

    return(res)
}


draw_annotation_legend <- function(annotation,
                                   annotation_colors,
                                   border_color,
                                   ...) {
    y = unit(1, "npc")
    text_height = unit(1, "grobheight", textGrob("FGH", gp = gpar(...)))

    res = gList()
    for (i in names(annotation)) {
        res[[i]] =
            textGrob(i,
                     x = 0,
                     y = y,
                     vjust = 1,
                     hjust = 0,
                     gp = gpar(fontface = "bold", ...))

        y = y - 1.5 * text_height
        if (is.character(annotation[[i]]) | is.factor(annotation[[i]])) {
            n = length(annotation_colors[[i]])
            yy = y - (1:n - 1) * 2 * text_height

            res[[paste(i, "r")]] =
                rectGrob(x = unit(0, "npc"),
                         y = yy,
                         hjust = 0,
                         vjust = 1,
                         height = 2 * text_height,
                         width = 2 * text_height,
                         gp = gpar(col = border_color, fill = annotation_colors[[i]]))
            res[[paste(i, "t")]] =
                textGrob(names(annotation_colors[[i]]),
                         x = text_height * 2.4,
                         y = yy - text_height,
                         hjust = 0,
                         vjust = 0.5,
                         gp = gpar(...))

            y = y - n * 2 * text_height

        } else {
            yy = y - 8 * text_height + seq(0, 1, 0.25)[-1] * 8 * text_height
            h = 8 * text_height * 0.25

            res[[paste(i, "r")]] =
                rectGrob(x = unit(0, "npc"),
                         y = yy,
                         hjust = 0,
                         vjust = 1,
                         height = h,
                         width = 2 * text_height,
                         gp =
                             gpar(col = NA,
                                  fill = colorRampPalette(annotation_colors[[i]])(4)))
            
            res[[paste(i, "r2")]] =
                rectGrob(x = unit(0, "npc"),
                         y = y,
                         hjust = 0,
                         vjust = 1,
                         height = 8 * text_height,
                         width = 2 * text_height,
                         gp = gpar(col = border_color))

            txt = rev(range(grid.pretty(range(annotation[[i]], na.rm = TRUE))))
            yy = y - c(1, 7) * text_height
            res[[paste(i, "t")]] =
                textGrob(txt,
                         x = text_height * 2.4,
                         y = yy,
                         hjust = 0,
                         vjust = 0.5,
                         gp = gpar(...))
            y = y - 8 * text_height
        }
        y = y - 1.5 * text_height
    }

    res = gTree(children = res)

    return(res)
}


draw_main <- function(text, ...) {
    res = textGrob(text, gp = gpar(fontface = "bold", ...))

    return(res)
}


vplayout <- function(x, y) {
    return(viewport(layout.pos.row = x, layout.pos.col = y))
}


heatmap_motor <- function(matrix,
                          border_color,
                          cellwidth,
                          cellheight,
                          tree_col,
                          tree_row,
                          treeheight_col,
                          treeheight_row,
                          filename,
                          width,
                          height,
                          breaks,
                          color,
                          legend,
                          annotation_row,
                          annotation_col,
                          annotation_colors,
                          annotation_legend,
                          main,
                          fontsize,
                          fontsize_row,
                          fontsize_col,
                          fmat,
                          fontsize_number,
                          number_color,
                          gaps_col,
                          gaps_row,
                          labels_row,
                          labels_col,
                          legend.cex,
                          txt.stats, ...) {
    ## Set layout
    lo =
        lo(coln = labels_col,
           rown = labels_row,
           nrow = nrow(matrix),
           ncol = ncol(matrix),
           cellwidth = cellwidth,
           cellheight = cellheight,
           treeheight_col = treeheight_col,
           treeheight_row = treeheight_row,
           legend = legend,
           annotation_col = annotation_col,
           annotation_row = annotation_row,
           annotation_colors = annotation_colors,
           annotation_legend = annotation_legend,
           main = main,
           fontsize = fontsize,
           fontsize_row = fontsize_row,
           fontsize_col = fontsize_col,
           gaps_row = gaps_row,
           gaps_col = gaps_col,
           ...)

    res = lo$gt
    mindim = lo$mindim

    if (!is.na(filename)) {
        if (is.na(height)) {
            height = convertHeight(gtable_height(res), "inches", valueOnly = TRUE)
        }
        if (is.na(width)) {
            width = convertWidth(gtable_width(res), "inches", valueOnly = TRUE)
        }

        ## Get file type.
        
        r = regexpr("\\.[a-zA-Z]*$", filename)
        if (r == -1) stop("Improper filename")
        ending = substr(filename, r + 1, r + attr(r, "match.length"))

        f =
            switch(ending,
                   pdf = function(x, ...) pdf(x, ...),
                   png = function(x, ...) png(x, units = "in", res = 300, ...),
                   jpeg = function(x, ...) jpeg(x, units = "in", res = 300, ...),
                   jpg = function(x, ...) jpeg(x, units = "in", res = 300, ...),
                   tiff = function(x, ...) tiff(x, units = "in", res = 300, compression = "lzw", ...),
                   bmp = function(x, ...) bmp(x, units = "in", res = 300, ...),
                   stop("File type should be: pdf, png, bmp, jpg, tiff")
                   )

        ## print(sprintf("height:%f width:%f", height, width))

        ## gt = heatmap_motor(matrix, cellwidth = cellwidth,
        ## cellheight = cellheight, border_color = border_color,
        ## tree_col = tree_col, tree_row = tree_row, treeheight_col =
        ## treeheight_col, treeheight_row = treeheight_row, breaks =
        ## breaks, color = color, legend = legend, annotation_col =
        ## annotation_col, annotation_row = annotation_row,
        ## annotation_colors = annotation_colors, annotation_legend =
        ## annotation_legend, filename = NA, main = main, fontsize =
        ## fontsize, fontsize_row = fontsize_row, fontsize_col =
        ## fontsize_col, fmat = fmat, fontsize_number =
        ## fontsize_number, number_color = number_color, labels_row =
        ## labels_row, labels_col = labels_col, gaps_col = gaps_col,
        ## gaps_row = gaps_row, ...)

        f(filename, height = height, width = width)
        gt =
            heatmap_motor(matrix,
                          cellwidth = cellwidth,
                          cellheight = cellheight,
                          border_color = border_color,
                          tree_col = tree_col,
                          tree_row = tree_row,
                          treeheight_col = treeheight_col,
                          treeheight_row = treeheight_row,
                          breaks = breaks,
                          color = color,
                          legend = legend,
                          annotation_col = annotation_col,
                          annotation_row = annotation_row,
                          annotation_colors = annotation_colors,
                          annotation_legend = annotation_legend,
                          filename = NA,
                          main = main,
                          fontsize = fontsize,
                          fontsize_row = fontsize_row,
                          fontsize_col = fontsize_col,
                          fmat = fmat,
                          fontsize_number = fontsize_number,
                          number_color = number_color,
                          labels_row = labels_row,
                          labels_col = labels_col,
                          gaps_col = gaps_col,
                          gaps_row = gaps_row,
                          legend.cex = legend.cex,
                          txt.stats = txt.stats,
                          ...)
        grid.draw(gt)
        dev.off()

        return(gt)
    }

    ## Omit border color if cell size is too small.
    
    if (mindim < 3) border_color = NA

    ## Draw title.
    
    if (!is.na(main)) {
        elem = draw_main(main, fontsize = 1.3 * fontsize, ...)
        res = gtable_add_grob(res, elem, t = 1, l = 3, name = "main")
    }

    ## Draw tree for the columns.
    
    if (!is.na(tree_col[[1]][1]) & treeheight_col != 0) {
        elem = draw_dendrogram(tree_col, gaps_col, horizontal = TRUE)
        res = gtable_add_grob(res, elem, t = 2, l = 3, name = "col_tree")
    }

    ## Draw tree for the rows.
    
    if (!is.na(tree_row[[1]][1]) & treeheight_row != 0) {
        elem = draw_dendrogram(tree_row, gaps_row, horizontal = FALSE)
        res = gtable_add_grob(res, elem, t = 4, l = 1, name = "row_tree")
    }

    ## Draw matrix.
    
    elem = draw_matrix(matrix, border_color, gaps_row, gaps_col, fmat, fontsize_number, number_color)
    res = gtable_add_grob(res, elem, t = 4, l = 3, clip = "off", name = "matrix")


    ## Draw colnames.
    
    if (length(labels_col) != 0) {
        pars = list(labels_col, gaps = gaps_col, fontsize = fontsize_col, ...)
        elem = do.call(draw_colnames, pars)
        res = gtable_add_grob(res, elem, t = 5, l = 3, clip = "off", name = "col_names")
    }

    ## Giulio
    ##res = gtable_add_grob(
    ##  res, elem, t = 6, l = 3, clip = "off", name = "giulio"
    ##  )
    
    ## Draw rownames.

    if (length(labels_row) != 0) {
        pars = list(labels_row, gaps = gaps_row, fontsize = fontsize_row, ...)
        elem = do.call(draw_rownames, pars)
        res = gtable_add_grob(res, elem, t = 4, l = 4, clip = "off", name = "row_names")
    }

    ## Draw annotation tracks on cols.
    
    if (!is.na(annotation_col[[1]][1])) {
        ## Draw tracks.
        
        converted_annotation = convert_annotations(annotation_col, annotation_colors)
        elem = draw_annotations(converted_annotation, border_color, gaps_col, fontsize, horizontal = TRUE)
        res = gtable_add_grob(res, elem, t = 3, l = 3, clip = "off", name = "col_annotation")

        ## Draw names.
        
        elem = draw_annotation_names(annotation_col, fontsize, horizontal = TRUE)
        res = gtable_add_grob(res, elem, t = 3, l = 4, clip = "off", name = "row_annotation_names")
    }

    ## Draw annotation tracks on rows.
    
    if (!is.na(annotation_row[[1]][1])) {
        ## Draw tracks.
        
        converted_annotation = convert_annotations(annotation_row, annotation_colors)
        elem = draw_annotations(converted_annotation, border_color, gaps_row, fontsize, horizontal = FALSE)
        res = gtable_add_grob(res, elem, t = 4, l = 2, clip = "off", name = "row_annotation")

        ## Draw names.
        
        elem = draw_annotation_names(annotation_row, fontsize, horizontal = FALSE)
        res = gtable_add_grob(res, elem, t = 5, l = 2, clip = "off", name = "row_annotation_names")
    }

    ## Draw annotation legend.
    
    annotation = c(annotation_col[length(annotation_col):1], annotation_row[length(annotation_row):1])
    annotation = annotation[unlist(lapply(annotation, function(x) !is.na(x[1])))]

    if (length(annotation) > 0 & annotation_legend) {
        elem = draw_annotation_legend(annotation, annotation_colors, border_color, fontsize = fontsize, ...)

        t = ifelse(is.null(labels_row), 4, 3)
        res = gtable_add_grob(res, elem, t = t, l = 6, b = 5, clip = "off", name = "annotation_legend")
    }

    ## Draw legend.
    
    if (!is.na(legend[1])) {
        elem = draw_legend(color, breaks, txt.stats, legend, legend.cex = legend.cex, fontsize = fontsize, ...)

        t = ifelse(is.null(labels_row), 4, 3)
        res = gtable_add_grob(res, elem, t = t, l = 5, b= 5, clip = "off", name = "legend")
    }

    return(res)
}


generate_breaks <- function(x, n, center = FALSE) {
    if (center) {
        m = max(abs(c(min(x, na.rm = TRUE), max(x, na.rm = TRUE))))
        res = seq(-m, m, length.out = n + 1)
    }
    else{
        res = seq(min(x, na.rm = TRUE), max(x, na.rm = TRUE), length.out = n + 1)
    }

    return(res)
}

    scale_vec_colours = function(x, col = rainbow(10), breaks = NA) {
        return(col[as.numeric(cut(x, breaks = breaks, include.lowest = TRUE))])
    }

    scale_colours = function(mat, col = rainbow(10), breaks = NA) {
        mat = as.matrix(mat)
        return(matrix(scale_vec_colours(as.vector(mat),
                                        col = col,
                                        breaks = breaks),
                      nrow(mat),
                      ncol(mat),
                      dimnames =
                          list(rownames(mat),
                               colnames(mat))))
    }


cluster_mat <- function(mat, distance, method) {
    if (!(method %in% c("ward.D2",
                        "ward",
                        "single",
                        "complete",
                        "average",
                        "mcquitty",
                        "median",
                        "centroid"))) {
        stop("clustering method must be one of: 'ward', ",
             "'ward.D2', 'single', 'complete', 'average', 'mcquitty', 'median' or 'centroid'.")
    }
    if (!(distance[1] %in% c("correlation",
                             "euclidean",
                             "maximum",
                             "manhattan",
                             "canberra",
                             "binary",
                             "minkowski"))
        & is(distance, "dist")) {
        stop("distance has to be a dissimilarity structure as produced by dist ",
             "or one measure from the list: ",
             "'correlation', 'euclidean', 'maximum', 'manhattan', 'canberra', 'binary', 'minkowski'")
    }
    if (distance[1] == "correlation") {
        d = as.dist(1 - cor(t(mat)))
    } else {
        if (is(distance, "dist")) {
            d = distance
        } else {
            d = dist(mat, method = distance)
        }
    }

    return(hclust(d, method = method))
}


scale_rows <- function(x) {
    m = apply(x, 1, mean, na.rm = TRUE)
    s = apply(x, 1, sd, na.rm = TRUE)
    return((x - m) / s)
}


scale_mat <- function(mat, scale) {
    if (!(scale %in% c("none", "row", "column"))) {
        stop("scale argument shoud take values: 'none', 'row' or 'column'")
    }
    mat = switch(scale,
        none = mat,
        row = scale_rows(mat),
        column = t(scale_rows(t(mat))))
    return(mat)
}


generate_annotation_colours <- function(annotation, annotation_colors, drop) {
    if (is.na(annotation_colors)[[1]][1]) {
        annotation_colors = list()
    }
    count = 0
    for (i in 1:length(annotation)) {
        if (is.character(annotation[[i]]) | is.factor(annotation[[i]])) {
            if (is.factor(annotation[[i]]) & !drop) {
                count = count + length(levels(annotation[[i]]))
            } else {
                count = count + length(unique(annotation[[i]]))
            }
        }
    }

    factor_colors = dscale(factor(1:count), hue_pal(l = 75))

    set.seed(3453)

    cont_counter = 2
    for (i in 1:length(annotation)) {
        if (!(names(annotation)[i] %in% names(annotation_colors))) {
            if (is.character(annotation[[i]]) | is.factor(annotation[[i]])) {
                n = length(unique(annotation[[i]]))
                if (is.factor(annotation[[i]]) & !drop) {
                    n = length(levels(annotation[[i]]))
                }
                ind = sample(1:length(factor_colors), n)
                annotation_colors[[names(annotation)[i]]] = factor_colors[ind]
                l = levels(as.factor(annotation[[i]]))
                l = l[l %in% unique(annotation[[i]])]
                if (is.factor(annotation[[i]]) & !drop) {
                    l = levels(annotation[[i]])
                }

                names(annotation_colors[[names(annotation)[i]]]) = l
                factor_colors = factor_colors[-ind]
            } else {
                annotation_colors[[names(annotation)[i]]] = brewer_pal("seq", cont_counter)(5)[1:4]
                cont_counter = cont_counter + 1
            }
        }
    }
    return(annotation_colors)
}


kmeans_pheatmap <- function(mat, k = min(nrow(mat), 150), sd_limit = NA, ...) {
    ## Filter data.
    
    if (!is.na(sd_limit)) {
        s = apply(mat, 1, sd)
        mat = mat[s > sd_limit, ]    
    }

    ## Cluster data.
    
    set.seed(1245678)
    km = kmeans(mat, k, iter.max = 100)
    mat2 = km$centers

    ## Compose rownames.
    
    t = table(km$cluster)
    rownames(mat2) = sprintf("cl%s_size_%d", names(t), t)

    ## Draw heatmap.
    
    pheatmap(mat2, ...)
}


find_gaps <- function(tree, cutree_n) {
    v = cutree(tree, cutree_n)[tree$order]
    gaps = which((v[-1] - v[-length(v)]) != 0)
}


#' A function to draw clustered heatmaps.
#' 
#' A function to draw clustered heatmaps where one has better control over some graphical 
#' parameters such as cell size, etc. 
#' 
#' The function also allows to aggregate the rows using kmeans clustering. This is 
#' advisable if number of rows is so big that R cannot handle their hierarchical 
#' clustering anymore, roughly more than 1000. Instead of showing all the rows 
#' separately one can cluster the rows in advance and show only the cluster centers. 
#' The number of clusters can be tuned with parameter kmeans_k.
#'
#' This is a modified version of the original pheatmap 
#' (https://cran.r-project.org/web/packages/pheatmap/index.html)
#' edited in accordance with GPL-2.
#'
#' @param mat numeric matrix of the values to be plotted.
#' @param color vector of colors used in heatmap.
#' @param kmeans_k the number of kmeans clusters to make, if we want to agggregate the 
#' rows before drawing heatmap. If NA then the rows are not aggregated.
#' @param breaks a sequence of numbers that covers the range of values in mat and is one 
#' element longer than color vector. Used for mapping values to colors. Useful, if needed 
#' to map certain values to certain colors, to certain values. If value is NA then the 
#' breaks are calculated automatically.
#' @param border_color color of cell borders on heatmap, use NA if no border should be 
#' drawn.
#' @param cellwidth individual cell width in points. If left as NA, then the values 
#' depend on the size of plotting window.
#' @param cellheight individual cell height in points. If left as NA, 
#' then the values depend on the size of plotting window.
#' @param scale character indicating if the values should be centered and scaled in 
#' either the row direction or the column direction, or none. Corresponding values are 
#' \code{"row"}, \code{"column"} and \code{"none"}
#' @param cluster_rows boolean values determining if rows should be clustered,
#' @param cluster_cols boolean values determining if columns should be clustered.
#' @param clustering_distance_rows distance measure used in clustering rows. Possible 
#' values are \code{"correlation"} for Pearson correlation and all the distances 
#' supported by \code{\link{dist}}, such as \code{"euclidean"}, etc. If the value is none 
#' of the above it is assumed that a distance matrix is provided.
#' @param clustering_distance_cols distance measure used in clustering columns. Possible 
#' values the same as for clustering_distance_rows.
#' @param clustering_method clustering method used. Accepts the same values as 
#' \code{\link{hclust}}.
#' @param cutree_rows number of clusters the rows are divided into, based on the
#'  hierarchical clustering (using cutree), if rows are not clustered, the 
#' argument is ignored
#' @param cutree_cols similar to \code{cutree_rows}, but for columns
#' @param treeheight_row the height of a tree for rows, if these are clustered. 
#' Default value 50 points.
#' @param treeheight_col the height of a tree for columns, if these are clustered. 
#' Default value 50 points.
#' @param legend logical to determine if legend should be drawn or not.
#' @param legend_breaks vector of breakpoints for the legend.
#' @param legend_labels vector of labels for the \code{legend_breaks}.
#' @param annotation_row data frame that specifies the annotations shown on left
#'  side of the heatmap. Each row defines the features for a specific row. The 
#' rows in the data and in the annotation are matched using corresponding row
#'  names. Note that color schemes takes into account if variable is continuous
#'  or discrete.
#' @param annotation_col similar to annotation_row, but for columns. 
#' @param annotation deprecated parameter that currently sets the annotation_col if it is missing
#' @param annotation_colors list for specifying annotation_row and 
#' annotation_col track colors manually. It is  possible to define the colors 
#' for only some of the features. Check examples for  details.
#' @param annotation_legend boolean value showing if the legend for annotation 
#' tracks should be drawn. 
#' @param drop_levels logical to determine if unused levels are also shown in 
#' the legend
#' @param show_rownames boolean specifying if column names are be shown.
#' @param show_colnames boolean specifying if column names are be shown.
#' @param main the title of the plot
#' @param fontsize base fontsize for the plot 
#' @param legend.cex Default 0.5; determines legend size if \code{legend = TRUE}
#' @param fontsize_row fontsize for rownames (Default: fontsize) 
#' @param fontsize_col fontsize for colnames (Default: fontsize) 
#' @param display_numbers logical determining if the numeric values are also printed to 
#' the cells. If this is a matrix (with same dimensions as original matrix), the contents
#' of the matrix are shown instead of original values.
#' @param number_format format strings (C printf style) of the numbers shown in cells. 
#' For example "\code{\%.2f}" shows 2 decimal places and "\code{\%.1e}" shows exponential 
#' notation (see more in \code{\link{sprintf}}).
#' @param number_color color of the text    
#' @param fontsize_number fontsize of the numbers displayed in cells
#' @param gaps_row vector of row indices that show shere to put gaps into
#'  heatmap. Used only if the rows are not clustered. See \code{cutree_row}
#'  to see how to introduce gaps to clustered rows. 
#' @param gaps_col similar to gaps_row, but for columns.
#' @param labels_row custom labels for rows that are used instead of rownames.
#' @param labels_col similar to labels_row, but for columns.
#' @param filename file path where to save the picture. Filetype is decided by 
#' the extension in the path. Currently following formats are supported: png, pdf, tiff,
#'  bmp, jpeg. Even if the plot does not fit into the plotting window, the file size is 
#' calculated so that the plot would fit there, unless specified otherwise.
#' @param width manual option for determining the output file width in inches.
#' @param height manual option for determining the output file height in inches.
#' @param silent do not draw the plot (useful when using the gtable output)
#' @param txt.stats By default, shows a summary statistics for shown data (n,m, |G| and |P|)
#' @param \dots graphical parameters for the text used in plot. Parameters passed to 
#' \code{\link{grid.text}}, see \code{\link{gpar}}. 
#' 
#' @return 
#' Invisibly a list of components 
#' \itemize{
#'     \item \code{tree_row} the clustering of rows as \code{\link{hclust}} object 
#'     \item \code{tree_col} the clustering of columns as \code{\link{hclust}} object
#'     \item \code{kmeans} the kmeans clustering of rows if parameter \code{kmeans_k} was 
#' specified 
#' }
#' 
#' @author  Raivo Kolde <rkolde@@gmail.com>
#' @examples
#' # Create test matrix
#' test = matrix(rnorm(200), 20, 10)
#' test[1:10, seq(1, 10, 2)] = test[1:10, seq(1, 10, 2)] + 3
#' test[11:20, seq(2, 10, 2)] = test[11:20, seq(2, 10, 2)] + 2
#' test[15:20, seq(2, 10, 2)] = test[15:20, seq(2, 10, 2)] + 4
#' colnames(test) = paste("Test", 1:10, sep = "")
#' rownames(test) = paste("Gene", 1:20, sep = "")
#' 
#' # Draw heatmaps
#' pheatmap(test)

#' @export pheatmap
#' @importFrom grid unit textGrob gpar unit.c viewport convertWidth
#' @importFrom grid convertHeight gList gTree rectGrob grobTree polylineGrob
#' @importFrom grid grid.draw grid.pretty grid.newpage
#' @importFrom scales dscale hue_pal brewer_pal
#' @importFrom RColorBrewer brewer.pal
#' @importFrom stats as.dist cor hclust dist cutree sd kmeans sd
#' @importFrom grDevices colorRampPalette pdf png jpeg tiff bmp dev.off rainbow
#' @importFrom graphics strwidth
#' 
pheatmap <- function(mat,
                     color = colorRampPalette(rev(brewer.pal(n = 7, name = "RdYlBu")))(100),
                     kmeans_k = NA,
                     breaks = NA,
                     border_color = "grey60",
                     cellwidth = NA,
                     cellheight = NA,
                     scale = "none",
                     cluster_rows = TRUE,
                     cluster_cols = TRUE,
                     clustering_distance_rows = "euclidean",
                     clustering_distance_cols = "euclidean",
                     clustering_method = "complete",
                     cutree_rows = NA,
                     cutree_cols = NA,
                     treeheight_row = ifelse(cluster_rows, 50, 0),
                     treeheight_col = ifelse(cluster_cols, 50, 0),
                     legend = TRUE,
                     legend_breaks = NA,
                     legend_labels = NA,
                     annotation_row = NA,
                     annotation_col = NA,
                     annotation = NA,
                     annotation_colors = NA,
                     annotation_legend = TRUE,
                     drop_levels = TRUE,
                     show_rownames = TRUE,
                     show_colnames = TRUE,
                     main = NA,
                     fontsize = 10,
                     fontsize_row = fontsize,
                     fontsize_col = fontsize,
                     display_numbers = FALSE,
                     number_format = "%.2f",
                     number_color = "grey30",
                     fontsize_number = 0.8 * fontsize,
                     gaps_row = NULL,
                     gaps_col = NULL,
                     labels_row = NULL,
                     labels_col = NULL,
                     filename = NA,
                     width = NA,
                     height = NA,
                     silent = FALSE,
                     legend.cex = 1,
                     txt.stats = NA,
                     ...) {
    ## Set labels.
    
    if (is.null(labels_row)) {
        labels_row = rownames(mat)
    }
    if (is.null(labels_col)) {
        labels_col = colnames(mat)
    }

    ## Preprocess matrix.
    
    mat = as.matrix(mat)
    if (scale != "none") {
        mat = scale_mat(mat, scale)
        if (is.na(breaks)) {
            breaks = generate_breaks(mat, length(color), center = TRUE)
        }
    }

    ## Kmeans.
    
    if (!is.na(kmeans_k)) {
        ## Cluster data.
        
        km = kmeans(mat, kmeans_k, iter.max = 100)
        mat = km$centers

        ## Compose rownames.
        
        t = table(km$cluster)
        labels_row = sprintf("Cluster: %s Size: %d", names(t), t)
    } else {
        km = NA
    }

    ## Format numbers to be displayed in cells.
    
    if (is.matrix(display_numbers) | is.data.frame(display_numbers)) {
        if (nrow(display_numbers) != nrow(mat) | ncol(display_numbers) != ncol(mat)) {
            stop("If display_numbers provided as matrix, its dimensions have to match with mat")
        }

        display_numbers = as.matrix(display_numbers)
        fmat = matrix(as.character(display_numbers), nrow = nrow(display_numbers), ncol = ncol(display_numbers))
        fmat_draw = TRUE
    } else {
        if (display_numbers) {
            fmat = matrix(sprintf(number_format, mat), nrow = nrow(mat), ncol = ncol(mat))
            fmat_draw = TRUE
        } else {
            fmat = matrix(NA, nrow = nrow(mat), ncol = ncol(mat))
            fmat_draw = FALSE
        }
    }

    ## Do clustering.
    
    if (cluster_rows) {
        tree_row = cluster_mat(mat, distance = clustering_distance_rows, method = clustering_method)
        mat = mat[tree_row$order, , drop = FALSE]
        fmat = fmat[tree_row$order, , drop = FALSE]
        labels_row = labels_row[tree_row$order]
        if (!is.na(cutree_rows)) {
            gaps_row = find_gaps(tree_row, cutree_rows)
        } else {
            gaps_row = NULL
        }
    } else {
        tree_row = NA
        treeheight_row = 0
    }

    if (cluster_cols) {
        tree_col = cluster_mat(t(mat), distance = clustering_distance_cols, method = clustering_method)
        mat = mat[, tree_col$order, drop = FALSE]
        fmat = fmat[, tree_col$order, drop = FALSE]
        labels_col = labels_col[tree_col$order]
        if (!is.na(cutree_cols)) {
            gaps_col = find_gaps(tree_col, cutree_cols)
        } else {
            gaps_col = NULL
        }
    } else {
        tree_col = NA
        treeheight_col = 0
    }

    attr(fmat, "draw") = fmat_draw

    ## Colors and scales.
    
    if (!is.na(legend_breaks[1]) & !is.na(legend_labels[1])) {
        if (length(legend_breaks) != length(legend_labels)) {
            stop("Lengths of legend_breaks and legend_labels must be the same")
        }
    }

    if (is.na(breaks[1])) {
        breaks = generate_breaks(as.vector(mat), length(color))
    }
    
    if (legend & is.na(legend_breaks[1])) {
        legend = grid.pretty(range(as.vector(breaks)))
        names(legend) = legend
    } else if (legend & !is.na(legend_breaks[1])) {
        legend = legend_breaks[legend_breaks >= min(breaks) & legend_breaks <= max(breaks)]

        if (!is.na(legend_labels[1])) {
            legend_labels = legend_labels[legend_breaks >= min(breaks) & legend_breaks <= max(breaks)]
            names(legend) = legend_labels
        } else {
            names(legend) = legend
        }
    } else {
        legend = NA
    }
    mat = scale_colours(mat, col = color, breaks = breaks)

    ## Preparing annotations.
    
    if (is.na(annotation_col[[1]][1]) & !is.na(annotation[[1]][1])) {
        annotation_col = annotation
    }
    
    ## Select only the ones present in the matrix.
    
    if (!is.na(annotation_col[[1]][1])) {
        annotation_col = annotation_col[colnames(mat), , drop = FALSE]
    }

    if (!is.na(annotation_row[[1]][1])) {
        annotation_row = annotation_row[rownames(mat), , drop = FALSE]
    }

    annotation = c(annotation_row, annotation_col)
    annotation = annotation[unlist(lapply(annotation, function(x) !is.na(x[1])))]
    if (length(annotation) != 0) {
        annotation_colors = generate_annotation_colours(annotation, annotation_colors, drop = drop_levels)
    } else {
        annotation_colors = NA
    }

    if (!show_rownames) {
        labels_row = NULL
    }

    if (!show_colnames) {
        labels_col = NULL
    }

    ## Draw heatmap
    gt =
        heatmap_motor(mat,
                      border_color = border_color,
                      cellwidth = cellwidth,
                      cellheight = cellheight,
                      treeheight_col = treeheight_col,
                      treeheight_row = treeheight_row,
                      tree_col = tree_col,
                      tree_row = tree_row,
                      filename = filename,
                      width = width,
                      height = height,
                      breaks = breaks,
                      color = color,
                      legend = legend,
                      annotation_row = annotation_row,
                      annotation_col = annotation_col,
                      annotation_colors = annotation_colors,
                      annotation_legend = annotation_legend,
                      main = main,
                      fontsize = fontsize,
                      fontsize_row = fontsize_row,
                      fontsize_col = fontsize_col,
                      fmat = fmat,
                      fontsize_number = fontsize_number,
                      number_color = number_color,
                      gaps_row = gaps_row,
                      gaps_col = gaps_col,
                      labels_row = labels_row,
                      labels_col = labels_col,
                      legend.cex = legend.cex,
                      txt.stats = txt.stats,
                      ...)

    if (is.na(filename) & !silent) {
        grid.newpage()
        grid.draw(gt)
    }

    invisible(list(tree_row = tree_row, tree_col = tree_col, kmeans = km, gtable = gt))
}


#' tronco.pattern.plot : plot a genotype
#' 
#' @title tronco.pattern.plot
#' @param x A TRONCO compliant dataset
#' @param group A list of events (see as.events() for details)
#' @param to A target event
#' @param gap.cex cex parameter for gap
#' @param legend.cex cex parameter for legend
#' @param label.cex cex parameter for label
#' @param title title
#' @param mode can be 'circos' or 'barplot'
#' @export tronco.pattern.plot
#' @importFrom circlize circos.clear circos.par chordDiagram circos.text
#' @importFrom circlize circos.trackPlotRegion get.cell.meta.data
#' @importFrom graphics layout par barplot plot.new legend
#' @importFrom grDevices rgb col2rgb
#' 
tronco.pattern.plot <- function(x,
                         group=as.events(x),
                         to,
                         gap.cex = 1.0,
                         legend.cex = 1.0,
                         label.cex = 1.0,
                         title=paste(to[1], to[2]),
                         mode = 'barplot') {
    ## Keys.
    
    events = as.events(x)

    keys = NULL
    for (i in 1:nrow(group))
        keys =
            c(keys,
              rownames(as.events(x,
                                 genes = group[i, 'event'],
                                 types = group[i, 'type'])))

    cat('Group:\n')
    events.names = events[keys, , drop = FALSE]
    print(events.names)

    cat('Group tested against:', to[1], to[2], '\n')

    ## HARD exclusivity: 1 for each pattern element
    ## CO-OCCURRENCE: 1
    ## SOFT exclusivity: 1
    ## OTHERS: 1
    
    matrix = matrix(0, nrow = length(keys) + 3, ncol = 1)
    rownames(matrix) = c(keys, 'soft', 'co-occurrence', 'other')
    ## colnames(matrix) = paste(to, collapse = ':')
    colnames(matrix) = to[1]

    to.samples = which.samples(x, gene=to[1], type=to[2])
    cat('Pattern conditioned to samples:', paste(to.samples, collapse = ', '), '\n')

    pattern.samples = list()
    hard.pattern.samples = list()
    for (i in 1:length(keys)) {
        ## Samples
        samples = which.samples(x, gene= events.names[i, 'event'], type= events.names[i, 'type'])

        ## Pattern samples
        pattern.samples = append(pattern.samples, list(samples))

        ## Other negative samples
        negative.samples = list()
        for (j in 1:length(keys))
            if (i != j)
                negative.samples =
                    append(negative.samples, 
                           list(which.samples(x,
                                              gene = events.names[j, 'event'],
                                              type = events.names[j, 'type'],
                                              neg = TRUE)))

        pattern.negative = Reduce(intersect, negative.samples)
        hard.pattern.samples = append(hard.pattern.samples, list(intersect(samples, pattern.negative)))   
    }
    names(pattern.samples) = keys
    names(hard.pattern.samples) = keys

    ## print(hard.pattern.samples)
    
    ## print('tutti:')
    ## print(unlist(pattern.samples))

    ## CO-OCCURRENCES.
    
    pattern.co.occurrences = Reduce(intersect, pattern.samples)
    co.occurrences = intersect(to.samples, pattern.co.occurrences)
    matrix['co-occurrence', ] = length(co.occurrences)
    cat('Co-occurrence in #samples: ', matrix['co-occurrence', ], '\n')

    ## HARD EXCLUSIVITY.
    
    for (i in 1:length(keys))
        matrix[keys[i], ] = length(intersect(to.samples, hard.pattern.samples[[keys[i]]])) 
    cat('Hard exclusivity in #samples:', matrix[keys, ], '\n')  

    ## OTHER.
    
    union = unique(unlist(pattern.samples))
    union = setdiff(as.samples(x), union)
    matrix['other', ] = length(intersect(to.samples, union))
    cat('Other observations in #samples:', matrix['other', ], '\n') 

    ## SOFT EXCLUSIVITY.
    
    matrix['soft', ] = length(to.samples) - colSums(matrix)
    cat('Soft exclusivity in #samples:', matrix['soft', ], '\n')  

    ## Choose colors according to event type.
    
    sector.color = rep('gray', nrow(matrix) + 1) 
    link.color = rep('gray', nrow(matrix)) 

    names(sector.color) = c(rownames(matrix), colnames(matrix))
    names(link.color) = rownames(matrix)

    add.alpha <- function(col, alpha = 1) {
        if (missing(col)) stop("Please provide a vector of colours.")
        apply(sapply(col, col2rgb)/255, 2, function(x) rgb(x[1], x[2], x[3], alpha=alpha))  
    }

    ## Link colors - event types .
    
    for (i in 1:length(keys))
        link.color[keys[i]] = as.colors(x)[events.names[keys[i], 'type' ]]

    ## print(link.color)

    link.color['soft'] = 'orange'
    link.color['co-occurrence'] = 'darkgreen'


    idx.max = which(matrix == max(matrix))
    link.style = matrix(0, nrow=nrow(matrix), ncol=ncol(matrix))
    rownames(link.style) = rownames(matrix)
    colnames(link.style) = colnames(matrix)
    link.style[idx.max, 1] = 5
    
    ## print(link.style)

    ## link.color[idx.max] = add.alpha(link.color[idx.max], .3)
    ## print(link.color)


    ## Sector colors - event types.
    
    sector.color[1:length(keys)] = 'red' # XOR
    sector.color['soft'] = 'orange'      # OR
    sector.color['co-occurrence'] = 'darkgreen' # AND
    sector.color[colnames(matrix)] = as.colors(x)[as.events(x, genes = to[1], types=to[2])[, 'type' ]]

    ## print(link.color)
    ## print(sector.color)

    ## Informative labels
    for (i in 1:length(keys)) {
        ## rownames(matrix)[i] = paste(paste(rep(' ', i), collapse = ''), events.names[i, 'event' ])
        if (nevents(x, genes = events.names[i, 'event' ]) > 1)
            rownames(matrix)[i] = paste(paste(rep(' ', i), collapse = ''), events.names[i, 'event' ])
        else rownames(matrix)[i] = events.names[i, 'event' ]

        names(sector.color)[i] = rownames(matrix)[i]    
    }

    if ( mode == 'circos') { 

        cat('Circlize matrix.\n')
        print(matrix)

        circos.clear()

        gaps = c(rep(2, length(keys) - 2), rep(15 * gap.cex, 4), rep(40 * gap.cex, 2))

        circos.par(gap.degree = gaps)

        chordDiagram(matrix, 
                     grid.col = sector.color,
                     annotationTrack = "grid", 
                     preAllocateTracks = list(track.height = 0.3),
                     row.col = link.color,
                     link.border = 'black',
                     link.lty = link.style,
                     link.lwd = 0.3,
                     reduce = 0
                     )

        circos.trackPlotRegion(track.index = 1, 
                               panel.fun = function(x, y) {
                                   xlim = get.cell.meta.data("xlim")
                                   ylim = get.cell.meta.data("ylim")
                                   sector.name = get.cell.meta.data("sector.index")
                                   circos.text(mean(xlim),
                                               cex = 1.0 * label.cex,
                                               ylim[1],
                                               sector.name,
                                               facing = "clockwise",
                                               niceFacing = TRUE,
                                               adj = c(0, 0.5)) 
                               },
                               bg.border = NA)
        
    } else { # barplot
        
        layout(matrix(c(1,2,3,3), ncol = 2, byrow = TRUE), heights = c(4, 1))
        par(mai=rep(0.5, 4))

        basic.unit = 2
        widths =
            c(rep(basic.unit, length(keys)), 
              rep(basic.unit * 2, 3)
              )

        spaces =
            c(rep(0, length(keys)),
              3, 
              rep(.5, 2)
              )

        print(matrix)

        ## barplot(matrix[, 1], widths, space = 0)

        rownames(matrix)[length(keys) + 1] = '2 or more\n(soft-exclusivity)'
        rownames(matrix)[length(keys) + 2] = 'all together\n(co-occurrence)'

        rownames(matrix)[nrow(matrix)] = 'none of the\nevents'

        summary = matrix[ (length(keys) + 1):nrow(matrix), 1, drop = FALSE]
        summary = rbind( sum(matrix[1:length(keys),]), summary)
        rownames(summary) = NULL
        colnames(summary) = paste(to[1], to[2])
        print(summary)  

        ## Plot, but suppress the labels.

        midpts =
            barplot(summary,
                    5,
                    col = c('red', 'orange', 'darkgreen', 'gray'),
                    horiz = FALSE,
                    space = 1,
                    las = 1,
                    main =
                        paste0('Combination of group events \n in ',
                               sum(summary),
                               ' samples with ',
                               to[1],
                               ' ',
                               to[2]),
                    cex.main = .6,
                    cex.names = .5,
                    cex.axis = .5 )

        exclus = matrix[1:length(keys), 1, drop = FALSE]
        ## print(exclus)


        midpts =
            barplot(exclus[, 1],
                    ## widths,
                    col = link.color[1:length(keys)],
                    horiz = TRUE,
                    space = 1,
                    las = 2,
                    main = paste0('Observations supporting \n hard-exclusivity'),
                    cex.main = 0.6,
                    ## xlab = 'number of observations (given KRAS)',
                    cex.names = 0.5,
                    cex.axis = 0.5 )

        par(mai = c(0, 0, 0, 0))
        plot.new()
        
        }

        events.legend.pch = rep(19, length(unique(events.names[, 'type'])))

        legend("topleft",
               cex = 0.6  * legend.cex,
               pt.cex = 1,
               title = expression(bold('Table of observations')),
               horiz=FALSE,
               bty='n',
               legend =
                   c(paste(sum(matrix[1:length(keys) ,]), 'with 1 event (hard exclusivity)'),
                     paste(matrix[length(keys) + 1, ],  'with 2 or more events'),
                     paste(matrix[length(keys) + 2, ], 'with all events (co-occurrence)'),
                     paste(matrix[length(keys) + 3, ], 'with no events')
                     ),
               fill = c('red', 'orange', 'darkgreen', 'gray')
               )

        group.legend = apply(group, 1, paste, collapse = ' in ')

        legend('top',
               cex = .6 * legend.cex,
               pt.cex = 1,
               title = expression(bold('Input group')),
               bty = 'n',
               legend = group.legend
               )

        legend(x = 'topright',
               legend = unique(events.names[, 'type']),
               title = expression(bold('Events type')),
               bty = 'n',
               inset  = +.05,
               cex = .6 * legend.cex,
               pt.cex = 1,
               pch = events.legend.pch,
               col = as.colors(x)[unique(events.names[, 'type'])]
               ## pt.bg = pt_bg
               )
    
}

#### end of file -- visualization.R

## ----include=FALSE------------------------------------------------------------
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)

## -----------------------------------------------------------------------------
dataset = change.color(aCML, 'Ins/Del', 'dodgerblue4')
dataset = change.color(dataset, 'Missense point', '#7FC97F')
as.colors(dataset)

## -----------------------------------------------------------------------------
consolidate.data(dataset)

## -----------------------------------------------------------------------------
alterations = events.selection(as.alterations(aCML), filter.freq = .05)

## -----------------------------------------------------------------------------
gene.hypotheses = c('KRAS', 'NRAS', 'IDH1', 'IDH2', 'TET2', 'SF3B1', 'ASXL1')
aCML.clean = events.selection(aCML,
    filter.in.names=c(as.genes(alterations), gene.hypotheses))
aCML.clean = annotate.description(aCML.clean, 
    'CAPRI - Bionformatics aCML data (selected events)')

## ----fig.width=8, fig.height=5.5, fig.cap="Data selected for aCML reconstruction annotated with the events which are part of a pattern that we will input to CAPRI."----
oncoprint(aCML.clean, gene.annot = list(priors = gene.hypotheses), sample.id = TRUE)

## -----------------------------------------------------------------------------
aCML.hypo = hypothesis.add(aCML.clean, 'NRAS xor KRAS', XOR('NRAS', 'KRAS'))

## ----eval=FALSE---------------------------------------------------------------
# aCML.hypo = hypothesis.add(aCML.hypo, 'NRAS or KRAS',  OR('NRAS', 'KRAS'))

## ----fig.width=6, fig.height=1, fig.cap="Oncoprint output to show the perfect (hard) exclusivity among NRAS/KRAS mutations in aCML"----
oncoprint(events.selection(aCML.hypo,
    filter.in.names = c('KRAS', 'NRAS')),
    font.row = 8,
    ann.hits = FALSE)

## -----------------------------------------------------------------------------
aCML.hypo = hypothesis.add(aCML.hypo, 'SF3B1 xor ASXL1', XOR('SF3B1', XOR('ASXL1')),
    '*')

## -----------------------------------------------------------------------------
as.events(aCML.hypo, genes = 'TET2') 
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 xor IDH2',
    XOR('TET2', 'IDH2'),
    '*')
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 or IDH2',
    OR('TET2', 'IDH2'),
    '*')

## ----fig.width=7, fig.height=2, fig.cap="{oncoprint} output to show the soft exclusivity among NRAS/KRAS mutations in aCML"----
oncoprint(events.selection(aCML.hypo,
    filter.in.names = c('TET2', 'IDH2')),
    font.row = 8,
    ann.hits = FALSE)

## -----------------------------------------------------------------------------
aCML.hypo = hypothesis.add.homologous(aCML.hypo)

## -----------------------------------------------------------------------------
dataset = hypothesis.add.group(aCML.clean, OR, group = c('SETBP1', 'ASXL1', 'CBL'))

## ----fig.width=8, fig.height=6.5, fig.cap="oncoprint} output of the a dataset that has patterns that could be given as input to CAPRI to retrieve a progression model."----
oncoprint(aCML.hypo, gene.annot = list(priors = gene.hypotheses), sample.id = TRUE, 
    font.row=10, font.column=5, cellheight=15, cellwidth=4)

## -----------------------------------------------------------------------------
npatterns(dataset)
nhypotheses(dataset)

## -----------------------------------------------------------------------------
as.patterns(dataset)
as.events.in.patterns(dataset)
as.genes.in.patterns(dataset)
as.types.in.patterns(dataset)

## -----------------------------------------------------------------------------
head(as.hypotheses(dataset))
dataset = delete.hypothesis(dataset, event = 'TET2')
dataset = delete.pattern(dataset, pattern = 'OR_ASXL1_CBL')

## ----pattern-plot,fig.show='hide', fig.width=4, fig.height=2.2, fig.cap="Barplot to show an hypothesis: here we test genes SETBP1 and ASXL1 versus Missense point mutations of  CSF3R, which suggests that  that pattern does not 'capture' all the samples with  CSF3R mutations."----
tronco.pattern.plot(aCML,
    group = as.events(aCML, genes=c('SETBP1', 'ASXL1')),
    to = c('CSF3R', 'Missense point'),
    legend.cex=0.8,
    label.cex=1.0)

## ----pattern-plot-circos, fig.width=6, fig.height=6, fig.cap="Circos to show an hypothesis: here we test genes SETBP1 and ASXL1 versus Missense point mutations of  CSF3R. The combination of this and the previous  plots should allow to understand which pattern we shall write in an attempt to capture a potential causality relation between the pattern and the event."----
tronco.pattern.plot(aCML,
    group = as.events(aCML, genes=c('TET2', 'ASXL1')),
    to = c('CSF3R', 'Missense point'),
    legend = 1.0,
    label.cex = 0.8,
    mode='circos')

## -----------------------------------------------------------------------------
model.capri = tronco.capri(aCML.hypo, boot.seed = 12345, nboot = 5)
model.capri = annotate.description(model.capri, 'CAPRI - aCML')

## -----------------------------------------------------------------------------
model.caprese = tronco.caprese(aCML.clean)
model.caprese = annotate.description(model.caprese, 'CAPRESE - aCML')

## -----------------------------------------------------------------------------
model.edmonds = tronco.edmonds(aCML.clean, nboot = 5, boot.seed = 12345)
model.edmonds = annotate.description(model.edmonds, 'MST Edmonds - aCML')

## -----------------------------------------------------------------------------
model.gabow = tronco.gabow(aCML.clean, nboot = 5, boot.seed = 12345)
model.gabow = annotate.description(model.gabow, 'MST Gabow - aCML')

## -----------------------------------------------------------------------------
model.chowliu = tronco.chowliu(aCML.clean, nboot = 5, boot.seed = 12345)
model.chowliu = annotate.description(model.chowliu, 'MST Chow Liu - aCML')

## -----------------------------------------------------------------------------
model.prim = tronco.prim(aCML.clean, nboot = 5, boot.seed = 12345)
model.prim = annotate.description(model.prim, 'MST Prim - aCML data')

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

# reconstruct the best topology based on probabilistic causation and maximum likelihood estimation
# @title gabow.fit
# @param dataset a dataset describing a progressive phenomenon
# @param regularization regularizators to be used for the likelihood fit
# @param score the score to be used, could be either pointwise mutual information (pmi) or conditional entropy (entropy)
# @param do.boot should I perform bootstrap? Yes if TRUE, no otherwise
# @param nboot integer number (greater than 0) of bootstrap sampling to be performed
# @param pvalue pvalue for the tests (value between 0 and 1)
# @param min.boot minimum number of bootstrapping to be performed
# @param min.stat should I keep bootstrapping untill I have nboot valid values?
# @param boot.seed seed to be used for the sampling
# @param silent should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @param hypotheses hypotheses to be considered in the reconstruction. This should be NA for this algorithms. 
# @return topology: the reconstructed tree topology
#
gabow.fit <- function(dataset,
                      regularization = "no_reg",
                      score = "pmi",
                      do.boot = TRUE,
                      nboot = 100,
                      pvalue = 0.05,
                      min.boot = 3,
                      min.stat = TRUE,
                      boot.seed = NULL,
                      silent = FALSE,
                      epos = 0.0,
                      eneg = 0.0,
                      do.raising = TRUE,
                      hypotheses = NA) {

    ## Start the clock to measure the execution time.
    
    ptm = proc.time();

    ## Structure with the set of valid edges
    ## I start from the complete graph, i.e., I have no prior and all
    ## the connections are possibly causal.
    
    adj.matrix = array(1, c(ncol(dataset), ncol(dataset)));
    colnames(adj.matrix) = colnames(dataset);
    rownames(adj.matrix) = colnames(dataset);

    ## The diagonal of the adjacency matrix should not be considered,
    ## i.e., no self cause is allowed.
    
    diag(adj.matrix) = 0;

    ## Consider any hypothesis.
    
    adj.matrix = hypothesis.adj.matrix(hypotheses, adj.matrix);

    ## Check if the dataset is valid.
    
    valid.dataset = check.dataset(dataset, adj.matrix, FALSE, epos, eneg)
    adj.matrix = valid.dataset$adj.matrix;
    invalid.events = valid.dataset$invalid.events;

    ## Reconstruct the prima facie topology
    ## Should I perform bootstrap? Yes if TRUE, no otherwise.
    
    if (do.boot == TRUE) {
        if (!silent)
            cat('*** Bootstraping selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.do.boot(dataset,
                                            hypotheses,
                                            nboot,
                                            pvalue,
                                            adj.matrix,
                                            min.boot,
                                            min.stat,
                                            boot.seed,
                                            silent,
                                            epos,
                                            eneg);
    } else {
        if (!silent)
            cat('*** Computing selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.no.boot(dataset,
                                            hypotheses,
                                            adj.matrix,
                                            silent,
                                            epos,
                                            eneg);
    }

    ## Add back in any connection invalid for the probability raising
    ## theory.
    
    if (length(invalid.events) > 0) {
        # save the correct acyclic matrix
        adj.matrix.cyclic.tp.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp
        adj.matrix.cyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic
        adj.matrix.acyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.acyclic
        for (i in 1:nrow(invalid.events)) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.cyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.acyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
        }
        # if the new cyclic.tp contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp = adj.matrix.cyclic.tp.valid
        }
        # if the new cyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic = adj.matrix.cyclic.valid
        }
        # if the new acyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.acyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.acyclic = adj.matrix.acyclic.valid
        }
    }
    
    adj.matrix.prima.facie =
        prima.facie.parents$adj.matrix$adj.matrix.acyclic
        
    # set the input prior matrix for Gabow
    if(do.raising==TRUE) {
        adj.matrix.gabow = prima.facie.parents$adj.matrix$adj.matrix.cyclic
    }
    else {
        adj.matrix.gabow = prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp
    }

    ## Perform the likelihood fit with the required strategy
    model = list();
    for(my_score in score) {
        
        # as Gabow algorithm is computationally expensive, I first reconstruct with "no_reg" for each score
        best.parents =
                perform.likelihood.fit.gabow(dataset,
                                       adj.matrix.gabow,
                                       regularization = "no_reg",
                                       score = my_score,
                                       marginal.probs = prima.facie.parents$marginal.probs,
                                       joint.probs = prima.facie.parents$joint.probs)
        
        # if Gabow was not performed due to memory issues, use Edmonds instead
        fallback.edmonds = FALSE
        if(is.null(best.parents)) {
            fallback.edmonds = TRUE
            best.parents =
                perform.likelihood.fit.edmonds(dataset,
                                       adj.matrix.prima.facie,
                                       regularization = "no_reg",
                                       score = my_score,
                                       marginal.probs = prima.facie.parents$marginal.probs,
                                       joint.probs = prima.facie.parents$joint.probs)
        }
                                       
        # now I perform the likelihood fit with each regularizator
        for (reg in regularization) {

            ## Perform the likelihood fit with the chosen regularization
            ## score on the prima facie topology
            if (!silent)
                cat('*** Performing likelihood-fit with regularization:', reg, '.\n')
                
            
            # adjacency matrix of the topology reconstructed by likelihood fit
            curr_adj.matrix.fit = array(0,c(nrow(best.parents$adj.matrix$adj.matrix.fit),ncol(best.parents$adj.matrix$adj.matrix.fit)))
            rownames(curr_adj.matrix.fit) = colnames(dataset)
            colnames(curr_adj.matrix.fit) = colnames(dataset)
            
            curr_adj.matrix.fit = lregfit(as.categorical.dataset(dataset),
                                    best.parents$adj.matrix$adj.matrix.fit,
                                    curr_adj.matrix.fit,
                                    reg,
                                    command = "hc")
                                    
                                    
            ## Save the results and return them.
            adj.matrix_reg = list(adj.matrix.pf = adj.matrix.gabow,
                                  adj.matrix.fit = curr_adj.matrix.fit)
            curr_best.parents = list(adj.matrix = adj.matrix_reg)
            
            ## Set the structure to save the conditional probabilities of the reconstructed topology
            reconstructed.model = create.model(dataset,
                curr_best.parents,
                prima.facie.parents)
    
            model.name = paste('gabow', reg, my_score, sep='_')
            model[[model.name]] = reconstructed.model
            
        }
        
    }

    ## Set the execution parameters
    parameters =
        list(algorithm = "GABOW",
             regularization = regularization,
             score = score,
             do.boot = do.boot,
             nboot = nboot,
             pvalue = pvalue,
             min.boot = min.boot,
             min.stat = min.stat,
             boot.seed = boot.seed,
             silent = silent,
             error.rates = list(epos=epos,eneg=eneg),
             do.raising = do.raising,
             fallback.edmonds = fallback.edmonds)

    ## Return the results
    topology =
        list(dataset = dataset,
             hypotheses = hypotheses,
             adj.matrix.prima.facie = adj.matrix.prima.facie,
             confidence = prima.facie.parents$pf.confidence,
             model = model,
             parameters = parameters,
             execution.time = (proc.time() - ptm))
    topology = rename.reconstruction.fields(topology, dataset)
    return(topology)
}


# reconstruct the best causal topology by Gabow algorithm combined with probabilistic causation
# @title perform.likelihood.fit.gabow
# @param dataset a valid dataset
# @param adj.matrix the adjacency matrix of the prima facie causes
# @param regularization regularization term to be used in the likelihood fit
# @param command type of search, either hill climbing (hc) or tabu (tabu)
# @return topology: the adjacency matrix of both the prima facie and causal topologies
#
perform.likelihood.fit.gabow = function(dataset,
                                        adj.matrix,
                                        regularization,
                                        score,
                                        command = "hc",
                                        marginal.probs,
                                        joint.probs) {

    data = as.categorical.dataset(dataset)
    adj.matrix.prima.facie = adj.matrix
    
    # adjacency matrix of the topology reconstructed by likelihood fit
    adj.matrix.fit = array(0,c(nrow(adj.matrix),ncol(adj.matrix)))
    rownames(adj.matrix.fit) = colnames(dataset)
    colnames(adj.matrix.fit) = colnames(dataset)
    
    # detect the acyclic parts of the graph,
    # being the strongly connected components of degree 1
    my_graph = graph_from_adjacency_matrix(adj.matrix)
    strongly_connected = clusters(my_graph,mode="strong")

    # perform Gabow only if the maximum strongly connected component 
    # has size less than 9
    if(max(strongly_connected$csize)<9) {
        valid_nodes = c()
        for(i in 1:strongly_connected$no) {
            if(length(which(strongly_connected$membership==i))==1) {
                valid_nodes = c(valid_nodes,which(strongly_connected$membership==i))
            }
        }
           
        # set at most one parent per node based on mutual information
        # only for the acyclic parts of the graph
        if(length(valid_nodes)>0) {
            for (i in valid_nodes) {
                    
                # consider the parents of i
                curr_parents = which(adj.matrix[,i] == 1)
                
                # if I have more then one valid parent
                if (length(curr_parents) > 1) {
                    
                    # find the best parent
                    curr_best_parent = -1
                    curr_best_score = NA
                    for (j in curr_parents) {
                        
                        # if the event is valid
                        if(joint.probs[i,j]>=0) {
                            # compute the chosen score
                            new_score = compute.edmonds.score(joint.probs[i,j],marginal.probs[i],marginal.probs[j],score)
                        }
                        # else, if the two events are indistinguishable
                        else if(joint.probs[i,j]<0) {
                            new_score = Inf
                        }
                        
                        if (is.na(curr_best_score) || new_score > curr_best_score) {
                            curr_best_parent = j
                            curr_best_score = new_score
                        }
                    }
                    
                    # set the best parent
                    for (j in curr_parents) {
                        if (j != curr_best_parent) {
                            adj.matrix[j,i] = 0
                        }
                    }
                }
            }
        }
           
        # now consider the acyclic parts of the graph
        if(length(valid_nodes)<nrow(adj.matrix)) {
            # now I consider each strongly connected componet with size greater then 1
            for(i in 1:strongly_connected$no) {
                if(length(which(strongly_connected$membership==i))>1) {
                    adj.matrix = find.best.subtree(adj.matrix,which(strongly_connected$membership==i),marginal.probs,joint.probs,score)
                }
            }
        }
        
        # perform the likelihood fit if requested
        adj.matrix.fit = lregfit(data,
            adj.matrix,
            adj.matrix.fit,
            regularization,
            command)
        
        ## Save the results and return them.
        
        adj.matrix =
            list(adj.matrix.pf = adj.matrix.prima.facie,
                 adj.matrix.fit = adj.matrix.fit)
        topology = list(adj.matrix = adj.matrix)
    }
    else {
        warning("Too big strongly connected components: Gabow can not be performed, using Edmonds instead.")
        topology = NULL
    }
        
    return(topology)

}

find.best.subtree = function( adj.matrix, subtree.nodes, marginal.probs, joint.probs, score ) {
    
    # enumerate the possible orderings within subtree.nodes
    possible.orderings = permutations(length(subtree.nodes),length(subtree.nodes))
    
    # consider all the possible trees within subtree.nodes
    all_tree_scores = sapply(1:nrow(possible.orderings),FUN=function(x) {
        curr.ordering = subtree.nodes[possible.orderings[x,]]
        curr.adj.matrix = adj.matrix
        # remove any loop accordingly to this ordering
        for(j in 2:length(curr.ordering)) {
            curr_invalid = curr.ordering[1:(j-1)]
            curr.adj.matrix[curr.ordering[j],curr_invalid] = 0
        }
        # compute the score of curr.adj.matrix
        curr_score = get.best.scored.tree(curr.adj.matrix,subtree.nodes,marginal.probs,joint.probs,score)$score
        return(curr_score)
    })
    
    # pick the best scored tree
    best.ordering = subtree.nodes[possible.orderings[which(all_tree_scores==max(all_tree_scores))[1],]]
    
    # remove any loop accordingly to this ordering
    best_adj.matrix = adj.matrix
    for(j in 2:length(best.ordering)) {
        curr_invalid = best.ordering[1:(j-1)]
        best_adj.matrix[best.ordering[j],curr_invalid] = 0
    }
    
    # compute the score of curr.adj.matrix
    best_adj.matrix = get.best.scored.tree(best_adj.matrix,subtree.nodes,marginal.probs,joint.probs,score)$adj.matrix
    
    return(best_adj.matrix)
}

get.best.scored.tree = function( adj.matrix, subtree.nodes, marginal.probs, joint.probs, score_type ) {
    
    score = 0
    total.arcs = 0
    attached.arcs = 0
    
    # set at most one parent per node based on mutual information
    for (i in subtree.nodes) {
        
        # consider the parents of i
        curr_parents = which(adj.matrix[,i] == 1)
        
        # if I have more then one valid parent
        if (length(curr_parents) > 1) {
            
            # find the best parent
            curr_best_parent = -1
            curr_best_score = NA
            for (j in curr_parents) {
                
                # if the event is valid
                if(joint.probs[i,j]>=0) {
                    # compute the chosen score
                    new_score = compute.edmonds.score(joint.probs[i,j],marginal.probs[i],marginal.probs[j],score_type)
                }
                # else, if the two events are indistinguishable
                else if(joint.probs[i,j]<0) {
                    new_score = Inf
                }
                
                if (is.na(curr_best_score) || new_score > curr_best_score) {
                    curr_best_parent = j
                    curr_best_score = new_score
                }
            }
            
            if(curr_best_score!=Inf) {
                score = score + curr_best_score
                total.arcs = total.arcs + 1
            }
            
            # set the best parent
            invalid.curr.parents = which(curr_parents!=curr_best_parent)
            if(length(invalid.curr.parents)>0) {
                adj.matrix[curr_parents[invalid.curr.parents],i] = 0
            }
            
            attached.arcs = attached.arcs + 1
            
        }
    }
    
    # if I have an empty topology, it has the worst score
    if(attached.arcs==0) {
        score = -Inf
    }
    # if I have only arcs between indistinguishable events
    else if(attached.arcs>0 && total.arcs==0) {
        score = Inf
    }
    # if I have a valid topology which I want to weight
    # for the number of arcs
    else {
        mean_score = score / total.arcs
        score = score + (mean_score * (attached.arcs-total.arcs))
    }
    
    res = list(adj.matrix=adj.matrix,score=score)
    
    return(res)
    
}


#### end of file -- gabow.algorithm.R

## ----include=FALSE------------------------------------------------------------
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)
gene.hypotheses = c('KRAS', 'NRAS', 'IDH1', 'IDH2', 'TET2', 'SF3B1', 'ASXL1')
alterations = events.selection(as.alterations(aCML), filter.freq = .05)
aCML.clean = events.selection(aCML,
    filter.in.names=c(as.genes(alterations), gene.hypotheses))
aCML.clean = annotate.description(aCML.clean, 
    'CAPRI - Bionformatics aCML data (selected events)')
aCML.hypo = hypothesis.add(aCML.clean, 'NRAS xor KRAS', XOR('NRAS', 'KRAS'))
aCML.hypo = hypothesis.add(aCML.hypo, 'SF3B1 xor ASXL1', XOR('SF3B1', XOR('ASXL1')),
    '*')
as.events(aCML.hypo, genes = 'TET2') 
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 xor IDH2',
    XOR('TET2', 'IDH2'),
    '*')
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 or IDH2',
    OR('TET2', 'IDH2'),
    '*')
aCML.hypo = hypothesis.add.homologous(aCML.hypo)
aCML.hypo = annotate.description(aCML.hypo, '')
aCML.clean = annotate.description(aCML.clean, '')
model.capri = tronco.capri(aCML.hypo, boot.seed = 12345, nboot = 5)
model.capri = annotate.description(model.capri, 'CAPRI - aCML')
model.caprese = tronco.caprese(aCML.clean)
model.caprese = annotate.description(model.caprese, 'CAPRESE - aCML')
model.edmonds = tronco.edmonds(aCML.clean, nboot = 5, boot.seed = 12345)
model.edmonds = annotate.description(model.edmonds, 'MST Edmonds - aCML')
model.gabow = tronco.gabow(aCML.clean, nboot = 5, boot.seed = 12345)
model.gabow = annotate.description(model.gabow, 'MST Gabow - aCML')
model.chowliu = tronco.chowliu(aCML.clean, nboot = 5, boot.seed = 12345)
model.chowliu = annotate.description(model.chowliu, 'MST Chow Liu - aCML')
model.prim = tronco.prim(aCML.clean, nboot = 5, boot.seed = 12345)
model.prim = annotate.description(model.prim, 'MST Prim - aCML data')

## -----------------------------------------------------------------------------
view(model.capri)

## ----fig.width=4,fig.height=4,warning=FALSE-----------------------------------
tronco.plot(model.capri, 
    fontsize = 12, 
    scale.nodes = 0.6, 
    confidence = c('tp', 'pr', 'hg'), 
    height.logic = 0.25, 
    legend.cex = 0.35, 
    pathways = list(priors = gene.hypotheses), 
    label.edge.size = 10)

## ----fig.width=7,fig.height=7,warning=FALSE, fig.cap="aCML data processed model by algorithms to extract models from individual patients, we show the otput of  CAPRESE,  and all algorithms based on Minimum Spanning Trees (Edmonds, Chow Liu and Prim). Only the  model retrieved by Chow Liu has two different edge colors as it was regularized with two different strategies: AIC and BIC."----
par(mfrow = c(2,2))
tronco.plot(model.caprese, fontsize = 22, scale.nodes = 0.6, legend = FALSE)
tronco.plot(model.edmonds, fontsize = 22, scale.nodes = 0.6, legend = FALSE)
tronco.plot(model.chowliu, fontsize = 22, scale.nodes = 0.6, legend.cex = .7)
tronco.plot(model.prim, fontsize = 22, scale.nodes = 0.6, legend = FALSE)

## -----------------------------------------------------------------------------
as.data.frame(as.parameters(model.capri))
has.model(model.capri)
dataset = delete.model(model.capri)

## -----------------------------------------------------------------------------
str(as.adj.matrix(model.capri))

## -----------------------------------------------------------------------------
marginal.prob = as.marginal.probs(model.capri)
head(marginal.prob$capri_bic)

## -----------------------------------------------------------------------------
joint.prob = as.joint.probs(model.capri, models='capri_bic')
joint.prob$capri_bic[1:3, 1:3]

## -----------------------------------------------------------------------------
conditional.prob = as.conditional.probs(model.capri, models='capri_bic')
head(conditional.prob$capri_bic)

## -----------------------------------------------------------------------------
str(as.confidence(model.capri, conf = c('tp', 'pr', 'hg')))

## ----selective-advantage------------------------------------------------------
as.selective.advantage.relations(model.capri)

## -----------------------------------------------------------------------------
model.boot = tronco.bootstrap(model.capri, nboot = 3, cores.ratio = 0)
model.boot = tronco.bootstrap(model.boot, nboot = 3, cores.ratio = 0, type = 'statistical')

## ----fig.width=4, fig.height=4, warning=FALSE, fig.cap="aCML model reconstructed by CAPRI with AIC / BIC as regolarizators and annotated with both non-parametric and statistical bootstrap scores. Edge thickness is proportional to the non-parametric scores."----

tronco.plot(model.boot, 
    fontsize = 12, 
    scale.nodes = .6,   
    confidence=c('sb', 'npb'), 
    height.logic = 0.25, 
    legend.cex = .35, 
    pathways = list(priors= gene.hypotheses), 
    label.edge.size=10)

## -----------------------------------------------------------------------------
as.bootstrap.scores(model.boot)
view(model.boot)

## ----fig.width=7, fig.height=7, fig.cap="Heatmap of the bootstrap scores for the CAPRI aCML model (via AIC regularization)."----

pheatmap(keysToNames(model.boot, as.confidence(model.boot, conf = 'sb')$sb$capri_aic) * 100, 
           main =  'Statistical bootstrap scores for AIC model',
           fontsize_row = 6,
           fontsize_col = 6,
           display_numbers = TRUE,
           number_format = "%d"
           )

## -----------------------------------------------------------------------------
model.boot = tronco.kfold.eloss(model.boot)
model.boot = tronco.kfold.prederr(model.boot, runs = 2, cores.ratio = 0)
model.boot = tronco.kfold.posterr(model.boot, runs = 2, cores.ratio = 0)

## -----------------------------------------------------------------------------
as.kfold.eloss(model.boot)
as.kfold.prederr(model.boot)
as.kfold.posterr(model.boot)

## -----------------------------------------------------------------------------
tabular = function(obj, M){
    tab = Reduce(
        function(...) merge(..., all = TRUE), 
            list(as.selective.advantage.relations(obj, models = M),
                as.bootstrap.scores(obj, models = M),
                as.kfold.prederr(obj, models = M),
                as.kfold.posterr(obj,models = M)))
  
    # merge reverses first with second column
    tab = tab[, c(2,1,3:ncol(tab))]
    tab = tab[order(tab[, paste(M, '.NONPAR.BOOT', sep='')], na.last = TRUE, decreasing = TRUE), ]
    return(tab)
}

head(tabular(model.boot, 'capri_bic'))

## ----fig.width=4,fig.height=4, warning=FALSE, fig.cap="aCML model reconstructed by CAPRI with  AIC/BIC as regolarizators and annotated with  non-parametric, as well as with entropy loss, prediction and posterior classification errors computed via cross-validation. Edge thickness is proportional to the non-parametric  scores."----
tronco.plot(model.boot, 
    fontsize = 12, 
    scale.nodes = .6, 
    confidence=c('npb', 'eloss', 'prederr', 'posterr'), 
    height.logic = 0.25, 
    legend.cex = .35, 
    pathways = list(priors= gene.hypotheses), 
    label.edge.size=10)

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


# reconstruct the best topology based on probabilistic causation and Chow Liu algorithm
# @title chow.liu.fit
# @param dataset a dataset describing a progressive phenomenon
# @param regularization regularizators to be used for the likelihood fit
# @param do.boot should I perform bootstrap? Yes if TRUE, no otherwise
# @param nboot integer number (greater than 0) of bootstrap sampling to be performed
# @param pvalue pvalue for the tests (value between 0 and 1)
# @param min.boot minimum number of bootstrapping to be performed
# @param min.stat should I keep bootstrapping untill I have nboot valid values?
# @param boot.seed seed to be used for the sampling
# @param silent should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @param hypotheses hypotheses to be considered in the reconstruction. This should be NA for this algorithms. 
# @return topology: the reconstructed tree topology
#
chow.liu.fit <- function(dataset,
                         regularization = c("bic","aic"),
                         do.boot = TRUE,
                         nboot = 100,
                         pvalue = 0.05,
                         min.boot = 3,
                         min.stat = TRUE,
                         boot.seed = NULL,
                         silent = FALSE,
                         epos = 0.0,
                         eneg = 0.0,
                         hypotheses = NA) {

    ## Start the clock to measure the execution time.
    
    ptm = proc.time();

    ## Structure with the set of valid edges
    ## I start from the complete graph, i.e., I have no prior and all
    ## the connections are possibly causal.
    
    adj.matrix = array(1, c(ncol(dataset), ncol(dataset)));
    colnames(adj.matrix) = colnames(dataset);
    rownames(adj.matrix) = colnames(dataset);

    ## The diagonal of the adjacency matrix should not be considered,
    ## i.e., no self cause is allowed.
    
    diag(adj.matrix) = 0;

    ## Consider any hypothesis.
    
    adj.matrix = hypothesis.adj.matrix(hypotheses, adj.matrix);

    ## Check if the dataset is valid.
    
    valid.dataset = check.dataset(dataset, adj.matrix, FALSE, epos, eneg)
    adj.matrix = valid.dataset$adj.matrix;
    invalid.events = valid.dataset$invalid.events;

    ## Reconstruct the prima facie topology
    ## Should I perform bootstrap? Yes if TRUE, no otherwise.
    
    if (do.boot == TRUE) {
        if (!silent)
            cat('*** Bootstraping selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.do.boot(dataset,
                                            hypotheses,
                                            nboot,
                                            pvalue,
                                            adj.matrix,
                                            min.boot,
                                            min.stat,
                                            boot.seed,
                                            silent,
                                            epos,
                                            eneg);
    } else {
        if (!silent)
            cat('*** Computing selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.no.boot(dataset,
                                            hypotheses,
                                            adj.matrix,
                                            silent,
                                            epos,
                                            eneg);
    }

    ## Add back in any connection invalid for the probability raising
    ## theory.
    
    if (length(invalid.events) > 0) {
        # save the correct acyclic matrix
        adj.matrix.cyclic.tp.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp
        adj.matrix.cyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic
        adj.matrix.acyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.acyclic
        for (i in 1:nrow(invalid.events)) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.cyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.acyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
        }
        # if the new cyclic.tp contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp = adj.matrix.cyclic.tp.valid
        }
        # if the new cyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic = adj.matrix.cyclic.valid
        }
        # if the new acyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.acyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.acyclic = adj.matrix.acyclic.valid
        }
    }
    
    adj.matrix.prima.facie =
        prima.facie.parents$adj.matrix$adj.matrix.acyclic

    ## Perform the likelihood fit with the required strategy.
    
    model = list();
    for (reg in regularization) {

        ## Perform the likelihood fit with the chosen regularization
        ## score on the prima facie topology.
        
        if (!silent)
            cat('*** Performing likelihood-fit with regularization',reg,'.\n')
        best.parents =
            perform.likelihood.fit.chow.liu(dataset,
                                   adj.matrix.prima.facie,
                                   regularization = reg);

        ## Set the structure to save the conditional probabilities of
        ## the reconstructed topology.
        
        reconstructed.model = create.model(dataset,
            best.parents,
            prima.facie.parents)

        model.name = paste('chow_liu', reg, sep='_')
        model[[model.name]] = reconstructed.model
    }

    ## Set the execution parameters.
    
    parameters =
        list(algorithm = "CHOW_LIU",
             regularization = regularization,
             do.boot = do.boot,
             nboot = nboot,
             pvalue = pvalue,
             min.boot = min.boot,
             min.stat = min.stat,
             boot.seed = boot.seed,
             silent = silent,
             error.rates = list(epos=epos,eneg=eneg));

    ## Return the results.
    
    topology =
        list(dataset = dataset,
             hypotheses = hypotheses,
             adj.matrix.prima.facie = adj.matrix.prima.facie,
             confidence = prima.facie.parents$pf.confidence,
             model = model,
             parameters = parameters,
             execution.time = (proc.time() - ptm))
    topology = rename.reconstruction.fields(topology, dataset)
    return(topology)
}


# reconstruct the best causal topology by Chow Liu algorithm combined with probabilistic causation
# @title perform.likelihood.fit.chow.liu
# @param dataset a valid dataset
# @param adj.matrix the adjacency matrix of the prima facie causes
# @param regularization regularization term to be used in the likelihood fit
# @param command type of search, either hill climbing (hc) or tabu (tabu)
# @return topology: the adjacency matrix of both the prima facie and causal topologies
#
perform.likelihood.fit.chow.liu = function( dataset, 
                                            adj.matrix,
                                            regularization,
                                            command = "hc") {

    ## Each variable should at least have 2 values: I'm ignoring
    ## connection to invalid events but, still, need to make the
    ## dataset valid for bnlearn.
    
    for (i in 1:ncol(dataset)) {
        if (sum(dataset[, i]) == 0) {
            dataset[sample(1:nrow(dataset), size=1), i] = 1;
        } else if (sum(dataset[, i]) == nrow(dataset)) {
            dataset[sample(1:nrow(dataset), size=1), i] = 0;
        }
    }

    # adjacency matrix of the topology reconstructed by likelihood fit
    adj.matrix.fit = array(0,c(nrow(adj.matrix),ncol(adj.matrix)))
    rownames(adj.matrix.fit) = colnames(dataset)
    colnames(adj.matrix.fit) = colnames(dataset)

    ## Create the blacklist based on the prima facie topology
    ## and the tree-structure assumption
    cont = 0
    parent = -1
    child = -1

    fully_disconnected = NULL
    for (i in rownames(adj.matrix)) {
        if((sum(adj.matrix[,i])+sum(adj.matrix[i,]))==0) {
            fully_disconnected = c(fully_disconnected,i)
        }
        for (j in colnames(adj.matrix)) {
            if(i != j && adj.matrix[i, j] == 0 && adj.matrix[j, i] == 0) {
                cont = cont + 1
                if (cont == 1) {
                    parent = i
                    child = j
                } else {
                    parent = c(parent, i)
                    child = c(child, j)
                }
            }
        }
    }

    # compute the best Chow-Liu tree among the valid edges
    if(length(fully_disconnected)<ncol(dataset)) {

        valid_data_entries = colnames(dataset)
        if(length(fully_disconnected)>0) {
            valid_data_entries = colnames(dataset)[which(!colnames(dataset)%in%fully_disconnected)]
        }

        if (cont > 0) {

            blacklist = data.frame(from = parent,to = child)
            if(length(fully_disconnected)>0) {
                blacklist = blacklist[which(blacklist$from%in%valid_data_entries&blacklist$to%in%valid_data_entries),]
            }

            if(dim(blacklist)[1]>0) {
                best_chow_liu_tree = tryCatch({
                            chow.liu(x=as.categorical.dataset(dataset[,valid_data_entries]),blacklist=blacklist)
                        }, error = function(e) {
                            NA
                        })
            }
            else {
                best_chow_liu_tree = tryCatch({
                            chow.liu(x=as.categorical.dataset(dataset[,valid_data_entries]))
                        }, error = function(e) {
                            NA
                        })
            }

        } else {
            best_chow_liu_tree = tryCatch({
                            chow.liu(x=as.categorical.dataset(dataset[,valid_data_entries]))
                        }, error = function(e) {
                            NA
                        })
        }

    }

    if(!any(is.na(best_chow_liu_tree)) && length(fully_disconnected)<ncol(dataset)) {

        # get the best topology considering both the priors and the Chow-Liu tree
        my.arcs = best_chow_liu_tree$arcs
        
        ## build the adjacency matrix of the reconstructed topology
        if (length(nrow(my.arcs)) > 0 && nrow(my.arcs) > 0) {
            for (i in 1:nrow(my.arcs)) {
                # [i,j] refers to causation i --> j
                if(adj.matrix[my.arcs[i,1],my.arcs[i,2]]==1) {
                    adj.matrix.fit[my.arcs[i,1],my.arcs[i,2]] = 1
                }
            }
        }

    }

    if(!any(is.na(best_chow_liu_tree))) {

        ## Perform the likelihood fit if requested
        adj.matrix.fit = lregfit(as.categorical.dataset(dataset),
            adj.matrix,
            adj.matrix.fit,
            regularization,
            command)

    }
    else {
        warning("A tree spanning all the node could not be obtained.")
    }
    
    ## Save the results and return them.
    
    adj.matrix =
        list(adj.matrix.pf = adj.matrix,
             adj.matrix.fit = adj.matrix.fit);
    topology = list(adj.matrix = adj.matrix);
    return(topology)

}

#### end of file -- chow.liu.algorithm.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' Perform a k-fold cross-validation using the function bn.cv
#' to estimate the entropy loss. For details and examples 
#' regarding the statistical assesment of an inferred model, 
#' we refer to the Vignette Section 7. 
#'
#' @title tronco.kfold.eloss
#'
#' @examples
#' data(test_model)
#' tronco.kfold.eloss(test_model, k = 2, runs = 2)
#'
#' @param x A reconstructed model (the output of tronco.capri or tronco.caprese)
#' @param models The names of the selected regularizers (bic, aic or caprese)
#' @param runs a positive integer number, the number of times cross-validation will be run
#' @param k a positive integer number, the number of groups into which the data will be split
#' @param silent A parameter to disable/enable verbose messages.
#' @importFrom bnlearn bn.cv empty.graph set.arc
#' @importFrom stats sd
#' @export tronco.kfold.eloss
#'
tronco.kfold.eloss = function(x, 
                              models = names(as.models(x)),
                              runs = 10,
                              k = 10, 
                              silent = FALSE) {   

    ## Check if there is a reconstructed model.

    if (!has.model(x)) {
        stop('The input TRONCO object does not contain a model, you should first do that -- won\'t perform cross-validation!')
    }

    ## Check if the selected regularization is used in the model
    if (!"kfold" %in% names(x)) {
        x$kfold = NULL
    }

    for (model in models) {
        if (!model %in% names(as.models(x))) {
            stop(paste(model, " was not used to infer the input TRONCO object -- won\'t perform cross-validation!"))
        }

        ## Get bnlearn network.
        bn = as.bnlearn.network(x, model)
        bndata = bn$data
        bnnet = bn$net

        ## Calculating the eloss with bn.cv
        if (!silent) {
            cat('Calculating entropy loss with k-fold cross-validation \n\t[ k =', k,
                '| runs =', runs, '| regularizer =', model, '] ... ')
        }

        ## Scutari fix
        
        bn.kcv.list = bnlearn::bn.cv(bndata, 
                            bnnet,
                            loss = 'logl',
                            runs = runs,
                            k = k,
                            fit = "bayes",
                            fit.args = list(iss = 1))

        losses = NULL

        for(i in 1:length(bn.kcv.list)) {
            losses = c(losses, attr(bn.kcv.list[[i]], "mean"))
        }
        if (any(is.na(losses))) {
            warning("Some folds returned NA")
        }
        eloss = losses[!is.na(losses)]
    
        meanll = mean(eloss)
        ll = get(model, x$model)$logLik
        ratio = meanll / abs(ll) * 100
    
        if (!silent) {
            cat(' DONE\n')
            cat('  Model logLik =', ll, '\n')
            cat('  Mean   eloss =', meanll,' | ', ratio,'% \n')
            cat('  Stdev  eloss = ', sd(eloss) ,'\n')
        }
    
        x$kfold[[model]]$bn.kcv.list = bn.kcv.list
        x$kfold[[model]]$eloss = eloss
    }
    return(x)
}


#' Perform a k-fold cross-validation using the function bn.cv
#' and scan every node to estimate its prediction error. For details and examples 
#' regarding the statistical assesment of an inferred model, 
#' we refer to the Vignette Section 7. 
#' 
#' @title tronco.kfold.prederr
#'
#' @examples
#' data(test_model)
#' tronco.kfold.prederr(test_model, k = 2, runs = 2, cores.ratio = 0)
#'
#' @param x A reconstructed model (the output of tronco.capri)
#' @param models The names of the selected regularizers (bic, aic or caprese)
#' @param events a list of event 
#' @param runs a positive integer number, the number of times cross-validation will be run
#' @param k a positive integer number, the number of groups into which the data will be split
#' @param cores.ratio Percentage of cores to use. coresRate * (numCores - 1)
#' @param silent A parameter to disable/enable verbose messages.
#' @importFrom bnlearn bn.cv empty.graph set.arc
#' @importFrom doParallel registerDoParallel  
#' @importFrom foreach foreach %dopar%
#' @importFrom iterators icount
#' @importFrom parallel stopCluster makeCluster detectCores
#' @importFrom stats sd
#' @export tronco.kfold.prederr
#'
tronco.kfold.prederr <- function(x,
                                 models = names(as.models(x)),
                                 events = as.events(x),
                                 runs = 10,
                                 k = 10,
                                 cores.ratio = 1,
                                 silent = FALSE) {

    ## Check if there is a reconstructed model
    if(!has.model(x)) {
        stop('This object does not have a model.')
    }

    ## Integrity check over nodes.
    as.adj.matrix(x,
                  events = events,
                  models = models,
                  type = 'fit')
  
    events = apply(events, 1, function(z){paste(z, collapse = ' ')})

    if (!"kfold" %in% names(x)) {
        x$kfold = NULL
    }

    cores = as.integer(cores.ratio * (detectCores() - 1))
    if (cores < 1) {
        cores = 1
    }

    cl = makeCluster(cores)
    registerDoParallel(cl)

    if (!silent) {
        cat('*** Using', cores, 'cores via "parallel" \n')
    }


    for (model in models) {
        
        ## Check if the selected regularization is used in the model.
        
        if (!model %in% names(as.models(x))) {
            stop(paste(model, " was not used to infer the input TRONCO object -- won\'t perform cross-validation!"))
        }

        ## Get bnlearn network and the adj matrix.

        bn = as.bnlearn.network(x, model)
        bndata = bn$data
        bnnet = bn$net

        adj.matrix = get(model, as.adj.matrix(x))
        adj.matrix = keysToNames(x, adj.matrix)
        names(colnames(adj.matrix)) = NULL
        names(rownames(adj.matrix)) = NULL
  
        pred = list()

        ## Perform the estimation of the prediction error. 
        if (!silent) {
            cat('\tScanning',
                length(events),
                'nodes for their prediction error (all parents). \n\tRegularizer: ',
                model,
                '\n')
        }
        

        r = foreach(i = 1:length(events), .inorder = TRUE) %dopar% {

            ## Scutari fix

            event = events[i]

            comp = bnlearn::bn.cv(bndata,
                         bnnet, 
                         loss = 'pred', 
                         loss.args = list(target = event),
                         runs = runs,
                         k = k,
                         fit = "bayes",
                         fit.args = list(iss = 1))
            
            res = NULL
            for(i in 1:runs) {
                res = c(res, attributes(comp[[i]])$mean)
            }            
            res  
        }
        if (!silent) {
            cat("*** Reducing results\n")
        }
        names(r) = events      
        x$kfold[[model]]$prederr = r
    }

    stopCluster(cl)

    return(x)
}

#' Perform a k-fold cross-validation using the function bn.cv
#' and scan every node to estimate its posterior classification error. 
#' @title tronco.kfold.posterr. For details and examples 
#' regarding the statistical assesment of an inferred model, 
#' we refer to the Vignette Section 7. 
#'
#' @examples
#' data(test_model)
#' tronco.kfold.posterr(test_model, k = 2, runs = 2, cores.ratio = 0)
#'
#' @param x A reconstructed model (the output of tronco.capri)
#' @param models The names of the selected regularizers (bic, aic or caprese)
#' @param events a list of event 
#' @param runs a positive integer number, the number of times cross-validation will be run
#' @param k a positive integer number, the number of groups into which the data will be split
#' @param cores.ratio Percentage of cores to use. coresRate * (numCores - 1)
#' @param silent A parameter to disable/enable verbose messages.
#' @importFrom bnlearn bn.cv empty.graph set.arc
#' @importFrom stats sd
#' @export tronco.kfold.posterr
#'
tronco.kfold.posterr <- function(x,
                                 models = names(as.models(x)),
                                 events = as.events(x),
                                 runs = 10,
                                 k = 10,
                                 cores.ratio = 1,
                                 silent = FALSE) {

    ## Check if there is a reconstructed model.

    if(!has.model(x)) {
        stop('This object does not have a model.')
    }

    ## Integrity check over nodes.
    as.adj.matrix(x,
                  events = events,
                  models = models,
                  type = 'fit')
  
    events = apply(events, 1, function(z){paste(z, collapse = ' ')})

    if (!"kfold" %in% names(x)) {
        x$kfold = NULL
    }


    cores = as.integer(cores.ratio * (detectCores() - 1))
    if (cores < 1) {
        cores = 1
    }

    cl = makeCluster(cores)

    registerDoParallel(cl)
    if (!silent) {
        cat('*** Using', cores, 'cores via "parallel" \n')
    }

    for (model in models) {
        
        ## Check if the selected regularization is used in the model.
        
        if (!model %in% names(as.models(x))) {
            stop(paste(model, " was not used to infer the input TRONCO object -- won\'t perform cross-validation!"))
        }

        ## Get bnlearn network and the adj matrix.

        bn = as.bnlearn.network(x, model)
        bndata = bn$data
        bnnet = bn$net

        adj.matrix = get(model, as.adj.matrix(x))
        adj.matrix = keysToNames(x, adj.matrix)
        names(colnames(adj.matrix)) = NULL
        names(rownames(adj.matrix)) = NULL

        ## Perform the estimation of the prediction error. 
        
        if (!silent) {
            cat('\tScanning',
                sum(adj.matrix == 1),
                'edges for posterior classification error. \n\tRegularizer:',
                model,
                '\n')
        }    

        r = foreach(i = 1:length(events), .inorder = TRUE, .combine = cbind) %dopar% {
            event = events[i]
            posterr.adj.col = array(list(NA), c(nrow(adj.matrix), 1))
            colnames(posterr.adj.col) = event
            rownames(posterr.adj.col) = rownames(adj.matrix)
                        
            for (pre in rownames(posterr.adj.col)) {
                if (adj.matrix[pre,event] == 1) {

                    comp = bnlearn::bn.cv(bndata,
                                 bnnet, 
                                 loss = 'pred', 
                                 loss.args = list(target = event,
                                                  from = pre,
                                                  predict = "bayes-lw"),
                                 runs = runs,
                                 k = k,
                                 fit = "bayes",
                                 fit.args = list(iss = 1))
                    
                    res = NULL
                    for(i in 1:runs) {
                        res = c(res, attributes(comp[[i]])$mean)
                    }
                    posterr.adj.col[[pre,event]] = res  
                }
            }
            posterr.adj.col
        }
        if (!silent) {
            cat("*** Reducing results\n")
        }
        x$kfold[[model]]$posterr = r
    }

    stopCluster(cl)
    return(x)
}

#### end of file -- statistics.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


# reconstruct the best tree-like topology
# @title caprese.fit
# @param dataset a dataset describing a progressive phenomenon
# @param lambda shrinkage parameter (value in [0,1])
# @param silent execute the algorithm in silent mode
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @param hypotheses hypotheses to be considered in the reconstruction. This should be NA for this algorithms. 
# @return topology: the reconstructed tree-like topology
#
caprese.fit <- function(dataset,
                        lambda = 0.5,
                        silent = FALSE,
                        epos = 0.0,
                        eneg = 0.0,
                        hypotheses = NA) {
    
    ## Start the clock to measure the execution time.
    
    ptm <- proc.time();
    
    ## Structure with the set of valid edges.
    ## I start from the complete graph, i.e., I have no prior and all
    ## the connections are possibly causal.
    
    adj.matrix = array(1, c(ncol(dataset), ncol(dataset)));
    colnames(adj.matrix) = colnames(dataset);
    rownames(adj.matrix) = colnames(dataset);
    
    ## The diagonal of the adjacency matrix should not be considered,
    ## i.e., no self cause is allowed.
    
    diag(adj.matrix) = 0;

    ## Consider any hypothesis.
    
    adj.matrix = hypothesis.adj.matrix(hypotheses, adj.matrix);
    
    ## Check if the dataset is valid.
    
    valid.dataset = check.dataset(dataset,adj.matrix,FALSE, epos, eneg)
    adj.matrix = valid.dataset$adj.matrix
    
    ## marginal.probs is an array of the observed marginal
    ## probabilities.
    
    marginal.probs <- valid.dataset$marginal.probs;
    
    ## joint.probs is an array of the observed joint probabilities.
    
    joint.probs <- valid.dataset$joint.probs;
    
    ## Reconstruct the causal topology.
    
    best.parents =
        get.tree.parents(adj.matrix,
                         marginal.probs,
                         joint.probs,
                         lambda)
    
    ## Create the structures where to save the results.
    
    parents.pos <-
        best.parents$parents;
    conditional.probs <-
        array(-1, dim = c(length(parents.pos), 1));
    adj.matrix <-
        array(0, dim = c(length(parents.pos), length(parents.pos)));
    colnames(adj.matrix) = colnames(dataset);
    rownames(adj.matrix) = colnames(dataset);
    
    confidence <- array(list(), c(3, 1));
    confidence[[1, 1]] =
        array(0, dim = c(length(parents.pos), length(parents.pos)));
    confidence[[2, 1]] =
        best.parents$pr.score;
    confidence[[3, 1]] =
        array(0, dim = c(length(parents.pos), length(parents.pos)));
    
    ## Set the parents names and the structures.
    
    hypergeometric.pvalues = vector();
    
    for (i in 1:ncol(dataset)) {
        
        ## If the node has a parent.
        
        if (parents.pos[i, 1] != -1) {

            ## Note: [i,j] = 1 means that i is causing j.

            adj.matrix[parents.pos[i, 1], i] = 1;
            
            ## Compute the conditional probability of
            ## P(CHILD=1|PARENT=1)
            
            conditional.probs[i, 1] =
                best.parents$joint.probs[parents.pos[i, 1], i] /
                    best.parents$marginal.probs[parents.pos[i]];
        } else {
            
            ## If the node has no parent, its conditional probability
            ## is set to 1.
            
            conditional.probs[i, 1] = 1;
        }

        ## Compute the hypergeometric test.
        
        for (j in i:ncol(dataset)) {
            if (i != j) {
                
                ## Confidence in temporal priority.
                
                confidence[[1, 1]][i, j] =
                    min(best.parents$marginal.probs[i] /
                            best.parents$marginal.probs[j],
                        1);
                confidence[[1, 1]][j, i] =
                    min(best.parents$marginal.probs[j] /
                            best.parents$marginal.probs[i],
                        1);
                
                ## Compute the confidence by hypergeometric test
                
                confidence[[3, 1]][i, j] =
                    phyper(best.parents$joint.probs[i, j] * nrow(dataset),
                           best.parents$marginal.probs[i] * nrow(dataset),
                           nrow(dataset) -
                               best.parents$marginal.probs[i] *
                                   nrow(dataset),
                           best.parents$marginal.probs[j] *
                               nrow(dataset),
                           lower.tail = FALSE);
                
                confidence[[3, 1]][j, i] = confidence[[3, 1]][i, j];
                
                ## Save all the valid pvalues.
                
                hypergeometric.pvalues =
                    append(hypergeometric.pvalues,
                           confidence[[2, 1]][i, j]);
            }
        }
    }
    
    estimated.error.rates = list(error.fp = NA, error.fn = NA)
    estimated.probabilities = list(marginal.probs = NA, 
                                   joint.probs = NA,
                                   conditional.probs = NA)
    error.rates = estimated.error.rates
    estimated.probabilities.fit = estimated.probabilities
    
    ## Structures where to save the results.
    
    model = list();
    adj.matrix.fit = list();
    adj.matrix.fit$adj.matrix.fit = adj.matrix;
    
    probabilities.observed =
        list(marginal.probs = marginal.probs,
             joint.probs = joint.probs,
             conditional.probs = conditional.probs);
    
    probabilities.fit =
        list(estimated.marginal.probs = estimated.probabilities.fit$marginal.probs,
             estimated.joint.probs = estimated.probabilities.fit$joint.probs,
             estimated.conditional.probs = estimated.probabilities.fit$conditional.probs);

    probabilities =
        list(probabilities.observed = probabilities.observed,
             probabilities.fit = probabilities.fit);
    
    ## Save the results for the model
    
    model[["caprese"]] =
        list(probabilities = probabilities,
             parents.pos = parents.pos,
             error.rates = error.rates,
             adj.matrix = adj.matrix.fit);
    
    ## Set the execution parameters.
    
    parameters =
        list(algorithm = "CAPRESE",
             lambda = lambda,
             silent = silent,
             error.rates = list(epos=epos,eneg=eneg))
    
    ## Return the results.
    
    topology =
        list(dataset = dataset,
             hypotheses = hypotheses,
             confidence = confidence,
             model = model,
             parameters = parameters,
             execution.time = (proc.time() - ptm))

    topology = rename.reconstruction.fields(topology, dataset)
    return(topology)
}


# select at the most one parent for each node based on the probability raising criteria
# @title get.tree.parents
# @param adj.matrix adjacency matrix of the valid edges
# @param marginal.probs observed marginal probabilities
# @param joint.probs observed joint probabilities
# @param lambda shrinkage parameter (value between 0 and 1)
# @return best.parents list of the best parents
#
get.tree.parents <- function(adj.matrix,
                             marginal.probs,
                             joint.probs,
                             lambda) {
    
    ## Compute the scores for each edge.
    
    scores =
        get.tree.scores(adj.matrix,
                        marginal.probs,
                        joint.probs,
                        lambda); 
    pr.score = scores$pr.score;
    
    ## Set to -1 the scores where there is no causation according to
    ## Suppes' condition [i,j] means i is causing j.
    
    for (i in 1:ncol(pr.score)) {
        for (j in i:ncol(pr.score)) {
            
            ## The diagonal has not to be considered (no self-cause)
            
            if (i == j) {
                pr.score[i, j] = -1;
            } else {
                ## Otherwise, apply Suppes's criteria for prima facie
                ## cause.
                
                ## If both the scores are not greater then 0, they are
                ## not valid in this case the events are causally
                ## irrelevant, i.e., independent.
                
                if (pr.score[i, j] <= 0 && pr.score[j, i] <= 0) {
                    pr.score[i, j] = -1;
                    pr.score[j, i] = -1;
                }
                ## If at least one score is greater then 0, I keep the
                ## greater one; in this way I give a (time) direction
                ## to the progression.  Furthermore, this constrain
                ## the topology to be acyclic by construction.
                
                else {
                    if (pr.score[i, j] > pr.score[j, i]) {
                        pr.score[j, i] = -1;
                    } else {
                        pr.score[i, j] = -1;
                    }
                }
            }
        }
    }
    
    ## Chose at the most one parent per node.
    ## Here I suppose that each node has a parent spurious causes are
    ## considered (and removed) later.
    
    best.parents = array(-1, dim = c(ncol(pr.score), 1));
    for (i in 1:ncol(pr.score)) {
        
        ## -1 means that the best parent is the Root
        
        curr.best = -1;
        
        ## Find the best parent for the current node.
        
        best = which.max(pr.score[,i]);
        if (pr.score[best,i]>0) {
            curr.best = best;
        }
        ## Set the best parent for the current node.
        
        best.parents[i,1] = curr.best;
    }
    
    # remove any loop in the reconstruction
    caprese.adj.matrix = array(0,c(nrow(best.parents),nrow(best.parents)))
    list.of.arcs = list()
    list.of.scores = NULL
    for(nodes in 1:nrow(best.parents)) {
        if(best.parents[nodes,1]!=-1) {
            caprese.adj.matrix[best.parents[nodes,1],nodes] = 1
            list.of.arcs[[(length(list.of.arcs)+1)]] = list(parent=best.parents[nodes,1],child=nodes)
            list.of.scores = c(list.of.scores,pr.score[best.parents[nodes,1],nodes])
        }
    }
    best.parents = remove.caprese.loops(best.parents,caprese.adj.matrix,list.of.arcs,list.of.scores)
    
    ## Check for spurious causes by the independent progression filter
    ## and complete the parents list.
    
    parents =
        verify.parents(best.parents,
                       marginal.probs,
                       joint.probs);
    
    ## Save the results
    best.parents =
        list(parents = parents,
             marginal.probs = marginal.probs,
             joint.probs = joint.probs,
             pr.score = scores$pr.score);
    return(best.parents);
}


remove.caprese.loops <- function( best.parents, adj.matrix, edges, scores ) {
    
    # if I have at least one edge
    if(length(edges)>0) {
        
        ordered.scores = sort(scores,decreasing=FALSE,index.return=TRUE)
        ordered.edges = edges[ordered.scores$ix]
        
        # go through the edges in decreasing order of confidence
        for (i in 1:length(edges)) {
            
            # consider any edge i --> j
            curr.edge = ordered.edges[[i]]
            curr.edge.i = as.numeric(curr.edge$parent)
            curr.edge.j = as.numeric(curr.edge$child)
            
            # search for loops between curr.edge.i and curr.edge.j
            curr.graph = graph.adjacency(adj.matrix,mode="directed")
            is.path = suppressWarnings(get.shortest.paths(curr.graph,
                                       curr.edge.j,
                                       curr.edge.i)$vpath)
            is.path = length(unlist(is.path))

            # if there is a path between the two nodes, remove edge i --> j
            if (is.path > 0) {
                adj.matrix[curr.edge.i,curr.edge.j] = 0
                best.parents[curr.edge.j,1] = -1
            }
            
        }
        
    }
    
    return(best.parents)
    
}


# compute the probability raising based scores
# @title get.tree.scores
# @param adj.matrix adjacency matrix of the valid edges
# @param marginal.probs observed marginal probabilities
# @param joint.probs observed joint probabilities
# @param lambda shrinkage parameter (value between 0 and 1)
# @return scores: probability raising based scores
#
get.tree.scores <- function(adj.matrix,
                            marginal.probs,
                            joint.probs,
                            lambda) {
    
    ## Structure where to save the probability raising scores.
    
    pr.score =
        array(-1, dim = c(nrow(marginal.probs), nrow(marginal.probs)))
    
    ## Compute the probability raising based scores.
    
    for (i in 1:ncol(pr.score)) {
        for (j in 1:ncol(pr.score)) {
            
            ## If the edge is valid.
            
            if (adj.matrix[i, j] == 1) {
                
                ## alpha is the probability raising model of causation
                ## (raw model estimate).
                
                alpha =
                    ((joint.probs[i, j] / marginal.probs[i]) -
                         ((marginal.probs[j] - joint.probs[i, j]) /
                              (1 - marginal.probs[i]))) /
                                  ((joint.probs[i,j] /
                                        marginal.probs[i]) +
                                            ((marginal.probs[j] - joint.probs[i,j]) /
                                                 (1 - marginal.probs[i])));
                
                ## beta is the correction factor (based on time
                ## distance in terms of statistical dependence).
                
                beta =
                    (joint.probs[i, j] -
                         marginal.probs[i] * marginal.probs[j]) /
                        (joint.probs[i, j] + marginal.probs[i] * marginal.probs[j]);
                
                ## The overall estimator is a shrinkage-like
                ## combination of alpha and beta.
                
                ## The scores are saved in the convention used for an
                ## ajacency matrix, i.e. [i, j] means causal edge
                ## i-->j.
                
                pr.score[i, j] = (1 - lambda) * alpha + lambda * beta;
            }
        }
    }
    scores =
        list(marginal.probs = marginal.probs,
             joint.probs = joint.probs,
             pr.score = pr.score);
    return(scores);
}


# verify the independent progression filter
# @title verify.parents
# @param best.parents best edges to be verified
# @param marginal.probs observed marginal probabilities
# @param joint.probs observed joint probabilities
# @return best.parents: list of the best valid parents
#
verify.parents <- function(best.parents, marginal.probs, joint.probs) {
    
    ## Verify the condition for the best parent of each node.
    
    for (i in 1:length(best.parents)) {
        
        ## If there is a connection, i.e. the node is not already
        ## attached to the Root.
        
        if (best.parents[i] != -1) {
            
            ## Score for the root as the parent of this node.
            
            w.root.node = 1 / (1 + marginal.probs[i]);
            
            ## Compute the scores for the edges to all the other
            ## upstream nodes.
            
            attach.to.root = 1;
            for (j in 1:length(marginal.probs)) {
                
                ## If the connection is valid and the parent node has
                ## greater probability, i.e. it is before the child in
                ## temporal order.
                
                if (i != j && marginal.probs[j] > marginal.probs[i]) {
                    w.parent.node =
                        (marginal.probs[j] / (marginal.probs[i] + marginal.probs[j])) *
                            (joint.probs[i, j] / (marginal.probs[i] * marginal.probs[j]));
                    
                    ## The parent is valid if this condition is valid
                    ## at least one time (i.e. for at least one of the
                    ## upstream nodes), meaning that if we find out
                    ## that a connection is not spurious for any node,
                    ## the best parent is not spurious as well.
                    
                    if (w.root.node <= w.parent.node) {
                        attach.to.root = 0;
                        break;
                    }
                }
            }
            
            ## Connect the node to the Root if the flag is true.
            
            if (attach.to.root==1) {
                best.parents[i] = -1;
            }
        }
    }
    return(best.parents);
}


#### end of file -- caprese.algorithm.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


# perform non-parametric or statistical bootstrap to evalutate the confidence of the reconstruction
# @title bootstrap.capri
# @param reconstruction Result of a previous reconstruction
# @param command should I perform non-parametric or statistical bootstrap?
# @param nboot.algorithm integer number (greater than 0) of bootstrap sampling to be performed
# @param cores.ratio Percentage of cores to use
# @return bootstrap.statistics: statistics of the bootstrap
#
bootstrap <- function(reconstruction, 
                      command = "non-parametric",
                      nboot = 100,
                      cores.ratio = 1,
                      silent = FALSE) {
    

    parameters = as.parameters(reconstruction)
    type = parameters$algorithm

    dataset = as.genotypes(reconstruction)

    bootstrap.statistics = list()
    if (!is.null(reconstruction$bootstrap)) {
        bootstrap.statistics = reconstruction$bootstrap
    }
    
    ## Get CAPRI parameters.
    
    if (type == 'CAPRI') {
        command.capri = parameters$command
        restart = parameters$restart
    }

    ## Get CAPRESE parameters.

    if (type == 'CAPRESE') {
        lambda = parameters$lambda
        epos = parameters$error.rates$epos
        eneg = parameters$error.rates$eneg
        hypotheses = NA
        if (nhypotheses(reconstruction) > 0) {
            hypotheses = reconstruction$hypotheses
        }
    }

    ## Get other parameters.

    if (type %in% c('CAPRI', 'EDMONDS', 'GABOW', 'CHOW_LIU', 'PRIM')) {
        regularization = parameters$regularization
        do.boot = parameters$do.boot
        nboot.algorithm = parameters$nboot
        pvalue = parameters$pvalue
        min.boot = parameters$min.boot
        min.stat = parameters$min.stat
        boot.seed = parameters$boot.seed
        if(type %in% c('EDMONDS', 'GABOW')) {
            score.param = parameters$score
        }
        if(type=='GABOW') {
            do.raising = parameters$do.raising
        }
        epos = parameters$error.rates$epos
        eneg = parameters$error.rates$eneg
        hypotheses = NA
        if (nhypotheses(reconstruction) > 0) {
            hypotheses = reconstruction$hypotheses
        }
    }

    ## Start the clock to measure the execution time

    ptm <- proc.time();
    
    ## Structure to save the results of the bootstrap.
    
    curr.bootstrap.results = array(list(-1), c(nboot,nevents(reconstruction)))
    colnames(curr.bootstrap.results) = rownames(as.events(reconstruction))
    bootstrap.results = list()
    bootstrap.results[names(as.models(reconstruction))] = list(curr.bootstrap.results)
    
    curr.bootstrap.adj.matrix = array(list(0), c(nevents(reconstruction)+1,nevents(reconstruction)+1))
    colnames(curr.bootstrap.adj.matrix) = c("None",rownames(as.events(reconstruction)))
    rownames(curr.bootstrap.adj.matrix) = c("None",rownames(as.events(reconstruction)))
    bootstrap.adj.matrix = list()
    bootstrap.adj.matrix[names(as.models(reconstruction))] = list(curr.bootstrap.adj.matrix)
    
    bootstrap.adj.matrix.frequency = list()
    bootstrap.adj.matrix.frequency[names(as.models(reconstruction))] = list(curr.bootstrap.adj.matrix)
    
    curr.edge.confidence = array(list(0), c(nevents(reconstruction),nevents(reconstruction)))
    colnames(curr.edge.confidence) = rownames(as.events(reconstruction))
    rownames(curr.edge.confidence) = rownames(as.events(reconstruction))
    bootstrap.edge.confidence = list()
    bootstrap.edge.confidence[names(as.models(reconstruction))] = list(curr.edge.confidence)
    
    overall.confidence = list()
    overall.confidence[names(as.models(reconstruction))] = list(0)
    overall.frequency = list()
    overall.frequency[names(as.models(reconstruction))] = list(0)

    cores = as.integer(cores.ratio * (detectCores() - 1))
    if (cores < 1) {
        cores = 1
    }


    expected.execution.time =
        round(((reconstruction$execution.time[3] * nboot) / (cores)), digits = 0)
    if (!silent) {
        cat("\tExpected completion in approx.",
            format(.POSIXct(expected.execution.time, tz="GMT"),
                   "%Hh:%Mm:%Ss"),
            "\n")
    }


    cl = makeCluster(cores)    
    registerDoParallel(cl)
    if (!silent) {
        cat('\tUsing', cores, 'cores via "parallel" \n')
    }
    ## Perform nboot bootstrap resampling
    
    r = foreach(num = 1:nboot ) %dopar% {    
            
        ## Performed the bootstrapping procedure.
        
        curr.reconstruction = list()

        if (command == "non-parametric") {
            
            ## Perform the sampling for the current step of bootstrap.
            
            samples = sample(1:nrow(dataset), 
                             size = nrow(dataset),
                             replace = TRUE)
            resampled.dataset = dataset[samples,]
            rownames(resampled.dataset) = 1:nrow(resampled.dataset)
            curr.reconstruction$genotypes = resampled.dataset


        } else if (command == "statistical") {
            curr.reconstruction$genotypes = dataset
        }

        curr.reconstruction$annotations = reconstruction$annotations
        curr.reconstruction$types = reconstruction$types

        if (type == 'CAPRI') {
            curr.reconstruction$hypotheses = hypotheses
            bootstrapped.topology =
                tronco.capri(curr.reconstruction,
                             command.capri, 
                             regularization,
                             do.boot,
                             nboot.algorithm,
                             pvalue,
                             min.boot,
                             min.stat,
                             boot.seed,
                             silent = TRUE,
                             epos,
                             eneg,
                             restart)
        } else if (type == 'CAPRESE') {
            curr.reconstruction$hypotheses = hypotheses
            bootstrapped.topology =
                tronco.caprese(curr.reconstruction,
                               lambda, 
                               silent = TRUE,
                               epos,
                               eneg)
        } else if (type == 'CHOW_LIU') {
            curr.reconstruction$hypotheses = hypotheses
            bootstrapped.topology = 
                tronco.chowliu(curr.reconstruction,
                                   regularization,
                                   do.boot,
                                   nboot.algorithm,
                                   pvalue,
                                   min.boot,
                                   min.stat,
                                   boot.seed,
                                   silent = TRUE,
                                   epos,
                                   eneg)
        } else if (type == 'PRIM') {
            curr.reconstruction$hypotheses = hypotheses
            bootstrapped.topology =
                tronco.prim(curr.reconstruction,
                                regularization,
                                do.boot,
                                nboot.algorithm,
                                pvalue,
                                min.boot,
                                min.stat,
                                boot.seed,
                                silent = TRUE,
                                epos,
                                eneg)
        } else if (type == 'EDMONDS') {
            curr.reconstruction$hypotheses = hypotheses
            bootstrapped.topology =
                tronco.edmonds(curr.reconstruction,
                                   regularization,
                                   score.param,
                                   do.boot,
                                   nboot.algorithm,
                                   pvalue,
                                   min.boot,
                                   min.stat,
                                   boot.seed,
                                   silent = TRUE,
                                   epos,
                                   eneg)
        } else if (type == 'GABOW') {
            curr.reconstruction$hypotheses = hypotheses
            bootstrapped.topology =
                tronco.gabow(curr.reconstruction,
                                 regularization,
                                 score.param,
                                 do.boot,
                                 nboot.algorithm,
                                 pvalue,
                                 min.boot,
                                 min.stat,
                                 boot.seed,
                                 silent = TRUE,
                                 epos,
                                 eneg,
                                 do.raising)
        }
            
        curr.reconstruction = bootstrapped.topology
        
        ## Set the reconstructed selective advantage edges.
        
        bootstrap.results = list()
        for (m in names(as.models(curr.reconstruction))) {
            
            ## Get the parents pos.
            parents.pos =
                array(list(), c(nevents(curr.reconstruction), 1))
            
            
            curr.adj.matrix =
                as.adj.matrix(curr.reconstruction, models = m)[[m]]
            for (i in 1:nevents(curr.reconstruction)) {
                for (j in 1:nevents(curr.reconstruction)) {
                    if (i != j && curr.adj.matrix[i, j] == 1) {
                        parents.pos[j, 1] =
                            list(c(unlist(parents.pos[j, 1]), i))
                    }
                }
            }
            
            parents.pos[unlist(lapply(parents.pos,is.null))] = list(-1)
            
            ## Save the results.
            
            bootstrap.results[[m]] = t(parents.pos);
        }
        bootstrap.results
    }

    stopCluster(cl)
    
    if (!silent) {
        cat("\tReducing results\n")
    }
    for (m in names(bootstrap.results)) {
        y = Reduce(rbind, lapply(r, function(z, type) { get(type, z) }, type = m))
        bootstrap.results[[m]] = y
    }

    ## Set the statistics of the bootstrap.
    
    for (m in names(as.models(reconstruction))) {
        curr.bootstrap.adj.matrix = bootstrap.adj.matrix[[m]]
        
        for (i in 2:ncol(curr.bootstrap.adj.matrix)) {
            curr.result = bootstrap.results[[m]][ , i - 1]
            
            for (j in 1:length(curr.result)) {
                curr.val = curr.result[[j]]
                
                for (k in 1:length(curr.val)) {
                    if (length(curr.val[k]) == 1 && curr.val[k] == -1) {
                        curr.bootstrap.adj.matrix[[1, i]] =
                            curr.bootstrap.adj.matrix[[1, i]] + 1
                    } else {
                        curr.bootstrap.adj.matrix[[curr.val[k] + 1, i]] =
                            curr.bootstrap.adj.matrix[[curr.val[k] + 1, i]] + 1
                    }
                }
            }
        }
        
        bootstrap.adj.matrix[[m]] = curr.bootstrap.adj.matrix;
        rownames(bootstrap.results[[m]]) =
            paste("Iteration ", 1:nrow(bootstrap.results[[m]]), sep= "")
        
    }
    
    ## Evalutate the overall confidence.
    
    for (m in names(as.models(reconstruction))) {
        
        curr.bootstrap.results = bootstrap.results[[m]]
        
        for (i in 1:nrow(curr.bootstrap.results)) {
            
            curr.adj.matrix = array(0, c(nevents(reconstruction), nevents(reconstruction)))
            
            for (j in 1:ncol(curr.bootstrap.results)) {
                
                curr.result = curr.bootstrap.results[i, j]
                
                for (k in 1:length(curr.result)) {
                    
                    curr.val = curr.result[[k]]
                    
                    for (l in 1:length(curr.val)) {
                        if (length(curr.val[l]) > 1 || curr.val[l] != -1) {
                            curr.adj.matrix[curr.val[l], j] = 1
                        }
                    }
                }
            }
            
            ## If I have a perfect match between the reconstructed
            ## topologies, increase the count.
            
            reconstructed.topology =
                as.adj.matrix(reconstruction, models = m)[[m]]
            flag = TRUE;

            for (j in 1:nrow(reconstructed.topology)) {
                for (k in 1:ncol(reconstructed.topology)) {
                    if (reconstructed.topology[j, k] !=
                        curr.adj.matrix[j, k]) {
                        flag = FALSE
                        next()
                    }
                }
            }

            if (flag == TRUE) {
                overall.confidence[[m]] = overall.confidence[[m]] + 1
                overall.frequency[[m]] = overall.confidence[[m]] / nboot
            }
        }
    }
    
    ## Save the edge confidence and the frequency of the bootstrap
    ## adj.matrix.
    
    for (m in names(as.models(reconstruction))) {
        
        curr.adj.matrix = as.adj.matrix(reconstruction, models = m)[[m]];
        
        ## Save the edge confidence.
        
        curr.bootstrap.matrix =
            bootstrap.adj.matrix[[m]][-1, -1];
        curr.edge.confidence =
            array(0, c(ncol(curr.bootstrap.matrix), nrow(curr.bootstrap.matrix)))
        colnames(curr.edge.confidence) =
            colnames(curr.bootstrap.matrix);
        rownames(curr.edge.confidence) =
            rownames(curr.bootstrap.matrix);
        
        for (i in 1:ncol(curr.bootstrap.matrix)) {
            for (j in 1:nrow(curr.bootstrap.matrix)) {
                curr.edge.confidence[i, j] =
                    (curr.adj.matrix[i, j] *
                         as.numeric(curr.bootstrap.matrix[i, j])) / nboot
            }
        }
        bootstrap.edge.confidence[[m]] = curr.edge.confidence
        
        ## Save the frequency of the bootstrap adj.matrix.
        curr.bootstrap.matrix = bootstrap.adj.matrix[[m]];
        curr.adj.matrix.frequency =
            array(0, c(ncol(curr.bootstrap.matrix), nrow(curr.bootstrap.matrix)))
        colnames(curr.adj.matrix.frequency) =
            colnames(curr.bootstrap.matrix);
        rownames(curr.adj.matrix.frequency) =
            rownames(curr.bootstrap.matrix);
        
        for (i in 1:ncol(curr.bootstrap.matrix)) {
            for (j in 1:nrow(curr.bootstrap.matrix)) {
                curr.adj.matrix.frequency[i, j] =
                    as.numeric(as.numeric(curr.bootstrap.matrix[i, j])) / nboot
            }
        }
        bootstrap.adj.matrix.frequency[[m]] = curr.adj.matrix.frequency
    }
    
    ## Save the statistics of the bootstrap.
    for (m in names(as.models(reconstruction))) {
        if (command == "non-parametric") {
            
            bootstrap.statistics[[m]]$npb$bootstrap.results =
                bootstrap.results[[m]]
            
            bootstrap.statistics[[m]]$npb$bootstrap.adj.matrix =
                list(count = bootstrap.adj.matrix[[m]],
                     frequency = bootstrap.adj.matrix.frequency[[m]])
            
            bootstrap.statistics[[m]]$npb$bootstrap.edge.confidence =
                bootstrap.edge.confidence[[m]]
            
            bootstrap.statistics[[m]]$npb$overall.confidence =
                list(count = overall.confidence[[m]],
                     frequency = overall.frequency[[m]])
            
            bootstrap.statistics[[m]]$npb$bootstrap.settings =
                list(type = command, nboot = nboot)
        } else if (command == "parametric") {
            
            bootstrap.statistics[[m]]$pb$bootstrap.results =
                bootstrap.results[[m]]
            
            bootstrap.statistics[[m]]$pb$bootstrap.adj.matrix =
                list(count = bootstrap.adj.matrix[[m]],
                     frequency = bootstrap.adj.matrix.frequency[[m]])
            
            bootstrap.statistics[[m]]$pb$bootstrap.edge.confidence =
                bootstrap.edge.confidence[[m]]
            
            bootstrap.statistics[[m]]$pb$overall.confidence =
                list(count = overall.confidence[[m]],
                     frequency = overall.frequency[[m]])
            
            bootstrap.statistics[[m]]$pb$bootstrap.settings =
                list(type = command, nboot = nboot)
        } else if (command == "statistical") {
            
            bootstrap.statistics[[m]]$sb$bootstrap.results =
                bootstrap.results[[m]]
            
            bootstrap.statistics[[m]]$sb$bootstrap.adj.matrix =
                list(count = bootstrap.adj.matrix[[m]],
                     frequency = bootstrap.adj.matrix.frequency[[m]])
            
            bootstrap.statistics[[m]]$sb$bootstrap.edge.confidence =
                bootstrap.edge.confidence[[m]]
            
            bootstrap.statistics[[m]]$sb$overall.confidence =
                list(count = overall.confidence[[m]],
                     frequency = overall.frequency[[m]])
            
            bootstrap.statistics[[m]]$sb$bootstrap.settings =
                list(type = command, nboot = nboot)
        }
    }
    
    ## Save the execution time of the bootstrap.
    
    if (command == "non-parametric") {
        bootstrap.statistics$npb$execution.time = (proc.time() - ptm)
    } else if (command == "parametric") {
        bootstrap.statistics$pb$execution.time = (proc.time() - ptm)
    } else if (command == "statistical") {
        bootstrap.statistics$sb$execution.time = (proc.time() - ptm)
    }
    
    return(bootstrap.statistics)
}


# convert an integer decimal number to binary
# @title decimal.to.binary.dag
# @param num.decimal decimal integer to be converted
# @param num.bits number of bits to be used
# @return num.binary: binary conversion of num.decimal
#
decimal.to.binary.dag <- function(num.decimal, num.bits) {
    
    ## Structure where to save the result.
    
    num.binary = rep(0, num.bits)
    
    ## Convert the integer decimal number to binary.
    
    pos = 0
    while (num.decimal > 0) {
        
        ## Compute the value of the current step.
        
        num.binary[num.bits-pos] = num.decimal %% 2;
        
        ## Divide the number by 2 for the next iteration.
        
        num.decimal = num.decimal %/% 2
        pos = pos + 1
    }
    return(num.binary)
}


#### end of file -- capri.bootstrap.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' @name maf
#' @title MAF example data
#' @description This dataset contains a standard MAF input for TRONCO 
#' @docType data
#' @usage data(maf)
#' @format Manual Annotated Format
#' @source fake data
#' @author Luca De Sano
#' @return A standard TRONCO object
NULL

#' @name muts
#' @title Simple mutation dataset
#' @description A simple mutation dataset without hypotheses
#' @docType data
#' @usage data(muts)
#' @format TRONCO compliant dataset
#' @source fake data
#' @author Luca De Sano
#' @return A standard TRONCO object
NULL

#' @name test_dataset_no_hypos
#' @title A complete dataset
#' @description This dataset contains a complete test dataset
#' @docType data
#' @usage data(test_dataset_no_hypos)
#' @format TRONCO compliant dataset
#' @source fake data
#' @author Luca De Sano
#' @return A standard TRONCO object
NULL

#' @name test_dataset
#' @title A complete dataset with hypotheses
#' @description This dataset contains a complete test dataset
#' @docType data
#' @usage data(test_dataset)
#' @format TRONCO compliant dataset
#' @source fake data
#' @author Luca De Sano
#' @return A standard TRONCO object
NULL

#' @name stage
#' @title Stage information for test_dataset
#' @description This dataset contains stage information for patient in test_dataset
#' @docType data
#' @usage data(stage)
#' @format Vector of stages
#' @source fake data
#' @author Luca De Sano
#' @return A list of stages
NULL

#' @name test_model
#' @title A complete dataset with a reconstructed model
#' @description This dataset contains a model reconstructed with CAPRI
#' @docType data
#' @usage data(test_model)
#' @format TRONCO compliant dataset
#' @source fake data
#' @author Luca De Sano
#' @return A standard TRONCO object
NULL

#' @name test_model_kfold
#' @title A complete dataset with a reconstructed model and crossvalidation informations
#' @description This dataset contains a model reconstructed with CAPRI
#' @docType data
#' @usage data(test_model_kfold)
#' @format TRONCO compliant dataset
#' @source fake data
#' @author Luca De Sano
#' @return A standard TRONCO object
NULL

#' @name aCML
#' @title Atypical chronic myeloid leukemia dataset
#' @description This file contains a TRONCO compliant dataset 
#' @docType data
#' @usage data(aCML)
#' @format TRONCO compliant dataset
#' @source data from http://www.nature.com/ng/journal/v45/n1/full/ng.2495.html
#' @author Luca De Sano
#' @return A standard TRONCO object
NULL

#' @name crc_maf
#' @title MAF example data
#' @description This dataset contains an example of MAF input of a crc cohort of patients 
#' @docType data
#' @usage data(crc_maf)
#' @format Manual Annotated Format
#' @source data from http://www.nature.com/nature/journal/v487/n7407/full/nature11252.html
#' @author Daniele Ramazzotti
#' @return A MAF file
NULL

#' @name crc_gistic
#' @title GISTIC example data
#' @description This dataset contains an example of GISTIC input of a crc cohort of patients 
#' @docType data
#' @usage data(crc_gistic)
#' @format GISTIC score
#' @source data from http://www.nature.com/nature/journal/v487/n7407/full/nature11252.html
#' @author Daniele Ramazzotti
#' @return A gistic file
NULL

#' @name crc_plain
#' @title Plain mutation dataset
#' @description This dataset contains an example of plain input of a crc cohort of patients
#' @docType data
#' @usage data(crc_plain)
#' @format plain data
#' @source data from http://www.nature.com/nature/journal/v487/n7407/full/nature11252.html
#' @author Daniele Ramazzotti
#' @return A plain input
NULL

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


# reconstruct the best topology based on probabilistic causation and Prim algorithm
# @title prim.fit
# @param dataset a dataset describing a progressive phenomenon
# @param regularization regularizators to be used for the likelihood fit
# @param do.boot should I perform bootstrap? Yes if TRUE, no otherwise
# @param nboot integer number (greater than 0) of bootstrap sampling to be performed
# @param pvalue pvalue for the tests (value between 0 and 1)
# @param min.boot minimum number of bootstrapping to be performed
# @param min.stat should I keep bootstrapping untill I have nboot valid values?
# @param boot.seed seed to be used for the sampling
# @param silent should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @param hypotheses hypotheses to be considered in the reconstruction. This should be NA for this algorithms. 
# @return topology: the reconstructed tree topology
#
prim.fit <- function(dataset,
                      regularization = "no_reg",
                      do.boot = TRUE,
                      nboot = 100,
                      pvalue = 0.05,
                      min.boot = 3,
                      min.stat = TRUE,
                      boot.seed = NULL,
                      silent = FALSE,
                      epos = 0.0,
                      eneg = 0.0,
                      hypotheses = NA) {

    ## Start the clock to measure the execution time.
    
    ptm = proc.time();

    ## Structure with the set of valid edges
    ## I start from the complete graph, i.e., I have no prior and all
    ## the connections are possibly causal
    adj.matrix = array(1, c(ncol(dataset), ncol(dataset)));
    colnames(adj.matrix) = colnames(dataset);
    rownames(adj.matrix) = colnames(dataset);

    ## The diagonal of the adjacency matrix should not be considered,
    ## i.e., no self cause is allowed
    diag(adj.matrix) = 0;

    ## Consider any hypothesis.
    
    adj.matrix = hypothesis.adj.matrix(hypotheses, adj.matrix);

    ## Check if the dataset is valid
    valid.dataset = check.dataset(dataset, adj.matrix, FALSE, epos, eneg)
    adj.matrix = valid.dataset$adj.matrix;
    invalid.events = valid.dataset$invalid.events;

    ## Reconstruct the prima facie topology
    ## Should I perform bootstrap? Yes if TRUE, no otherwise
    if (do.boot == TRUE) {
        if (!silent)
            cat('*** Bootstraping selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.do.boot(dataset,
                                            hypotheses,
                                            nboot,
                                            pvalue,
                                            adj.matrix,
                                            min.boot,
                                            min.stat,
                                            boot.seed,
                                            silent,
                                            epos,
                                            eneg);
    } else {
        if (!silent)
            cat('*** Computing selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.no.boot(dataset,
                                            hypotheses,
                                            adj.matrix,
                                            silent,
                                            epos,
                                            eneg);
    }

    ## Add back in any connection invalid for the probability raising
    ## theory.
    
    if (length(invalid.events) > 0) {
        # save the correct acyclic matrix
        adj.matrix.cyclic.tp.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp
        adj.matrix.cyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic
        adj.matrix.acyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.acyclic
        for (i in 1:nrow(invalid.events)) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.cyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.acyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
        }
        # if the new cyclic.tp contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp = adj.matrix.cyclic.tp.valid
        }
        # if the new cyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic = adj.matrix.cyclic.valid
        }
        # if the new acyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.acyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.acyclic = adj.matrix.acyclic.valid
        }
    }
    
    adj.matrix.prima.facie =
        prima.facie.parents$adj.matrix$adj.matrix.acyclic

    ## Perform the likelihood fit with the required strategy.
    
    model = list();
    for (reg in regularization) {

        ## Perform the likelihood fit with the chosen regularization
        ## score on the prima facie topology.
        
        if (!silent) {
            cat('*** Performing likelihood-fit with regularization:',reg,'.\n')
        }
        best.parents =
            perform.likelihood.fit.prim(dataset,
                adj.matrix.prima.facie,
                regularization = reg,
                marginal.probs = prima.facie.parents$marginal.probs,
                joint.probs = prima.facie.parents$joint.probs)

        ## Set the structure to save the conditional probabilities of
        ## the reconstructed topology.

        reconstructed.model = create.model(dataset,
            best.parents,
            prima.facie.parents)

        model.name = paste('prim', reg, sep='_')
        model[[model.name]] = reconstructed.model
    }

    ## Set the execution parameters.
    
    parameters =
        list(algorithm = "PRIM",
             regularization = regularization,
             do.boot = do.boot,
             nboot = nboot,
             pvalue = pvalue,
             min.boot = min.boot,
             min.stat = min.stat,
             boot.seed = boot.seed,
             silent = silent,
             error.rates = list(epos=epos,eneg=eneg))

    ## Return the results.
    
    topology =
        list(dataset = dataset,
             hypotheses = hypotheses,
             adj.matrix.prima.facie = adj.matrix.prima.facie,
             confidence = prima.facie.parents$pf.confidence,
             model = model,
             parameters = parameters,
             execution.time = (proc.time() - ptm))
    topology = rename.reconstruction.fields(topology, dataset)
    return(topology)
}


# reconstruct the best causal topology by Prim algorithm combined with probabilistic causation
# title perform.likelihood.fit.prim
# param dataset a valid dataset
# param adj.matrix the adjacency matrix of the prima facie causes
# param regularization regularization term to be used in the likelihood fit
# param command type of search, either hill climbing (hc) or tabu (tabu)
# return topology: the adjacency matrix of both the prima facie and causal topologies
#
perform.likelihood.fit.prim = function(dataset,
                                       adj.matrix,
                                       regularization,
                                       command = "hc",
                                       marginal.probs,
                                       joint.probs){

    data = as.categorical.dataset(dataset)
    adj.matrix.prima.facie = adj.matrix
    
    # adjacency matrix of the topology reconstructed by likelihood fit
    adj.matrix.fit = array(0,c(nrow(adj.matrix),ncol(adj.matrix)))
    rownames(adj.matrix.fit) = colnames(dataset)
    colnames(adj.matrix.fit) = colnames(dataset)
       
    # create the graph of valid edges
    curr.valid.adj.matrix = array(0, c(nrow(adj.matrix), ncol(adj.matrix)))
    cont = 0
    for (i in 1:nrow(curr.valid.adj.matrix)) {
        for (j in i:nrow(curr.valid.adj.matrix)) {
            # we want an undirected prima facie graph, 
            # so we consider all the edges where there is a directed arc
            if (!(adj.matrix[i,j] == 0 && adj.matrix[j,i] == 0)) {
                cont = cont + 1
                curr.valid.adj.matrix[i,j] = 1
                curr.valid.adj.matrix[j,i] = 1
            }
        }
    }
    
    # if the graph has at least one edge, build to weighted igraph object
    if (cont > 0) {
        curr.graph = graph.adjacency(curr.valid.adj.matrix, mode = "undirected")
        all_edges = get.edgelist(curr.graph)
        # set the weights to the edges
        new_weights = NULL
        
        if (cont == 1) {
            # consider the current arc
            i = all_edges[1,1]
            j = all_edges[1,2]
            # if the event is valid
            if(joint.probs[i,j]>=0) {
                new_score = compute.mi.score(joint.probs[i,j],marginal.probs[i],marginal.probs[j])
            }
            # else, if the two events are indistinguishable put the higher score
            else if(joint.probs[i,j]<0) {
                new_score = 1
            }
            new_weights = new_score
        } else {
            for (i in 1:nrow(all_edges)) {
                # consider the current arc
                curr_i = all_edges[i,1]
                curr_j = all_edges[i,2]
                # if the event is valid
                if(joint.probs[curr_i,curr_j]>=0) {
                    new_score = compute.mi.score(joint.probs[curr_i,curr_j],marginal.probs[curr_i],marginal.probs[curr_j])
                }
                # else, if the two events are indistinguishable put the higher score
                else if(joint.probs[curr_i,curr_j]<0) {
                    new_score = 1 # Inf
                }
                new_weights = c(new_weights,new_score)
            }
        }
        
        # set the weights to the graph
        E(curr.graph)$weight = 1 - new_weights
        
        # get the minimum spanning tree by Prim algorithm
        curr.valid.adj.matrix = as.matrix(get.adjacency(minimum.spanning.tree(curr.graph,algorithm="prim")))

        # build the matrix of the priors
        new_prior_matrix = array(0, c(nrow(adj.matrix), ncol(adj.matrix)))
        rownames(new_prior_matrix) = colnames(dataset)
        colnames(new_prior_matrix) = colnames(dataset)
        for (i in 1:nrow(curr.valid.adj.matrix)) {
            for (j in i:ncol(curr.valid.adj.matrix)) {
                if (i != j) {
                    if (curr.valid.adj.matrix[i,j] == 1) {
                        if (adj.matrix[i,j] == 1) {
                            new_prior_matrix[i,j] = 1
                        } else if(adj.matrix[j,i] == 1) {
                            new_prior_matrix[j,i] = 1
                        }
                    }
                }
            }
        }
        adj.matrix = new_prior_matrix

        ## Perform the likelihood fit if requested
        adj.matrix.fit = lregfit(data,
            adj.matrix,
            adj.matrix.fit,
            regularization,
            command)
    }
    
    
    adj.matrix = list(adj.matrix.pf = adj.matrix.prima.facie,
        adj.matrix.fit = adj.matrix.fit)
    topology = list(adj.matrix = adj.matrix)
    return(topology)
}

# compute the mutual information score for the prim algorithm
compute.mi.score = function ( p_i_j, p_i, p_j ) {
    
    # compute the needed measures
    p_i_not_j = p_i - p_i_j
    p_not_i_j = p_j - p_i_j
    p_not_i_not_j = 1 - p_i - p_j + p_i_j
    
    # compute the 4 terms of mutual information
    m_i_j = p_i_j * log(p_i_j/(p_i*p_j))
    m_i_not_j = p_i_not_j * log(p_i_not_j/(p_i*(1-p_j)))
    m_not_i_j = p_not_i_j * log(p_not_i_j/((1-p_i)*p_j))
    m_not_i_not_j = p_not_i_not_j * log(p_not_i_not_j/((1-p_i)*(1-p_j)))
    
    # NOTE: in our case p_i and p_j are positive numbers in (0,1) with intervals excluded, 
    # hence, the denominator of the scores cannot be 0. 
    # the numerators are numbers in [0,1). So the fraction is a number between [0,Inf). 
    # specifically the log of the fraction is a number in [-Inf,Inf) with Inf excluded and 
    # -Inf when any numerator is 0. 
    # BUT: when numerator is 0, the multiplier of the logarithm is 0 hence givine 0*(-Inf) = NA
    # SO: we will replace any NA with 0 (multiplier wins over logarithm)
    if(is.nan(m_i_j)) {
        m_i_j = 0
    }
    if(is.nan(m_i_not_j)) {
        m_i_not_j = 0
    }
    if(is.nan(m_not_i_j)) {
        m_not_i_j = 0
    }
    if(is.nan(m_not_i_not_j)) {
        m_not_i_not_j = 0
    }
    
    # compute the complete mutual information
    mutual_information = m_i_j + m_i_not_j + m_not_i_j + m_not_i_not_j
    
    return(mutual_information)
}


#### end of file -- prim.algorithm.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.

# reconstruct the best dag topology running CAPRI algorithm
# @title capri.fit
# @param dataset a dataset describing a progressive phenomenon
# @param hypotheses hypotheses to be considered in the reconstruction
# @param command type of search for the likelihood fit, either hill climbing (hc) or tabu (tabu)
# @param regularization regularizators to be used for the likelihood fit
# @param do.boot should I perform bootstrap? Yes if TRUE, no otherwise
# @param nboot integer number (greater than 0) of bootstrap sampling to be performed
# @param pvalue pvalue for the tests (value between 0 and 1)
# @param min.boot minimum number of bootstrapping to be performed
# @param min.stat should I keep bootstrapping untill I have nboot valid values?
# @param boot.seed seed to be used for the sampling
# @param silent should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @return topology: the reconstructed tree topology
#
capri.fit <- function(dataset,
                      hypotheses = NA,
                      command = "hc",
                      regularization = c("bic", "aic"),
                      do.boot = TRUE,
                      nboot = 100,
                      pvalue = 0.05,
                      min.boot = 3,
                      min.stat = TRUE,
                      boot.seed = NULL,
                      silent = FALSE,
                      epos = 0.0,
                      eneg = 0.0,
                      restart = 100) {

    ## Start the clock to measure the execution time.
    
    ptm = proc.time();

    ## Structure with the set of valid edges
    ## I start from the complete graph, i.e., I have no prior and all
    ## the connections are possibly causal.
    
    adj.matrix = array(1, c(ncol(dataset), ncol(dataset)));
    colnames(adj.matrix) = colnames(dataset);
    rownames(adj.matrix) = colnames(dataset);

    ## The diagonal of the adjacency matrix should not be considered,
    ## i.e., no self cause is allowed.
    
    diag(adj.matrix) = 0;

    ## Consider any hypothesis.
    
    adj.matrix = hypothesis.adj.matrix(hypotheses, adj.matrix);

    ## Check if the dataset is valid.
    
    valid.dataset = check.dataset(dataset, adj.matrix, FALSE, epos, eneg)
    adj.matrix = valid.dataset$adj.matrix;
    invalid.events = valid.dataset$invalid.events;

    ## Reconstruct the prima facie topology
    ## Should I perform bootstrap? Yes if TRUE, no otherwise.
    
    if (do.boot == TRUE) {
        if (!silent)
            cat('*** Bootstraping selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.do.boot(dataset,
                                            hypotheses,
                                            nboot,
                                            pvalue,
                                            adj.matrix,
                                            min.boot,
                                            min.stat,
                                            boot.seed,
                                            silent,
                                            epos,
                                            eneg);
    } else {
        if (!silent)
            cat('*** Computing selective advantage scores (prima facie).\n')
        prima.facie.parents =
            get.prima.facie.parents.no.boot(dataset,
                                            hypotheses,
                                            adj.matrix,
                                            silent,
                                            epos,
                                            eneg);
    }

    ## Add back in any connection invalid for the probability raising
    ## theory.
    
    if (length(invalid.events) > 0) {
        # save the correct acyclic matrix
        adj.matrix.cyclic.tp.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp
        adj.matrix.cyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.cyclic
        adj.matrix.acyclic.valid = prima.facie.parents$adj.matrix$adj.matrix.acyclic
        for (i in 1:nrow(invalid.events)) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.cyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
            prima.facie.parents$adj.matrix$adj.matrix.acyclic[invalid.events[i, "cause"],invalid.events[i, "effect"]] = 1
        }
        # if the new cyclic.tp contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic.tp = adj.matrix.cyclic.tp.valid
        }
        # if the new cyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.cyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.cyclic = adj.matrix.cyclic.valid
        }
        # if the new acyclic contains cycles use the previously computed matrix
        if (!is.dag(graph.adjacency(prima.facie.parents$adj.matrix$adj.matrix.acyclic))) {
            prima.facie.parents$adj.matrix$adj.matrix.acyclic = adj.matrix.acyclic.valid
        }
    }

    adj.matrix.prima.facie =
        prima.facie.parents$adj.matrix$adj.matrix.acyclic

    ## Perform the likelihood fit with the required regularization
    ## scores.
    
    model = list();
    for (reg in regularization) {

        ## Perform the likelihood fit with the chosen regularization
        ## score on the prima facie topology.
        
        if (!silent)
            cat(paste0('*** Performing likelihood-fit with regularization ',reg,'.\n'))
        best.parents =
            perform.likelihood.fit.capri(dataset,
                prima.facie.parents$adj.matrix$adj.matrix.acyclic,
                command,
                regularization = reg,
                restart)

        ## Set the structure to save the conditional probabilities of
        ## the reconstructed topology.

        reconstructed.model = create.model(dataset,
            best.parents,
            prima.facie.parents)

        model.name = paste('capri', reg, sep='_')
        model[[model.name]] = reconstructed.model
    }

    ## Set the execution parameters.
    
    parameters =
        list(algorithm = "CAPRI",
             command = command,
             regularization = regularization,
             do.boot = do.boot,
             nboot = nboot,
             pvalue = pvalue,
             min.boot = min.boot,
             min.stat = min.stat,
             boot.seed = boot.seed,
             silent = silent,
             error.rates = list(epos=epos,eneg=eneg),
             restart = restart)

    ## Return the results.
    
    topology =
        list(dataset = dataset,
             hypotheses = hypotheses,
             adj.matrix.prima.facie = adj.matrix.prima.facie,
             confidence = prima.facie.parents$pf.confidence,
             model = model,
             parameters = parameters,
             execution.time = (proc.time() - ptm))
    topology = rename.reconstruction.fields(topology, dataset)
    return(topology)
}


# check if the dataset is valid accordingly to the probability raising
# @title check.dataset
# @param dataset a dataset describing a progressive phenomenon
# @param adj.matrix adjacency matrix of the topology
# @param verbose should I print the warnings? Yes if TRUE, no otherwise
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @return valid.dataset: a dataset valid accordingly to the probability raising
check.dataset <- function(dataset, adj.matrix, verbose, epos, eneg ) {

    ## Perform the preprocessing only if I have at least two binary
    ## events and two samples.
    
    if (length(ncol(dataset)) > 0
        && ncol(dataset) > 1
        && length(nrow(dataset)) > 0
        && nrow(dataset) > 1
        && length(dataset[dataset == 0 | dataset == 1]) == nrow(dataset) * ncol(dataset)) {

        ## Structure to compute the observed and observed joint
        ## probabilities.
        
        pair.count <- array(0, dim = c(ncol(dataset), ncol(dataset)));
        
        ## Compute the probabilities on the dataset.
        
        for (i in 1:ncol(dataset)) {
            for (j in 1:ncol(dataset)) {
                val1 = dataset[ ,i];
                val2 = dataset[ ,j];
                pair.count[i,j] = (t(val1) %*% val2);
            }
        }
        
        # minimum probability to be represented in the dataset
        minimum.prob = 0 #max(.Machine$double.eps,(1/nrow(dataset))/2)
        
        ## marginal.probs is an array with the marginal probabilities.
        
        marginal.probs <-
            array(as.matrix(diag(pair.count) / nrow(dataset)),
                  dim = c(ncol(dataset), 1))
                  
        ## joint.probs is an array with the joint observed probabilities.
        
        joint.probs <- as.matrix(pair.count / nrow(dataset))
        
        # apply the noise model to estimate the theoretical joint.probs
        for (p1 in 1:nrow(joint.probs)) {
            for (p2 in p1:ncol(joint.probs)) {
                estimated.prob = estimate.theoretical.probs(joint.probs[p1,p2],type="joint",prob1=marginal.probs[p1],
                                                            prob2=marginal.probs[p2],
                                                            min.prob=minimum.prob,epos=epos,eneg=eneg)
                joint.probs[p1,p2] = estimated.prob
                joint.probs[p2,p1] = estimated.prob
            }
        }
        
        
        # apply the noise model to estimate the theoretical marginal.probs
        marginal.probs = sapply(marginal.probs,FUN=function(x) { return(estimate.theoretical.probs(x,
                                                                         type="marginal",min.prob=minimum.prob,
                                                                         epos=epos,eneg=eneg)) })
                                                                         
        marginal.probs = array(marginal.probs, dim = c(length(marginal.probs), 1))
    
        # verify the probabilities to be corret
        res.probs = verify.constraints.probs(marginal.probs,joint.probs)
        marginal.probs = res.probs$marginal.probs
        joint.probs = res.probs$joint.probs
        
        ## Evaluate the connections.
        
        invalid.events = vector();
        for (i in 1:ncol(adj.matrix)) {
            for (j in 1:nrow(adj.matrix)) {

                ## if i --> j is valid
                if (i != j && adj.matrix[i, j] == 1) {
                                        
                    if (marginal.probs[i,1] == 1) {
                        ## the potential cause is always present
                        adj.matrix[i, j] = 0;
                    } else if (marginal.probs[i,1] == 0) {
                        ## the potential cause is always missing
                        adj.matrix[i, j] = 0;
                    } else if (marginal.probs[j,1] == 1) {
                        ## the potential child is always present
                        adj.matrix[i, j] = 0;
                    } else if (marginal.probs[j,1] == 0) {
                        ## the potential child is always missing
                        adj.matrix[i, j] = 0;
                    } else if ((joint.probs[i,j] / marginal.probs[i]) == 1
                                && (joint.probs[i,j] / marginal.probs[j]) == 1) {
                        
                        ## the two events are equals
                        adj.matrix[i, j] = 0;                        
                        
                        #invalid.events = rbind(invalid.events,t(c(i,j)));
                        
                        # if we have 2 indistinguishable events both connected to create a cycle
                        # I choose one direction randomly
                        # by keeping only the edge from the lower to the higher positioned node
                        if (i<j) {
                            invalid.events = rbind(invalid.events,t(c(i,j)))
                        }
                    }
                }
            }
        }
        
        if (length(invalid.events) > 0) {
            warning("The dataset contains indistinguishable events that are left disconnected in the progression model.\n")
            colnames(invalid.events) = c("cause","effect");
        }
        valid.dataset =
            list(dataset = dataset,
                 adj.matrix = adj.matrix,
                 invalid.events = invalid.events,
                 marginal.probs = marginal.probs,
                 joint.probs = joint.probs);
    }

    ## if the dataset is not valid, we stop here
    else {
        if (verbose == TRUE) {
            warning("The dataset must contain at least two binary events and two samples.");
        }
        valid.dataset =
            list(dataset = NA,
                 adj.matrix = NA,
                 invalid.events = NA,
                 marginal.probs = NA,
                 joint.probs = NA);
    }
    return(valid.dataset);
}

# verify the constraints over the probabilities after noise is applied
verify.constraints.probs = function( marginal.probs, joint.probs ) {
    
    for (i in 1:nrow(joint.probs)) {
        for (j in i:ncol(joint.probs)) {
            if(i!=j) {
                max.joint.prob = min(marginal.probs[i,1],marginal.probs[j,1])
                if(joint.probs[i,j]>max.joint.prob) {
                    joint.probs[i,j] = max.joint.prob
                    joint.probs[j,i] = joint.probs[i,j]
                }
            }
        }
    }
    
    res.probs = list(marginal.probs=marginal.probs,joint.probs=joint.probs)
    
}


# compute a robust estimation of the scores using rejection sampling bootstrap
# @title get.bootstrapped.scores
# @param dataset a valid dataset
# @param nboot number of bootstrap resampling to be performed
# @param adj.matrix adjacency matrix of the initially valid edges
# @param min.boot minimum number of bootstrapping to be performed
# @param min.stat should I keep bootstrapping untill I have nboot valid values?
# @param boot.seed seed to be used for the sampling
# @param silent Should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @return scores: list structure with the scores and the data generated by bootstrap
#
get.bootstrapped.scores <- function(dataset,
                                   nboot,
                                   adj.matrix,
                                   min.boot = 3,
                                   min.stat = TRUE,
                                   boot.seed = NULL,
                                   silent = FALSE,
                                   epos,
                                   eneg) {

    ## Structures to save the distributions generated by the
    ## bootstrapped datasets.
    
    marginal.probs.distributions <-
        array(list(-1), c(ncol(dataset), 1));
    joint.probs.distributions <-
        array(list(-1), c(ncol(dataset), ncol(dataset)));
    prima.facie.model.distributions <-
        array(list(-1), c(ncol(dataset), ncol(dataset)));
    prima.facie.null.distributions <-
        array(list(-1), c(ncol(dataset), ncol(dataset)));
    
    ## Structures to save the number of performed valid (not rejected)
    ## sampling.
    
    sampled.marginal.probs.distributions <-
        array(0, dim = c(ncol(dataset),1));
    sampled.joint.probs.distributions <-
        array(0, dim = c(ncol(dataset), ncol(dataset)));
    sampled.prima.facie.distributions <-
        array(0, dim = c(ncol(dataset),ncol(dataset)));

    ## I require a minimum of min.boot (default = 3) sampling of
    ## bootstrap.
    
    nboot = max(nboot,min.boot);

    ## Set not to sample for the invalid edges.
    
    for (i in 1:nrow(adj.matrix)) {
        for (j in 1:ncol(adj.matrix)) {
            if (adj.matrix[i, j] == 0) {
                sampled.prima.facie.distributions[i,j] = nboot;
            }
        }
    }

    ## Perform bootstrap estimation based on a number of bootstrapped
    ## (>= nboot) datasets.

    ###
    ### START BOOTSTRAP HERE
    ###

    
    curr.iteration = min(sampled.prima.facie.distributions);
    boot.counter = 0;

    ## Set the seed to be used for the sampling.
    
    set.seed(boot.seed);

    #if (silent == FALSE) {
    #    ## Create a progress bar.
    #    flush.console();
    #    pb <- txtProgressBar(curr.iteration, nboot, style = 3);
    #}

    dot = 0
    if (!silent) {
        cat('\t')
    }

    while (curr.iteration<nboot) {

        ## Define the dataset to be used in this iteration and compute
        ## the scores on it.
        
        sampled.data =
            dataset[sample(1:nrow(dataset),
                           size = nrow(dataset),
                           replace = TRUE),
                    ];
        boot.counter = boot.counter + 1;

        ## Compute the scores on the sampled data.
        
        curr.scores = get.dag.scores(sampled.data,adj.matrix,epos,eneg);
        curr.marginal.probs = curr.scores$marginal.probs;
        curr.joint.probs = curr.scores$joint.probs;
        curr.prima.facie.model = curr.scores$prima.facie.model;
        curr.prima.facie.null = curr.scores$prima.facie.null;

        ## Save the (valid) scores for each edge.
        
        for (i in 1:nrow(curr.prima.facie.model)) {

            ## Get the marginal probabilities from the sampled data.
            
            if (sampled.marginal.probs.distributions[i, 1] == 0) {
                sampled.marginal.probs.distributions[i, 1] = 1;
                marginal.probs.distributions[i,1] =
                    curr.marginal.probs[i, 1];
            } else {
                marginal.probs.distributions[i, 1] =
                    list(c(unlist(marginal.probs.distributions[i, 1]),
                           curr.marginal.probs[i, 1]));
            }

            for (j in 1:ncol(curr.prima.facie.model)) {

                ## Get the joint probs from the sampled data.
                
                if (sampled.joint.probs.distributions[i, j] == 0) {
                    sampled.joint.probs.distributions[i, j] = 1;
                    joint.probs.distributions[i, j] =
                        curr.joint.probs[i, j];
                } else {
                    joint.probs.distributions[i, j] =
                        list(c(unlist(joint.probs.distributions[i, j]),
                               curr.joint.probs[i, j]));
                }

                ## Get the prima facie estimations from the sampled
                ## data.
                
                if (curr.prima.facie.model[i, j] != -1) {
                    
                    ## Count the valid values per edge.
                    
                    sampled.prima.facie.distributions[i, j] =
                        sampled.prima.facie.distributions[i, j] + 1;

                    ## Save the scores.
                    
                    if (sampled.prima.facie.distributions[i, j] == 1) {
                        ## scores for i --> j
                        prima.facie.model.distributions[i, j] =
                            list(curr.prima.facie.model[i, j]);
                        prima.facie.null.distributions[i, j] =
                            list(curr.prima.facie.null[i, j]);
                    } else {
                        ## Scores for i --> j
                        prima.facie.model.distributions[i, j] =
                            list(c(unlist(prima.facie.model.distributions[i, j]),
                                   curr.prima.facie.model[i, j]));
                        prima.facie.null.distributions[i, j] =
                            list(c(unlist(prima.facie.null.distributions[i, j]),
                                   curr.prima.facie.null[i, j]));
                    }
                }
            }
        }

        ## Set the number of performed iterations after the last
        ## bootstrap sampling.
        
        curr.iteration = min(sampled.prima.facie.distributions);

        ## If the flag min.stat is FALSE, even if after nboot
        ## iterations I don't have nboot valid entries, as soon as I
        ## have at least min.boot values, I stop anyway.
        
        if (min.stat == FALSE
            && boot.counter >= nboot
            && curr.iteration >= min.boot) {
            curr.iteration = nboot;
        }

        #if (silent == FALSE) {
        #    ## Increment the progress bar.
        #    if (min.stat == FALSE) {
        #        setTxtProgressBar(pb, boot.counter);
        #    } else {
        #        setTxtProgressBar(pb, curr.iteration);
        #    }
        #}
        if (!silent && (curr.iteration %% 5 == 0)) {
            cat('.')
            dot = dot + 1
            if (dot %% 50 == 0) {
                cat('\n\t')
            }
        }
    }

    #if (silent == FALSE) {
    #    ## Close the progress bar.
    #    close(pb);
    #}
    if (!silent) {
        cat('\n')
    }

    ## Save the results and return them.
    scores =
        list(marginal.probs.distributions = marginal.probs.distributions,
             joint.probs.distributions = joint.probs.distributions,
             prima.facie.model.distributions = prima.facie.model.distributions,
             prima.facie.null.distributions = prima.facie.null.distributions);
    return(scores);
}


# compute the observed probabilities and the prima facie scores on the dataset
# @title get.dag.scores
# @param dataset a valid dataset
# @param adj.matrix adjacency matrix of the initially valid edges
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @return scores: observed probabilities and prima facie scores
#
get.dag.scores <- function( dataset, adj.matrix, epos, eneg ) {

    ## Structure to save the prima facie scores.
    
    prima.facie.model <- array(-1, dim = c(ncol(dataset), ncol(dataset)));
    prima.facie.null  <- array(-1, dim = c(ncol(dataset), ncol(dataset)));

    ## Structure to save the observed and observed-joint
    ## probabilities.
    
    pair.count <- array(0, dim = c(ncol(dataset), ncol(dataset)));

    ## Compute the observed probabilities on the dataset.
    
    for (i in 1:ncol(dataset)) {
        for (j in 1:ncol(dataset)) {
            val1 = dataset[ ,i];
            val2 = dataset[ ,j];
            pair.count[i,j] = (t(val1) %*% val2);
        }
    }
    
    # minimum probability to be represented in the dataset
    minimum.prob = 0 #max(.Machine$double.eps,(1/nrow(dataset))/2)
    
    ## marginal.probs is an array with the marginal probabilities.
    
    marginal.probs <-
        array(as.matrix(diag(pair.count) / nrow(dataset)),
              dim = c(ncol(dataset), 1))
              
    ## joint.probs is an array with the joint observed probabilities.
    
    joint.probs <- as.matrix(pair.count / nrow(dataset))
    
    # apply the noise model to estimate the theoretical joint.probs
    for (p1 in 1:nrow(joint.probs)) {
        for (p2 in p1:ncol(joint.probs)) {
            estimated.prob = estimate.theoretical.probs(joint.probs[p1,p2],type="joint",prob1=marginal.probs[p1],
                                                        prob2=marginal.probs[p2],
                                                        min.prob=minimum.prob,epos=epos,eneg=eneg)
            joint.probs[p1,p2] = estimated.prob
            joint.probs[p2,p1] = estimated.prob
        }
    }
    
    
    # apply the noise model to estimate the theoretical marginal.probs
    marginal.probs = sapply(marginal.probs,FUN=function(x) { return(estimate.theoretical.probs(x,
                                                                     type="marginal",min.prob=minimum.prob,
                                                                     epos=epos,eneg=eneg)) })

    marginal.probs = array(marginal.probs, dim = c(length(marginal.probs), 1))
    
    # verify the probabilities to be corret
    res.probs = verify.constraints.probs(marginal.probs,joint.probs)
    marginal.probs = res.probs$marginal.probs
    joint.probs = res.probs$joint.probs
    
    ## Compute the prima facie scores based on the probability raising
    ## model.
    
    for (i in 1:nrow(prima.facie.model)) {
        for (j in 1:ncol(prima.facie.model)) {
            
            ## The scores are saved in the convention of the adjacency
            ## matrix, i.e., [i,j] means i is causing j  the diagonal
            ## (self cause) and the other invalid edges have not to be
            ## considered.
            
            if (adj.matrix[i, j] != 0) {
                
                ## Check if the connections from j to i and from i to
                ## j can be evaluated on this dataset.
                
                if (marginal.probs[i,1] > 0
                    && marginal.probs[i,1] < 1
                    && marginal.probs[j,1] > 0
                    && marginal.probs[j,1] < 1) {
                    
                    ## Check if the two events i and j are
                    ## distinguishable.
                    
                    if ((joint.probs[i, j] / marginal.probs[j,1]) < 1
                        || (joint.probs[i, j] / marginal.probs[i,1]) < 1) {
                        
                        ## prima facie scores of i --> j
                        
                        prima.facie.model[i, j] =
                            joint.probs[j, i] / marginal.probs[i,1];
                        prima.facie.null[i, j] =
                            (marginal.probs[j,1] - joint.probs[j, i]) / (1 - marginal.probs[i,1]);
                    }
                }
            }
        }
    }

    ## Save the results and return them.
    scores =
        list(marginal.probs = marginal.probs,
             joint.probs = joint.probs,
             prima.facie.model = prima.facie.model,
             prima.facie.null = prima.facie.null);
    return(scores);
}

# apply the noise model to estimate the either the theoretical marginal or joint probability
estimate.theoretical.probs = function( observed.prob, type, prob1 = NA, prob2 = NA, min.prob = 0, epos, eneg ) {
    
    estimated.prob = NA
    
    # estimate the probability
    if(type=="marginal") {
        
        estimated.prob = (observed.prob - epos) / (1 - epos - eneg)
    
        # set a minimum/maximum value for the marginal probabilities in [min.prob,(1-min.prob)]
        if(estimated.prob<min.prob) {
            estimated.prob = min.prob
        }
        if(estimated.prob>(1-min.prob)) {
            estimated.prob = (1-min.prob)
        }
    
    }
    else if(type=="joint") {
        
        estimated.prob = (observed.prob - epos * (prob1 + prob2 - epos)) / ((1 - epos - eneg)^2)
    
        # set a minimum/maximum value for the joint probabilities in [0,(1-min.prob)]
        if(estimated.prob<0) {
            estimated.prob = 0
        }
        if(estimated.prob>(1-min.prob)) {
            estimated.prob = (1-min.prob)
        }
        
    }
    
    return(estimated.prob)
    
}


# select the best set of prima facie causes per node
# @title get.prima.facie.causes.do.boot
# @param adj.matrix adjacency matrix of the initially valid edges
# @param hypotheses hypotheses to be considered
# @param marginal.probs.distributions distributions of the bootstrapped marginal probabilities
# @param prima.facie.model.distributions distributions of the prima facie model
# @param prima.facie.null.distributions distributions of the prima facie null
# @param pvalue minimum pvalue for the Mann-Whitney U tests to be significant
# @param dataset a valid dataset
# @param marginal.probs observed marginal probabilities
# @param joint.probs observed joint probabilities
# @param silent Should I be verbose?
# @return prima.facie.topology: list describing the topology of the prima facie causes
#
get.prima.facie.causes.do.boot <- function(adj.matrix,
                                           hypotheses,
                                           marginal.probs.distributions,
                                           prima.facie.model.distributions,
                                           prima.facie.null.distributions,
                                           pvalue,
                                           dataset,
                                           marginal.probs,
                                           joint.probs,
                                           silent = FALSE) {

    ## Structure to save the confidence of the edges.
    
    edge.confidence.matrix <- array(list(), c(3, 1));
    edge.confidence.matrix[[1,1]] =
        array(NA, c(ncol(prima.facie.model.distributions),
                    ncol(prima.facie.model.distributions)));
    edge.confidence.matrix[[2,1]] =
        array(NA, c(ncol(prima.facie.model.distributions),
                    ncol(prima.facie.model.distributions)));
    edge.confidence.matrix[[3,1]] =
        array(NA, c(ncol(prima.facie.model.distributions),
                    ncol(prima.facie.model.distributions)));

    ## Verify Suppes' conditions for prima facie causes;
    ## i.e., i --> j implies P(i)>P(j) (temporal priority) and
    ## P(j|i)>P(j|not i) (probability raising).
    
    ## Verify the temporal priority condition.
    
    if (!silent)
        cat(paste0('\tEvaluating \"temporal priority\" (Wilcoxon, p-value ',
                   pvalue,
                   ')\n'));
    temporal.priority =
        verify.temporal.priority.do.boot(marginal.probs.distributions,
                                         pvalue,adj.matrix,
                                         edge.confidence.matrix);

    ## Verify the probability raising condition.
    
    if (!silent)
        cat(paste0('\tEvaluating \"probability raising\" (Wilcoxon, p-value ',
                   pvalue,
                   ')\n'));
    probability.raising =
        verify.probability.raising.do.boot(prima.facie.model.distributions,
                                           prima.facie.null.distributions,
                                           pvalue,temporal.priority$adj.matrix,temporal.priority$edge.confidence.matrix);

    ## Perform the hypergeometric test for each pair of events.
    
    for (i in 1:ncol(adj.matrix)) {
        for (j in i:nrow(adj.matrix)) {

            ## The diagonal (self cause) and the other invalid edges
            ## have not to be considered.
            
            if (adj.matrix[i, j] != 0 || adj.matrix[j, i] != 0) {
                
                ## Compute the confidence by hypergeometric test for
                ## both j --> i and i --> j.
                
                probability.raising$edge.confidence.matrix[[3, 1]][i, j] =
                    phyper(joint.probs[i, j] * nrow(dataset),
                           marginal.probs[i] * nrow(dataset),
                           nrow(dataset) - marginal.probs[i] * nrow(dataset),
                           marginal.probs[j] * nrow(dataset),
                           lower.tail = FALSE);
                probability.raising$edge.confidence.matrix[[3, 1]][j, i] =
                    probability.raising$edge.confidence.matrix[[3, 1]][i, j];
            } else {
                probability.raising$edge.confidence.matrix[[3, 1]][i, j] = 1;
                probability.raising$edge.confidence.matrix[[3, 1]][j, i] = 1;
            }
        }
    }

    ## Remove any cycle.
    
    adj.matrix.cyclic = probability.raising$adj.matrix
    if (length(temporal.priority$not.ordered) > 0
        || !is.na(hypotheses[1])) {

        if (!silent)
            cat('*** Loop detection found loops to break.\n')

        weights.temporal.priority =
            probability.raising$edge.confidence.matrix[[1, 1]] +
                probability.raising$edge.confidence.matrix[[2, 1]];
        weights.matrix =
            probability.raising$edge.confidence.matrix[[2, 1]] +
                probability.raising$edge.confidence.matrix[[3, 1]];
        acyclic.topology =
            remove.cycles(probability.raising$adj.matrix,
                          weights.temporal.priority,
                          weights.matrix,
                          temporal.priority$not.ordered,
                          hypotheses,
                          silent);
        adj.matrix.acyclic = acyclic.topology$adj.matrix;
    } else {
        adj.matrix.acyclic = probability.raising$adj.matrix;
    }
    adj.matrix =
        list(adj.matrix.cyclic.tp = temporal.priority$adj.matrix,
             adj.matrix.cyclic = adj.matrix.cyclic,
             adj.matrix.acyclic = adj.matrix.acyclic)

    ## Save the results and return them.

    
    prima.facie.topology =
        list(adj.matrix = adj.matrix,
             edge.confidence.matrix = probability.raising$edge.confidence.matrix);
    return(prima.facie.topology);
}


# select the best set of prima facie causes per node without bootstrap
# @title get.prima.facie.causes.no.boot
# @param adj.matrix adjacency matrix of the initially valid edges
# @param hypotheses hypotheses object related to adjacency matrix
# @param marginal.probs observed marginal probabilities
# @param prima.facie.model prima facie model
# @param prima.facie.null prima facie null
# @param dataset a valid dataset
# @param joint.probs observed joint probabilities
# @param silent Should I be verbose?
# @return prima.facie.topology: adjacency matrix of the prima facie causes
#
get.prima.facie.causes.no.boot <- function(adj.matrix,
                                           hypotheses,
                                           marginal.probs,
                                           prima.facie.model,
                                           prima.facie.null,
                                           dataset,
                                           joint.probs,
                                           silent = FALSE) {

    ## Structure to save the confidence of the edges.
    
    edge.confidence.matrix <- array(list(), c(3, 1));
    edge.confidence.matrix[[1, 1]] = array(NA, c(ncol(adj.matrix), ncol(adj.matrix)));
    edge.confidence.matrix[[2, 1]] = array(NA, c(ncol(adj.matrix), ncol(adj.matrix)));
    edge.confidence.matrix[[3, 1]] = array(NA, c(ncol(adj.matrix), ncol(adj.matrix)));

    ## Verify Suppes' conditions for prima facie causes;
    ## i.e., i --> j implies P(i)>P(j) (temporal priority) and
    ## P(j|i)>P(j|not i) (probability raising).
    
    ## Verify the temporal priority condition.
    
    if (!silent)
        cat(paste0('\tEvaluating \"temporal priority\".\n'));
    temporal.priority =
        verify.temporal.priority.no.boot(marginal.probs,
                                         adj.matrix,
                                         edge.confidence.matrix);

    ## Verify the probability raising condition.
    
    if (!silent)
        cat(paste0('\tEvaluating \"probability raising\".\n'));
    probability.raising =
        verify.probability.raising.no.boot(prima.facie.model,
                                           prima.facie.null,
                                           temporal.priority$adj.matrix,temporal.priority$edge.confidence.matrix);

    ## Perform the hypergeometric test for each pair of events.
    
    for (i in 1:ncol(adj.matrix)) {
        for (j in i:nrow(adj.matrix)) {

            ## The diagonal (self cause) and the other invalid edges
            ## have not to be considered.
            
            if (adj.matrix[i, j] != 0
                || adj.matrix[j, i] != 0) {
                
                ## Compute the confidence by hypergeometric test for
                ## both j --> i and i --> j.
                
                probability.raising$edge.confidence.matrix[[3, 1]][i, j] =
                    phyper(joint.probs[i,j] * nrow(dataset),
                           marginal.probs[i] * nrow(dataset),
                           nrow(dataset) - marginal.probs[i] * nrow(dataset),
                           marginal.probs[j] * nrow(dataset),lower.tail = FALSE);

                probability.raising$edge.confidence.matrix[[3, 1]][j, i] =
                    probability.raising$edge.confidence.matrix[[3,1]][i,j];
            } else {
                probability.raising$edge.confidence.matrix[[3, 1]][i, j] = 1;
                probability.raising$edge.confidence.matrix[[3, 1]][j, i] = 1;
            }
        }
    }

    ## Remove any cycle.
    
    adj.matrix.cyclic = probability.raising$adj.matrix
    if (length(temporal.priority$not.ordered) > 0
        || !is.na(hypotheses[1])) {

        if (!silent)
            cat('*** Loop detection found loops to break.\n')

        weights.temporal.priority =
            probability.raising$edge.confidence.matrix[[2, 1]];
        weights.matrix =
            probability.raising$edge.confidence.matrix[[2, 1]] +
                probability.raising$edge.confidence.matrix[[3, 1]];
        acyclic.topology =
            remove.cycles(probability.raising$adj.matrix,
                          weights.temporal.priority,
                          weights.matrix,
                          temporal.priority$not.ordered,
                          hypotheses,
                          silent);
        adj.matrix.acyclic = acyclic.topology$adj.matrix;
    } else {
        adj.matrix.acyclic = probability.raising$adj.matrix;
    }
    adj.matrix =
        list(adj.matrix.cyclic.tp = temporal.priority$adj.matrix,
             adj.matrix.cyclic = adj.matrix.cyclic,
             adj.matrix.acyclic = adj.matrix.acyclic)

    ## Save the results and return them.
    
    prima.facie.topology <-
        list(adj.matrix = adj.matrix,
             edge.confidence.matrix = probability.raising$edge.confidence.matrix);
    return(prima.facie.topology);
}


# select the set of the prima facie parents (with bootstrap) for each node
# based on Suppes' definition of causation
# @title get.prima.facie.parents.do.boot
# @param dataset a valid dataset
# @param hypotheses hypotheses object related to dataset
# @param nboot integer number (greater than 0) of bootstrap sampling to be performed
# @param pvalue pvalue for the tests (value between 0 and 1)
# @param adj.matrix adjacency matrix of the initially valid edges
# @param min.boot minimum number of bootstrapping to be performed
# @param min.stat should I keep bootstrapping untill I have nboot valid values?
# @param boot.seed seed to be used for the sampling
# @param silent Should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @return prima.facie.parents list of the set (if any) of prima facie parents for each node
#
get.prima.facie.parents.do.boot <- function(dataset,
                                            hypotheses,
                                            nboot,
                                            pvalue,
                                            adj.matrix,
                                            min.boot,
                                            min.stat,
                                            boot.seed,
                                            silent,
                                            epos,
                                            eneg ) {

    ## Perform a robust estimation of the scores using rejection
    ## sampling bootstrap.
    
    scores =
        get.bootstrapped.scores(dataset,
                               nboot,
                               adj.matrix,
                               min.boot,
                               min.stat,
                               boot.seed,
                               silent,
                               epos,
                               eneg);

    ## Compute the observed and joint probabilities as the mean of the
    ## bootstrapped values.
    
    marginal.probs = array(-1, dim = c(ncol(dataset), 1));
    joint.probs = array(-1, dim = c(ncol(dataset), ncol(dataset)));
    for (i in 1:ncol(dataset)) {
        marginal.probs[i, 1] =
            mean(unlist(scores$marginal.probs.distributions[i, 1]));
        for (j in i:ncol(dataset)) {
            joint.probs[i,j] =
                mean(unlist(scores$joint.probs.distributions[i, j]));
            if (i != j) {
                joint.probs[j, i] = joint.probs[i, j];
            }
        }
    }

    ## Remove all the edges not representing a prima facie cause.
    prima.facie.topology =
        get.prima.facie.causes.do.boot(adj.matrix,
                                       hypotheses,
                                       scores$marginal.probs.distributions,
                                       scores$prima.facie.model.distributions,
                                       scores$prima.facie.null.distributions,
                                       pvalue,
                                       dataset,
                                       marginal.probs,joint.probs,
                                       silent);

    # remove from adj.matrix.cyclic.tp any edge between events where P(i,j) = 0
    for (i in 1:nrow(prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp)) {
        for (j in i:ncol(prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp)) {
            if(joint.probs[i,j] == 0) {
                prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp[i,j] = 0
                prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp[j,i] = 0
            }
        }
    }

    ## Save the results and return them.
    
    prima.facie.parents <-
        list(marginal.probs = marginal.probs,
             joint.probs = joint.probs,
             adj.matrix = prima.facie.topology$adj.matrix,
             pf.confidence = prima.facie.topology$edge.confidence.matrix);
    return(prima.facie.parents);
}


# select the set of the prima facie parents (without bootstrap) for each node based on Suppes' definition of causation
# @title get.prima.facie.parents.no.boot
# @param dataset a valid dataset
# @param hypotheses hypotheses object associated to dataset
# @param adj.matrix adjacency matrix of the initially valid edges
# @param silent Should I be verbose?
# @param epos error rate of false positive errors
# @param eneg error rate of false negative errors
# @return prima.facie.parents: list of the set (if any) of prima facie parents for each node
#
get.prima.facie.parents.no.boot <- function(dataset,
                                            hypotheses,
                                            adj.matrix,
                                            silent,
                                            epos,
                                            eneg) {

    ## Compute the scores from the dataset.
    
    scores = get.dag.scores(dataset,adj.matrix,epos,eneg);

    ## Remove all the edges not representing a prima facie causes.
    
    prima.facie.topology =
        get.prima.facie.causes.no.boot(adj.matrix,
                                       hypotheses,
                                       scores$marginal.probs,
                                       scores$prima.facie.model,
                                       scores$prima.facie.null,
                                       dataset,
                                       scores$joint.probs,
                                       silent);

    # remove from adj.matrix.cyclic.tp any edge between events where P(i,j) = 0
    for (i in 1:nrow(prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp)) {
        for (j in i:ncol(prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp)) {
            if(scores$joint.probs[i,j] == 0) {
                prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp[i,j] = 0
                prima.facie.topology$adj.matrix$adj.matrix.cyclic.tp[j,i] = 0
            }
        }
    }

    ## Save the results return them.
    
    prima.facie.parents <-
        list(marginal.probs = scores$marginal.probs,
             joint.probs = scores$joint.probs,
             adj.matrix = prima.facie.topology$adj.matrix,
             pf.confidence = prima.facie.topology$edge.confidence.matrix);
    return(prima.facie.parents);
}


# reconstruct the best causal topology by likelihood fit
# @title perform.likelihood.fit.capri
# @param dataset a valid dataset
# @param adj.matrix the adjacency matrix of the prima facie causes
# @param command type of search, either hill climbing (hc) or tabu (tabu)
# @param regularization regularization term to be used in the likelihood fit
# @return topology: the adjacency matrix of both the prima facie and causal topologies
#
perform.likelihood.fit.capri <- function(dataset,
                                         adj.matrix,
                                         command,
                                         regularization,
                                         restart) {

    ## Adjacency matrix of the topology reconstructed by likelihood
    ## fit.
    adj.matrix.fit = array(0, c(nrow(adj.matrix), ncol(adj.matrix)))
    rownames(adj.matrix.fit) = colnames(dataset)
    colnames(adj.matrix.fit) = colnames(dataset)

    ## Create a categorical data frame from the dataset.
    data = as.categorical.dataset(dataset)

    # Perform the likelihood fit
    adj.matrix.fit = lregfit(data,
        adj.matrix,
        adj.matrix.fit,
        regularization,
        command,
        restart)

    ## Save the results and return them.
    
    adj.matrix =
        list(adj.matrix.pf = adj.matrix,
             adj.matrix.fit = adj.matrix.fit);
    topology = list(adj.matrix = adj.matrix);
    return(topology);
}


# remove any cycle from a given cyclic topology
# @title remove.cycles
# @param adj.matrix adjacency matrix of the topology
# @param weights.temporal.priority weighted matrix to be used to remove the cycles involving atomic events
# @param weights.matrix weighted matrix to be used to remove the cycles involving hypotheses
# @param not.ordered list of the nodes to be orderd
# @param hypotheses hypotheses to evaluate potential cycles
# @param silent Should I be verbose?
# @return acyclic.topology: structure representing the best acyclic topology
#
remove.cycles <- function(adj.matrix,
                          weights.temporal.priority,
                          weights.matrix,
                          not.ordered,
                          hypotheses = NA,
                          silent) {

    
    total.edges = length(which(adj.matrix == 1))
    removed = 0

    ## Evaluate the possible cycles involving atomic events.

    if (length(not.ordered) > 0) {

        ## Consider only the edges that were not ordered by temporal
        ## priority.
        
        curr.edge.pos = 0;
        for (i in 1:length(not.ordered)) {

            ## Consider the events i and j.
            
            curr.edge = not.ordered[[i]]
            curr.edge.i = curr.edge[1, 1]
            curr.edge.j = curr.edge[2, 1]

            ## check if i and j still create a cycle.
            
            if (adj.matrix[curr.edge.i, curr.edge.j] == 1
                && adj.matrix[curr.edge.j, curr.edge.i] == 1) {

                ## Get the scores of the two edges.
                
                curr.score.i.j =
                    weights.temporal.priority[curr.edge.i, curr.edge.j]
                curr.score.j.i =
                    weights.temporal.priority[curr.edge.j, curr.edge.i]

                ## Choose an edge based on the score.
                
                if (curr.score.i.j < curr.score.j.i) {
                    
                    ## if i --> j is more confident (lower score) then
                    ## j --> i
                    
                    removed = removed + 1
                    adj.matrix[curr.edge.j, curr.edge.i] = 0
                } else {
                    ## otherwise
                    removed = removed + 1
                    adj.matrix[curr.edge.i, curr.edge.j] = 0
                }
            }
        }
    }

    ## Create the structures where to save the weights in increasing
    ## order of confidence.
    ordered.weights <- vector();
    ordered.edges <- list();

    ## Select the edges to be evaluated during
    ## the loop removal.   
    curr.edge.pos = 0;
    for (i in 1:nrow(adj.matrix)) {
        for (j in 1:nrow(adj.matrix)) {
            if (adj.matrix[i, j] == 1) {
                ordered.weights =
                    rbind(ordered.weights, weights.matrix[i, j]);
                curr.edge.pos = curr.edge.pos + 1;
                new.edge <- array(0, c(2, 1));
                new.edge[1, 1] = i;
                new.edge[2, 1] = j;
                ordered.edges[curr.edge.pos] = list(new.edge);
            }
        }
    }

    ## Sort the edges in increasing order of confidence (i.e. the
    ## edges with lower pvalue are the most confident).
    ordered.weights = sort(unlist(ordered.weights), 
        decreasing = TRUE,
        index.return = TRUE)
    ordered.edges = ordered.edges[ordered.weights$ix]

    ## Consider the patterns related the hypotheses.

    consider.hypotheses = FALSE
    if (!is.na(hypotheses[1])) {
        consider.hypotheses = TRUE
    }


    ## Visit the ordered edges and remove the ones that are causing
    ## any cycle.
    
    if (length(ordered.edges) > 0) {

        ## Expanded matrix to be considered in removing the loops
        ## if hypotheses are present.
       
        if (consider.hypotheses) {
            expansion = hypotheses.expansion(input_matrix = adj.matrix,
                map=hypotheses$hstructure,
                expand = TRUE,
                skip.disconnected = FALSE)
            curr.adj.matrix = expansion[[1]]
            hypos.new.name = expansion[[2]]
        } else {
            curr.adj.matrix = adj.matrix
        }


        for (i in 1:length(ordered.edges)) {

            ## Consider the edge i-->j
            curr.edge = ordered.edges[[i]]
            curr.edge.i = curr.edge[1,1]
            curr.edge.j = curr.edge[2,1]

            ## Resolve the mapping from the adj.matrix to the expanded
            ## one both for curr.edge.i and curr.edge.j
            
            if (consider.hypotheses && colnames(adj.matrix)[curr.edge.i] %in% hypos.new.name) {
                curr.edge.i.exp = which(colnames(curr.adj.matrix) %in% names(hypos.new.name)[
                    which(hypos.new.name %in% colnames(adj.matrix)[curr.edge.i])
                ])
            } else {
                curr.edge.i.exp = which(
                    colnames(curr.adj.matrix) %in% colnames(adj.matrix)[curr.edge.i]
                )
            }
            
            if (consider.hypotheses && colnames(adj.matrix)[curr.edge.j] %in% hypos.new.name) {
                curr.edge.j.exp = which(colnames(curr.adj.matrix) %in% names(hypos.new.name)[
                    which(hypos.new.name %in% colnames(adj.matrix)[curr.edge.j])
                ])
            } else {
                curr.edge.j.exp = which(
                    colnames(curr.adj.matrix) %in% colnames(adj.matrix)[curr.edge.j]
                )
            }

            ## Search for loops between curr.edge.i and curr.edge.j
            
            curr.graph = graph.adjacency(curr.adj.matrix, mode = "directed")

            is.path = suppressWarnings(get.shortest.paths(curr.graph,
                                       curr.edge.j.exp,
                                       curr.edge.i.exp)$vpath)

            is.path = length(unlist(is.path))

            ## If there is a path between the two nodes, remove edge i --> j
            
            if (is.path > 0) {
                removed = removed + 1

                ## cat("Removing edge ",colnames(adj.matrix)[curr.edge.i]," to ",colnames(adj.matrix)[curr.edge.j],"\n");

                curr.adj.matrix[curr.edge.i.exp, curr.edge.j.exp] = 0
                adj.matrix[curr.edge.i, curr.edge.j] = 0
            }
        }

        if (!silent)
            cat(paste0('\tRemoved ',
                       removed,
                       ' edges out of ',
                       total.edges,
                       ' (',
                       round(100 * removed/total.edges, 0),
                       '%)\n'))
    }

    ## Save the results and return them.
    
    acyclic.topology = list(adj.matrix = adj.matrix)
    return(acyclic.topology)
}


# verify the probability raising condition
# @title verify.probability.raising.do.boot
# @param prima.facie.model.distributions distributions of the prima facie model
# @param prima.facie.null.distributions distributions of the prima facie null
# @param pvalue minimum pvalue for the Mann-Whitney U tests to be significant
# @param adj.matrix adjacency matrix of the topology
# @param edge.confidence.matrix matrix of the confidence of each edge
# @return probability.raising: list describing the causes where probability raising is verified
#
verify.probability.raising.do.boot <- function(prima.facie.model.distributions,
                                               prima.facie.null.distributions,
                                               pvalue,
                                               adj.matrix,
                                               edge.confidence.matrix) {

    ## Evaluate the probability raising condition.
    
    for (i in 1:nrow(adj.matrix)) {
        for (j in i:ncol(adj.matrix)) {

            ## The diagonal (self cause) and the other invalid edges
            ## have not to be considered.
            
            if (adj.matrix[i, j] != 0
                || adj.matrix[j, i] != 0) {

                ## pvalue for the probability raising condition for i --> j

                second.pvalue.i.j = suppressWarnings(
                    wilcox.test(unlist(prima.facie.model.distributions[i, j]),
                                unlist(prima.facie.null.distributions[i,j]),
                                alternative = "greater",
                                mu = 0))$p.value;
                if (is.na(second.pvalue.i.j)
                    || is.nan(second.pvalue.i.j)) {

                    ## In this case the two distributions are exactly
                    ## identical.
                    
                    second.pvalue.i.j = 1;
                }

                ## In this case i --> j is not valid
                
                if (second.pvalue.i.j >= pvalue) {
                    adj.matrix[i, j] = 0;
                }

                ## pvalue for the probability raising condition for j --> i

                second.pvalue.j.i = suppressWarnings(
                    wilcox.test(unlist(prima.facie.model.distributions[j, i]),
                                unlist(prima.facie.null.distributions[j,i]),
                                alternative = "greater", mu = 0))$p.value;
                if (is.na(second.pvalue.j.i)
                    || is.nan(second.pvalue.j.i)) {

                    ## In this case the two distributions are exactly
                    ## identical.
                    
                    second.pvalue.j.i = 1;
                }

                ## In this case j --> i is not valid
                
                if (second.pvalue.j.i >= pvalue) {
                    adj.matrix[j, i] = 0;
                }

                ## Save the confidence for i--> j and j --> i

                tmp = edge.confidence.matrix[[2, 1]];
                tmp[i, j] = second.pvalue.i.j;
                tmp[j, i] = second.pvalue.j.i;
                edge.confidence.matrix[2, 1] = list(tmp);
            } else {
                tmp = edge.confidence.matrix[[2, 1]];
                tmp[i, j] = 1;
                tmp[j, i] = 1;
                edge.confidence.matrix[2, 1] = list(tmp);
            }
        }
    }

    ## Save the results and return them.
    
    probability.raising <-
        list(adj.matrix = adj.matrix,
             edge.confidence.matrix = edge.confidence.matrix);
    return(probability.raising);
}


# verify the probability raising condition without bootstrap
# @title verify.probability.raising.no.boot
# @param prima.facie.model prima facie model
# @param prima.facie.null prima facie null
# @param adj.matrix adjacency matrix of the topology
# @param edge.confidence.matrix matrix of the confidence of each edge
# @return probability.raising: adjacency matrix where temporal priority is verified

verify.probability.raising.no.boot <- function(prima.facie.model,
                                               prima.facie.null,
                                               adj.matrix,
                                               edge.confidence.matrix) {

    ## Evaluate the probability raising condition.
    
    for (i in 1:nrow(adj.matrix)) {
        for (j in i:ncol(adj.matrix)) {

            ## The diagonal (self cause) and the other invalid edges
            ## have not to be considered probability raising
            ## condition: if P(j|i)>P(j|not i) the edge i --> j is
            ## valid for probability raising.
            
            if (adj.matrix[i, j] != 0
                || adj.matrix[j, i] != 0) {

                ## In this case i --> j is not valid
                
                if (prima.facie.model[i, j] <= prima.facie.null[i, j]) {
                    adj.matrix[i, j] = 0;
                }

                ## In this case j --> i is not valid
                if (prima.facie.model[j, i] <= prima.facie.null[j, i]) {
                    adj.matrix[j, i] = 0;
                }

                ## Save the confidence for i-->j and j --> i

                tmp = edge.confidence.matrix[[2, 1]];
                tmp[i, j] = min(prima.facie.null[i, j] / prima.facie.model[i, j],1);
                tmp[j, i] = min(prima.facie.null[j, i] / prima.facie.model[j, i],1);
                edge.confidence.matrix[2, 1] = list(tmp);
            } else {
                tmp = edge.confidence.matrix[[2, 1]];
                tmp[i, j] = 1;
                tmp[j, i] = 1;
                edge.confidence.matrix[2, 1] = list(tmp);
            }
        }
    }

    ## Save the results and return them.
    
    probability.raising <-
        list(adj.matrix = adj.matrix,
             edge.confidence.matrix = edge.confidence.matrix);
    return(probability.raising);
}


# verify the temporal priority condition with bootstrap
# @title verify.temporal.priority.do.boot
# @param marginal.probs.distributions distributions of the bootstrapped marginal probabilities
# @param pvalue minimum pvalue for the Mann-Whitney U tests to be significant
# @param adj.matrix adjacency matrix of the topology
# @param edge.confidence.matrix matrix of the confidence of each edge
# @return temporal.priority: list describing the causes where temporal priority is verified
#
verify.temporal.priority.do.boot <- function(marginal.probs.distributions,
                                             pvalue,
                                             adj.matrix,
                                             edge.confidence.matrix) {

    ## Evalutate the temporal priority condition for each pair of
    ## edges.
    
    not.ordered = list();
    counter = 0;
    for (i in 1:nrow(adj.matrix)) {
        for (j in i:ncol(adj.matrix)) {

            ## The diagonal (self cause) and the other invalid edges
            ## have not to be considered.
            
            if (adj.matrix[i, j] != 0
                || adj.matrix[j, i] != 0) {

                ## [i,j] refers to causation i --> j
                ## Temporal priority condition: if P(i) > P(j) the edge
                ## i --> j is valid for temporal priority.
                
            
                ## Test i --> j
                first.pvalue.i.j = suppressWarnings(
                    wilcox.test(unlist(marginal.probs.distributions[i, 1]),
                                unlist(marginal.probs.distributions[j, 1]),
                                alternative = "greater",
                                mu = 0))$p.value;
                if (is.na(first.pvalue.i.j)
                    || is.nan(first.pvalue.i.j)) {

                    ## In this case the two distributions are exactly
                    ## identical.
                    
                    first.pvalue.i.j = 1;
                }

                ## Test j --> i
                
                first.pvalue.j.i = suppressWarnings(
                    wilcox.test(unlist(marginal.probs.distributions[j, 1]),
                                unlist(marginal.probs.distributions[i, 1]),
                                alternative = "greater",
                                mu = 0))$p.value;
                if (is.na(first.pvalue.j.i)
                    || is.nan(first.pvalue.j.i)) {
                    
                    ## In this case the two distributions are exactly
                    ## identical.
                    
                    first.pvalue.j.i = 1;
                }

                ## In this case i is before j and j --> i is not valid.

                if (first.pvalue.j.i >= pvalue
                    && first.pvalue.i.j < pvalue) {

                    ## [j,i] = 0 means j is after i, i.e. it can not be causing i
                    adj.matrix[j,i] = 0;
                }

                ## In this case j is before i and i --> j is not valid.

                else if (first.pvalue.j.i < pvalue
                         && first.pvalue.i.j >= pvalue) {
                    ## [i,j] = 0 means i is after j, i.e. it can not be causing j
                    adj.matrix[i,j] = 0;
                }
                
                ## In this case, a total time order between i and j
                ## can not be defined.
                
                else {
                    
                    ## No temporal priority induced by the topology
                    ## can be inferred.
                    
                    counter = counter + 1;
                    curr.not.ordered = array(-1, c(2, 1));
                    curr.not.ordered[1, 1] = i;
                    curr.not.ordered[2, 1] = j;
                    not.ordered[counter] = list(curr.not.ordered);
                }

                ## Save the confidence for i --> j and j --> i

                tmp = edge.confidence.matrix[[1, 1]];
                tmp[i, j] = first.pvalue.i.j;
                tmp[j, i] = first.pvalue.j.i;
                edge.confidence.matrix[1, 1] = list(tmp);
            } else {
                tmp = edge.confidence.matrix[[1, 1]];
                tmp[i, j] = 1;
                tmp[j, i] = 1;
                edge.confidence.matrix[1, 1] = list(tmp);
            }
        }
    }

    ## Save the results and return them.
    
    temporal.priority <-
        list(adj.matrix = adj.matrix,
             edge.confidence.matrix = edge.confidence.matrix,
             not.ordered = not.ordered);
    return(temporal.priority);
}


# verify the temporal priority condition without bootstrap
# @title verify.temporal.priority.no.boot
# @param marginal.probs marginal probabilities
# @param adj.matrix adjacency matrix of the topology
# @param edge.confidence.matrix matrix of the confidence of each edge
# @return temporal.priority: adjacency matrix where temporal priority is verified
#
verify.temporal.priority.no.boot <- function(marginal.probs,
                                             adj.matrix,
                                             edge.confidence.matrix) {

    ## Evalutate the temporal priority condition for each pair of
    ## edges.
    
    not.ordered = list();
    counter = 0;
    for (i in 1:nrow(adj.matrix)) {
        for (j in i:ncol(adj.matrix)) {

            ## The diagonal (self cause) and the other invalid edges
            ## have not to be considered.
            
            if (adj.matrix[i,j] != 0
                || adj.matrix[j, i] != 0) {

                ## [i,j] refers to causation i --> j

                ## Temporal priority condition: if P(i)>P(j) the edge
                ## i --> j is valid for temporal priority in this case
                ## i is before j and j --> i is not valid.
                
                if (marginal.probs[i, 1] > marginal.probs[j, 1]) {
                    ## [j,i] = 0 means j is after i, i.e. it can not be causing i

                    adj.matrix[j, i] = 0;
                }

                ## In this case j is before i and i --> j is not valid
                
                else if (marginal.probs[j, 1] > marginal.probs[i, 1]) {
                    ## [i,j] = 0 means i is after j, i.e. it can not be causing j

                    adj.matrix[i,j] = 0;
                }
                
                ## In this case, a total time order between i and j
                ## can not be defined.
                
                else {
                    
                    ## No temporal priority induced by the topology
                    ## can be inferred.
                    
                    counter = counter + 1;
                    curr.not.ordered = array(-1, c(2, 1));
                    curr.not.ordered[1, 1] = i;
                    curr.not.ordered[2, 1] = j;
                    not.ordered[counter] = list(curr.not.ordered);
                }

                ## Save the confidence for i --> j and j --> i

                tmp = edge.confidence.matrix[[1, 1]];
                tmp[i, j] = min(marginal.probs[j, 1] / marginal.probs[i, 1], 1);
                tmp[j, i] = min(marginal.probs[i,1]/marginal.probs[j,1],1);
                edge.confidence.matrix[1, 1] = list(tmp);

            } else {
                tmp = edge.confidence.matrix[[1, 1]];
                tmp[i, j] = 1;
                tmp[j, i] = 1;
                edge.confidence.matrix[1, 1] = list(tmp);
            }
        }
    }

    ## Save the results and return them.
    
    temporal.priority <-
        list(adj.matrix = adj.matrix,
             edge.confidence.matrix = edge.confidence.matrix,
             not.ordered = not.ordered);
    return(temporal.priority);
}


#### end of file -- capri.algorithm.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


# Internal function

#' Verify if the input data are consolidate, i.e., if there are events
#' with 0 or 1 probability or indistinguishable in terms of
#' observations
#' 
#' @title consolidate.data
#' 
#' @examples
#' data(test_dataset)
#' consolidate.data(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @param print A boolean value stating whether to print of not the summary
#' @return The list of any 0 probability, 1 probability and indistinguishable.
#' @export consolidate.data
#' 
consolidate.data <- function(x, print = FALSE) {

    is.compliant(x)

    ind = list()
    zeros = list()
    ones = list()

    ## Get the list of indistinguishible events for each entry.
    duplicated.names =
        unlist(lapply(colnames(x$genotypes),
                      function(gene)
                          paste(as.vector(which(apply(t(x$genotypes),
                                                      1,
                                                      function(y)
                                                          all(y == as.vector(x$genotypes[ , gene]))))),
                                collapse = "-")))

    ## Get the unique set of indistinguishible events
    duplicated.events =
        unique(duplicated.names[which(unlist(lapply(duplicated.names,
                                                    ## The next
                                                    ## deserves to be
                                                    ## sent to dailywtf!
                                                    function(x)
                                                        if (length(unlist(strsplit(x, "-"))) > 1)
                                                            TRUE
                                                        else
                                                            FALSE)))])

    ## Evaluate the indistinguishible events.
    if (length(duplicated.events) > 0) {
        for (i in duplicated.events) {
            ev = as.numeric(unlist(strsplit(i, "-")))

            ind.pool = as.events(x)[ev,]

            if (any(x$genotypes[, ev[1]] != 0) && any(x$genotypes[, ev[1]] != 1)) {
                ind = append(ind, list(ind.pool))
            }

            if (print) {
                cat('\nIndistinguishable events:\n')

                print(ind.pool)
                cat('Total number of events for these genes: ')
                cat(paste(nevents(x, as.events(x, genes=c(ind.pool)))), '\n')
            }
        }
    }

    ## Evaluate any event at 0 or 1 probability.

    for (i in 1:ncol(x$genotypes)) {

        ev.pool = as.events(x)[i,]

        if (sum(x$genotypes[, i]) == 0) {
            zeros = append(zeros, list(ev.pool))
            if (print) {

                cat('\nIndistinguishable events:\n')

                print(ev.pool)
                cat('Total number of events for these genes: ')
                cat(paste(nevents(x, as.events(x, genes = c(ev.pool)))), '\n')

            }
        }
        if (sum(x$genotypes[, i]) == nrow(x$genotypes)) {
            ones = append(ones, list(ev.pool))
            if (print) {
                cat('\nEvents with no alterations across samples:\n')

                print(ev.pool)
                cat('Total number of events for these genes: ')
                cat(paste(nevents(x, as.events(x, genes=c(ev.pool)))), '\n')
            }
        }
    }

    ret = NULL
    ret$indistinguishable = ind
    ret$zeroes = zeros
    ret$ones = ones

    return(ret)
}

#' Annotate a description on the selected dataset
#' @title annotate.description
#'
#' @examples
#' data(test_dataset)
#' annotate.description(test_dataset, 'new description')
#'
#' @param x A TRONCO compliant dataset.
#' @param label A string
#' @return A TRONCO compliant dataset.
#' @export annotate.description
#' 
annotate.description <- function(x, label) {
    if (as.description(x) != "")
        warning(paste('Old description substituted: ', as.description(x), '.'))

    x$name = label
    return(x)
}


#' Annotate stage information on the selected dataset
#' @title annotate.stages
#'
#' @examples
#' data(test_dataset)
#' data(stage)
#' test_dataset = annotate.stages(test_dataset, stage)
#' as.stages(test_dataset)
#' 
#' @param x A TRONCO compliant dataset.
#' @param stages A list of stages. Rownames must match samples list of x
#' @param match.TCGA.patients Match using TCGA notations (only first 12 characters)
#' @return A TRONCO compliant dataset.
#' @export annotate.stages
#' 
annotate.stages <- function(x, stages, match.TCGA.patients = FALSE) {
    if (is.null(rownames(stages))) {
        stop('Stages have no rownames - will not add annotation.')
    }

    samples = as.samples(x)

    ## Just for temporary - will be shortened to make a simple check...
    
    samples.temporary = samples
    if (match.TCGA.patients) {
        samples.temporary = substring(samples.temporary, 0, 12)
    }

    if (!any(samples.temporary %in% rownames(stages)))
        stop('There are no stages for samples in input dataset - will not add annotation.')

    ## Notify if something gets lost.
    
    if (has.stages(x)) {
        warning('Stages in input dataset overwritten.')
    }

    ## Actual stages.
    
    x$stages = data.frame(row.names = samples, stringsAsFactors = FALSE)
    x$stages[ , 'stage'] = as.character(NA)

    for (i in 1:nsamples(x)) {
        if (!match.TCGA.patients) {
            x$stages[i, ] = as.character(stages[as.samples(x)[i], ])
        } else {
            
            ## Potential match if x's samples are long TCGA barcodes
            ## and stages are TCGA patients barcdodes (short).
            
            short.name = substring(as.samples(x)[i], 0 , 12)
            x$stages[i, 'stage'] = as.character(stages[short.name, ])
        }
    }

    count.na = is.na(x$stages)
    if (any(count.na)) {
        warning(paste(length(which(count.na)),
                      ' missing stages were added as NA.'))
    }

    return(x)
}


#' Change the color of an event type
#' @title change.color
#'
#' @examples
#' data(test_dataset)
#' dataset = change.color(test_dataset, 'ins_del', 'red')
#'
#' @param x A TRONCO compliant dataset.
#' @param type An event type
#' @param new.color The new color (either HEX or R Color)
#' @return A TRONCO complian dataset.
#' @export change.color
#' 
change.color <- function(x, type, new.color) {
    is.compliant(x)
    if (type %in% as.types(x)) {
        x$types[type, ] = new.color
    } else {
        stop('type: \"', type, '\" not in dataset')
    }

    is.compliant(x)
    return(x)
}


#' Rename an event type
#' @title rename.type
#'
#' @examples
#' data(test_dataset)
#' test_dataset = rename.type(test_dataset, 'ins_del', 'deletion')
#'
#' @param x A TRONCO compliant dataset.
#' @param old.name The type of event to rename.
#' @param new.name The new name
#' @return A TRONCO complian dataset.
#' @export rename.type
#' 
rename.type <- function(x, old.name, new.name) {
    is.compliant(x, 'rename.type: input dataset')
    types = as.types(x)

    if (old.name == 'Pattern'
        || new.name == 'Pattern') {
        stop("Pattern is a reserved keyword in TRONCO")
    }

    if (old.name == new.name) {
        return(x)
    }


    if (old.name %in% types) {
        x$annotations[which(x$annotations[ , 'type'] == old.name), 'type'] = new.name
        if (! new.name %in% types) {
            rownames(x$types)[which(rownames(x$types) == old.name)] = new.name
        } else {
            x$types =
                x$types[!rownames(x$types) %in% list(old.name), , drop = FALSE]
        }
    } else {
        stop(paste(old.name, 'not in as.types(x)'))
    }

    is.compliant(x, err.fun = 'rename.type: output')
    return(x)
}


#' Rename a gene
#' @title rename.gene
#'
#' @examples
#' data(test_dataset)
#' test_dataset = rename.gene(test_dataset, 'TET2', 'gene x')
#'
#' @param x A TRONCO compliant dataset.
#' @param old.name The name of the gene to rename.
#' @param new.name The new name
#' @return A TRONCO complian dataset.
#' @export rename.gene
#' 
rename.gene <- function(x, old.name, new.name) {
    ## If is compliant x
    is.compliant(x)

    if (old.name %in% as.genes(x)) {
        x$annotations[ which(x$annotations[,'event'] == old.name), 'event' ] = new.name

    } else {
        stop(paste(old.name, 'not in as.genes(x)'))
    }

    is.compliant(x)
    return(x)
}


#' Delete an event type
#' @title delete.type
#'
#' @examples
#' data(test_dataset)
#' test_dataset = delete.type(test_dataset, 'Pattern')
#'
#' @param x A TRONCO compliant dataset.
#' @param type The name of the type to delete.
#' @return A TRONCO complian dataset.
#' @export delete.type
#' 
delete.type <- function(x, type) {
    ## if is compliant x
    is.compliant(x)

    if (has.model(x)) {
        stop("There's a reconstructed model, a type cannot be deleted now. \nUse delete.model()")
    }

    for (pattern in as.patterns(x)) {
        if (type %in% as.types.in.patterns(x, patterns=pattern)) {
            stop('Found type \"',
                 type,
                 '\" in pattern \"',
                 pattern,
                 '\". Delete that pattern first.\n')
        }
    }

    if (type %in% as.types(x)) {
        events = as.events(x, types = setdiff(as.types(x), type))

        x$genotypes = as.genotypes(x)[ , rownames(events), drop = FALSE]
        x$annotations = events
        x$types = x$types[which(rownames(x$types) != type), , drop = FALSE]
    } else {
        stop(paste(type, 'not in as.types(x)'))
    }

    if (type == 'Pattern') {
        x$hypotheses = NA
    }

    is.compliant(x)
    return(x)
}


#' Delete a gene
#' @title delete.gene
#'
#' @examples
#' data(test_dataset)
#' test_dataset = delete.gene(test_dataset, 'TET2')
#'
#' @param x A TRONCO compliant dataset.
#' @param gene The name of the gene to delete.
#' @return A TRONCO complian dataset.
#' @export delete.gene
#' 
delete.gene <- function(x, gene) {
    ## if is compliant x
    is.compliant(x, 'delete:gene: input')

    if (has.model(x)) {
        stop("There's a reconstructed model, a type cannot be deleted now. \nUse delete.model()")
    }

    if (all(gene %in% as.genes(x))) {
        for (pattern in as.patterns(x)) {
            for (g in gene) {
                if (g %in% as.genes.in.patterns(x, patterns=pattern)) {
                    stop('Found gene \"',
                         g,
                         '\" in pattern \"',
                         pattern,
                         '\". Delete that pattern first.\n')
                }
            }
        }


        drops = rownames(as.events(x, genes = gene))
        x$genotypes = x$genotypes[ , -which(colnames(x$genotypes) %in% drops)]
        x$annotations = x$annotations[ -which(rownames(x$annotations) %in% drops), ]

        ## TO DO: something better than this t(t(...))
        x$types = x$types[which(rownames(x$types) %in% unique(x$annotations[,"type"])), , drop = FALSE]
        colnames(x$types) = 'color'
    } else {
        stop(paste(gene[!(gene %in% as.genes(x))], collapse= ','),
             'are not in the dataset -- as.genes(x).')
    }

    is.compliant(x, 'delete:gene: output')
    return(x)
}


#' Delete an event from the dataset
#' @title delete.event
#'
#' @examples
#' data(test_dataset)
#' test_dataset = delete.event(test_dataset, 'TET2', 'ins_del')
#'
#' @param x A TRONCO compliant dataset.
#' @param gene The name of the gene to delete.
#' @param type The name of the type to delete.
#' @return A TRONCO complian dataset.
#' @export delete.event
#' 
delete.event <- function(x, gene, type) {

    is.compliant(x, 'delete.event: input')

    if (has.model(x)) {
        stop("There's a reconstructed model, a type cannot be deleted now. \nUse delete.model()")
    }

    if (type == 'Pattern') {
        stop("Pattern is a reserved keyword in TRONCO. See delete.pattern")
    }

    for (pattern in as.patterns(x)) {
        events = as.events.in.patterns(x, patterns=pattern)
        if (length(which(events[,'type'] == type & events[,'event'] == gene)) > 0) {
            stop('Found event \"(',
                 gene,
                 ', ',
                 type,
                 ')\" in pattern \"',
                 pattern,
                 '\". Delete that pattern first.\n')
        }
    }


    if (all(c(type, gene) %in% as.events(x))) {
        drops = rownames(as.events(x, genes = gene, types = type))
        x$genotypes = x$genotypes[, -which( colnames(x$genotypes) %in% drops ), drop = FALSE]
        x$annotations = x$annotations[ -which (rownames(x$annotations) %in% drops), , drop = FALSE]

        ## TO DO: something better than this t(t(...))
        x$types = x$types[ which(rownames(x$types) %in% unique(x$annotations[,"type"])), , drop = FALSE]
    } else {
        stop(paste(type, gene, ' - not in as.events(x)'))
    }

    is.compliant(x, 'delete:gene: output')
    return(x)
}


#' Delete an hypothesis from the dataset based on a selected event.
#' Check if the selected event exist in the dataset and delete his associated hypothesis
#' @title delete.hypothesis
#'
#' @examples
#' data(test_dataset)
#' delete.hypothesis(test_dataset, event='TET2')
#' delete.hypothesis(test_dataset, cause='EZH2')
#' delete.hypothesis(test_dataset, event='XOR_EZH2')
#'
#' @param x A TRONCO compliant dataset.
#' @param event Can be an event or pattern name
#' @param cause Can be an event or pattern name
#' @param effect Can be an event or pattern name
#' @return A TRONCO complian dataset.
#' @export delete.hypothesis
#' 
delete.hypothesis <- function(x, event = NA, cause = NA, effect = NA) {

    is.compliant(x)
    if (has.model(x)) {
        stop("There's a reconstructed model, hypotheses cannot be deleted now.\nUse delete.model()")
    }

    hypo_map = as.hypotheses(x)
    to_remove = c()

    if (!any(is.na(event))) {
        if (length(event) == 1 && event %in% as.events(x)[ , 'event']) {
            cause_del = which(hypo_map[ , 'cause event'] == event)
            effect_del = which(hypo_map[ , 'effect event'] == event)
            to_remove = unique(c(to_remove, cause_del, effect_del))
        } else {
            stop('Select only one event present in as.events')
        }
    }

    if (! any(is.na(cause))) {
        if (all(cause %in% as.events(x)[ , 'event'])) {
            cause_del = which(hypo_map[ , 'cause event'] == cause)
            to_remove = unique(c(to_remove, cause_del))
        } else {
            stop('Wrong cause, select only events present in as.events')
        }
    }

    if (! any(is.na(effect))) {
        if (all(effect %in% as.events(x)[ , 'event'])) {
            effect_del = which(hypo_map[ , 'effect event'] == effect)
            to_remove = unique(c(to_remove, effect_del))
        } else {
            stop('Wrong effect, select only events present in as.events')
        }
    }

    if (length(to_remove) == 0) {
        warning("Nothing to remove")
        return(x)
    }

    x$hypotheses$num.hypotheses = x$hypotheses$num.hypotheses - 1
    x$hypotheses$hlist = x$hypotheses$hlist[-to_remove, ,drop=FALSE]

    is.compliant(x)

    return(x)
}


#' Delete a pattern and every associated hypotheses from the dataset
#' @title delete.pattern
#'
#' @examples
#' data(test_dataset)
#' delete.pattern(test_dataset, pattern='XOR_EZH2')
#'
#' @param x A TRONCO compliant dataset.
#' @param pattern A pattern name
#' @return A TRONCO complian dataset.
#' @export delete.pattern
#' 
delete.pattern <- function(x, pattern) {
    is.compliant(x)
    if (has.model(x)) {
        stop("There's a reconstructed model, a pattern cannot be deleted now. \nUse delete.model()")
    }

    if (! pattern %in% as.patterns(x)) {
        stop(paste(pattern, " not in as.patterns()"))
    }

    x = delete.hypothesis(x, pattern)
    x$annotations = x$annotations[-which(rownames(x$annotations) == pattern), , drop = FALSE]
    x$genotypes = x$genotypes[, -which(colnames(x$genotypes) == pattern), drop = FALSE]
    x$hypotheses$patterns[pattern] = NULL
    x$hypotheses$pvalues = NULL

    for (atom in names(x$hypotheses$atoms)) {
        if (x$hypotheses$atoms[atom] == pattern) {
            x$hypotheses$atoms[atom] = NULL
        }
    }

    x$hypotheses$hstructure[pattern] = NULL

    if (! 'Pattern' %in% unique(x$annotations[,'type'])) {
        x$types = x$types[-which(rownames(x$types) == 'Pattern'),,drop=FALSE]

    }

    is.compliant(x)

    return(x)
}


#' Delete a reconstructed model from the dataset
#' @title delete.model
#'
#' @examples
#' data(test_model)
#' model = delete.model(test_model)
#' has.model(model)
#'
#' @param x A TRONCO compliant dataset.
#' @return A TRONCO complian dataset.
#' @export delete.model
#' 
delete.model <- function(x) {
    is.compliant(x)
    if (! has.model(x)) {
        stop("No model to delete in dataset")
    }
    x$model = NULL
    x$bootstrap = NULL
    x$confidence = NULL
    x$parameters = NULL
    x$adj.matrix.prima.facie = NULL
    x$execution.time = NULL
    x$kfold = NULL

    is.compliant(x)

    return(x)
}


#' Delete samples from selected dataset
#' @title delete.samples
#'
#' @examples
#' data(test_dataset)
#' dataset = delete.samples(test_dataset, c('patient 1', 'patient 4'))
#'
#' @param x A TRONCO compliant dataset.
#' @param samples An array of samples name
#' @return A TRONCO complian dataset.
#' @export delete.samples
#' 
delete.samples <- function(x, samples) {
    is.compliant(x, 'delete.samples input')

    if (has.model(x)) {
        stop("There's a reconstructed model, a sample cannot be deleted now. \nUse delete.model()")
    }

    samples = unique(samples)
    if (!all(samples %in% as.samples(x))) {
        stop("Not all sample in as.samples(x)")
    }

    genotypes = x$genotypes[!rownames(x$genotypes) %in% samples, , drop = FALSE]

    if (has.stages(x)) {
        stages = x$stages[!rownames(x$stages) %in% samples, , drop = FALSE]
        x$stages = stages
    }

    x$genotypes = genotypes

    is.compliant(x, 'delete.samples output')

    return(x)
}


#' Intersect samples and events of two dataset
#' @title intersect.datasets
#'
#' @examples
#' data(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @param y A TRONCO compliant dataset.
#' @param intersect.genomes If False -> just samples
#' @return A TRONCO complian dataset.
#' @export intersect.datasets
#' 
intersect.datasets <- function(x,y, intersect.genomes = TRUE) {
    is.compliant(x)
    is.compliant(y)

    ## Common samples and genes (according to intersect.genomes)
    samples = intersect(as.samples(x), as.samples(y))
    genes =
        ifelse(intersect.genomes,
               intersect(as.genes(x), as.genes(y)), # intersect.genomes -> INTERSECTION
               unique(c(as.genes(x), as.genes(y)))) # !intersect.genomes -> UNION

    report = data.frame(row.names = c('Samples', 'Genes'))
    report$x = c(nsamples(x), ngenes(x))
    report$y = c(nsamples(y), ngenes(y))

    ## Restrict genes - only if intersect.genomes = T
    
    if (intersect.genomes) {
        x = events.selection(x, filter.in.names=genes)
        y = events.selection(y, filter.in.names=genes)
    }

    ## TODO: check they have no events in common!
    ## if (as.events(x) )

    ## Restrict stamples.
    
    x = samples.selection(x, samples)
    y = samples.selection(y, samples)

    ## Result.
    
    z = ebind(x,y)

    cat('*** Intersect dataset [ intersect.genomes =', intersect.genomes, ']\n')
    report$result = c(nsamples(z), ngenes(z))
    print(report)

    return(z)
}


#' Binds events from one or more datasets, which must be defined over the same set of samples.
#' @title ebind
#'
#' @param ... the input datasets
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO complian dataset.
#' @export ebind
#' 
ebind <- function(...,
                  silent = FALSE) {
    
    ## Merge two  datasets at a time.
    events.pairwise.bind <- function(x, y) {
        is.compliant(x, 'ebind: input x')
        is.compliant(y, 'ebind: input y')

        samples.intersect = intersect(as.samples(x), as.samples(y))
        if (!(setequal(samples.intersect, as.samples(x))
              && setequal(samples.intersect, as.samples(y))))
            stop('Datasets have different samples, won\'t bind!')

        z = list()

        y$genotypes = y$genotypes[rownames(x$genotypes), , drop = FALSE]
        y$stages = y$stages[rownames(y$genotypes), , drop = FALSE]

        ## Copy genotype matrix, and sets its rownames (samples).
        
        z$genotypes = cbind(x$genotypes, y$genotypes)
        colnames(z$genotypes) = paste('G', 1:ncol(z$genotypes), sep='')

        ## Copy annotations for gene symbols etc.
        
        z$annotations = rbind(x$annotations, y$annotations)
        rownames(z$annotations) = colnames(z$genotypes)

        ## Copy types.
        
        z$types = unique(rbind(x$types, y$types))

        ## Copy stages, if present.
        
        if (has.stages(x) && has.stages(y)) {
            stages.x = as.stages(x)
            stages.y = as.stages(y)

            stages.x = stages.x[!is.na(stages.x)]
            stages.y = stages.y[!is.na(stages.y)]

            if (any(stages.x != stages.y)) {
                stop('Patients have different stages, won\'t merge!')
            }
        }

        if (has.stages(x)) {
            z = annotate.stages(z, as.stages(x))
        }

        is.compliant(z, 'ebind: output')

        return(z)
    }

    input = list(...)

    if (!silent) {
        cat('*** Binding events for', length(input), 'datasets.\n')
    }
    return(Reduce(events.pairwise.bind, input))
}


#' Binds samples from one or more datasets, which must be defined over the same set of events
#' @title sbind
#'
#' @param ... the input datasets
#' @return A TRONCO complian dataset.
#' @export sbind
#' 
sbind <- function(...) {

    ## Merge two datasets at a time.
    
    samples.pairwise.bind <- function(x, y) {
        is.compliant(x, 'sbind: input x')
        is.compliant(y, 'sbind: input y')

        if (!all(as.events(x) == as.events(y))) {
            stop('Datasets have different events, can not bind!')
        }
        
        z = list()

        ## Copy genotypes and annotations.
        
        z$genotypes = rbind(x$genotypes, y$genotypes)
        z$annotations = x$annotations
        z$types = x$types

        ## Copy stages, if present.
        
        if (has.stages(x) || has.stages(y)) {
            if (has.stages(x)) xstages = as.stages(x)
            else xstages = matrix(rep('NA', nsamples(x)), nrow = nsamples(x))

            if (has.stages(y)) ystages = as.stages(y)
            else ystages = matrix(rep('NA', nsamples(y)), nrow = nsamples(y))

            z$stages = (rbind(x$stages, y$stages))

            colnames(z$stages) = 'stage'
        }
        is.compliant(z, 'sbind: output')

        return(z)
    }

    input = list(...)
    return(Reduce(samples.pairwise.bind, input))
}


#' For an input dataset merge all the events of two or more distincit types
#' (e.g., say that missense and indel mutations are events
#' of a unique "mutation" type)
#' @title join.types
#'
#' @examples
#' data(test_dataset_no_hypos)
#' join.types(test_dataset_no_hypos, 'ins_del', 'missense_point_mutations')
#' join.types(test_dataset_no_hypos, 'ins_del',
#'      'missense_point_mutations', new.type='mut', new.color='green')
#'
#' @param x A TRONCO compliant dataset.
#' @param ... type to merge
#' @param new.type label for the new type to create
#' @param new.color color for the new type to create
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO compliant dataset.
#' @export join.types
#' 
join.types <- function(x,
                       ...,
                       new.type = "new.type",
                       new.color = "khaki",
                       silent = FALSE) {

    ## Check if x is compliant.
    
    is.compliant(x)

    if(has.model(x)) {
        stop("There's a reconstructed model, types cannot be merged now. \nUse delete.model()")
    }

    if(length(as.patterns(x)) > 0) {
        stop('Patterns found. Delete patterns first.\n')
    }

    input = list(...)

    ## TODO Change this in a better way (deafult ellipsis?)
    if (length(input) == 1 && is.null(input[[1]])) {
        input = as.list(as.types(x))
    }

    if (!silent) {
        cat("*** Aggregating events of type(s) {",
            paste(unlist(input), collapse = ", ", sep = ""),
            "}\nin a unique event with label \"",
            new.type,
            "\".\n")
    }

    if (length(input) <= 1) {
        if (!silent) {
            cat("One input type provided, using renaming functions.\n")
        }

        x = rename.type(x, input[[1]], new.type)
        x = change.color(x, new.type, new.color)
        return(x)
    }


    types.check =
        lapply(input,
               function(type) {
                   type %in% as.types(x)
               })

    if (!all(unlist(types.check))) {
        t = (input[!unlist(types.check)])

        stop("No events of type '", t, "' in input dataset, will not merge.")
    }

    if (any(duplicated(input))) {
        stop("No duplicated types are allowed, will not merge.")
    }

    if (new.type %in% as.types(x)) {
        stop(paste0(new.type, "is already used in input dataset, will not merge"))
    }

    input = unlist(input)

    genes = as.genes(x, types = input)
    if (!silent) {
        cat("Dropping event types",
            paste(input, collapse = ", ", sep = ""),
            "for",
            length(genes),
            "genes.\n")
    }
    geno.matrix = matrix(, nrow = nsamples(x), ncol = length(genes))

    #if (!silent) {
    #    pb = txtProgressBar(1, length(genes), style = 3)
    #    flush.console()
    #}
    
    for (i in 1:length(genes)) {
        #if (!silent) {
        #    setTxtProgressBar(pb, i)
        #}
        if (!silent) {
            cat('.')
        }

        geno = as.matrix(rowSums(as.gene(x, genes[i], types = input)))
        geno[geno > 1] = 1

        geno.matrix[, i] = geno
    }
    if (!silent) {
        cat('\n')
    }

    rownames(geno.matrix) = as.samples(x)
    colnames(geno.matrix) = genes
    #if (!silent) {
    #    close(pb)
    #}

    z = import.genotypes(geno.matrix, event.type = new.type, color = new.color)
    if (has.stages(x)) {
        z = annotate.stages(z, as.stages(x))
    }

    y = x
    for (i in input) {
        y = delete.type(y, i)
    }

    w = ebind(y, z, silent = silent)
    is.compliant(w)

    return(w)
}


#' Deletes all events which have frequency 0 in the dataset.
#' @title trim
#'
#' @examples
#' data(test_dataset)
#' test_dataset = trim(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return A TRONCO compliant dataset.
#' @export trim
#' 
trim <- function(x) {
    is.compliant(x, 'trim: input')

    x = enforce.numeric(x)

    del = names(which(colSums(x$genotypes) == 0))

    x$genotypes = x$genotypes[ , !colnames(x$genotypes) %in% del, drop = FALSE]
    x$annotations = x$annotations[!rownames(x$annotations) %in% del, , drop = FALSE]

    x$types = matrix(x$types[unique(x$annotations[ , 'type', drop = FALSE]), , drop = FALSE ], ncol = 1)
    rownames(x$types) = unique(x$annotations[,'type', drop = FALSE])
    colnames(x$types) = 'color'

    is.compliant(x, 'trim: output')
    return(x)
}


# TODO: check
#' Split cohort (samples) into groups, return either all groups or a specific group.
#' @title ssplit
#'
#' @param x A TRONCO compliant dataset.
#' @param clusters A list of clusters. Rownames must match samples list of x
#' @param idx ID of a specific group present in stages. If NA all groups will be extracted
#' @return A TRONCO compliant dataset.
#' @export ssplit
#' 
ssplit <- function(x, clusters, idx = NA) {
    is.compliant(x)

    data = x$genotypes

    cat('*** Splitting cohort into groups.\n')

    ## TODO: check that clusters has at least 2 columns

    ## Check that map has correct size.

    if (nsamples(x) != nrow(clusters))
        stop(paste("Error: cannot split, number of samples (",
                   nsamples(x) ,
                   ") and groups (",
                   nrow(clusters),
                   ") do not match.",
                   sep = ''));

    ## Check it is an actual map.
    
    if (!all(rownames(clusters) %in% rownames(data)))
        stop(paste('Error: samples',
                   paste(rownames(clusters)[!rownames(clusters) %in% as.samples(x)],
                         collapse=', ',
                         sep = ''),
                   'are not assigned to a group',
                   sep = ''));

    ## Groups info.
    
    cluster.labels = unique(clusters)
    num.clusters = nrow(cluster.labels)

    ## Extract a specific group.
    
    if (!is.na(idx)) {
        y = list()
        samples.in.cluster = rownames(clusters)[clusters == idx]

        cat(paste('Group \"', idx, '\" has ', length(samples.in.cluster),
                  ' samples, returning this group.\n', sep=''))

        y$genotypes = data[samples.in.cluster, ];
        y$annotations = x$annotations
        y$types = x$types

        if (!is.null(x$stages)) {
            y$stages = as.matrix(x$stages[samples.in.cluster, ])
            rownames(y$stages) = samples.in.cluster
        }

        is.compliant(y, 'ssplit.split with index')
        return(y)
    }

    ## Extract all groups.
    
    partitions = list()
    for (i in 1:num.clusters) {
        y = list()
        samples.in.cluster = rownames(clusters)[clusters == cluster.labels[i, 1]]

        cat(paste('Group \"', cluster.labels[i,1], '\" has ', length(samples.in.cluster),
                  ' samples.\n', sep=''))

        y$genotypes = data[samples.in.cluster, ];
        y$annotations = x$annotations
        y$types = x$types

        if (!is.null(x$stages)) {
            y$stages = as.matrix(x$stages[samples.in.cluster, ])
            rownames(y$stages) = samples.in.cluster
        }

        is.compliant(y, 'subtypes.split partitionig')
        partitions = append(partitions, list(y))
        names(partitions)[i] = cluster.labels[i,1]
    }

    return(partitions)
}


# Split events into groups according to their types.
#
# x: cohort
# @export

tsplit <- function(x) {
    ## Parametro per estrarre un tipo solo di evento?
}


#' Merge a list of events in an unique event
#' @title join.events
#'
#' @examples
#' data(muts)
#' dataset = join.events(muts, 'G1', 'G2', new.event='test', new.type='banana', event.color='yellow')
#'
#' @param x A TRONCO compliant dataset.
#' @param ... A list of events to merge
#' @param new.event The name of the resultant event
#' @param new.type The type of the new event
#' @param event.color The color of the new event
#' @return A TRONCO compliant dataset.
#' @export join.events
#' 
join.events <- function(x, ..., new.event, new.type, event.color) {

    events = list(...)

    if (length(events) < 2) {
        stop('Badly formed events.')
    }

    for (pattern in as.patterns(x)) {
        if (any(events %in% rownames(as.events.in.patterns(x, patterns=pattern)))) {
            stop('Found event in pattern \"', pattern, '\". Delete that pattern first.\n')
        }
    }

    x = enforce.numeric(x)

    y = x
    as_ev = as.events(x)

    for (event in events) {
        y = delete.event(y, gene = as_ev[event, 'event'], type = as_ev[event, 'type'])
    }

    genos = x$genotypes[, unlist(events)]
    genos = matrix(rowSums(genos), nrow = nsamples(x))
    genos[genos > 1] = 1
    colnames(genos) = new.event
    rownames(genos) = as.samples(x)

    genos = import.genotypes(genos, event.type = new.type, color = event.color)
    y = ebind(y, genos)

    return(y)
}


# rename field of reconstruction
rename.reconstruction.fields <- function(reconstruction, genotypes){
    rownames(reconstruction$confidence) =
        c("temporal priority",
          "probability raising",
          "hypergeometric test")
    colnames(reconstruction$confidence) = "confidence"
    rownames(reconstruction$confidence[[1,1]]) = colnames(genotypes)
    colnames(reconstruction$confidence[[1,1]]) = colnames(genotypes)
    rownames(reconstruction$confidence[[2,1]]) = colnames(genotypes)
    colnames(reconstruction$confidence[[2,1]]) = colnames(genotypes)
    rownames(reconstruction$confidence[[3,1]]) = colnames(genotypes)
    colnames(reconstruction$confidence[[3,1]]) = colnames(genotypes)

    for (i in 1:length(reconstruction$model)) {

        ## Set rownames and colnames to the probabilities.
        rownames(reconstruction$model[[i]]$probabilities$probabilities.observed$marginal.probs) = colnames(genotypes)
        colnames(reconstruction$model[[i]]$probabilities$probabilities.observed$marginal.probs) = "marginal probability"
        rownames(reconstruction$model[[i]]$probabilities$probabilities.observed$joint.probs) = colnames(genotypes)
        colnames(reconstruction$model[[i]]$probabilities$probabilities.observed$joint.probs) = colnames(genotypes)
        rownames(reconstruction$model[[i]]$probabilities$probabilities.observed$conditional.probs) = colnames(genotypes)
        colnames(reconstruction$model[[i]]$probabilities$probabilities.observed$conditional.probs) = "conditional probability"

        ## Set rownames and colnames to the parents positions.
        rownames(reconstruction$model[[i]]$parents.pos) = colnames(genotypes)
        colnames(reconstruction$model[[i]]$parents.pos) = "parents"

        ## Set rownames and colnames to the adjacency matrices.
        rownames(reconstruction$model[[i]]$adj.matrix$adj.matrix.fit) = colnames(genotypes)
        colnames(reconstruction$model[[i]]$adj.matrix$adj.matrix.fit) = colnames(genotypes)

    }
    return(reconstruction)
}


#### end of file -- editing.functions.R

data(test_dataset_no_hypos)
data(test_model)
data(test_dataset)

context("change.color")

test_that("change.color is working", {
    expect_equal(length(change.color(test_dataset, 'ins_del', 'red')), 5)
    expect_error(change.color(test_dataset_no_hypos, "XXX", "red"))
})

context("rename.type")

test_that("rename.type is working", {
    expect_equal(length(rename.type(test_dataset, 'ins_del', 'banana')), 5)
    expect_equal(length(rename.type(test_dataset, 'ins_del', 'ins_del')), 5)
    expect_equal(length(rename.type(test_dataset, 'ins_del', 'missense_point')), 5)
    expect_error(rename.type(test_dataset, 'Pattern', 'missense_point'))
    expect_error(rename.type(test_dataset, 'ins_del', 'Pattern'))
    expect_error(rename.type(test_dataset, 'banana', 'Pattern'))
})

context("rename.gene")

test_that("rename.gene is working", {
    expect_equal(length(rename.gene(test_dataset, 'TET2', 'banana')), 5)
    expect_error(rename.gene(test_dataset, 'XXX', 'banana'))
})

context("rename.gene")

test_that("rename.gene is working", {
    expect_equal(length(rename.gene(test_dataset, 'TET2', 'banana')), 5)
    expect_error(rename.gene(test_dataset, 'XXX', 'banana'))
})

context("delete.type")

test_that("delete.type is working", {
    expect_equal(length(delete.type(test_dataset_no_hypos, 'ins_del')), 5)
    expect_equal(length(delete.type(test_dataset, 'Pattern')), 5)
    expect_error(delete.type(test_model, 'ins_del'))
    expect_error(delete.type(test_dataset, 'ins_del'))
    expect_error(delete.type(test_dataset_no_hypos, 'banana'))
})

context("delete.gene")

test_that("delete.gene is working", {
    expect_equal(length(delete.gene(test_dataset_no_hypos, 'TET2')), 5)
    expect_error(delete.gene(test_model, 'TET2'))
    expect_error(delete.gene(test_dataset, 'EZH2'))
    expect_error(delete.gene(test_dataset_no_hypos, 'XXX'))
})

context("delete.event")

test_that("delete.event is working", {
    expect_equal(length(delete.event(test_dataset_no_hypos, 'TET2', 'ins_del')), 5)
    expect_error(delete.event(test_dataset, 'TET2', 'Pattern'))
    expect_error(delete.event(test_model, 'TET2', 'ins_del'))
    expect_error(delete.event(test_dataset, 'EZH2', 'ins_del'))
    expect_error(delete.event(test_dataset, 'XXX', 'Pattern'))
})

context("delete.hypothesis")

test_that("delete.hypothesis is working", {
    expect_equal(length(delete.hypothesis(test_dataset, event = 'TET2')), 5)
    expect_equal(length(delete.hypothesis(test_dataset, cause = c('TET2', 'EZH2'))), 5)
    expect_equal(length(delete.hypothesis(test_dataset, effect = c('TET2', 'EZH2'))), 5)
    expect_warning(delete.hypothesis(test_dataset))
    expect_error(delete.hypothesis(test_dataset, event = c('TET2', 'EZH2')))
    expect_error(delete.hypothesis(test_model, event = 'TET2'))
    expect_error(delete.hypothesis(test_dataset, cause = 'XXX'))
    expect_error(delete.hypothesis(test_dataset, effect = 'XXX'))
})

context("delete.pattern")

test_that("delete.pattern is working", {
    expect_equal(length(delete.pattern(test_dataset, pattern = 'XOR_EZH2')), 5)
    expect_error(delete.pattern(test_model, pattern = 'XOR_EZH2'))
    expect_error(delete.pattern(test_dataset, pattern = 'banana'))
    a =  delete.pattern(test_dataset, pattern = 'XOR_EZH2')
    expect_equal(length(delete.pattern(a, pattern = 'OR_CSF3R')), 5)
})

context("delete.model")

test_that("delete.model is working", {
    expect_equal(length(delete.model(test_model)), 5)
    expect_error(delete.model(test_dataset))
})

context("delete.samples")

test_that("delete.model is working", {
    expect_equal(length(delete.samples(test_dataset_no_hypos, samples='patient 1')), 5)
    expect_error(delete.samples(test_dataset_no_hypos, samples='EROS'))
    expect_error(delete.samples(test_model))
})

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' Reconstruct a progression model using CAPRESE algorithm. For details and examples
#' regarding the inference process and on the algorithm implemented in the package,
#' we refer to the Vignette Section 6.
#'
#' @examples
#' data(test_dataset_no_hypos)
#' recon = tronco.caprese(test_dataset_no_hypos)
#'
#' @title tronco caprese
#' @param data A TRONCO compliant dataset.
#' @param lambda Coefficient to combine the raw estimate with a correction factor into a shrinkage estimator.
#' @param silent A parameter to disable/enable verbose messages.
#' @param epos Error rate of false positive errors.
#' @param eneg Error rate of false negative errors.
#' @return A TRONCO compliant object with reconstructed model
#' @export tronco.caprese
#' @importFrom stats phyper
#' @importFrom bnlearn empty.graph set.arc
#' @importFrom igraph graph.adjacency get.shortest.paths
#'
tronco.caprese <- function(data,
                           lambda = 0.5,
                           silent = FALSE,
                           epos = 0.0,
                           eneg = 0.0) {
  ## Check for the inputs to be correct
  if (is.null(data) || is.null(data$genotypes)) {
    stop("The dataset given as input is not valid.")
  }
  if (lambda < 0 || lambda > 1) {
    stop("The value of the shrinkage parameter lambda has to be in [0:1]!",
         call. = FALSE)
    
  }
  
  ## check for the input to be compliant
  is.compliant(data)
  
  ## check if there are hypotheses
  if (npatterns(data) > 0) {
    warning("Patters found in input for tronco.caprese\n")
  }

  if (is.null(data$hypotheses)) {
    data$hypotheses = NA
    
  }
  
  ## Reconstruct the reconstruction with CAPRESE
  if (silent == FALSE) {
    cat('*** Checking input events.\n')
    invalid = consolidate.data(data, TRUE)
    if (length(unlist(invalid)) > 0)
      warning("Input events should be consolidated - see consolidate.data.")
    
    
    cat(
      paste0(
        '*** Inferring a progression model with the following settings.\n',
        '\tDataset size: n = ',
        nsamples(data),
        ', m = ',
        nevents(data),
        '.\n',
        '\tAlgorithm: CAPRESE with shrinkage coefficient: ',
        lambda,
        '.\n'
      )
    )
  }
  reconstruction = caprese.fit(
    dataset = data$genotypes,
    lambda = lambda,
    silent = silent,
    epos = epos,
    eneg = eneg,
    hypotheses = data$hypotheses
  )
  
  ## Structure to save the results
  results = data
  
  results$confidence = reconstruction$confidence
  
  results$model = reconstruction$model
  
  results$parameters = reconstruction$parameters
  
  results$execution.time = reconstruction$execution.time
  
  
  if (!silent) {
    cat('*** Evaluating LogLik informations.\n')
    
  }
  
  
  
  bayes.net = as.bnlearn.network(results, model = 'caprese')
  logLik = logLik(bayes.net$net, data = bayes.net$data)
  results$model$caprese$logLik = logLik
  
  ## The reconstruction has been completed.
  if (!silent)
    cat(paste(
      "The reconstruction has been successfully completed in",
      format(.POSIXct(
        round(reconstruction$execution.time[3],
              digits = 0),
        tz = "GMT"
      ),
      "%Hh:%Mm:%Ss"),
      "\n"
    ))
  
  
  return(results)
  
}


#' Reconstruct a progression model using CAPRI algorithm. For details and examples
#' regarding the inference process and on the algorithm implemented in the package,
#' we refer to the Vignette Section 6.
#'
#' @examples
#' data(test_dataset)
#' recon = tronco.capri(test_dataset, nboot = 1)
#'
#' @title tronco capri
#' @param data A TRONCO compliant dataset.
#' @param command Parameter to define to heuristic search to be performed. Hill Climbing and Tabu search are currently available.
#' @param regularization Select the regularization for the likelihood estimation, e.g., BIC, AIC.
#' @param do.boot A parameter to disable/enable the estimation of the error rates give the reconstructed model.
#' @param nboot Number of bootstrap sampling (with rejection) to be performed when estimating the selective advantage scores.
#' @param pvalue Pvalue to accept/reject the valid selective advantage relations.
#' @param min.boot Minimum number of bootstrap sampling to be performed.
#' @param min.stat A parameter to disable/enable the minimum number of bootstrap sampling required besides nboot if any sampling is rejected.
#' @param boot.seed Initial seed for the bootstrap random sampling.
#' @param silent A parameter to disable/enable verbose messages.
#' @param epos Error rate of false positive errors.
#' @param eneg Error rate of false negative errors.
#' @param restart An integer, the number of random restarts.
#' @return A TRONCO compliant object with reconstructed model
#' @export tronco.capri
#' @importFrom bnlearn hc tabu empty.graph set.arc
#' @importFrom igraph graph.adjacency get.adjacency graph.union edge
#' @importFrom igraph get.shortest.paths is.dag
#' @importFrom stats phyper AIC BIC wilcox.test
#'
tronco.capri <- function(data,
                         command = "hc",
                         regularization = c("bic", "aic"),
                         do.boot = TRUE,
                         nboot = 100,
                         pvalue = 0.05,
                         min.boot = 3,
                         min.stat = TRUE,
                         boot.seed = NULL,
                         silent = FALSE,
                         epos = 0.0,
                         eneg = 0.0,
                         restart = 100) {
  ## Check for the inputs to be correct
  if (is.null(data) || is.null(data$genotypes)) {
    stop("The dataset given as input is not valid.")
    
  }
  
  ## Enforce data to be numeric
  data = enforce.numeric(data)
  
  if (is.null(data$hypotheses)) {
    data$hypotheses = NA
    
  }
  
  if (!command %in% c("hc", "tabu")) {
    stop("The inference can be performed either by hill climbing or tabu search!",
         call. = FALSE)
  }
  
  if (pvalue < 0 || pvalue > 1) {
    stop("The value of the pvalue has to be in [0:1]!", call. = FALSE)
    
  }
  
  if (!all(regularization %in% c('loglik', 'bic', 'aic'))) {
    stop("Possible regularization are loglik, bic or aic", call. = FALSE)
    
  }
  
  if (epos < 0 || epos >= 0.5 || eneg < 0 || eneg >= 0.5) {
    stop("The values of the error rates have to be in [0:0.5)!",
         call. = FALSE)
  }
  
  ## Check for the input to be compliant
  is.compliant(data)
  
  ## Reconstruct the reconstruction with CAPRI
  if (is.null(boot.seed)) {
    my.seed = "NULL"
  }
  else {
    my.seed = boot.seed
    
  }
  if (silent == FALSE) {
    cat('*** Checking input events.\n')
    invalid = consolidate.data(data, TRUE)
    if (length(unlist(invalid)) > 0)
      warning("Input events should be consolidated - see consolidate.data.")
    
    
    cat(
      paste0(
        '*** Inferring a progression model with the following settings.\n',
        '\tDataset size: n = ',
        nsamples(data),
        ', m = ',
        nevents(data),
        '.\n',
        '\tAlgorithm: CAPRI with \"',
        paste0(regularization, collapse = ", "),
        '\" regularization and \"',
        command,
        '\" likelihood-fit strategy.\n',
        '\tRandom seed: ',
        my.seed,
        '.\n',
        '\tBootstrap iterations (Wilcoxon): ',
        ifelse(do.boot, nboot, 'disabled'),
        '.\n',
        ifelse(
          do.boot,
          paste0(
            '\t\texhaustive bootstrap: ',
            min.stat,
            '.\n\t\tp-value: ',
            pvalue,
            '.\n\t\tminimum bootstrapped scores: ',
            min.boot,
            '.\n'
          ),
          ''
        )
      )
    )
  }
  
  reconstruction =
    capri.fit(
      data$genotypes,
      data$hypotheses,
      command = command,
      regularization = regularization,
      do.boot = do.boot,
      nboot = nboot,
      pvalue = pvalue,
      min.boot = min.boot,
      min.stat = min.stat,
      boot.seed = boot.seed,
      silent = silent,
      epos = epos,
      eneg = eneg,
      restart = restart
    )
  
  ## Structure to save the results.
  
  results = data
  
  results$adj.matrix.prima.facie = reconstruction$adj.matrix.prima.facie
  results$confidence = reconstruction$confidence
  
  results$model = reconstruction$model
  
  results$parameters = reconstruction$parameters
  
  results$execution.time = reconstruction$execution.time
  
  
  ## Add BIC/AIC/LogLik informations
  
  if (!silent) {
    cat('*** Evaluating BIC / AIC / LogLik informations.\n')
  }
  
  if ("loglik" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'capri_loglik')
    score = logLik(bayes.net$net, data = bayes.net$data)
    logLik = score
    results$model$capri_loglik$score = score
    results$model$capri_loglik$logLik = logLik
  }
  
  if ("bic" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'capri_bic')
    score = BIC(bayes.net$net, data = bayes.net$data)
    logLik = logLik(bayes.net$net, data = bayes.net$data)
    results$model$capri_bic$score = score
    results$model$capri_bic$logLik = logLik
  }
  
  if ("aic" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'capri_aic')
    score = AIC(bayes.net$net, data = bayes.net$data)
    logLik = logLik(bayes.net$net, data = bayes.net$data)
    results$model$capri_aic$score = score
    results$model$capri_aic$logLik = logLik
  }
  
  ## the reconstruction has been completed.
  
  if (!silent)
    cat(paste(
      "The reconstruction has been successfully completed in",
      format(.POSIXct(
        round(reconstruction$execution.time[3],
              digits = 0),
        tz = "GMT"
      ),
      "%Hh:%Mm:%Ss"),
      "\n"
    ))
  
  
  return(results)
  
}


#' Reconstruct a progression model using Edmonds algorithm combined
#' with probabilistic causation. For details and examples
#' regarding the inference process and on the algorithm implemented in the package,
#' we refer to the Vignette Section 6.
#'
#' @examples
#' data(test_dataset_no_hypos)
#' recon = tronco.edmonds(test_dataset_no_hypos, nboot = 1)
#'
#' @title Tronco Edmonds
#' @param data A TRONCO compliant dataset.
#' @param regularization Select the regularization for the
#' likelihood estimation, e.g., BIC, AIC.
#' @param score Select the score for the estimation of
#' the best tree, e.g., pointwise mutual information (pmi), conditional entropy (entropy).
#' @param do.boot A parameter to disable/enable the estimation
#' of the error rates give the reconstructed model.
#' @param nboot Number of bootstrap sampling (with rejection)
#' to be performed when estimating the selective advantage scores.
#' @param pvalue Pvalue to accept/reject the valid selective
#' advantage relations.
#' @param min.boot Minimum number of bootstrap sampling to be
#' performed.
#' @param min.stat A parameter to disable/enable the minimum number
#' of bootstrap sampling required besides nboot if any sampling
#' is rejected.
#' @param boot.seed Initial seed for the bootstrap random sampling.
#' @param silent A parameter to disable/enable verbose messages.
#' @param epos Error rate of false positive errors.
#' @param eneg Error rate of false negative errors.
#' @return A TRONCO compliant object with reconstructed model
#' @export tronco.edmonds
#' @importFrom bnlearn hc tabu empty.graph set.arc
#' @importFrom igraph graph.adjacency get.adjacency graph.union edge
#' @importFrom igraph get.shortest.paths is.dag
#### @importFrom infotheo mutinformation
#' @importFrom stats phyper AIC BIC
#'
tronco.edmonds <- function(data,
                           regularization = "no_reg",
                           score = "pmi",
                           do.boot = TRUE,
                           nboot = 100,
                           pvalue = 0.05,
                           min.boot = 3,
                           min.stat = TRUE,
                           boot.seed = NULL,
                           silent = FALSE,
                           epos = 0.0,
                           eneg = 0.0) {
  if (is.null(data) || is.null(data$genotypes)) {
    stop("The dataset given as input is not valid.")
    
  }
  
  ## Enforce data to be numeric
  data = enforce.numeric(data)
  
  ## Check for the inputs to be correct.
  
  if (is.null(data$hypotheses)) {
    data$hypotheses = NA
    
  }
  
  if (pvalue < 0 || pvalue > 1) {
    stop("The value of the pvalue has to be in [0:1]!", call. = FALSE)
    
  }
  
  if (!all(regularization %in% c('no_reg', 'loglik', 'bic', 'aic'))) {
    stop("Possible regularization are no-reg, loglik, bic or aic",
         call. = FALSE)
    
  }
  
  if (!all(score %in% c('pmi', 'mi', 'entropy', 'cpmi'))) {
    stop("Possible scores are pmi, mi, entropy or cpmi", call. = FALSE)
    
  }
  
  if (epos < 0 || epos >= 0.5 || eneg < 0 || eneg >= 0.5) {
    stop("The values of the error rates have to be in [0:0.5)!",
         call. = FALSE)
  }
  
  ## Check for the input to be compliant.
  
  is.compliant(data)
  
  ## check if there are hypotheses
  
  if (npatterns(data) > 0) {
    warning("Patters found in input for tronco.edmonds\n")
  }
  
  ## Reconstruct the reconstruction with Edmonds.
  
  if (is.null(boot.seed)) {
    my.seed = "NULL"
  }
  else {
    my.seed = boot.seed
    
  }
  if (silent == FALSE) {
    cat('*** Checking input events.\n')
    invalid = consolidate.data(data, TRUE)
    if (length(unlist(invalid)) > 0)
      warning("Input events should be consolidated - see consolidate.data.")
    
    
    cat(
      paste0(
        '*** Inferring a progression model with the following settings.\n',
        '\tDataset size: n = ',
        nsamples(data),
        ', m = ',
        nevents(data),
        '.\n',
        '\tAlgorithm: Edmonds with \"',
        paste0(regularization, collapse = ", "),
        '\" regularization',
        '\tRandom seed: ',
        my.seed,
        '.\n',
        '\tBootstrap iterations (Wilcoxon): ',
        ifelse(do.boot, nboot, 'disabled'),
        '.\n',
        ifelse(
          do.boot,
          paste0(
            '\t\texhaustive bootstrap: ',
            min.stat,
            '.\n\t\tp-value: ',
            pvalue,
            '.\n\t\tminimum bootstrapped scores: ',
            min.boot,
            '.\n'
          ),
          ''
        )
      )
    )
  }
  
  reconstruction =
    edmonds.fit(
      data$genotypes,
      regularization = regularization,
      score = score,
      do.boot = do.boot,
      nboot = nboot,
      pvalue = pvalue,
      min.boot = min.boot,
      min.stat = min.stat,
      boot.seed = boot.seed,
      silent = silent,
      epos = epos,
      eneg = eneg,
      hypotheses = data$hypotheses
    )
  
  ## Structure to save the results.
  results = data
  results$adj.matrix.prima.facie = reconstruction$adj.matrix.prima.facie
  results$adj.matrix.prima.facie.cyclic = reconstruction$adj.matrix.prima.facie.cyclic
  results$confidence = reconstruction$confidence
  results$model = reconstruction$model
  results$parameters = reconstruction$parameters
  results$execution.time = reconstruction$execution.time
  
  ## Add BIC/AIC/LogLik informations
  
  if (!silent) {
    cat('*** Evaluating BIC / AIC / LogLik informations.\n')
  }
  
  search_scores = score
  
  if ("no_reg" %in% regularization) {
    for (my_s in search_scores) {
      bayes.net = as.bnlearn.network(results, model = paste('edmonds_no_reg', my_s, sep =
                                                              "_"))
      score = logLik(bayes.net$net, data = bayes.net$data)
      logLik = score
      results$model[[paste('edmonds_no_reg', my_s, sep = "_")]]$score = score
      results$model[[paste('edmonds_no_reg', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  if ("loglik" %in% regularization) {
    for (my_s in search_scores) {
      bayes.net = as.bnlearn.network(results, model  = paste('edmonds_loglik', my_s, sep =
                                                               "_"))
      score = logLik(bayes.net$net, data = bayes.net$data)
      logLik = score
      results$model[[paste('edmonds_loglik', my_s, sep = "_")]]$score = score
      results$model[[paste('edmonds_loglik', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  if ("bic" %in% regularization) {
    for (my_s in search_scores) {
      bayes.net = as.bnlearn.network(results, model  = paste('edmonds_bic', my_s, sep =
                                                               "_"))
      score = BIC(bayes.net$net, data = bayes.net$data)
      logLik = logLik(bayes.net$net, data = bayes.net$data)
      results$model[[paste('edmonds_bic', my_s, sep = "_")]]$score = score
      results$model[[paste('edmonds_bic', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  if ("aic" %in% regularization) {
    for (my_s in search_scores) {
      bayes.net = as.bnlearn.network(results, model = paste('edmonds_aic', my_s, sep =
                                                              "_"))
      score = AIC(bayes.net$net, data = bayes.net$data)
      logLik = logLik(bayes.net$net, data = bayes.net$data)
      results$model[[paste('edmonds_aic', my_s, sep = "_")]]$score = score
      results$model[[paste('edmonds_aic', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  ## the reconstruction has been completed
  if (!silent)
    cat(paste(
      "The reconstruction has been successfully completed in",
      format(.POSIXct(
        round(reconstruction$execution.time[3],
              digits = 0),
        tz = "GMT"
      ),
      "%Hh:%Mm:%Ss"),
      "\n"
    ))
  
  
  return(results)
  
}


#' Reconstruct a progression model using Gabow algorithm combined
#' with probabilistic causation. For details and examples
#' regarding the inference process and on the algorithm implemented in the package,
#' we refer to the Vignette Section 6.
#'
#' @examples
#' data(test_dataset_no_hypos)
#' recon = tronco.gabow(test_dataset_no_hypos, nboot = 1)
#'
#' @title Tronco Gabow
#' @param data A TRONCO compliant dataset.
#' @param regularization Select the regularization for the
#' likelihood estimation, e.g., BIC, AIC.
#' @param score Select the score for the estimation of
#' the best tree, e.g., pointwise mutual information (pmi), conditional entropy (entropy).
#' @param do.boot A parameter to disable/enable the estimation
#' of the error rates give the reconstructed model.
#' @param nboot Number of bootstrap sampling (with rejection)
#' to be performed when estimating the selective advantage scores.
#' @param pvalue Pvalue to accept/reject the valid selective
#' advantage relations.
#' @param min.boot Minimum number of bootstrap sampling to be
#' performed.
#' @param min.stat A parameter to disable/enable the minimum number
#' of bootstrap sampling required besides nboot if any sampling
#' is rejected.
#' @param boot.seed Initial seed for the bootstrap random sampling.
#' @param silent A parameter to disable/enable verbose messages.
#' @param epos Error rate of false positive errors.
#' @param eneg Error rate of false negative errors.
#' @param do.raising Whether to use or not the raising condition as a prior.
#' @return A TRONCO compliant object with reconstructed model
#' @export tronco.gabow
#' @importFrom bnlearn hc tabu empty.graph set.arc score amat<- amat
#' @importFrom igraph graph.adjacency get.adjacency graph.union edge
#' @importFrom igraph get.shortest.paths graph_from_adjacency_matrix clusters unfold.tree
#' @importFrom igraph is.dag
#' @importFrom gtools permutations
#' @importFrom stats phyper AIC BIC logLik runif
#'
tronco.gabow <- function(data,
                         regularization = "no_reg",
                         score = "pmi",
                         do.boot = TRUE,
                         nboot = 100,
                         pvalue = 0.05,
                         min.boot = 3,
                         min.stat = TRUE,
                         boot.seed = NULL,
                         silent = FALSE,
                         epos = 0.0,
                         eneg = 0.0,
                         do.raising = TRUE) {
  if (is.null(data) || is.null(data$genotypes)) {
    stop("The dataset given as input is not valid.")
    
  }
  
  ## Enforce data to be numeric
  data = enforce.numeric(data)
  
  ## Check for the inputs to be correct.
  
  if (is.null(data$hypotheses)) {
    data$hypotheses = NA
    
  }
  
  if (pvalue < 0 || pvalue > 1) {
    stop("The value of the pvalue has to be in [0:1]!", call. = FALSE)
    
  }
  
  if (!all(regularization %in% c('no_reg', 'loglik', 'bic', 'aic'))) {
    stop("Possible regularization are no-reg, loglik, bic or aic",
         call. = FALSE)
    
  }
  
  if (!all(score %in% c('pmi', 'mi', 'entropy', 'cpmi'))) {
    stop("Possible scores are pmi, mi, entropy or cpmi", call. = FALSE)
  }
  
  if (epos < 0 || epos >= 0.5 || eneg < 0 || eneg >= 0.5) {
    stop("The values of the error rates have to be in [0:0.5)!",
         call. = FALSE)
  }
  
  ## Check for the input to be compliant.
  
  is.compliant(data)
  
  ## check if there are hypotheses
  
  if (npatterns(data) > 0) {
    warning("Patters found in input for tronco.gabow\n")
  }
  
  ## Reconstruct the reconstruction with MLE.
  
  if (is.null(boot.seed)) {
    my.seed = "NULL"
  }
  else {
    my.seed = boot.seed
    
  }
  if (silent == FALSE) {
    cat('*** Checking input events.\n')
    invalid = consolidate.data(data, TRUE)
    if (length(unlist(invalid)) > 0)
      warning("Input events should be consolidated - see consolidate.data.")
    
    
    cat(
      paste0(
        '*** Inferring a progression model with the following settings.\n',
        '\tDataset size: n = ',
        nsamples(data),
        ', m = ',
        nevents(data),
        '.\n',
        '\tAlgorithm: Gabow with \"',
        paste0(regularization, collapse = ", "),
        '\" regularization',
        '\tRandom seed: ',
        my.seed,
        '.\n',
        '\tBootstrap iterations (Wilcoxon): ',
        ifelse(do.boot, nboot, 'disabled'),
        '.\n',
        ifelse(
          do.boot,
          paste0(
            '\t\texhaustive bootstrap: ',
            min.stat,
            '.\n\t\tp-value: ',
            pvalue,
            '.\n\t\tminimum bootstrapped scores: ',
            min.boot,
            '.\n'
          ),
          ''
        )
      )
    )
  }
  
  reconstruction =
    gabow.fit(
      data$genotypes,
      regularization = regularization,
      score = score,
      do.boot = do.boot,
      nboot = nboot,
      pvalue = pvalue,
      min.boot = min.boot,
      min.stat = min.stat,
      boot.seed = boot.seed,
      silent = silent,
      epos = epos,
      eneg = eneg,
      do.raising = do.raising,
      hypotheses = data$hypotheses
    )
  
  ## Structure to save the results.
  results = data
  results$adj.matrix.prima.facie = reconstruction$adj.matrix.prima.facie
  results$confidence = reconstruction$confidence
  results$model = reconstruction$model
  results$parameters = reconstruction$parameters
  results$execution.time = reconstruction$execution.time
  
  ## Add BIC/AIC/LogLik informations
  
  if (!silent) {
    cat('*** Evaluating BIC / AIC / LogLik informations.\n')
  }
  
  search_scores = score
  
  is.acyclic = TRUE
  
  models.adj.matrix = as.adj.matrix(results)
  
  if ("no_reg" %in% regularization) {
    for (my_s in search_scores) {
      mod.name = paste('gabow_no_reg', my_s, sep = "_")
      this.matrix = models.adj.matrix[[mod.name]]
      if (is.dag(graph.adjacency(this.matrix))) {
        bayes.net = as.bnlearn.network(results, model = mod.name)
        score = logLik(bayes.net$net, data = bayes.net$data)
        logLik = score
      } else {
        score = -1
        logLik = -1
        is.acyclic = FALSE
      }
      results$model[[paste('gabow_no_reg', my_s, sep = "_")]]$score = score
      results$model[[paste('gabow_no_reg', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  if ("loglik" %in% regularization) {
    for (my_s in search_scores) {
      mod.name = paste('gabow_loglik', my_s, sep = "_")
      this.matrix = models.adj.matrix[[mod.name]]
      if (is.dag(graph.adjacency(this.matrix))) {
        bayes.net = as.bnlearn.network(results, model = mod.name)
        score = logLik(bayes.net$net, data = bayes.net$data)
        logLik = score
      } else {
        score = -1
        logLik = -1
        is.acyclic = FALSE
      }
      results$model[[paste('gabow_loglik', my_s, sep = "_")]]$score = score
      results$model[[paste('gabow_loglik', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  if ("bic" %in% regularization) {
    for (my_s in search_scores) {
      mod.name = paste('gabow_bic', my_s, sep = "_")
      this.matrix = models.adj.matrix[[mod.name]]
      if (is.dag(graph.adjacency(this.matrix))) {
        bayes.net = as.bnlearn.network(results, model = mod.name)
        score = BIC(bayes.net$net, data = bayes.net$data)
        logLik = logLik(bayes.net$net, data = bayes.net$data)
      } else {
        score = -1
        logLik = -1
        is.acyclic = FALSE
      }
      results$model[[paste('gabow_bic', my_s, sep = "_")]]$score = score
      results$model[[paste('gabow_bic', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  if ("aic" %in% regularization) {
    for (my_s in search_scores) {
      mod.name = paste('gabow_aic', my_s, sep = "_")
      this.matrix = models.adj.matrix[[mod.name]]
      
      if (is.dag(graph.adjacency(this.matrix))) {
        bayes.net = as.bnlearn.network(results, model = mod.name)
        score = AIC(bayes.net$net, data = bayes.net$data)
        logLik = logLik(bayes.net$net, data = bayes.net$data)
      } else {
        score = -1
        logLik = -1
        is.acyclic = FALSE
      }
      results$model[[paste('gabow_aic', my_s, sep = "_")]]$score = score
      results$model[[paste('gabow_aic', my_s, sep = "_")]]$logLik = logLik
    }
  }
  
  ## the reconstruction has been completed.
  
  if (!is.acyclic) {
    save(results, file = paste0('result_',
                                as.character(as.integer(runif(
                                  1
                                ) * 10000)),
                                '.RData'))
  }
  
  if (!silent)
    cat(paste(
      "The reconstruction has been successfully completed in",
      format(.POSIXct(
        round(reconstruction$execution.time[3],
              digits = 0),
        tz = "GMT"
      ),
      "%Hh:%Mm:%Ss"),
      "\n"
    ))
  
  
  return(results)
}


#' Reconstruct a progression model using Chow Liu
#' algorithm combined with probabilistic causation. For details and examples
#' regarding the inference process and on the algorithm implemented in the package,
#' we refer to the Vignette Section 6.
#'
#' @examples
#' data(test_dataset_no_hypos)
#' recon = tronco.chowliu(test_dataset_no_hypos, nboot = 1)
#'
#' @title Tronco Chow Liu
#' @param data A TRONCO compliant dataset.
#' @param regularization Select the regularization for the
#' likelihood estimation, e.g., BIC, AIC.
#' @param do.boot A parameter to disable/enable the estimation
#' of the error rates give the reconstructed model.
#' @param nboot Number of bootstrap sampling (with rejection)
#' to be performed when estimating the selective advantage scores.
#' @param pvalue Pvalue to accept/reject the valid selective
#' advantage relations.
#' @param min.boot Minimum number of bootstrap sampling
#' to be performed.
#' @param min.stat A parameter to disable/enable the minimum
#' number of bootstrap sampling required besides nboot if
#' any sampling is rejected.
#' @param boot.seed Initial seed for the bootstrap random
#' sampling.
#' @param silent A parameter to disable/enable verbose
#' messages.
#' @param epos Error rate of false positive errors.
#' @param eneg Error rate of false negative errors.
#' @return A TRONCO compliant object with reconstructed
#' model
#' @export tronco.chowliu
#' @importFrom bnlearn hc tabu empty.graph set.arc chow.liu
#' @importFrom igraph graph.adjacency get.adjacency graph.union edge
#' @importFrom igraph get.shortest.paths is.dag
#' @importFrom stats phyper AIC BIC
#'
tronco.chowliu <- function(data,
                           regularization = c("bic", "aic"),
                           do.boot = TRUE,
                           nboot = 100,
                           pvalue = 0.05,
                           min.boot = 3,
                           min.stat = TRUE,
                           boot.seed = NULL,
                           silent = FALSE,
                           epos = 0.0,
                           eneg = 0.0) {
  ## Check for the inputs to be correct.
  
  if (is.null(data) || is.null(data$genotypes)) {
    stop("The dataset given as input is not valid.")
    
  }
  
  ## Enforce data to be numeric
  data = enforce.numeric(data)
  
  if (is.null(data$hypotheses)) {
    data$hypotheses = NA
    
  }
  
  if (pvalue < 0 || pvalue > 1) {
    stop("The value of the pvalue has to be in [0:1]!", call. = FALSE)
    
  }
  
  if (!all(regularization %in% c('loglik', 'bic', 'aic'))) {
    stop("Possible regularization are loglik, bic or aic", call. = FALSE)
    
  }
  
  if (epos < 0 || epos >= 0.5 || eneg < 0 || eneg >= 0.5) {
    stop("The values of the error rates have to be in [0:0.5)!",
         call. = FALSE)
  }
  
  ## Check for the input to be compliant.
  
  is.compliant(data)
  
  ## check if there are hypotheses
  
  if (npatterns(data) > 0) {
    warning("Patters found in input for tronco.chow.liu\n")
  }
  
  ## Reconstruct the reconstruction with Chow Liu.
  
  if (is.null(boot.seed)) {
    my.seed = "NULL"
  }
  else {
    my.seed = boot.seed
    
  }
  if (silent == FALSE) {
    cat('*** Checking input events.\n')
    invalid = consolidate.data(data, TRUE)
    if (length(unlist(invalid)) > 0)
      warning("Input events should be consolidated - see consolidate.data.")
    
    
    cat(
      paste0(
        '*** Inferring a progression model with the following settings.\n',
        '\tDataset size: n = ',
        nsamples(data),
        ', m = ',
        nevents(data),
        '.\n',
        '\tAlgorithm: Chow Liu with \"',
        paste0(regularization, collapse = ", "),
        '\" regularization',
        '\tRandom seed: ',
        my.seed,
        '.\n',
        '\tBootstrap iterations (Wilcoxon): ',
        ifelse(do.boot, nboot, 'disabled'),
        '.\n',
        ifelse(
          do.boot,
          paste0(
            '\t\texhaustive bootstrap: ',
            min.stat,
            '.\n\t\tp-value: ',
            pvalue,
            '.\n\t\tminimum bootstrapped scores: ',
            min.boot,
            '.\n'
          ),
          ''
        )
      )
    )
  }
  
  reconstruction =
    chow.liu.fit(
      data$genotypes,
      regularization = regularization,
      do.boot = do.boot,
      nboot = nboot,
      pvalue = pvalue,
      min.boot = min.boot,
      min.stat = min.stat,
      boot.seed = boot.seed,
      silent = silent,
      epos = epos,
      eneg = eneg,
      hypotheses = data$hypotheses
    )
  
  ## Structure to save the results.
  
  results = data
  
  results$adj.matrix.prima.facie = reconstruction$adj.matrix.prima.facie
  results$confidence = reconstruction$confidence
  
  results$model = reconstruction$model
  
  results$parameters = reconstruction$parameters
  
  results$execution.time = reconstruction$execution.time
  
  
  ## Add BIC/AIC/LogLik informations
  
  if (!silent) {
    cat('*** Evaluating BIC / AIC / LogLik informations.\n')
  }
  
  if ("loglik" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'chow_liu_loglik')
    score = logLik(bayes.net$net, data = bayes.net$data)
    logLik = score
    results$model$chow_liu_loglik$score = score
    results$model$chow_liu_loglik$logLik = logLik
  }
  
  if ("bic" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'chow_liu_bic')
    score = BIC(bayes.net$net, data = bayes.net$data)
    logLik = logLik(bayes.net$net, data = bayes.net$data)
    results$model$chow_liu_bic$score = score
    results$model$chow_liu_bic$logLik = logLik
  }
  
  if ("aic" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'chow_liu_aic')
    score = AIC(bayes.net$net, data = bayes.net$data)
    logLik = logLik(bayes.net$net, data = bayes.net$data)
    results$model$chow_liu_aic$score = score
    results$model$chow_liu_aic$logLik = logLik
  }
  
  ## the reconstruction has been completed.
  
  if (!silent)
    cat(paste(
      "The reconstruction has been successfully completed in",
      format(.POSIXct(
        round(reconstruction$execution.time[3],
              digits = 0),
        tz = "GMT"
      ),
      "%Hh:%Mm:%Ss"),
      "\n"
    ))
  
  
  return(results)
  
}


#' Reconstruct a progression model using Prim algorithm combined with probabilistic causation. For details and examples
#' regarding the inference process and on the algorithm implemented in the package,
#' we refer to the Vignette Section 6.
#'
#' @examples
#' data(test_dataset_no_hypos)
#' recon = tronco.prim(test_dataset_no_hypos, nboot = 1)
#'
#' @title Tronco Prim
#' @param data A TRONCO compliant dataset.
#' @param regularization Select the regularization for the
#' likelihood estimation, e.g., BIC, AIC.
#' @param do.boot A parameter to disable/enable the estimation
#' of the error rates give the reconstructed model.
#' @param nboot Number of bootstrap sampling (with rejection)
#' to be performed when estimating the selective advantage scores.
#' @param pvalue Pvalue to accept/reject the valid selective
#' advantage relations.
#' @param min.boot Minimum number of bootstrap sampling to
#' be performed.
#' @param min.stat A parameter to disable/enable the minimum
#' number of bootstrap sampling required besides nboot if
#' any sampling is rejected.
#' @param boot.seed Initial seed for the bootstrap random sampling.
#' @param silent A parameter to disable/enable verbose messages.
#' @param epos Error rate of false positive errors.
#' @param eneg Error rate of false negative errors.
#' @return A TRONCO compliant object with reconstructed model
#' @export tronco.prim
#' @importFrom bnlearn hc tabu empty.graph set.arc
#' @importFrom igraph get.edgelist E E<-
#' @importFrom igraph graph.adjacency get.adjacency graph.union edge
#' @importFrom igraph get.shortest.paths minimum.spanning.tree is.dag
#### @importFrom infotheo mutinformation
#' @importFrom stats phyper AIC BIC
#'
tronco.prim <- function(data,
                        regularization = "no_reg",
                        do.boot = TRUE,
                        nboot = 100,
                        pvalue = 0.05,
                        min.boot = 3,
                        min.stat = TRUE,
                        boot.seed = NULL,
                        silent = FALSE,
                        epos = 0.0,
                        eneg = 0.0) {
  ## Check for the inputs to be correct.
  
  if (is.null(data) || is.null(data$genotypes)) {
    stop("The dataset given as input is not valid.")
    
  }
  
  ## Enforce data to be numeric
  data = enforce.numeric(data)
  
  if (is.null(data$hypotheses)) {
    data$hypotheses = NA
    
  }
  
  if (pvalue < 0 || pvalue > 1) {
    stop("The value of the pvalue has to be in [0:1]!", call. = FALSE)
    
  }
  
  if (!all(regularization %in% c('no_reg', 'loglik', 'bic', 'aic'))) {
    stop("Possible regularization are no-reg, loglik, bic or aic",
         call. = FALSE)
    
  }
  
  if (epos < 0 || epos >= 0.5 || eneg < 0 || eneg >= 0.5) {
    stop("The values of the error rates have to be in [0:0.5)!",
         call. = FALSE)
  }
  
  ## Check for the input to be compliant.
  
  is.compliant(data)
  
  ## check if there are hypotheses
  
  if (npatterns(data) > 0) {
    warning("Patters found in input for tronco.prim\n")
  }
  
  ## Reconstruct the reconstruction with Prim.
  
  if (is.null(boot.seed)) {
    my.seed = "NULL"
  }
  else {
    my.seed = boot.seed
    
  }
  if (silent == FALSE) {
    cat('*** Checking input events.\n')
    invalid = consolidate.data(data, TRUE)
    if (length(unlist(invalid)) > 0)
      warning("Input events should be consolidated - see consolidate.data.")
    
    
    cat(
      paste0(
        '*** Inferring a progression model with the following settings.\n',
        '\tDataset size: n = ',
        nsamples(data),
        ', m = ',
        nevents(data),
        '.\n',
        '\tAlgorithm: Prim with \"',
        paste0(regularization, collapse = ", "),
        '\" regularization',
        '\tRandom seed: ',
        my.seed,
        '.\n',
        '\tBootstrap iterations (Wilcoxon): ',
        ifelse(do.boot, nboot, 'disabled'),
        '.\n',
        ifelse(
          do.boot,
          paste0(
            '\t\texhaustive bootstrap: ',
            min.stat,
            '.\n\t\tp-value: ',
            pvalue,
            '.\n\t\tminimum bootstrapped scores: ',
            min.boot,
            '.\n'
          ),
          ''
        )
      )
    )
  }
  
  reconstruction =
    prim.fit(
      data$genotypes,
      regularization = regularization,
      do.boot = do.boot,
      nboot = nboot,
      pvalue = pvalue,
      min.boot = min.boot,
      min.stat = min.stat,
      boot.seed = boot.seed,
      silent = silent,
      epos = epos,
      eneg = eneg,
      hypotheses = data$hypotheses
    )
  
  ## Structure to save the results.
  
  results = data
  
  results$adj.matrix.prima.facie = reconstruction$adj.matrix.prima.facie
  results$confidence = reconstruction$confidence
  
  results$model = reconstruction$model
  
  results$parameters = reconstruction$parameters
  
  results$execution.time = reconstruction$execution.time
  
  
  ## Add BIC/AIC/LogLik informations
  
  if (!silent) {
    cat('*** Evaluating BIC / AIC / LogLik informations.\n')
  }
  
  if ("no_reg" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'prim_no_reg')
    score = logLik(bayes.net$net, data = bayes.net$data)
    logLik = score
    results$model$prim_no_reg$score = score
    results$model$prim_no_reg$logLik = logLik
  }
  
  if ("loglik" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'prim_loglik')
    score = logLik(bayes.net$net, data = bayes.net$data)
    logLik = score
    results$model$prim_loglik$score = score
    results$model$prim_loglik$logLik = logLik
  }
  
  if ("bic" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'prim_bic')
    score = BIC(bayes.net$net, data = bayes.net$data)
    logLik = logLik(bayes.net$net, data = bayes.net$data)
    results$model$prim_bic$score = score
    results$model$prim_bic$logLik = logLik
  }
  
  if ("aic" %in% regularization) {
    bayes.net = as.bnlearn.network(results, model = 'prim_aic')
    score = AIC(bayes.net$net, data = bayes.net$data)
    logLik = logLik(bayes.net$net, data = bayes.net$data)
    results$model$prim_aic$score = score
    results$model$prim_aic$logLik = logLik
  }
  
  ## the reconstruction has been completed
  if (!silent)
    cat(paste(
      "The reconstruction has been successfully completed in",
      format(.POSIXct(
        round(reconstruction$execution.time[3],
              digits = 0),
        tz = "GMT"
      ),
      "%Hh:%Mm:%Ss"),
      "\n"
    ))
  
  
  return(results)
  
}


#' Bootstrap a reconstructed progression model. For details and examples
#' regarding the statistical assesment of an inferred model,
#' we refer to the Vignette Section 7.
#'
#' @examples
#' data(test_model)
#' boot = tronco.bootstrap(test_model, nboot = 1, cores.ratio = 0)
#'
#' @title tronco bootstrap
#' @param reconstruction The output of tronco.capri or
#' tronco.caprese
#' @param type Parameter to define the type of sampling
#' to be performed, e.g., non-parametric for uniform sampling.
#' @param nboot Number of bootstrap sampling to be performed
#' when estimating the model confidence.
#' @param cores.ratio Percentage of cores to use
#' coresRate * (numCores - 1)
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO compliant object with reconstructed model
#' @importFrom doParallel registerDoParallel
#' @importFrom foreach foreach %dopar%
#' @importFrom iterators icount
#' @importFrom parallel stopCluster makeCluster detectCores
#' @export tronco.bootstrap
#'
tronco.bootstrap <- function(reconstruction,
                             type = "non-parametric",
                             nboot = 100,
                             cores.ratio = 1,
                             silent = FALSE) {
  ## Check for the input to be compliant.
  is.compliant(reconstruction)
  
  ## Check for the inputs to be given.
  is.model(reconstruction)
  
  
  if (type == "statistical"
      && !((
        reconstruction$parameters$algorithm == "CAPRI"
        || reconstruction$parameters$algorithm == "PRIM"
        || reconstruction$parameters$algorithm == "CHOW_LIU"
        || reconstruction$parameters$algorithm == "EDMONDS"
      )
      && reconstruction$parameters$do.boot == TRUE
      )) {
    stop(
      paste(
        "To perform statistical bootstrap, the algorithm used for",
        "the reconstruction must be CAPRI, PRIM, CHOW_LIU or EDMONDS",
        "with bootstrap."
      ),
      call. = FALSE
    )
  }
  
  ## Set all the needed parameters to perform the bootstrap estimation
  if (!type %in% c("non-parametric", "statistical")) {
    stop(
      paste(
        "The types of bootstrap that can be performed are:",
        "non-parametric or statistical."
      ),
      call. = FALSE
    )
  }
  
  ## Perform the selected bootstrap procedure
  if (!silent) {
    cat("*** Executing now the bootstrap procedure, this may take a long time...\n")
  }
  
  parameters = as.parameters(reconstruction)
  
  if (parameters$algorithm == "CAPRESE") {
    lambda = parameters$lambda
    curr.boot = bootstrap(reconstruction,
                          type,
                          nboot,
                          cores.ratio,
                          silent = silent)
    if (!silent) {
      cat(
        "Performed",
        type,
        "bootstrap with",
        nboot,
        "resampling and",
        lambda,
        "as shrinkage parameter.\n"
      )
    }
    
  } else {
    curr.boot = bootstrap(reconstruction,
                          type,
                          nboot,
                          cores.ratio,
                          silent = silent)
    if (!silent) {
      cat("Performed", type,
          "bootstrap with", nboot,
          "resampling")
      if (parameters$do.boot == TRUE) {
        cat(" and",
            parameters$pvalue,
            "as pvalue for the statistical tests")
      }
      cat(".\n")
    }
  }
  reconstruction$bootstrap = curr.boot
  return(reconstruction)
}


#### end of file -- tronco.R

data(test_dataset)
data(test_dataset_no_hypos)

capri = tronco.capri(test_dataset, nboot = 1, silent = TRUE)
caprese = tronco.caprese(test_dataset_no_hypos, silent = TRUE)
chowliu = tronco.chowliu(test_dataset_no_hypos, nboot = 1, silent = TRUE)
edmonds = tronco.edmonds(test_dataset_no_hypos, nboot = 1, silent = TRUE)
gabow = tronco.gabow(test_dataset_no_hypos, nboot = 1, silent = TRUE)
prim = tronco.prim(test_dataset_no_hypos, nboot = 1, silent = TRUE)
npb = tronco.bootstrap(capri, nboot = 1, silent = TRUE, cores.ratio = 0)
sb = tronco.bootstrap(capri, nboot = 1, type = 'statistical', silent = TRUE, cores.ratio = 0)


context("CAPRESE")

test_that("tronco caprese is working", {
    expect_output(tronco.caprese(test_dataset_no_hypos), "CAPRESE")
    expect_warning(tronco.caprese(test_dataset))
    expect_error(tronco.caprese(NULL))
    expect_error(tronco.caprese(test_dataset_no_hypos, lambda = -1))
    expect_error(tronco.caprese(test_dataset_no_hypos, lambda = 2))
})

context("CAPRI")

test_that("tronco capri is working", {
    expect_output(tronco.capri(test_dataset,
        nboot = 1,
        regularization = c('bic', 'aic', 'loglik')),
    "CAPRI")
    expect_output(tronco.capri(test_dataset,
        nboot = 1,
        regularization = c('bic'),
        command = 'tabu',
        do.boot = FALSE),
    "CAPRI")
    expect_output(tronco.capri(test_dataset,
        nboot = 1,
        boot.seed = 1),
    "CAPRI")
    expect_error(tronco.capri(NULL))
    expect_error(tronco.capri(test_dataset,
        command = 'banana'))
    expect_error(tronco.capri(test_dataset,
        pvalue = -1))
    expect_error(tronco.capri(test_dataset,
        pvalue = 2))
    expect_error(tronco.capri(test_dataset,
        regularization = 'banana'))
})

context("EDMONDS")

test_that("tronco edmonds is working", {
    expect_output(tronco.edmonds(test_dataset_no_hypos,
        nboot = 1,
        regularization = c('no_reg', 'loglik', 'bic', 'aic')),
    "Edmonds")
    expect_output(tronco.edmonds(test_dataset_no_hypos,
        nboot = 1,
        boot.seed = 1),
    "Edmonds")
    expect_warning(tronco.edmonds(test_dataset,
        nboot = 1))
    expect_error(tronco.edmonds(NULL))
    expect_error(tronco.edmonds(test_dataset_no_hypos,
        command = 'banana'))
    expect_error(tronco.edmonds(test_dataset_no_hypos,
        pvalue = -1))
    expect_error(tronco.edmonds(test_dataset_no_hypos,
        pvalue = 2))
    expect_error(tronco.edmonds(test_dataset_no_hypos,
        regularization = 'banana'))
})

context("GABOW")

test_that("tronco gabow is working", {
    expect_output(tronco.gabow(test_dataset_no_hypos,
        nboot = 1,
        regularization = c('no_reg', 'loglik', 'bic', 'aic')),
    "Gabow")
    expect_output(tronco.gabow(test_dataset_no_hypos,
        nboot = 1,
        boot.seed = 1),
    "Gabow")
    expect_warning(tronco.gabow(test_dataset,
        nboot = 1))
    expect_error(tronco.gabow(NULL))
    expect_error(tronco.gabow(test_dataset_no_hypos,
        command = 'banana'))
    expect_error(tronco.gabow(test_dataset_no_hypos,
        pvalue = -1))
    expect_error(tronco.gabow(test_dataset_no_hypos,
        pvalue = 2))
    expect_error(tronco.gabow(test_dataset_no_hypos,
        regularization = 'banana'))
})

context("CHOW LIU")

test_that("tronco chow liu is working", {
    expect_output(tronco.chowliu(test_dataset_no_hypos,
        nboot = 1,
        regularization = c('loglik', 'bic', 'aic')),
    "Chow")
    expect_output(tronco.chowliu(test_dataset_no_hypos,
        nboot = 1,
        boot.seed = 1),
    "Chow")
    expect_warning(tronco.chowliu(test_dataset,
        nboot = 1))
    expect_error(tronco.chowliu(NULL))
    expect_error(tronco.chowliu(test_dataset_no_hypos,
        command = 'banana'))
    expect_error(tronco.chowliu(test_dataset_no_hypos,
        pvalue = -1))
    expect_error(tronco.chowliu(test_dataset_no_hypos,
        pvalue = 2))
    expect_error(tronco.chowliu(test_dataset_no_hypos,
        regularization = 'banana'))
})

context("PRIM")

test_that("tronco prim is working", {
    expect_output(tronco.prim(test_dataset_no_hypos,
        nboot = 1,
        regularization = c('no_reg', 'loglik', 'bic', 'aic')),
    "Prim")
    expect_output(tronco.prim(test_dataset_no_hypos,
        nboot = 1,
        boot.seed = 1),
    "Prim")
    expect_warning(tronco.prim(test_dataset,
        nboot = 1))
    expect_error(tronco.prim(NULL))
    expect_error(tronco.prim(test_dataset_no_hypos,
        command = 'banana'))
    expect_error(tronco.prim(test_dataset_no_hypos,
        pvalue = -1))
    expect_error(tronco.prim(test_dataset_no_hypos,
        pvalue = 2))
    expect_error(tronco.prim(test_dataset_no_hypos,
        regularization = 'banana'))
})

context("bootstrap")

test_that("tronco bootstrap is working", {
    expect_output(tronco.bootstrap(capri, nboot = 1, cores.ratio = 0), 'non-parametric')
    expect_output(tronco.bootstrap(capri, nboot = 1, type = 'statistical', cores.ratio = 0), 'statistical')
    expect_output(tronco.bootstrap(caprese, nboot = 1, cores.ratio = 0), 'non-parametric')
    expect_output(tronco.bootstrap(prim, nboot = 1, cores.ratio = 0), 'non-parametric')
    expect_output(tronco.bootstrap(prim, nboot = 1, type = 'statistical', cores.ratio = 0), 'statistical')
    expect_output(tronco.bootstrap(edmonds, nboot = 1, cores.ratio = 0), 'non-parametric')
    expect_output(tronco.bootstrap(edmonds, nboot = 1, type = 'statistical', cores.ratio = 0), 'statistical')
    expect_output(tronco.bootstrap(chowliu, nboot = 1, cores.ratio = 0), 'non-parametric')
    expect_output(tronco.bootstrap(chowliu, nboot = 1, type = 'statistical', cores.ratio = 0), 'statistical')
    expect_error(tronco.bootstrap(test_dataset, nboot = 1, cores.ratio = 0))
    expect_error(tronco.bootstrap(capri, type = 'supergiovane', cores.ratio = 0))
    expect_error(tronco.bootstrap(caprese, type = 'statistical', cores.ratio = 0))
})

context("confidences")

test_that('as.confidence work with reconstruction and bootstrap', {
    expect_equal(length(as.confidence(caprese, conf=c('pr', 'tp', 'hg'))), 3)
    expect_equal(length(as.confidence(npb, conf = 'npb')), 1)
    expect_error(as.confidence(capri, conf = 'npb'))
    expect_equal(length(as.confidence(sb, conf = 'sb')), 1)
    expect_error(as.confidence(capri, conf = 'sb'))
})

context("probabilities")

test_that('as.prob work with reconstruction', {
    expect_equal(length(as.joint.probs(capri)), 2)
    expect_error(as.joint.probs(capri, events = c('a')))
    expect_error(as.joint.probs(capri, type = 'banana'))
    expect_error(as.joint.probs(capri, models = 'banana'))
    expect_equal(length(as.conditional.probs(capri)), 2)
    expect_error(as.conditional.probs(capri, events = c('a')))
    expect_error(as.conditional.probs(capri, type = 'banana'))
    expect_error(as.conditional.probs(capri, models = 'banana'))
    expect_equal(length(as.marginal.probs(capri)), 2)
    expect_error(as.marginal.probs(capri, events = c('a')))
    expect_error(as.marginal.probs(capri, type = 'banana'))
    expect_error(as.marginal.probs(capri, models = 'banana'))
    expect_equal(length(as.error.rates(capri)), 2)
    expect_error(as.error.rates(capri, models = 'banana'))
})

context("as.bootstrap.scores")

test_that('as.bootstrap.scores return results', {
    expect_equal(length(as.bootstrap.scores(npb)), 2)
    expect_equal(length(as.bootstrap.scores(sb)), 2)
    expect_error(as.bootstrap.scores(npb, events = 'asd'))
    expect_error(as.bootstrap.scores(npb, models = 'banana'))
})

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2019, The TRONCO Team (www.troncopackage.org)
#### email: tronco@disco.unimib.it
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' Import a matrix of 0/1 alterations as a TRONCO compliant
#' dataset. Input "geno" can be either a dataframe or a file name. In
#' any case the dataframe or the table stored in the file must have a
#' column for each altered gene and a rows for each sample. Colnames
#' will be used to determine gene names, if data is loaded from file
#' the first column will be assigned as rownames. For details and
#' examples regarding the loading functions provided by the package we
#' refer to the Vignette Section 3.
#' 
#' @title import.genotypes
#' @param geno Either a dataframe or a filename
#' @param event.type Any 1 in "geno" will be interpreted as a an observed alteration labeled with type "event.type"
#' @param color This is the color used for visualization of events labeled as of "event.type"
#' @return A TRONCO compliant dataset
#' @export import.genotypes
#' @importFrom utils read.table
#'
import.genotypes <- function(geno, event.type = "variant", color = "Darkgreen") {
    if (!(is.data.frame(geno) || is.matrix(geno)) && is.character(geno)) {
        cat('*** Input "geno" is a character, interpreting it as a filename to load a table.
             Required table format:
             \t- one column for each gene, one row for each gene;
             \t- colnames/rownames properly defined.\n')

        data =
            read.table(geno,
                       header = TRUE,
                       check.names = FALSE,
                       stringsAsFactors = FALSE)

        if (any(is.null(colnames(data))))
            stop('Input table should have column names.')
        rownames(data) = data[, 1]
        data[, 1] = NULL
        geno = data
    }

    ## Avoid malformed datasets

    if (ncol(geno) == 0 || nrow(geno) == 0)
        stop("Empty genotypes (number of rows/columns 0), will not import.")
    nc = ncol(geno)
    nr = nrow(geno)

    ## Gather col/row names

    if (is.null(colnames(geno))) {
        cn = paste0("Gene", 1:ncol(geno))
        warning("Missing column names to identify genes. Will use labels \"Gene1\", \"Gene2\", .....")
    } else
        cn = colnames(geno)

    ## Gather col/row names
    if (is.null(rownames(geno))) {
        rn = paste0("Sample", 1:nrow(geno))
        warning("Missing row names to identify samples. Will use labels \"Sample1\", \"Sample2\",  .....")
    } else rn = rownames(geno)

    x = list()

    ## Access keys - G1, G2, ...
    keys = paste0("G", 1:ncol(geno))

    ## Genotype matrix

    x$genotypes = as.matrix(geno)
    colnames(x$genotypes) = keys
    rownames(x$genotypes) = rn

    ## Create attributes
    x$annotations = matrix(0, nrow = nc, ncol = 2)
    colnames(x$annotations) = c("type", "event")
    rownames(x$annotations) = keys

    x$annotations[, "type"] = event.type
    x$annotations[, "event"] = cn

    ## We create a map from types to colors
    x$types = matrix(color, nrow = 1, ncol = 1)
    rownames(x$types) = event.type
    colnames(x$types) = c("color")

    is.compliant(x, "import.genotypes: output")

    return(x)
}


#' Transform GISTIC scores for CNAs in a TRONCO compliant object. Input can be either a matrix, with columns
#' for each altered gene and rows for each sample; in this case colnames/rownames mut be provided. If input
#' is a character an attempt to load a table from file is performed. In this case the input table format
#' should be constitent with TCGA data for focal CNA; there should hence be: one column for each sample,
#' one row for each gene, a column Hugo_Symbol with every gene name and a column Entrez_Gene_Id with every
#'  gene\'s Entrez ID. A valid GISTIC score should be any value of: "Homozygous Loss" (-2), "Heterozygous
#'  Loss" (-1), "Low-level Gain" (+1), "High-level Gain" (+2). For details and examples 
#' regarding the loading functions provided by the package we refer to the Vignette Section 3. 
#'
#' @examples
#' data(crc_gistic)
#' gistic = import.GISTIC(crc_gistic)
#' 
#' @title import.GISTIC
#' @param x Either a dataframe or a filename
#' @param filter.genes A list of genes
#' @param filter.samples A list of samples
#' @param silent A parameter to disable/enable verbose messages.
#' @param trim Remove the events without occurrence
#' @param rna.seq.data Either a dataframe or a filename
#' @param rna.seq.up TODO
#' @param rna.seq.down TODO
#' @return A TRONCO compliant representation of the input CNAs.
#' @export import.GISTIC
#' @importFrom utils read.table
#' 
import.GISTIC <- function(x,
                          filter.genes = NULL,
                          filter.samples = NULL,
                          silent = FALSE,
                          trim = TRUE,
                          rna.seq.data = NULL,
                          rna.seq.up = NULL,
                          rna.seq.down = NULL) {

    if (!(is.data.frame(x) || is.matrix(x)) && is.character(x)) {
        if (!silent) {
            cat('*** Input "x" is a character, interpreting it as a filename to load a table.
                Required table format constitent with TCGA data for focal CNAs:
                \t- one column for each sample, one row for each gene;
                \t- a column Hugo_Symbol with every gene name;
                \t- a column Entrez_Gene_Id with every gene\'s Entrez ID.\n')
        }

        data =
            read.table(x,
                       header = TRUE,
                       check.names = FALSE,
                       stringsAsFactors = FALSE)

        if (!silent) {
            cat('Data loaded.\n')
        }
        if (any(is.null(colnames(data)))) {
            stop('Input table should have column names.')
        }
        if (!'Hugo_Symbol' %in% colnames(data)) {
            stop('Missing Hugo_Symbol column!')
        }
        if (!'Entrez_Gene_Id' %in% colnames(data)) {
            stop('Missing Hugo_Symbol column!')
        }
        data$Entrez_Gene_Id = NULL
        rownames(data) = data$Hugo_Symbol
        data$Hugo_Symbol = NULL
        x = t(data)
    }

    # Import RNASeq data
    if (!is.null(rna.seq.data) && !(is.data.frame(rna.seq.data) || is.matrix(rna.seq.data)) && is.character(rna.seq.data)) {

        if (!silent) {
            cat('*** Input "rna.seq.data" is a character, interpreting it as a filename to load a table.
                Required table format constitent with TCGA data for focal CNAs:
                \t- one column for each sample, one row for each gene;
                \t- a column Hugo_Symbol with every gene name;
                \t- a column Entrez_Gene_Id with every gene\'s Entrez ID.\n')
        }

        rna.seq.data.raw = 
            read.table(rna.seq.data,
                       header = TRUE,
                       check.names = FALSE,
                       stringsAsFactors = FALSE,
                       sep = ';')

        if (!silent) {
            cat('Data loaded.\n')
        }
        if (any(is.null(colnames(rna.seq.data.raw)))) {
            stop('Input table should have column names.')
        }
        if (!'Hugo_Symbol' %in% colnames(rna.seq.data.raw)) {
            stop('Missing Hugo_Symbol column!')
        }
        if (!'Entrez_Gene_Id' %in% colnames(rna.seq.data.raw)) {
            stop('Missing Hugo_Symbol column!')
        }
        rna.seq.data.raw$Entrez_Gene_Id = NULL
        rownames(rna.seq.data.raw) = rna.seq.data.raw$Hugo_Symbol
        rna.seq.data.raw$Hugo_Symbol = NULL
        rna.seq.data = t(rna.seq.data.raw)

        samples.intersection = intersect(rownames(x), rownames(rna.seq.data))
        rna.seq.data = rna.seq.data[samples.intersection, , drop = FALSE]

        gene.intersection = intersect(colnames(rna.seq.data), colnames(x))
        rna.seq.data = rna.seq.data[, gene.intersection, drop = FALSE]
    }

    if (!silent && is.null(filter.genes) && is.null(filter.samples)) {
        cat('*** Using full GISTIC: #dim ', nrow(x), ' x ', ncol(x), '\n' )
    } else if (!silent) {
        cat('*** Filtering full GISTIC: #dim ', nrow(x), ' x ', ncol(x), '\n' )
    }

    if(!is.null(filter.genes)) {
        if(!is.vector(filter.genes)) {
            stop('filter.genes - should be vector')
        }
        x = x[, which(colnames(x) %in% filter.genes), drop = FALSE ]

        if (!silent) {
            cat('*** Using reduced GISTIC: #dim ', nrow(x), ' x ', ncol(x), '\n' ) 
        }
    }

    if (!is.null(rna.seq.up) && !is.null(rna.seq.data)) {
        if (!silent) {
            cat('*** Checking expression information again positive GISTIC score\n' ) 
        }
        for (sample in rownames(x)) {
            for (gene in colnames(x)) {

                if (x[[sample, gene]] > 0 &&
                    gene %in% colnames(rna.seq.data) &&
                    sample %in% rownames(rna.seq.data) &&
                    !is.na(rna.seq.data[[sample, gene]])) {

                    if (rna.seq.data[[sample, gene]] < rna.seq.up) {
                        x[[sample, gene]] = 0
                    }
                }
            }
        }
    }

    if (!is.null(rna.seq.down) && !is.null(rna.seq.data)) {
        if (!silent) {
            cat('*** Checking expression information again negative GISTIC score\n' ) 
        }
        for (sample in rownames(x)) {
            for (gene in colnames(x)) {

                if (x[[sample, gene]] < 0 &&
                    gene %in% colnames(rna.seq.data) &&
                    sample %in% rownames(rna.seq.data) &&
                    !is.na(rna.seq.data[[sample, gene]])) {

                    if (rna.seq.data[[sample, gene]] > rna.seq.down) {
                        x[[sample, gene]] = 0
                    }
                }
            }
        }
    }

    if(!is.null(filter.samples)) {
        if(!is.vector(filter.samples)) {
            stop('filter.samples - should be vectors')
        }
        x = x[which(rownames(x) %in% filter.samples), ]
        
        if (!silent) {
            cat('*** Using reduced GISTIC: #dim ', nrow(x), ' x ', ncol(x), '\n' ) 
        }
    }

    if (!silent) {
        cat("*** GISTIC input format conversion started.\n")
    }

    ## For next operations it is convenient to have everything as
    ## 'char' rather than 'int'

    if (typeof(x[, 1]) != typeof("somechar")) {
        if (!silent) {
            cat("Converting input data to character for import speedup.\n")
        }
        rn = rownames(x)
        x = apply(x, 2, as.character)
        rownames(x) = rn
    }

    if (any(is.na(x)))  {
        warning("NA entries were replaced with 0s.\n")
    }
    x[is.na(x)] = 0

    if (!silent) {
        cat("Creating ", 4 * (ncol(x)), "events for", ncol(x), "genes \n")
    }

    # gene symbols
    enames <- colnames(x)
    if (is.null(enames)) {
        stop("Error: gistic file has no column names and can not imported, aborting!")
    }

    if (!silent) {
        cat("Extracting \"Homozygous Loss\" events (GISTIC = -2) \n")
    }
    d.homo = x
    d.homo[d.homo != -2] <- 0
    d.homo[d.homo == -2] <- 1

    if (!silent) {
        cat("Extracting \"Heterozygous Loss\" events (GISTIC = -1) \n")
    }
    d.het <- x
    d.het[d.het != -1] <- 0
    d.het[d.het == -1] <- 1

    if (!silent) {
        cat("Extracting \"Low-level Gain\" events (GISTIC = +1) \n")
    }
    d.low <- x
    d.low[d.low != 1] <- 0

    if (!silent) {
        cat("Extracting \"High-level Gain\" events (GISTIC = +2) \n")
    }
    d.high <- x
    d.high[d.high != 2] <- 0
    d.high[d.high == 2] <- 1

    if (!silent) {
        cat("Transforming events in TRONCO data types ..... \n")
    }
    d.homo = import.genotypes(d.homo, event.type = "Homozygous Loss", color = "dodgerblue4")
    d.het = import.genotypes(d.het, event.type = "Heterozygous Loss", color = "dodgerblue1")
    d.low = import.genotypes(d.low, event.type = "Low-level Gain", color = "firebrick1")
    d.high = import.genotypes(d.high, event.type = "High-level Gain", color = "firebrick4")

    d.cnv.all = ebind(d.homo, d.het, d.low, d.high, silent = silent)

    if (trim) {
        d.cnv.all = trim(d.cnv.all)
    }

    if (!silent) {
        cat("*** Data extracted, returning only events observed in at least one sample \n",
            "Number of events: n =", nevents(d.cnv.all), "\n",
            "Number of genes: |G| =", ngenes(d.cnv.all), "\n",
            "Number of samples: m =", nsamples(d.cnv.all), "\n")
    }
    is.compliant(d.cnv.all, "import.gistic: output")
    return(d.cnv.all)
}


#' Import mutation profiles from a Manual Annotation Format (MAF) file. All mutations are aggregated as a
#' unique event type labeled "Mutation" and assigned a color according to the default of function
#' \code{import.genotypes}. If this is a TCGA MAF file check for multiple samples per patient is performed
#' and a warning is raised if these occurr. Customized MAF files can be imported as well provided that 
#' they have columns Hugo_Symbol, Tumor_Sample_Barcode and Variant_Classification. 
#' Custom filters are possible (via filter.fun) to avoid loading the full MAF data. For details and examples 
#' regarding the loading functions provided by the package we refer to the Vignette Section 3. 
#'
#' @examples
#' data(maf)
#' mutations = import.MAF(maf)
#' mutations = annotate.description(mutations, 'Example MAF')
#' mutations = TCGA.shorten.barcodes(mutations)
#' oncoprint(mutations)
#' 
#' @title import.MAF
#' @param file  MAF filename
#' @param sep MAF separator, default \'\\t\'
#' @param is.TCGA TRUE if this MAF is from TCGA; thus its sample codenames can be interpreted
#' @param filter.fun A filter function applied to each row. This is expected to return TRUE/FALSE.
#' @param to.TRONCO If FALSE returns a dataframe with MAF data, not a TRONCO object
#' @param irregular If TRUE seeks only for columns Hugo_Symbol, Tumor_Sample_Barcode and Variant_Classification
#' @param paste.to.Hugo_Symbol If a list of column names, this will be pasted each Hugo_Symbol to yield names such as PHC2.chr1.33116215.33116215
#' @param merge.mutation.types If TRUE, all mutations are considered equivalent, regardless of their Variant_Classification value. Otherwise no.
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO compliant representation of the input MAF
#' @export import.MAF
#' @importFrom utils read.table read.delim count.fields flush.console
#' @importFrom grDevices colorRampPalette
#' 
import.MAF <- function(file,
                       sep = '\t',
                       is.TCGA = TRUE,
                       filter.fun = NULL,
                       to.TRONCO = TRUE,
                       irregular = FALSE,
                       paste.to.Hugo_Symbol = NULL,
                       merge.mutation.types = TRUE,
                       silent = FALSE) {

    ## Data loading.
    if (!(is.data.frame(file) || is.matrix(file)) && is.character(file)) {
        if (!silent) {
            cat("*** Importing from file: ", file, "\n")
        }
        if(!irregular) {
            if (!silent) {
                cat("Loading MAF file ...")
            }
            maf = read.delim(file, comment.char = "#", sep = sep, header = TRUE, stringsAsFactors = FALSE)
        } else {
            if (!silent) {
                cat("*** [irregular = TRUE] Seeking only Hugo_Symbol, Tumor_Sample_Barcode and Variant_Classification columns")
            }
            
            read.irregular <- function(filenm) {
                fileID <- file(filenm, open = "rt")
                nFields <- count.fields(fileID)
                mat <- matrix(nrow = length(nFields), ncol = max(nFields))
                invisible(seek(fileID, where = 0, origin = "start", rw = "read"))
                for (i in 1:nrow(mat)) {
                    mat[i, 1:nFields[i]] = scan(fileID, 
                                                what = "", 
                                                nlines = 1,
                                                quiet = TRUE)
                }
                close(fileID)
                df = data.frame(mat, stringsAsFactors = FALSE)
                return(df)
            }
            x = read.irregular(file)
          
            req.columns = c('Hugo_Symbol', 'Tumor_Sample_Barcode', 'Variant_Classification', paste.to.Hugo_Symbol)
        
            if(!all(req.columns %in% x[1,]))
                stop( paste(
                    '\nMAF file has no columns:', paste(req.columns, collapse = ' | '),
                    'Columns found:',
                    paste('\t', x[1,], collapse = '\n '), 
                    sep='\n '))
          
                maf = x[, which(x[1,] %in% req.columns)]
                colnames(maf) = maf[1,]
                maf = maf[2:nrow(maf),]
        }
        if (!silent) {
            cat("DONE\n")
        }
    } else {
        if (!silent) {
            cat("*** Importing from dataframe\n")
            cat("Loading MAF dataframe ...")
        }
        maf = file
        if (!silent) {
            cat("DONE\n")
        }
    }

    ## Build custom names
    names.columns = c('Hugo_Symbol', paste.to.Hugo_Symbol)
    if (is.vector(paste.to.Hugo_Symbol)) {
        if (!silent) {
            cat('*** Mutations names: augmenting Hugo_Symbol with values: ', paste(paste.to.Hugo_Symbol, sep = ', '), '\n')
        }
        maf[, 'Hugo_Symbol'] = 
            apply(maf[, names.columns], 
                  MARGIN = 1,
                  function(x) {return(paste(x, collapse = '.'))})
      
    } else if (!silent) {
        cat('*** Mutations names: using Hugo_Symbol\n')
    }
   
    if (is.null(filter.fun)) {
        if (!silent) {
            cat('*** Using full MAF: #entries ', nrow(maf), '\n')
        }
    } else {
        if (!is.function(filter.fun)) {
            stop('filter.fun - should be a function')
        }
        if (!silent) {
            cat('*** Filtering full MAF: #entries ', nrow(maf), '\n' ) 
        }
        maf = maf[apply(maf, 1, filter.fun), ]
        if (!silent) {
            cat('*** Using reduced MAF: #entries ', nrow(maf), '\n' )
        }
    }
  
    ## Auxiliary functions to extract information from the MAF file
    ## This is the possibly smallest type of information required to prepare a TRONCO file
    ## If any necessary information is missing, execution is aborted

    variants <- function(x) {
        if (!("Variant_Classification" %in% colnames(x)))
            warning("Missing Variant_Classification flag in MAF file.")

        return(unique(x[, "Variant_Classification"]))
    }

    samples <- function(x) {
        if (!("Tumor_Sample_Barcode" %in% colnames(x)))
            stop("Missing Tumor_Sample_Barcode flag in MAF file - will not import.")

        return(unique(x[, "Tumor_Sample_Barcode"]))
    }

    genes <- function(x) {
        if (!("Hugo_Symbol" %in% colnames(x)))
            stop("Missing Hugo_Symbol flag in MAF file - will not import.")

        return(unique(x[, "Hugo_Symbol"]))
    }

    valid.calls <- function(x) {
        if (!("Validation_Status" %in% colnames(x)))
            warning("Missing Validation_Status flag in MAF file.")
        else return(which(x[, "Validation_Status"] == "Valid"))
    }

    as.TCGA.patients <- function(x) {
        samples = samples(x)
        patients = substr(samples, 0, 12)

        return(unique(patients))
    }

    ## General report about the mutations stored in this MAF

    if (!silent) {
        cat("*** MAF report: ")
    }
    if (is.TCGA && !silent) {
        cat("TCGA=TRUE")
    }

    MAF.variants = variants(maf)
    MAF.samples = samples(maf)

    if (!silent) {
        cat("\nType of annotated mutations: \n")
        print(MAF.variants)
    }
    if(!silent && merge.mutation.types) {
        cat('*** [merge.mutation.types = T] Mutations will be merged and annotated as \'Mutation\'\n')
    } else if (!silent) {
        cat('*** [merge.mutation.types = F] Mutations will be distinguished by type\n')
    }
    
    if (!silent) {
        cat("Number of samples:", length(MAF.samples), "\n")
    }

    ## If it is TCGA you should check for multiple samples per patient

    if (is.TCGA) {
        TCGA.patients = as.TCGA.patients(maf)
        if (!silent) {
            cat("[TCGA = TRUE] Number of TCGA patients:", length(TCGA.patients), "\n")
        }

        if (length(TCGA.patients) != length(MAF.samples)) {
            warning("This MAF contains duplicate samples for some patients - use TCGA functions for further information")
        }
    }

    which.valid.calls = valid.calls(maf)
    n.valid = length(which.valid.calls)
    if (!silent) {
        cat("Number of annotated mutations:", nrow(maf), "\n")
    }

    if (!silent && ("Validation_Status" %in% colnames(maf))) {
        cat("Mutations annotated with \"Valid\" flag (%):", round(n.valid/nrow(maf) * 100, 0), "\n")
    } else if (!silent) {
        cat("Mutations annotated with \"Valid\" flag (%): missing flag\n")
    }
    
    
    MAF.genes = genes(maf)
    if (!silent) {
        cat("Number of genes (Hugo_Symbol):", length(MAF.genes), "\n")
    }
    
    if (!to.TRONCO) {
        return(maf)
    }
    
    if(merge.mutation.types) {
        if (!silent) {
            cat("Starting conversion from MAF to 0/1 mutation profiles (1 = mutation) :")
            cat(length(MAF.samples), "x", length(MAF.genes), "\n")
            #flush.console()
            #pb <- txtProgressBar(1, nrow(maf), style = 3)
        }


        ## Temporary binary matrix

        binary.mutations = matrix(0, nrow = length(MAF.samples), ncol = length(MAF.genes))

        colnames(binary.mutations) = MAF.genes
        rownames(binary.mutations) = MAF.samples

        for (i in 1:nrow(maf)) {
            #if (!silent) {
            #    setTxtProgressBar(pb, i)
            #}
            if (!silent) {
                cat('.')
            }
            binary.mutations[maf$Tumor_Sample_Barcode[i], maf$Hugo_Symbol[i]] = 1
        }

        if (!silent) {
            #close(pb)
            cat("\nStarting conversion from MAF to TRONCO data type.\n")
        }
        tronco.data = import.genotypes(binary.mutations, event.type = "Mutation")
        is.compliant(tronco.data)
    } else {
        if (!silent) {
            cat("Starting conversion from MAF to 0/1 mutation profiles (1 = mutation) :\n")
            #flush.console()
            #pb <- txtProgressBar(1, nrow(maf), style = 3)
        }


        # For this case -- separate mutation types -- it is less convenient to use import.genotypes 
        tronco.data = list()
        tronco.data$genotypes = NULL
        tronco.data$annotations = NULL
        tronco.data$types = NULL

        
        # First, build the list of events that we will be creating
        tronco.data$annotations = as.matrix(unique(maf[, c('Variant_Classification', 'Hugo_Symbol')]))
        colnames(tronco.data$annotations) = c('type', 'event')
        rownames(tronco.data$annotations) = paste('G', 1:nrow(tronco.data$annotations), sep='')
        
        tronco.data$genotypes = matrix(0, nrow = length(MAF.samples), ncol = nrow(tronco.data$annotations))
        colnames(tronco.data$genotypes) = rownames(tronco.data$annotations)
        rownames(tronco.data$genotypes) = MAF.samples

        for (i in 1:nrow(maf)) {
            #if (!silent) {
            #    setTxtProgressBar(pb, i)
            #}
            if (!silent) {
                cat('.')
            }
            ev = intersect(
                which(tronco.data$annotations[, 'event'] == maf$Hugo_Symbol[i]), 
                which(tronco.data$annotations[, 'type']  == maf$Variant_Classification[i])
                )

            tronco.data$genotypes[maf$Tumor_Sample_Barcode[i], ev] = 1
        }

        #if (!silent) {
        #    close(pb)
        #}
        if (!silent) {
            cat('\n')
        }
        
        
        colors = colorRampPalette(brewer.pal(8, 'Accent'))(length(MAF.variants))
        
        tronco.data$types = matrix(colors, ncol = 1)
        
        rownames(tronco.data$types) = MAF.variants
        colnames(tronco.data$types) = 'color'

        is.compliant(tronco.data)           
    }

    return(tronco.data)
}


#' Extract a map Hugo_Symbol -> Entrez_Gene_Id from a MAF input file. If some genes map to ID 0
#' a warning is raised.
#' 
#' @title extract.MAF.HuGO.Entrez.map
#' @param file  MAF filename
#' @param sep MAF separator, default \'\\t\'
#' @return A mapHugo_Symbol -> Entrez_Gene_Id.
#' @export extract.MAF.HuGO.Entrez.map
#' @importFrom utils read.delim
#' 
extract.MAF.HuGO.Entrez.map <- function(file, sep = "\t") {
    cat("*** Importing from file: ", file, "\n")
    cat("Loading MAF file ...")

    maf = read.delim(file, comment.char = "#", sep = sep, header = TRUE, stringsAsFactors = FALSE)
    cat("DONE\n")

    map = unique(maf[, c("Hugo_Symbol", "Entrez_Gene_Id")])
    map.missing = which(map[, "Entrez_Gene_Id"] == 0)
    map.missing = map[map.missing, "Hugo_Symbol", drop = FALSE]

    if (nrow(map.missing) > 0)
        warning("The are Hugo_Symbol with Entrez_Gene_Id equal to 0")
    else cat("Map seem consistent (non-zero Entrez_Gene_Id) for", nrow(map), "genes")

    return(map)
}


## Wrapper for the CGDS package to query the Cbio portal. This can
## work either automatically, if one sets \code{cbio.study},
## \code{cbio.dataset} or \code{cbio.profile}, or interactively
## otherwise. A list of genes to query with less than 900 entries should
## be provided. This function returns a list with two dataframe: the
## gentic profile required and clinical data for the Cbio study. Output
## is also saved to disk as Rdata file. See also
## http://www.cbioportal.org.
##
## @title cbio.query
## @param cbio.study  Cbio study ID
## @param cbio.dataset Cbio dataset ID
## @param cbio.profile Cbio genetic profile ID
## @param genes A list of < 900 genes to query
## @param file String containing filename for RData output. If NA no output will be provided
## @return A list with two dataframe: the gentic profile required and clinical data for the Cbio study.
## @export cbio.query
## @importFrom cgdsr CGDS getCancerStudies getCaseLists 
## @importFrom cgdsr getGeneticProfiles getProfileData getClinicalData
## @importFrom utils write.table
## 
#cbio.query <- function(cbio.study = NA,
#                       cbio.dataset = NA,
#                       cbio.profile = NA,
#                       genes,
#                       file = NA) {
#    cat("*** CGDS plugin for Cbio query.\n")
#    ## require("cgdsr")
#
#    if (is.null(genes) || is.na(genes) || length(genes) == 0) {
#        stop('Empty list of genes to query')
#    }
#    if (length(genes) > 900) {
#        stop('URL with more than 900 genes will not be accepted, please split it.')
#    }
#
#    if (is.na(cbio.study)) {
#        cat("\nAutomatic CBIO study assessment: off")
#    } else {
#        cat(paste("\nAutomatic CBIO study index: ", cbio.study, sep = ""))
#    }
#
#    if (is.na(cbio.dataset)) {
#        cat("\nAutomatic CBIO dataset assessment: off")
#    } else {
#        cat(paste("\nAutomatic CBIO dataset index: ", cbio.dataset, sep = ""))
#    }
#    
#    if (is.na(cbio.profile)) {
#        cat("\nAutomatic CBIO profile assessment: off")
#    } else {
#        cat(paste("\nAutomatic CBIO profile index: ", cbio.profile, sep = ""))
#    }
#    
#    ## mycgds = CGDS("http://www.cbioportal.org/public-portal/")
#    ## New version of the CGDS-R library does not want the
#    ## "public-portal".
#    
#    mycgds = CGDS("http://www.cbioportal.org/")
#
#    cs = getCancerStudies(mycgds)
#    if (is.na(cbio.study)) {
#        cat("\nAvailable studies at CBIO portal.\n")
#        print(cs[c("cancer_study_id", "name")])
#
#        repeat{
#            cbio.study <- readline(prompt = "Enter CBIO study id: ")
#            if (cbio.study %in% cs$cancer_study_id)
#                break
#        }
#    }
#
#    ## Get available case lists (collection of samples) for a given
#    ## cancer study
#
#    mycancerstudy <- cbio.study
#
#    if (is.na(mycancerstudy)) {
#        stop("CBIO study id invalid. Aborting.")
#    }
#
#    study <- cs[cs$cancer_study_id == cbio.study, , drop = FALSE]
#
#    cat(paste("\nCancer codename: ", study[, 1], sep = ""))
#    cat(paste("\nCancer Ref.: ", study[, 2], sep = ""))
#    cat(paste("\nCancer Syn.: ", study[, 3], sep = ""))
#
#    cutdescr = function(x, n) {
#        x[, ncol(x)] = ifelse(
#            nchar(x[, ncol(x)]) > n,
#            paste0(substr(x[, ncol(x)], 1, n), '....'),
#            x[, ncol(x)])
#        return(x)
#    }
#
#    ## Get dataset for the study
#
#    csl = getCaseLists(mycgds, cbio.study)
#    if (is.na(cbio.dataset)) {
#        cat("\nAvailable datasets for study:", cbio.study, "\n")
#        print(cutdescr(csl[c("case_list_id",  "case_list_description")], 90))
#
#        repeat{
#            cbio.dataset <- readline(prompt = "Enter study dataset id: ")
#            if (cbio.dataset %in% csl$case_list_id) break
#        }
#    }
#
#    caselist =  csl[csl$case_list_id == cbio.dataset, , drop = FALSE]
#
#    if (any(is.na(caselist))) stop("No data for selected study. Aborting.")
#
#
#    cat(paste("\nData codename: ", caselist[, 1], sep = ""))
#    cat(paste("\nData Ref.: ", caselist[, 2], sep = ""))
#    cat(paste("\nData Syn.: ", caselist[, 3], sep = ""))
#
#    ## Get available genetic profiles
#
#    gp = getGeneticProfiles(mycgds, cbio.study)
#    if (is.na(cbio.profile)) {
#        cat("\nAvailable genetic profiles for selected datasets.\n")
#        print(cutdescr(gp[c("genetic_profile_id", "genetic_profile_description")], 90))
#
#        repeat{
#            cbio.profile <- readline(prompt = "Enter genetic profile id: ")
#            if (cbio.profile %in% gp$genetic_profile_id) break
#        }
#    }
#
#    profile = gp[gp$genetic_profile_id == cbio.profile, , drop = FALSE]
#
#    if (any(is.na(cbio.profile))) {
#        stop("No samples for this profile. Aborting")
#    }
#
#    samples.name <- profile[1, 1]
#    samples.ref <- profile[1, 2]
#    samples.syn <- profile[1, 3]
#    samples.id <- profile[1, 4]
#
#    cat(paste("\nSamples codename: ", samples.name, sep = ""))
#    cat(paste("\nData Ref.: ", samples.ref, sep = ""))
#    cat(paste("\nData Syn.: ", samples.syn, sep = ""))
#
#    cat("\n\nQuerying the following list of genes: ")
#    cat(paste(genes, collapse = ", "), '\n')
#
#    ## Get data slices for a specified list of genes, genetic profile
#    ## and case list
#
#    data <- getProfileData(mycgds, genes, samples.name, cbio.dataset)
#    rownames(data) = gsub('\\.', '-', rownames(data))
#    cat('Symbol \".\" was replaced with "-" in sample IDs.\n')
#
#    ## Export
#
#    cat(paste("\nData retrieved: ", nrow(data), " samples, ", ncol(data), " genes.", sep = ""))
#
#    ## Get clinical data for the case list
#
#    cat("\nRetrieved also clinical data for samples:", cbio.dataset)
#
#    clinicaldata = getClinicalData(mycgds, cbio.dataset)
#    rownames(clinicaldata) = gsub('\\.', '-', rownames(clinicaldata))
#
#    ret = NULL
#    ret$profile = data
#    ret$clinical = clinicaldata
#
#    if (!is.na(file)) {
#        save(ret, file=file)
#    }
#
#    return(ret)
#}
#
#




#' Add an adjacency matrix as a model to a TRONCO compliant object.
#' Input model can be either a dataframe or a file name. 
#' 
#' @title import.model
#' @param tronco_object A TRONCO compliant object
#' @param model Either a dataframe or a filename
#' @param model.name Name of the imported model
#' @return A TRONCO compliant object
#' @export import.model
#' @importFrom utils read.table
#'
import.model <- function(tronco_object, model, model.name = "imported_model") {

    if (!(is.data.frame(model) || is.matrix(model)) && is.character(model)) {
        cat('*** Input "model" is a character, interpreting it as a filename to load a table.
             Required table format:
             \t- one column for each gene, one row for each gene, without header.\n')

        model =
            read.table(model,
                       header = FALSE,
                       check.names = FALSE,
                       stringsAsFactors = FALSE)
    }

    ## Check for the input to be compliant
    is.compliant(tronco_object)

    ## Check for the model to be of the correct dimention
    if(dim(model)[1]!=nevents(tronco_object)||dim(model)[2]!=nevents(tronco_object)) {
        stop('The adjacency matrix of the model to be imported does not have the exact size.')
    }

    ## Add the model
    results = tronco_object
    results$model[[model.name]] = list()
    rownames(model) = colnames(as.genotypes(results))
    colnames(model) = colnames(as.genotypes(results))
    results$model[[model.name]]$adj.matrix$adj.matrix.fit = model

	return(results)

}


#### end of file -- loading.R

#### TRONCO: a tool for TRanslational ONCOlogy
####
#### Copyright (c) 2015-2017, Marco Antoniotti, Giulio Caravagna, Luca De Sano,
#### Alex Graudenzi, Giancarlo Mauri, Bud Mishra and Daniele Ramazzotti.
####
#### All rights reserved. This program and the accompanying materials
#### are made available under the terms of the GNU GPL v3.0
#### which accompanies this distribution.


#' Return all genotypes for input 'x', which should be a TRONCO compliant dataset
#' see \code{is.compliant}. 
#' Function \code{keysToNames} can be used to translate colnames to events. 
#' @title as.genotypes
#'
#' @examples
#' data(test_dataset)
#' as.genotypes(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return A TRONCO genotypes matrix.
#' @export as.genotypes
#' 
as.genotypes <- function(x) {
    return(x$genotypes)
}


#' Return all sample IDs for input 'x', which should be a TRONCO compliant dataset - see \code{is.compliant}. 
#' @title as.samples
#'
#' @examples
#' data(test_dataset)
#' as.samples(test_dataset)
#' 
#' @param x A TRONCO compliant dataset.
#' @return A vector of sample IDs
#' @export as.samples
#' 
as.samples <- function(x) {
    return(rownames(x$genotypes))
}


#' Return all gene symbols for which a certain type of event exists in 'x', which should be a
#' TRONCO compliant dataset - see \code{is.compliant}. 
#' @title as.genes
#'
#' @examples
#' data(test_dataset)
#' as.genes(test_dataset)
#' 
#' @param x A TRONCO compliant dataset.
#' @param types The types of events to consider, if NA all available types are used.
#' @return A vector of gene symbols for which a certain type of event exists
#' @export as.genes
#' 
as.genes <- function(x, types = NA) {
    if ('Pattern' %in% types) {
        stop('"Pattern" is not a valid gene type, it is a reseverd keyword in TRONCO.')
    }
    
    if (npatterns(x) > 0) {
        ev = as.events(x, types = types)
        ev = ev[which(ev[ , 'type'] != 'Pattern'), 'event']
        return(unique(ev))
    } else
        return(unique(as.events(x, types = types)[ , 'event']))
}


#' Return all events involving certain genes and of a certain type in 'x', which should be a
#' TRONCO compliant dataset - see \code{is.compliant}. 
#' @title as.events
#'
#' @examples
#' data(test_dataset)
#' as.events(test_dataset)
#' as.events(test_dataset, types='ins_del')
#' as.events(test_dataset, genes = 'TET2')
#' as.events(test_dataset, types='Missing')
#'
#' @param x A TRONCO compliant dataset.
#' @param types The types of events to consider, if NA all available types are used.
#' @param genes The genes to consider, if NA all available genes are used.
#' @param keysToNames If TRUE return a list of mnemonic name composed by type + gene
#' @return A matrix with 2 columns (event type, gene name) for the events found.
#' @export as.events
#' 
as.events <- function(x, genes = NA, types = NA, keysToNames = FALSE) {
    ann = x$annotations[ , c('type', 'event'), drop = FALSE]

    if (!any(is.na(genes)))
        ann = ann[which(ann[, 'event', drop=FALSE] %in% genes), , drop = FALSE]
    
    if (!any(is.na(types)))
        ann = ann[which(ann[, 'type', drop = FALSE] %in% types), , drop = FALSE] 

    if (keysToNames) {
        ann = keysToNames(x, ann)
        names(colnames(ann)) = NULL
        names(rownames(ann)) = NULL
        return(rownames(ann))
    }  

    return(ann)
}


#' Return the association sample -> stage, if any. Input 'x' should be a
#' TRONCO compliant dataset - see \code{is.compliant}.
#' @title as.stages
#'
#' @examples
#' data(test_dataset)
#' data(stage)
#' test_dataset = annotate.stages(test_dataset, stage)
#' as.stages(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return A matrix with 1 column annotating stages and rownames as sample IDs.
#' @export as.stages
#' 
as.stages <- function(x) {
    if (has.stages(x)) {
        return(x$stages) 
    }
    return(NA)
}


#' Return the types of events for a set of genes which are in 'x', which should be a
#' TRONCO compliant dataset - see \code{is.compliant}.
#' @title as.types
#'
#' @examples
#' data(test_dataset)
#' as.types(test_dataset)
#' as.types(test_dataset, genes='TET2')
#' 
#' @param x A TRONCO compliant dataset.
#' @param genes A list of genes to consider, if NA all genes are used.
#' @return A matrix with 1 column annotating stages and rownames as sample IDs.
#' @export as.types
#' 
as.types <- function(x, genes = NA) {    
    return(unlist(unique(as.events(x, genes = genes)[, 'type'])))
}


#' Return the colors associated to each type of event in 'x', which should be a
#' TRONCO compliant dataset - see \code{is.compliant}.
#' @title as.colors
#' 
#' @examples
#' data(test_dataset)
#' as.colors(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return A named vector of colors.
#' @export as.colors
#' 
as.colors <- function(x) {
    return(x$types[, 'color'])
}


#' Return the genotypes for a certain set of genes and type of events. Input 'x' should be a
#' TRONCO compliant dataset - see \code{is.compliant}. In this case column names are substituted
#' with events' types.
#' @title as.gene
#'
#' @examples
#' data(test_dataset)
#' as.gene(test_dataset, genes = c('EZH2', 'ASXL1'))
#' 
#' @param x A TRONCO compliant dataset.
#' @param types The types of events to consider, if NA all available types are used.
#' @param genes The genes to consider, if NA all available genes are used.
#' @return A matrix, subset of \code{as.genotypes(x)} with colnames substituted  with events' types.
#' @export as.gene
#' 
as.gene <- function(x, genes, types = NA) {
    keys = as.events(x, genes = genes, types = types)

    data = data.frame(x$genotypes[, rownames(keys)],
                      row.names = as.samples(x))
    
    colnames(data) = apply(keys,
                           1,
                           FUN = paste,
                           collapse = ' ')

    return(data)
}


#' Return a dataset where all events for a gene are merged in a unique event, i.e., 
#' a total of gene-level alterations diregarding the event type. Input 'x' is checked
#' to be a TRONCO compliant dataset - see \code{is.compliant}. 
#' @title as.alterations
#'
#' @examples
#' data(muts)
#' as.alterations(muts)
#' 
#' @param x A TRONCO compliant dataset.
#' @param new.type The types label of the new event type, 'Alteration' by default.
#' @param new.color The color of the event \code{new.type}, default 'khaki'.
#' @param silent A parameter to disable/enable verbose messages.
#' @return A TRONCO compliant dataset with alteration profiles.
#' @export as.alterations
#' 
as.alterations <- function(x,
                           new.type = 'Alteration',
                           new.color = 'khaki',
                           silent = FALSE) {
    is.compliant(x)
    if(has.model(x)) {
        stop("There's a reconstructed model, types cannot be merged now. \nUse delete.model()")
    }

    if(length(as.patterns(x)) > 0) {
        stop('Patterns found. Delete patterns first.\n')
    }

    join.types(x,
               NULL,
               new.type = new.type,
               new.color = new.color,
               silent = silent)
}

#' Return the patterns in the dataset which constitute CAPRI's hypotheses.
#' @title as.patterns
#'
#' @examples
#' data(test_dataset)
#' as.patterns(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @return The patterns in the dataset which constitute CAPRI's hypotheses.
#' @export as.patterns
#' 
as.patterns <- function(x) {
    is.compliant(x)
    if (npatterns(x) == 0) {
        return(NULL)
    }
    return(names(x$hypotheses$hstructure))
}


#' Return the hypotheses in the dataset which constitute CAPRI's hypotheses.
#' @title as.hypotheses
#'
#' @examples
#' data(test_dataset)
#' as.hypotheses(test_dataset)
#'
#' @param x A TRONCO compliant dataset.
#' @param cause A list of genes to use as causes
#' @param effect A list of genes to use as effects
#' @return The hypotheses in the dataset which constitute CAPRI's hypotheses.
#' @export as.hypotheses
#' 
as.hypotheses <- function(x, cause = NA, effect = NA) {
    if (nhypotheses(x) < 1)
        return(NULL)

    hlist = x$hypotheses$hlist

    list_c = x$annotations[hlist[,'cause'], c('type', 'event'), drop = FALSE]
    colnames(list_c) = c('cause type', 'cause event')
    rownames(list_c) = NULL
    list_e = x$annotations[hlist[,'effect'], c('type', 'event'), drop = FALSE]
    colnames(list_e) = c('effect type', 'effect event')
    rownames(list_e) = NULL

    filtered_list = cbind(list_c, list_e)

    if (!is.na(cause)) {
        if (all(cause %in% as.events(x)[, 'event'])) {
            filtered_list = filtered_list[which(filtered_list[, 'cause event'] == cause), ]
        } else {
            stop('some cause not in as.events\n')
        }
    }

    if (!is.na(effect)) {
        if (all(effect %in% as.events(x)[, 'event'])) {
            filtered_list = filtered_list[which(filtered_list[, 'effect event'] == effect), ]
        } else {
            stop('some effect not in as.events\n')
        }
    }

    return(filtered_list)
}

#' Return the list of events present in selected patterns
#'
#' @examples
#' data(test_dataset)
#' as.events.in.patterns(test_dataset)
#' as.events.in.patterns(test_dataset, patterns='XOR_EZH2')
#'
#' @title as.events.in.patterns
#' @param x A TRONCO compliant dataset.
#' @param patterns A list of patterns for which the list will be returned
#' @return A list of events present in patterns which consitute CAPRI's hypotheses
#' @export as.events.in.patterns
#' 
as.events.in.patterns <- function(x, patterns = NULL) {
    is.compliant(x)
    ann = x$annotations[, c('type', 'event'), drop = FALSE]
    if (is.null(patterns)) {
        patterns = as.patterns(x)
    }

    genes_list = NULL
    for (h in patterns) {
        if (!h %in% as.patterns(x)) {
            stop('Hypothesis ', h, ' not in as.patterns(x)')
        }

        g =
            lapply(colnames(x$hypotheses$hstructure[[h]]),
                   function(x) { if (!is.logic.node(x)) return(x) })
        genes_list = append(genes_list, g)
    }  
    genes_list = unique(unlist(genes_list))

    if (!(is.null(genes_list)))
        ann = ann[ which(rownames(ann) %in% genes_list), , drop = FALSE] 

    return(ann)
}


#' Return the list of genes present in selected patterns
#'
#' @examples
#' data(test_dataset)
#' as.genes.in.patterns(test_dataset)
#' as.genes.in.patterns(test_dataset, patterns='XOR_EZH2')
#'
#' @title as.genes.in.patterns
#' @param x A TRONCO compliant dataset.
#' @param patterns A list of patterns for which the list will be returned
#' @return A list of genes present in patterns which consitute CAPRI's hypotheses
#' @export as.genes.in.patterns
#' 
as.genes.in.patterns <- function(x, patterns=NULL) {
    events = as.events.in.patterns(x, patterns)
    genes = unique(events[, 'event'])
    return(genes)
}


#' Return the list of types present in selected patterns
#'
#' @examples
#' data(test_dataset)
#' as.types.in.patterns(test_dataset)
#' as.types.in.patterns(test_dataset, patterns='XOR_EZH2')
#'
#' @title as.types.in.patterns
#' @param x A TRONCO compliant dataset.
#' @param patterns A list of patterns for which the list will be returned
#' @return A list of types present in patterns which consitute CAPRI's hypotheses
#' @export as.types.in.patterns
#' 
as.types.in.patterns <- function(x, patterns = NULL) {
    events = as.events.in.patterns(x, patterns)
    types = unique(events[, 'type'])
    return(types)
}


#' Return a list of events which are observed in the input samples list
#'
#' @examples
#' data(test_dataset)
#' as.events.in.sample(test_dataset, c('patient 1', 'patient 7'))
#'
#' @title as.events.in.sample
#' @param x A TRONCO compliant dataset
#' @param sample Vector of sample names
#' @return A list of events which are observed in the input samples list
#' @export as.events.in.sample
#' 
as.events.in.sample <- function(x, sample) {
    
    aux <- function(s) {
        sub.geno = as.genotypes(x)[s, , drop = FALSE]  
        sub.geno = sub.geno[, sub.geno == 1, drop = FALSE]
        return(as.events(x)[colnames(sub.geno), , drop = FALSE])
    }

    events = sapply(sample, FUN = aux, simplify = FALSE)
    merge = unique(Reduce(rbind, events))
    return(merge)
}


#' Return confidence information for a TRONCO model. Available information are: temporal priority (tp), 
#' probability raising (pr), hypergeometric test (hg), parametric (pb), non parametric (npb) or 
#' statistical (sb) bootstrap, entropy loss (eloss), prediction error (prederr).
#' Confidence is available only once a model has been reconstructed with any of the algorithms implemented
#' in TRONCO. If more than one model has been reconstructed - for instance via multiple regularizations - 
#' confidence information is appropriately nested. The requested confidence is specified via 
#' vector parameter \code{conf}.
#'
#' @examples
#' data(test_model)
#' as.confidence(test_model, conf='tp')
#' as.confidence(test_model, conf=c('tp', 'hg'))
#'
#' @title as.confidence
#' @param x A TRONCO model.
#' @param conf A vector with any of 'tp', 'pr', 'hg', 'npb', 'pb', 'sb', 'eloss', 'prederr' or 'posterr'. 
#' @param models The name of the models to extract, all by default. 
#' @return A list of matrices with the event-to-event confidence. 
#' @export as.confidence
#' 
as.confidence <- function(x, conf, models = names(x$model)) {
    is.compliant(x)
    is.model(x)
    if (!is.vector(conf)) stop('"conf" should be a vector.')

    keys = c('hg', 'tp', 'pr', 'npb', 'pb', 'sb', 'eloss', 'prederr', 'posterr')

    if (!all(conf %in% keys)) 
        stop('Confidence keyword unrecognized, \'conf\' should be any of:\n
            INPUT DATASET\n
            \t \"hg\" - hypergeometric test (randomness of observations)\n
            SELECTIVE ADVANTAGE SCORES\n
            \t \"tp\" - temporal priority (temporal ordering of events)\n
            \t \"pr\" - probability raising (selectivity among events)\n
            MODEL CONFIDENCE - requires post-reconstruction bootstrap or cross-validation\n
            \t \"npb\"     - non-parametric bootstrap,\n
            \t \"sb\"      - statistical bootstrap\n
            \t \"eloss\"   - entropy loss\n
            \t \"prederr\" - prediction error\n
            \t \"posterr\"- posterior classification error'
             )

    if (is.null(x$confidence)
        || any(is.na(x$confidence))
        || is.null(x$model)
        || all(is.na(x$model)))
        stop('Input \'x\' does not contain a TRONCO model. No confidence to show.\n')

    has.npb.bootstrap = is.null(x$bootstrap[[models[1]]]$npb)
    has.sb.bootstrap = is.null(x$bootstrap[[models[1]]]$sb)

    if ('npb' %in% conf && has.npb.bootstrap) {
        stop('Non-parametric bootstrap was not performed. Remove keyword\n')
    }

    if ('sb' %in% conf && has.sb.bootstrap) {
        stop('Statistical bootstrap was not performed. Remove keyword\n')
    }
    
    result = NULL

    if ('hg' %in% conf) {
        result$hg = x$confidence['hypergeometric test', ][[1]]
    }
    
    if ('tp' %in% conf) {
        result$tp = x$confidence['temporal priority', ][[1]]
    }
    
    if ('pr' %in% conf) {
        result$pr = x$confidence['probability raising', ][[1]]
    }
    
    if ('npb' %in% conf) {
        for (i in 1:length(models)) {
            result$npb[models[i]] =
                list(x$bootstrap[[models[i]]]$npb$bootstrap.edge.confidence)  
        }
    }
    
    if ('sb' %in% conf) { 
        for (i in 1:length(models)) {
            result$sb[models[i]] =
                list(x$bootstrap[[models[i]]]$sb$bootstrap.edge.confidence)  
        }
    }
    
    if ('eloss' %in% conf) { 
        result$eloss = as.kfold.eloss(x, models = models)
    }

    if ('prederr' %in% conf) {
        result$prederr = as.kfold.prederr(x, models = models)
    }

    if ('posterr' %in% conf) {
        result$posterr = as.kfold.posterr(x, models = models)
    }
    
    return(result)  
}


#' Extract the models from a reconstructed object.
#'
#' @examples
#' data(test_model)
#' as.models(test_model)
#'
#' @title as.models
#' @param x A TRONCO model.
#' @param models The name of the models to extract, e.g. 'bic', 'aic', 'caprese', all by default. 
#' @return The models in a reconstructed object. 
#' @export as.models
#' 
as.models <- function(x, models=names(x$model)) {
    is.compliant(x)
    is.model(x)

    for (model in models) {
        if ( !model %in% names(x$model)) {
            stop(paste('model:', model, 'not present'))
        }
    }

    return(x$model[models])
}


#' Return the description annotating the dataset, if any. Input 'x' should be
#' a TRONCO compliant dataset - see \code{is.compliant}. 
#'
#' @examples
#' data(test_dataset)
#' as.description(test_dataset)
#'
#' @title as.description
#' @param x A TRONCO compliant dataset.
#' @return The description annotating the dataset, if any.
#' @export as.description
#' 
as.description <- function(x) {
    if (!is.null(x$name))
        return(x$name)
    return("")
}


#' Given a cohort and a pathway, return the cohort with events restricted to genes 
#' involved in the pathway. This might contain a new 'pathway' genotype with an alteration mark if
#' any of the involved genes are altered. 
#'
#' @examples
#' data(test_dataset)
#' p = as.pathway(test_dataset, c('ASXL1', 'TET2'), 'test_pathway')
#'
#' @title as.pathway
#' @param x A TRONCO compliant dataset.
#' @param pathway.genes Gene (symbols) involved in the pathway.
#' @param pathway.name Pathway name for visualization.
#' @param pathway.color Pathway color for visualization.
#' @param aggregate.pathway If TRUE drop the events for the genes in the pathway.
#' @param silent A parameter to disable/enable verbose messages.
#' @return Extract the subset of events for genes which are part of a pathway.
#' @export as.pathway
#' 
as.pathway <- function(x, 
                       pathway.genes, 
                       pathway.name, 
                       pathway.color='yellow', 
                       aggregate.pathway = TRUE,
                       silent = FALSE) {
    
    is.compliant(x, 'as.pathway: input')
    data = x$genotypes
    if (!silent) {
        cat('*** Extracting events for pathway: ',
            pathway.name,
            '.\n')
    }

    ## Select only those events involving a gene in pathway.genes
    ## which is also in x.
    
    y = events.selection(x, NA, filter.in.names = pathway.genes, NA)

    ## Extend genotypes.
    
    y = enforce.numeric(y)

    pathway =
        data.frame(rowSums(as.genotypes(y)),
                   row.names = as.samples(y),
                   stringsAsFactors = FALSE)  
    pathway[pathway > 1, ] =  1
    colnames(pathway) = pathway.name 

    pathway =
        import.genotypes(pathway,
                         event.type = 'Pathway',
                         color = pathway.color)

    if (!silent) {
        cat('Pathway extracted succesfully.\n')
    }

    if (!aggregate.pathway) {
        pathway = ebind(pathway, y, silent = silent)
    }

    if (has.stages(y)) {
        pathway = annotate.stages(pathway, as.stages(y))
    }

    is.compliant(pathway, 'as.pathway: output')

    return(pathway)
}


#' Extract the adjacency matrix of a TRONCO model. The matrix is indexed with colnames/rownames which 
#' represent genotype keys - these can be resolved with function \code{keysToNames}. It is possible to
#' specify a subset of events to build the matrix, a subset of models if multiple reconstruction have
#' been performed. Also, either the prima facie matrix or the post-regularization matrix can be extracted.
#'
#' @examples
#' data(test_model)
#' as.adj.matrix(test_model)
#' as.adj.matrix(test_model, events=as.events(test_model)[5:15,])
#' as.adj.matrix(test_model, events=as.events(test_model)[5:15,], type='pf')
#' 
#' @title as.adj.matrix
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @param type Either the prima facie ('pf') or the post-regularization ('fit') matrix, 'fit' by default.
#' @return The adjacency matrix of a TRONCO model. 
#' @export as.adj.matrix
#' 
as.adj.matrix <- function(x,
                          events = as.events(x),
                          models = names(x$model),
                          type = 'fit') {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    if (!all(models %in% names(x$model))) {
        stop('not all "models" are reconstructed model.') 
    }
    
    if (!type %in% c('fit', 'pf')  ) {
        stop('"type" should be any of \'fit\' (post-regularization) or \'pf\' (prima facie).')
    }  

    m = as.models(x, models = models)

    ret = list()
    for (i in models) {
        if (type == 'fit') {
            mat = m[[i]]$adj.matrix$adj.matrix.fit
        } else if (type == 'pf') {
            mat = m[[i]]$adj.matrix$adj.matrix.pf
        }

        mat = mat[rownames(events), , drop = FALSE]
        mat = mat[, rownames(events), drop = FALSE]

        ret = append(ret, list(mat)) 
    }

    names(ret) = models
    return(ret) 
}


#' Extract the marginal probabilities from a TRONCO model. The return matrix is indexed with rownames which 
#' represent genotype keys - these can be resolved with function \code{keysToNames}. It is possible to
#' specify a subset of events to build the matrix, a subset of models if multiple reconstruction have
#' been performed. Also, either the observed or fit probabilities can be extracted.
#'
#' @examples
#' data(test_model)
#' as.marginal.probs(test_model)
#' as.marginal.probs(test_model, events=as.events(test_model)[5:15,])
#'
#' @title as.marginal.probs
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @param type observed.
#' @return The marginal probabilities in a TRONCO model. 
#' @export as.marginal.probs
as.marginal.probs <- function(x,
                              events = as.events(x),
                              models = names(x$model),
                              type = 'observed') {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)
    
    if (any(is.null(colnames(events)))) {
        stop('Events should have colnames to access the adjacency matrix - use \'as.events\' function?')
    }

    if (!all(models %in% names(x$model))) {
        stop('not all "models" are reconstructed model.') 
    }

    if (type != 'observed') {
        stop('Marginal probabilities are available for \'observed\' (empirical).') 
    }

    m = as.models(x, models = models)

    ret = list()
    for (i in models) {
        mat = m[[i]]$probabilities$probabilities.observed$marginal.probs
        mat = mat[rownames(events), , drop = FALSE]
        ret = append(ret, list(mat)) 
    }

    names(ret) = models
    return(ret) 
}


#' Extract the joint probabilities from a TRONCO model. The return matrix is indexed with rownames/colnames which 
#' represent genotype keys - these can be resolved with function \code{keysToNames}. It is possible to
#' specify a subset of events to build the matrix, a subset of models if multiple reconstruction have
#' been performed. Also, either the observed or fit probabilities can be extracted.
#'
#' @examples
#' data(test_model)
#' as.joint.probs(test_model)
#' as.joint.probs(test_model, events=as.events(test_model)[5:15,])
#'
#' @title as.joint.probs
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @param type observed
#' @return The joint probabilities in a TRONCO model. 
#' @export as.joint.probs
#'
as.joint.probs <- function(x,
                           events = as.events(x),
                           models = names(x$model),
                           type = 'observed') {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    if (any(is.null(colnames(events)))) {
        stop('Events should have colnames to access the adjacency matrix - use \'as.events\' function?')
    }

    if (!all(models %in% names(x$model))) {
        stop('not all "models" are reconstructed model.') 
    }

    if (type != 'observed') {
        stop('Marginal probabilities are available for \'observed\' (empirical).') 
    }

    m = as.models(x, models = models)

    ret = list()
    for (i in models) {
        mat = m[[i]]$probabilities$probabilities.observed$joint.probs
        mat = mat[rownames(events), , drop = FALSE]
        mat = mat[, rownames(events), drop = FALSE]
        ret = append(ret, list(mat)) 
    }

    names(ret) = models
    return(ret) 
}


#' Extract the conditional probabilities from a TRONCO model. The return matrix is indexed with rownames which 
#' represent genotype keys - these can be resolved with function \code{keysToNames}. It is possible to
#' specify a subset of events to build the matrix, a subset of models if multiple reconstruction have
#' been performed. Also, either the observed or fit probabilities can be extracted.
#' 
#' #' @examples
#' data(test_model)
#' as.conditional.probs(test_model)
#' as.conditional.probs(test_model, events=as.events(test_model)[5:15,])
#'
#' @title as.conditional.probs
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @param type observed ('observed') 
#' @return The conditional probabilities in a TRONCO model. 
#' @export as.conditional.probs
#' 
as.conditional.probs <- function(x,
                                 events = as.events(x),
                                 models = names(x$model),
                                 type = 'observed') {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    if (any(is.null(colnames(events)))) {
        stop('Events should have colnames to access the adjacency matrix - use \'as.events\' function?')
    }

    if (!all(models %in% names(x$model))) {
        stop('not all "models" are reconstructed model.') 
    }

    if (type != 'observed') {
        stop('Marginal probabilities are available for \'observed\' (empirical).') 
    }

    m = as.models(x, models = models)

    ret = list()
    for (i in models) {
        mat = m[[i]]$probabilities$probabilities.observed$conditional.probs
        mat = mat[rownames(events), , drop = FALSE]
        ret = append(ret, list(mat)) 
    }

    names(ret) = models
    return(ret) 
}


# Extract the estimated rates of false positives an negatives in the data, given the model. 
# A subset of models if multiple reconstruction have been performed can be extracted.
# 
# @examples
# data(test_model)
# as.error.rates(test_model)
#
# @title as.error.rates
# @param x A TRONCO model.
# @param models A subset of reconstructed models, all by default.
# @return The estimated rates of false positives an negatives in the data, given the model.
#
as.error.rates <- function(x, models = names(x$model)) {
    is.compliant(x)
    is.model(x)

    if (!all(models %in% names(x$model))) {
        stop('not all "models" are reconstructed model.') 
    }

    m = as.models(x, models = models)

    ret = list()
    for (i in models) {
        mat = m[[i]]$error.rates
        ret = append(ret, list(mat)) 
    }

    names(ret) = models
    return(ret) 
}


#' Returns a dataframe with all the selective advantage relations in a 
#' TRONCO model. Confidence is also shown - see \code{as.confidence}. It is possible to
#' specify a subset of events or models if multiple reconstruction have
#' been performed. 
#'
#' @examples
#' data(test_model)
#' as.selective.advantage.relations(test_model)
#' as.selective.advantage.relations(test_model, events=as.events(test_model)[5:15,])
#' as.selective.advantage.relations(test_model, events=as.events(test_model)[5:15,], type='pf')
#'
#' @title as.selective.advantage.relations
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @param type Either Prima Facie ('pf') or fit ('fit') probabilities, 'fit' by default.
#' @return All the selective advantage relations in a TRONCO model 
#' @export as.selective.advantage.relations
#' 
as.selective.advantage.relations <- function(x,
                                             events = as.events(x),
                                             models = names(x$model),
                                             type = 'fit') {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    ## TEMPORARY HORRIBLE FIX
    matrix = NULL
    
    if (type == 'pf')
        matrix$pf = x$adj.matrix.prima.facie   
    else
        matrix =
            as.adj.matrix(x,
                          events = events,
                          models = models,
                          type = type)
    
    matrix = lapply(matrix, keysToNames, x = x)

    conf = as.confidence(x, conf = c('tp', 'pr', 'hg'))
    conf = lapply(conf, keysToNames, x = x)

    matrix.to.df <- function(m) {    
        entries = length(which(m == 1))
        df = NULL
        df$SELECTS = NULL
        df$SELECTED = NULL
        df$OBS.SELECTS = NULL
        df$OBS.SELECTED = NULL
        df$HG = NULL
        df$TP = NULL
        df$PR = NULL
        
        if (entries == 0) {
            return(NULL)
        }

        for (i in 1:ncol(m)) {
            for (j in 1:nrow(m)) {
                if (m[i, j] == 1) {
                    df$SELECTS = c(df$SELECTS, rownames(m)[i])
                    df$SELECTED = c(df$SELECTED, colnames(m)[j])

                    df$OBS.SELECTS =
                        c(df$OBS.SELECTS,
                          sum(as.genotypes(x)[, nameToKey(x, rownames(m)[i])]))
                    df$OBS.SELECTED =
                        c(df$OBS.SELECTED,
                          sum(as.genotypes(x)[, nameToKey(x, colnames(m)[j])]))

                    df$TP = c(df$TP, conf$tp[rownames(m)[i], colnames(m)[j]])
                    df$PR = c(df$PR, conf$pr[rownames(m)[i], colnames(m)[j]])
                    df$HG = c(df$HG, conf$hg[rownames(m)[i], colnames(m)[j]])   
                }
            }
        }

        df =
            cbind(df$SELECTS,
                  df$SELECTED,
                  df$OBS.SELECTS,
                  df$OBS.SELECTED,
                  df$TP,
                  df$PR,
                  df$HG)

        colnames(df) =
            c('SELECTS',
              'SELECTED',
              'OBS.SELECTS',
              'OBS.SELECTED',
              'TEMPORAL.PRIORITY',
              'PROBABILITY.RAISING',
              'HYPERGEOMETRIC')
        
        rownames(df) = paste(1:nrow(df))

        df = data.frame(df, stringsAsFactors = FALSE) 
        df$OBS.SELECTS = as.numeric(df$OBS.SELECTS)
        df$OBS.SELECTED = as.numeric(df$OBS.SELECTED)
        df$HYPERGEOMETRIC = as.numeric(df$HYPERGEOMETRIC)
        df$TEMPORAL.PRIORITY = as.numeric(df$TEMPORAL.PRIORITY)
        df$PROBABILITY.RAISING = as.numeric(df$PROBABILITY.RAISING)

        return(df)
    }
    
    return(lapply(matrix, matrix.to.df))
}


#' Returns a dataframe with all the bootstrap score in a 
#' TRONCO model. It is possible to specify a subset of events
#' or models if multiple reconstruction have been performed. 
#'
#' @examples
#' data(test_model)
#' as.bootstrap.scores(test_model)
#' as.bootstrap.scores(test_model, events=as.events(test_model)[5:15,])
#'
#' @title as.bootstrap.scores
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @return All the bootstrap scores in a TRONCO model 
#' @export as.bootstrap.scores
#' 
as.bootstrap.scores <- function(x,
                                events = as.events(x),
                                models = names(x$model)) {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    if (!all(models %in% names(x$model))) {
        stop('not all "models" are reconstructed model.') 
    }

    matrix = 
    as.adj.matrix(x,
        events = events,
        models = models,
        type = 'fit')


    matrix = lapply(matrix, keysToNames, x = x)

    if (!(is.null(x$confidence) && is.na(x$confidence))) {
        models = names(x$model)
    }

    has.npb.bootstrap = !is.null(x$bootstrap[[models[1]]]$npb)
    has.sb.bootstrap = !is.null(x$bootstrap[[models[1]]]$sb)

    if(has.npb.bootstrap) {
        npb.boot.conf = lapply(as.confidence(x, conf = c('npb'))$npb, keysToNames, x = x)
    }
    if(has.sb.bootstrap) {
        sb.boot.conf = lapply(as.confidence(x, conf = c('sb'))$sb, keysToNames, x = x)
    }

    matrix.to.df <- function(z) {   
        m = matrix[[z]]

        entries = length(which(m == 1))
        df = NULL
        df$SELECTS = NULL
        df$SELECTED = NULL
        df$OBS.SELECTS = NULL
        df$OBS.SELECTED = NULL
        df$BOOT.NPB = NULL
        df$BOOT.PB = NULL
        df$BOOT.SB = NULL

        if (entries == 0) {
            return(NULL)
        }

        for (i in 1:ncol(m)) {
            for (j in 1:nrow(m)) {

                if (m[i, j] == 1) {
                    df$SELECTS = c(df$SELECTS, rownames(m)[i])
                    df$SELECTED = c(df$SELECTED, colnames(m)[j])

                    df$OBS.SELECTS = c(df$OBS.SELECTS,
                                       sum(as.genotypes(x)[, nameToKey(x, rownames(m)[i])]))
                    
                    df$OBS.SELECTED = c(df$OBS.SELECTED,
                                       sum(as.genotypes(x)[, nameToKey(x, colnames(m)[j])]))

                    if (has.npb.bootstrap) {
                        df$BOOT.NPB = c(df$BOOT.NPB,
                                        npb.boot.conf[[z]][ rownames(m)[i], colnames(m)[j] ] * 100)
                    } else {
                        df$BOOT.NPB = c(df$BOOT.NPB, NA)
                    }

                    if (has.sb.bootstrap) {
                        df$BOOT.SB = c(df$BOOT.SB,
                                       sb.boot.conf[[z]][ rownames(m)[i], colnames(m)[j] ] * 100)
                    } else {
                        df$BOOT.SB = c(df$BOOT.SB, NA)
                    }
                }
            }
        }

        df = cbind(df$SELECTS,
                   df$SELECTED,
                   df$OBS.SELECTS,
                   df$OBS.SELECTED,
                   df$BOOT.NPB,
                   df$BOOT.SB)

        colnames(df) = 
            c('SELECTS',
              'SELECTED',
              'OBS.SELECTS',
              'OBS.SELECTED',
              'NONPAR.BOOT',
              'STAT.BOOT')

        rownames(df) = paste(1:nrow(df))

        df = data.frame(df, stringsAsFactors = FALSE) 
        df$OBS.SELECTS = as.numeric(df$OBS.SELECTS)
        df$OBS.SELECTED = as.numeric(df$OBS.SELECTED)

        if (!has.npb.bootstrap) {
            df$NONPAR.BOOT = NULL
        } else {
            df$NONPAR.BOOT = as.numeric(df$NONPAR.BOOT)
        }

        if (!has.sb.bootstrap) {
            df$STAT.BOOT = NULL
        } else {
            df$STAT.BOOT = as.numeric(df$STAT.BOOT)
        }
        return(df)
    }

    res = lapply(seq_along(matrix), matrix.to.df)
    names(res) = names(matrix)

    return(res)
}


#' Returns a dataframe with all the average/stdev entropy loss score of a 
#' TRONCO model. It is possible to specify models if multiple
#' reconstruction have been performed. 
#'
#' @examples
#' data(test_model_kfold)
#' as.kfold.eloss(test_model_kfold)
#' as.kfold.eloss(test_model_kfold, models='capri_aic')
#'
#' @title as.kfold.eloss
#' @param x A TRONCO model.
#' @param models A subset of reconstructed models, all by default.
#' @param values If you want to see also the values
#' @return All the bootstrap scores in a TRONCO model 
#' @export as.kfold.eloss
#' @importFrom stats sd
#' 
as.kfold.eloss <- function(x,
                           models = names(x$model),
                           values = FALSE) {
    is.compliant(x)
    is.model(x)

    if(is.null(x$kfold)) {
        stop('Cross-validation was never performed on this model!')
    }

    ret = NULL
    ret$C1 = NULL
    ret$C2 = NULL
    ret$C3 = NULL
    ret$C4 = NULL

    for (model in models) {
        if (is.null(x$kfold[[model]]$eloss)) {
            stop('Entropy loss was not estimated for \'', model, '\'')
        }
        meanll = mean(get(model, x$kfold)$eloss)
        ll = get(model, x$model)$logLik
        ratio = meanll / abs(ll) * 100

        ret$C1 = c(ret$C1, meanll)
        ret$C2 = c(ret$C2, ratio)
        ret$C3 = c(ret$C3, sd(get(model, x$kfold)$eloss))
        ret$C4 = c(ret$C4, paste(round(get(model, x$kfold)$eloss, 2), sep = ', ', collapse = ', '))
    }

    ret = data.frame(ret)
    rownames(ret) = models
    colnames(ret) = c('Mean', '%-of-logLik', 'Stdev', 'Values (rounded, 2 digits)')
    
    if(!values) ret = ret[, 1:3]

    return(ret)
}


#' Returns a dataframe with all the prediction error score in a 
#' TRONCO model. It is possible to specify a subset of events
#' or models if multiple reconstruction have been performed. 
#'
#' @examples
#' data(test_model_kfold)
#' as.kfold.prederr(test_model_kfold)
#' as.kfold.prederr(test_model_kfold, models='capri_aic')
#'
#' @title as.kfold.prederr
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @param values If you want to see also the values
#' @param table Keep the original table (defaul false)
#' @return All the bootstrap scores in a TRONCO model 
#' @export as.kfold.prederr
#' @importFrom stats sd
#' 
as.kfold.prederr <- function(x,
                             events = as.events(x),
                             models = names(x$model),
                             values = FALSE,
                             table = FALSE) {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    if (is.null(x$kfold) ) {
        stop('Crossvalidation was not executed for this object!')
    }

    matrix.to.df <- function(z) {   
        if ( any(is.null(x$kfold[[z]]$prederr)) ) {
            stop('Crossvalidation was not executed for the required model: ', models[z] )
        }
        prederr.matrix = x$kfold[[z]]$prederr
        if (table) {
            names(prederr.matrix) = lapply(names(prederr.matrix), function(k) { nameToKey(x, k) })
            return(prederr.matrix)
        }

        # Already prepared data - just wrap it in a dataframe
        df = NULL
        df$prederr = t(as.data.frame(prederr.matrix)) # values

        df$MEAN.PREDERR = apply(df$prederr, 1, mean) # means
        df$SD.PREDERR = apply(df$prederr, 1, sd) # standard deviation
        df$VALUES.PREDERR = apply(df$prederr, 1, function(z){paste(round(z, 3), collapse = ', ')}) # collapse values

        df$SELECTED = gsub("\\.", " ", rownames(df$prederr)) # for later merge we use this

        df = data.frame(df, stringsAsFactors = FALSE) 
        rownames(df) = paste(1:nrow(df))
        df = df[, c('SELECTED', 'MEAN.PREDERR', 'SD.PREDERR', 'VALUES.PREDERR')]
        colnames(df)[4] = 'PREDERR Values (rounded, 3 digits)'

        ## Filter out events if required.

        sel.events = apply(events, 1, function(z){paste(z, collapse = ' ')})
        df = df[which(df$SELECTED %in% sel.events), , drop = FALSE]

        if(!values) df = df[, 1:3]
        
        return(df)
    }

    res = lapply(seq_along(models), matrix.to.df)
    names(res) = models

    return(res)
}


#' Returns a dataframe with all the posterior classification error
#' score in a TRONCO model. It is possible to specify a subset of events
#' or models if multiple reconstruction have been performed. 
#'
#' @examples
#' data(test_model_kfold)
#' data(test_model)
#' as.kfold.posterr(test_model_kfold)
#' as.kfold.posterr(test_model_kfold, events=as.events(test_model)[5:15,])
#'
#' @title as.kfold.posterr
#' @param x A TRONCO model.
#' @param events A subset of events as of \code{as.events(x)}, all by default.
#' @param models A subset of reconstructed models, all by default.
#' @param values If you want to see also the values
#' @param table Keep the original table (defaul false)
#' @return All the posterior classification error scores in a TRONCO model 
#' @export as.kfold.posterr
#' @importFrom stats sd
#' 
as.kfold.posterr <- function(x,
                             events = as.events(x),
                             models = names(x$model),
                             values = FALSE,
                             table = FALSE) {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    if (is.null(x$kfold) ) {
        stop('Crossvalidation was not executed for this object!')
    }

    matrix = 
    as.adj.matrix(x,
        events = events,
        models = models,
        type = 'fit')


    matrix = lapply(matrix, keysToNames, x = x)

    matrix.to.df <- function(z) {  
        if ( any(is.null(x$kfold[[z]]$posterr)) ) {
            stop('Posterior classification error was not executed for the required model: ', z )
        }

        m = matrix[[z]]
        posterr.matrix = get(z, x$kfold)$posterr

        if (table) {
            rownames(posterr.matrix) = lapply(rownames(posterr.matrix), function(k) { nameToKey(x, k) })
            colnames(posterr.matrix) = lapply(colnames(posterr.matrix), function(k) { nameToKey(x, k) })
            return(posterr.matrix)

        }

        posterr.matrix = keysToNames(posterr.matrix, x = x)


        entries = length(which(m == 1))
        df = NULL
        df$SELECTS = NULL
        df$SELECTED = NULL
        df$MEAN.POSTERR = NULL
        df$SD.POSTERR = NULL
        df$POSTERR = NULL

        if (entries == 0) {
            return(NULL)
        }

        for (i in 1:ncol(m)) {
            for (j in 1:nrow(m)) {
                if (m[i, j] == 1) {
                    select = rownames(m)[i]
                    df$SELECTS = c(df$SELECTS, select)
                    selected = colnames(m)[j]
                    val = posterr.matrix[[select,selected]]
                    df$SELECTED = c(df$SELECTED, colnames(m)[j])
                    df$MEAN.POSTERR = c(df$MEAN.POSTERR, mean(val))
                    df$SD.POSTERR = c(df$SD.POSTERR, sd(val))
                    posterr = paste(round(val, 3), collapse=', ')
                    df$POSTERR = c(df$POSTERR, posterr) 
                }
            }
        }

        
        df = cbind(df$SELECTS,
                   df$SELECTED,
                   df$MEAN.POSTERR,
                   df$SD.POSTERR,
                   df$POSTERR)

        rownames(df) = c(1:nrow(df))
        df = data.frame(df, stringsAsFactors = FALSE) 
        
        colnames(df) = 
            c('SELECTS',
              'SELECTED',
              'MEAN.POSTERR',
              'SD.POSTERR',
              'POSTERR Values (rounded, 3 digits)')

        df$MEAN.POSTERR = as.numeric(df$MEAN.POSTERR)
        df$SD.POSTERR = as.numeric(df$SD.POSTERR)

        if(!values) df = df[, 1:4]
        

        return(df)
    }

    res = lapply(models, matrix.to.df)
    names(res) = names(matrix)


    return(res)
}


# Get parents for each node
#
# @title as.parents.pos
# @param x A TRONCO model.
# @param events A subset of events as of \code{as.events(x)}, all by default.
# @param models A subset of reconstructed models, all by default.
# @return A list of parents for each node
#
as.parents.pos <- function(x,
                           events = as.events(x),
                           models = names(x$model)) {
    is.compliant(x)
    is.model(x)
    is.events.list(x, events)

    if (!all(models %in% names(x$model))) {
        stop('not all "models" are reconstructed model.') 
    }

    m = as.models(x, models = models)

    ret = list()
    for (i in models) {
        mat = m[[i]]$parents.pos
        mat = mat[rownames(events), , drop = FALSE]
        ret = append(ret, list(mat)) 
    }

    names(ret) = models
    return(ret) 
}


#' Get parameters of a model
#'
#' @title as.parameters
#' 
#' @examples
#' data(test_model)
#' as.parameters(test_model)
#' 
#' @param x A TRONCO model.
#' @return A list of parameters
#' @export as.parameters
#
as.parameters <- function(x) {
    is.compliant(x)
    is.model(x)

    return(x$parameters) 
}


#' Return true if the TRONCO dataset 'x', which should be a TRONCO compliant dataset 
#' - see \code{is.compliant} - has stage annotations for samples. Some sample stages 
#' might be annotated as NA, but not all.
#'
#' @examples
#' data(test_dataset)
#' has.stages(test_dataset)
#' data(stage)
#' test_dataset = annotate.stages(test_dataset, stage)
#' has.stages(test_dataset)
#'
#' @title has stages
#' @param x A TRONCO compliant dataset.
#' @return TRUE if the TRONCO dataset has stage annotations for samples.
#' @export has.stages
#' 
has.stages <- function(x) {
    return(!(all(is.null(x$stages)) || all(is.na(x$stages))))
}


#' Return true if there are duplicated events in the TRONCO dataset 'x', which should be
#' a TRONCO compliant dataset - see \code{is.compliant}. Events are identified by a gene 
#' name, e.g., a HuGO_Symbol, and a type label, e.g., c('SNP', 'KRAS')
#'
#' @examples
#' data(test_dataset)
#' has.duplicates(test_dataset)
#' 
#' @title has.duplicates
#' @param x A TRONCO compliant dataset.
#' @return TRUE if there are duplicated events in \code{x}.
#' @export has.duplicates
#' 
has.duplicates <- function(x) {  
    ## Find duplicate over the dataset.
    dup = duplicated(as.events(x))

    ## Return true if at least one duplicate is found.
    return(any(dup))
}


#' Return true if there is a reconstructed model in the TRONCO dataset 'x', which should be
#' a TRONCO compliant dataset - see \code{is.compliant}.
#'
#' @examples
#' data(test_dataset)
#' has.model(test_dataset)
#' 
#' @title has.model
#' @param x A TRONCO compliant dataset.
#' @return TRUE if there is a reconstructed model in \code{x}.
#' @export has.model
#' 
has.model <- function(x) {
    is.compliant(x)
    if (length(x$model) > 0)            # Isn't 'return(length(x$mode) > 0)' enough?
        return(TRUE)
    return(FALSE)
}


#' Return the events duplicated in \code{x}, if any. Input 'x' should be
#' a TRONCO compliant dataset - see \code{is.compliant}. 
#'
#' @examples
#' data(test_dataset)
#' duplicates(test_dataset)
#'
#' @title duplicates
#' @param x A TRONCO compliant dataset.
#' @return A subset of \code{as.events(x)} with duplicated events.
#' @export duplicates
#' 
duplicates <- function(x) {
    is.compliant(x)
    return(as.events(x)[duplicated(as.events(x)),])
}


#' Print to console a short report of a dataset 'x', which should be
#' a TRONCO compliant dataset - see \code{is.compliant}. 
#'
#' @examples
#' data(test_dataset)
#' view(test_dataset)
#' 
#' @title view
#' @param x A TRONCO compliant dataset.
#' @param view The firse \code{view} events are shown via \code{head}.
#' @export view
#' @importFrom utils head
#' 
view <- function(x, view = 5) {
    is.compliant(x)
    x = enforce.numeric(x)
    view = min(view, nevents(x))

    if (as.description(x) != "")
        cat(paste('Description: ', as.description(x), '.\n', sep = ''))

    cat(paste('-- TRONCO Dataset: n=',
              nsamples(x),
              ', m=',
              nevents(x),
              ', |G|=',
              ngenes(x),
              ', patterns=',
              npatterns(x),
              '.\n',
              sep = '')) 
    cat(paste('Events (types): ',
              paste(as.types(x), collapse=', '),
              '.\n',
              sep = ''))
    cat(paste('Colors (plot): ',
              paste(as.colors(x), collapse=', '),
              '.\n',
              sep = ''))

    if (has.stages(x)) {
        cat(paste('Stages: '))

        s = unlist(sort(unique(as.stages(x)[, 1])))
        cat((paste(paste(s, collapse=', ', sep = ''), '.\n', sep = '')))
    }

    cat(paste('Events (', view, ' shown):\n', sep = ''))
    to.show =
        paste( '\t',
              rownames(as.events(x)[1: view,]), ':',
              as.events(x)[1: view, 1],
              as.events(x)[1: view, 2],
              sep = ' ')

    cat(paste(to.show, collapse = '\n'))

    cat(paste('\nGenotypes (', view, ' shown):\n', sep = ''))

    if (has.model(x)) {
        cat('\n-- TRONCO Model(s): ', x$parameters$algorithm, '\n')

        if (as.parameters(x)$algorithm == 'CAPRI') {
            cat('Score optimization via ')
            if (x$parameters$command == 'hc') {
                cat('Hill-Climbing.\n')
            }

            if (x$parameters$command == 'tabu') {
                cat('Tabu Search.\n')
            }
        }

        if (as.parameters(x)$algorithm != 'CAPRESE') {
            
            cat(paste(toupper(x$parameters$regularization), sep = ', ', collapse = ', '),' regularizers.\n', sep = '')
            models = as.models(x)
            sel.adv.rel = as.selective.advantage.relations(x)


            for (reg in as.parameters(x)$regularization) {
                model = paste0(tolower(as.parameters(x)$algorithm), '_', reg)
                cat(toupper(reg), ': ', sep = '')
                score = get(model, models)$score
                logLik = get(model, models)$logLik
                row = nrow(get(model, sel.adv.rel))
                cat('score', score, '|',
                    'logLik', logLik, '|',
                    row, 'selective advantage relations.\n')

            }
        }
      
        cat('Available confidence measures:\n')
        cat('\tTemporal priority | Probability raising | Hypergeometric\n')
      
        if(!is.null(x$bootstrap)) {
            cat('Bootstrap estimation available.\n')
        }

        if(!is.null(x$kfold)) {
            cat('Cross-validation assessment available.\n')
        }

        cat('\n')
    }
} 



#' Return the number of types in the dataset.
#'
#' @examples
#' data(test_dataset)
#' ntypes(test_dataset)
#'
#' @title ntypes
#' @param x A TRONCO compliant dataset.
#' @return The number of types in the dataset.
#' @export ntypes
#' 
ntypes <- function(x) {
    return(length(as.types(x)))
}


#' Return the number of samples in the dataset.
#'
#' @examples
#' data(test_dataset)
#' nsamples(test_dataset)
#'
#' @title nsamples
#' @param x A TRONCO compliant dataset.
#' @return The number of samples in the dataset.
#' @export nsamples
#' 
nsamples <- function(x) {
    return(nrow(x$genotypes))
}


#' Return the number of events in the dataset involving a certain gene or type of event.
#'
#' @examples
#' data(test_dataset)
#' nevents(test_dataset)
#'
#' @title nevents
#' @param x A TRONCO compliant dataset.
#' @param genes The genes to consider, if NA all available genes are used.
#' @param types The types of events to consider, if NA all available types are used.
#' @return The number of events in the dataset involving a certain gene or type of event.
#' @export nevents
#' 
nevents <- function(x, genes = NA, types = NA) {
    return(nrow(as.events(x, genes, types)))
}


#' Return the number of genes in the dataset involving a certain type of event.
#'
#' @examples
#' data(test_dataset)
#' ngenes(test_dataset)
#'
#' @title ngenes
#' @param x A TRONCO compliant dataset.
#' @param types The types of events to consider, if NA all available types are used.
#' @return The number of genes in the dataset involving a certain type of event.
#' @export ngenes
#' 
ngenes <- function(x, types = NA) {
    return(length(as.genes(x, types = types)))
}


#' Return the number of patterns in the dataset
#'
#' @examples
#' data(test_dataset)
#' npatterns(test_dataset)
#'
#'
#' @param x the dataset.
#' @export npatterns
#' 
npatterns <- function(x) {
    if (any(is.null(x$hypotheses))) {
        return(0)
    }

    if ('hstructure' %in% names(x$hypotheses)) {
        return(length(ls(x$hypotheses$hstructure)))
    }
    return(0)
}


#' Return the number of hypotheses in the dataset
#'
#' @examples
#' data(test_dataset)
#' nhypotheses(test_dataset)
#'
#' @param x the dataset.
#' @export nhypotheses
#' 
nhypotheses <- function(x) {
    if (npatterns(x) < 1) {
        return(0)
    }

    if ('hlist' %in% names(x$hypotheses)) {
        return(length(x$hypotheses$hlist) / 2)
    }
    return(0)
}


#' Convert the internal reprensentation of genotypes to numeric, if not. 
#'
#' @examples
#' data(test_dataset)
#' test_dataset = enforce.numeric(test_dataset)
#'
#' @title enforce.numeric
#' @param x A TRONCO compliant dataset.
#' @return Convert the internal reprensentation of genotypes to numeric, if not.
#' @export enforce.numeric
#' 
enforce.numeric <- function(x) {
    is.compliant(x)
    if (!all(is.numeric(x$genotypes[1, ]))) {
        rn = as.samples(x)
        x$genotypes = apply(x$genotypes, 2, as.numeric)
        rownames(x$genotypes) = rn
    }

    return(x)
}


#' Convert the internal representation of genotypes to character, if not. 
#'
#' @examples
#' data(test_dataset)
#' test_dataset = enforce.string(test_dataset)
#'
#' @title enforce.string
#' @param x A TRONCO compliant dataset.
#' @return Convert the internal reprensentation of genotypes to character, if not.
#' @export enforce.string
#' 
enforce.string <- function(x) {
    is.compliant(x)
    if (!all(is.character(x$genotypes[1,]))) {
        rn = as.samples(x)
        x$genotypes = apply(x$genotypes, 2, as.character)
        rownames(x$genotypes) = rn
    }

    return(x)
}


#' Sort the internal genotypes according to event frequency.
#'
#' @examples
#' data(test_dataset)
#' order.frequency(test_dataset)
#'
#' @title order.frequency
#' @param x A TRONCO compliant dataset.
#' @param decreasing Inverse order. Default TRUE
#' @return A TRONCO compliant dataset with the internal genotypes sorted according to event frequency.
#' @export order.frequency
#' 
order.frequency <- function(x, decreasing = TRUE) {
    is.compliant(x)

    x = enforce.numeric(x)
    sums = colSums(x$genotypes)

    x$genotypes = x$genotypes[, order(sums, decreasing = decreasing), drop = FALSE]
    x$annotations = x$annotations[colnames(x$genotypes), , drop = FALSE]

    return(x)  
}


#' Convert colnames/rownames of a matrix into intelligible event names, e.g., change a key G23 in 'Mutation KRAS'.
#' If a name is not found, the original name is left unchanged.
#'
#' @examples
#' data(test_model)
#' adj_matrix = as.adj.matrix(test_model, events=as.events(test_model)[5:15,])$capri_bic
#' keysToNames(test_model, adj_matrix)
#'
#'
#' @title keysToNames
#' @param x A TRONCO compliant dataset.
#' @param matrix A matrix with colnames/rownames which represent genotypes keys. 
#' @return The matrix with intelligible colnames/rownames. 
#' @export keysToNames
#' 
keysToNames <- function(x, matrix) {
    is.compliant(x)
    if (!is.matrix(matrix)
        || any(is.null(colnames(matrix)))
        || any(is.null(rownames(matrix))))
        stop('"matrix" should be a matrix with rownames/colnames.')

    events = as.events(x)
    resolve <- function(y) { 
        if (y %in% rownames(events)) paste(events[y,], collapse=' ')
        else y
    }

    colnames(matrix) = sapply(colnames(matrix), resolve)
    rownames(matrix) = sapply(rownames(matrix), resolve)
    return(matrix)
}

#' Convert to key an intelligible event names, e.g., change 'Mutation KRAS' in G23.
#' If a name is not found, an error is raised!
#'
#' @examples
#' data(test_model)
#' adj_matrix = as.adj.matrix(test_model, events=as.events(test_model)[5:15,])$bic
#'
#' @title nameToKey
#' @param x A TRONCO compliant dataset.
#' @param name A intelligible event name
#' @return A TRONCO dataset key name
#' @export nameToKey
#'
nameToKey <- function(x, name) {
    is.compliant(x)

    types = as.types(x)
    for (i in types) {
        if (nchar(name) > nchar(i) && substr(name, 1, nchar(i)) == i)
            return(rownames(as.events(x, 
                                      genes = substr(name, nchar(i) + 2, nchar(name)),
                                      types = substr(name, 1, nchar(i)))))
    }
    
    stop('"name" is not a key!')
}


# Check if logic node down
#
# @title is logical node down
# @param node A node identifier
# @return boolean
#
is.logic.node.down <- function(node) {
    if (substr(node, start = 1, stop = 3) == 'OR_')
        return(TRUE)
    if (substr(node, start = 1, stop = 4) == 'XOR_')
        return(TRUE)
    if (substr(node, start = 1, stop = 4) == 'AND_')
        return(TRUE)
    if (substr(node, start = 1, stop = 4) == 'NOT_')
        return(TRUE)
    return(FALSE)
}


# Check if logic node up
#
# @title is logical node up
# @param node A node identifier
# @return boolean
is.logic.node.up <- function(node) {
    if (substr(node, start = 1, stop = 2) == 'UP')
        return(TRUE)
    return(FALSE)
}


# Check if logic node down or up
#
# @title is logical node
# @param node A node identifier
# @return boolean

is.logic.node <- function(node) {
    return(is.logic.node.up(node) || is.logic.node.down(node))
}


as.categorical.dataset <- function(dataset){

    ## Create a categorical data frame from the dataset
    data = array("missing", c(nrow(dataset), ncol(dataset)))

    for (i in 1:nrow(dataset)) {
        for (j in 1:ncol(dataset)) {
            if (dataset[i,j] == 1) {
                data[i,j] = "observed"
            }
        }
    }

    data = data.frame(data, stringsAsFactors = TRUE)
    for (n in names(data)) {
        levels(data[[n]]) = c('missing', 'observed')
    }

    ## Renaming
    colnames(data) = colnames(dataset)
    rownames(data) = rownames(dataset)
    return(data)
}



# Convert a TRONCO object in a Bnlearn network.
# title as.bnlearn.network
#
# examples
# data(test_model)
# as.bnlearn.network(test_model)
#
# param x A reconstructed model (the output of tronco.capri or tronco.caprese)
# param model The name of the selected regularization
# export as.bnlearn.network
# importFrom bnlearn empty.graph set.arc
#
as.bnlearn.network <- function(x, 
                               model = names(as.models(x))[1]) {

    ## Check if there is a reconstructed model.

    if(!has.model(x)) {
        stop('Input doesn\'t have a TRONCO object inside.')
    }

    ## Check if the selected regularization is used in the model.

    if (!model %in% names(as.models(x))) {
        stop(paste(model, " was not used to build the input TRONCO model!"))
    }

    ## Get genotypes and data.

    genotypes = as.genotypes(x)
    genotypes = as.matrix(genotypes)
    genotypes = keysToNames(x, genotypes)
    names(colnames(genotypes)) = NULL

    df = as.categorical.dataset(genotypes)

    adj.matrix = get(model, as.adj.matrix(x, models = model))
    adj.matrix = keysToNames(x, adj.matrix)

    
    bayes.net = NULL
    bayes.net$data = df
        
    net = empty.graph(colnames(genotypes))
    for (from in rownames(adj.matrix)) {
        for (to in colnames(adj.matrix)) {
            if (adj.matrix[from, to] == 1) {
                net = set.arc(net, from, to)
            }
        }
    }
    
    bayes.net$net = net
    return(bayes.net) 
}

#### end of file -- as.functions.R

data(maf)
muts = import.MAF(maf, silent = TRUE)
hypo = hypothesis.add(muts, 'test', OR('ABAT', 'ABCC3'))
no_hypo = delete.hypothesis(hypo, 'test')
data(crc_gistic)
gistic_err = import.GISTIC(crc_gistic, silent = TRUE)
gistic = delete.gene(gistic_err, 'APC')
gistic_model = tronco.caprese(gistic, silent = TRUE)
gistic_model_capri = tronco.capri(gistic, nboot = 2, silent = TRUE)

stages = list()
stages$stage = c('A', 'B', 'C')
names(stages$stage) = as.samples(muts)
stages = as.data.frame(stages)
muts_stages = annotate.stages(muts, stages=stages)

context("as.genotypes")

test_that("as.genotypes returns a genotypes matrix", {
    expect_equal(length(as.genotypes(muts)), 39)
    expect_equal(as.genotypes(NULL), NULL)
})

context("as.samples")

test_that("as.samples returns a list of samples", {
    expect_equal(as.samples(muts), 
                 unique(as.character(maf$Tumor_Sample_Barcode)))
    expect_equal(as.samples(NULL), NULL)
})

context("as.genes")

test_that("as.genes returns a list of samples", {
    expect_equal(as.genes(muts), unique(as.character(maf$Hugo_Symbol)))
    expect_equal(as.genes(hypo), unique(as.character(maf$Hugo_Symbol)))
    expect_equal(as.genes(NULL), NULL)
    expect_error(as.genes(hypo, types = 'Pattern'))
    expect_equal(length(as.genes(hypo)), 13)
})

context("as.events")

test_that("as.events returns a list of samples", {
    expect_equal(length(as.events(muts)), 26)
    expect_equal(length(as.events(hypo, types = 'Mutation')), 26)
    expect_equal(as.events(hypo, keysToNames = TRUE)[1],
                 "Mutation A2BP1")
    expect_equal(as.events(NULL), NULL)
})

context("as.stages")

test_that("as.stages returns a list of stages", {
    expect_equal(length(as.stages(muts_stages)), 1)
    expect_equal(as.stages(muts), NA)
    expect_equal(as.stages(NULL), NA)
})

context("as.types")

test_that("as.types returns a list of types", {
    expect_equal(as.types(muts), 'Mutation')
    expect_equal(as.types(NULL), NULL)
})

context("as.colors")

test_that("as.colors returns a list of types", {
    areColors <- function(x) {
        sapply(x, function(X) {
            tryCatch(is.matrix(col2rgb(X)), 
                error = function(e) FALSE)
        })
    }
    expect_true(all(areColors(as.colors(muts))))
    expect_equal(as.colors(NULL), NULL)
})

context("as.gene")

test_that("as.gene returns a list of types", {
    expect_equal(as.gene(muts, genes='A2BP1')[1,1], 1)
})

context("as.alteration")

test_that("as.alteration returns a compliant dataset", {
    expect_equal(unique(as.alterations(gistic, silent = TRUE)$annotations[,'type']),
                 "Alteration")
    expect_error(as.alterations(hypo))
    expect_error(as.alterations(gistic_model))
})

context("as.patterns")

test_that("as.patterns return a list of patterns", {
    expect_null(as.patterns(muts))
    expect_equal(as.patterns(hypo),
                 "test")
})

context("as.hypotheses")

test_that("as.hypotheses return a list of hypotheses", {
    expect_null(as.hypotheses(gistic))
    expect_equal(length(as.hypotheses(hypo)), 88)
    expect_equal(length(as.hypotheses(hypo, cause = 'A2BP1')), 4)
    expect_equal(length(as.hypotheses(hypo, effect = 'A2BP1')), 4)
    expect_error(as.hypotheses(hypo, cause = 'X'))
    expect_error(as.hypotheses(hypo, effect = 'X'))
})

context("as.events.in.patterns")

test_that("as.events.in.patterns return the right amount of genes", {
    expect_equal(length(as.events.in.patterns(hypo)), 4)
    expect_equal(length(as.events.in.patterns(hypo, pattern = 'test')), 4)
    expect_error(as.events.in.patterns(hypo, pattern = 'X'))
})

context("as.genes.in.patterns")

test_that("as.genes.in.patterns return the right amount of genes", {
    expect_equal(length(as.genes.in.patterns(hypo)), 2)
})

context("as.types.in.patterns")

test_that("as.types.in.patterns return the right amount of genes", {
    expect_equal(length(as.types.in.patterns(hypo)), 1)
})

context("as.events.in.sample")

test_that("as.events.in.sample return the right amount of genes", {
    expect_equal(length(as.events.in.sample(muts, as.samples(muts)[1])), 8)
})

context("as.models")

test_that("as.models return a model", {
    expect_equal(length(as.models(gistic_model)), 1)
    expect_error(as.models(gistic_model, models = 'X'))
})

context("as.description")

test_that("as.description return a description", {
    expect_equal(as.description(hypo), "")
    desc = annotate.description(hypo, 'desc')
    expect_equal(as.description(desc), "desc")
})

context("as.pathway")

test_that("as.pathway return compliant dataset", {
    expect_output(as.pathway(muts,
                      pathway.name = 'test',
                      pathway.genes = 'APC'),
                "succesfully.")
    expect_output(as.pathway(muts,
                      pathway.name = 'test',
                      pathway.genes = 'APC',
                      aggregate.pathway = FALSE),
                "succesfully.")
    expect_output(as.pathway(muts_stages,
                      pathway.name = 'test',
                      pathway.genes = 'APC'),
                "succesfully.")
})

context("as.adj.matrix")

test_that("as.adj.matrix return a matrix", {
    expect_equal(length(as.adj.matrix(gistic_model_capri, 
                                      type = 'fit')), 2)
    expect_equal(length(as.adj.matrix(gistic_model_capri,
                                      type = 'pf')), 2)
    expect_error(as.adj.matrix(gistic_model_capri,
                               type = 'X'))
    expect_error(as.adj.matrix(gistic_model_capri,
                               models = 'X'))
})

context("as.duplicates")

test_that("as.duplicates return false", {
    expect_false(has.duplicates(hypo))
})

context("as.parents.pos")

test_that("as.parents.pos return the right amount of models", {
    expect_equal(length(as.parents.pos(gistic_model_capri)), 2)
    expect_error(as.parents.pos(gistic_model_capri, models = 'banana'))
})

context("duplicates")

test_that("duplicates return 0", {
    expect_equal(length(duplicates(hypo)), 0)
})

context("npatterns")

test_that("npatterns return the right amount of pattern", {
    expect_equal(length(duplicates(hypo)), 0)
})

context("view")

test_that("view return output", {
    expect_output(view(gistic_model_capri), regexp=NULL)
})

context("order.frequency")

test_that("order.frequency reorder the genotypes", {
    expect_equal(length(order.frequency(gistic_model_capri)), 9)
})

context("enforce.string")

test_that("enforce.string return a string genotypes", {
    expect_equal(length(enforce.string(gistic_model_capri)), 9)
})

data(test_dataset)
data(test_dataset_no_hypos)

model = tronco.capri(test_dataset, nboot = 1, silent = TRUE)
eloss = tronco.kfold.eloss(model, k = 2, runs = 2, silent = TRUE)
prederr = tronco.kfold.prederr(model, k = 2, runs = 2, cores.ratio = 0, silent = TRUE)
posterr = tronco.kfold.posterr(model, k = 2, runs = 2, cores.ratio = 0, silent = TRUE)

context("eloss")

test_that("eloss", {
    expect_output(tronco.kfold.eloss(model, k = 2, runs = 2), 'loss')
    expect_equal(length(as.kfold.eloss(eloss)), 3) 
    expect_equal(length(as.confidence(eloss, conf = 'eloss')), 1)
})

context("prederr")

test_that("eloss", {
    expect_output(tronco.kfold.prederr(model, k = 2, runs = 2, cores.ratio = 0), 'prediction')
    expect_equal(length(as.kfold.prederr(prederr)), 2)
    expect_equal(length(as.kfold.prederr(prederr, table = TRUE)), 2)
    expect_equal(length(as.confidence(prederr, conf = 'prederr')), 1)
})


context("posterr")

test_that("posterr", {
    expect_output(tronco.kfold.posterr(model, k = 2, runs = 2, cores.ratio = 0), 'posterior')
    expect_equal(length(as.kfold.posterr(posterr)), 2)
    expect_equal(length(as.kfold.posterr(posterr, table = TRUE)), 2)
    expect_equal(length(as.confidence(posterr, conf = 'posterr')), 1)
})

---
title: "Loading data"
author: 
  - Luca De Sano
  - Daniele Ramazzotti
  - Giulio Caravagna
  - Alex Graudenxi
  - Marco Antoniotti
date: "`r format(Sys.time(), '%B %d, %Y')`"
graphics: yes
package: TRONCO
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Loading data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{TRONCO,BiocStyle}
---


```{r include=FALSE}
library(knitr)
opts_chunk$set(
concordance = TRUE,
background = "#f3f3ff"
)
```

## Preliminaries



```{r}
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)
```




TRONCO transforms input data in a sort of database-alike format, where three main fields are presente: *genotypes* which contains the genomic signatures of the input samples, *annotations* which provides an index to the events present in the data and *types*, a field mapping type of events (e.g., mutations, CNAs, etc.) to colors for display visualization. Other annotations are generated when a dataset is augmented with some metadata.  A TRONCO object shall be edited by using TRONCO functions,  to avoid to create inconsistencies in its internal representation. Function `is.compliant` can be used to test if a TRONCO object is consistent; the function is called by any TRONCO function before returning  a modified object, so to ensure that consistency is preserved -- `is.compliant`  will raise an error if this is not the case.

TRONCO supports the import of data from 3 formats. The Mutation Annotation Format (*MAF*) is a tab-delimited file containing somatic and/or germline mutation annotations; the *GISTIC* format for copy number alterations as defined by TCGA and a custom boolean matrix format where the user can directly specify the mutational profiles to be importend. Through some data included in the package we will show how to load your datasets in TRONCO.  

- **aCML** a TRONCO object that represents the *atypical Chronic Myeloid Leukemia* dataset by Piazza *et al.* (Nat. Gen. 2013 45(1):18-24).
- **crc_maf**  a shortened version of the *colorectal cancer mutation data* made available by the TCGA consortium within the COADREAD. See [this link](https://tcga-data.nci.nih.gov/docs/publications/coadread\_2012/) and our PicNiC case study for the real analysis of such data.
 - **crc_gistic** from the same TCGA  project, we also provide a shortened version of the focal CNAs in the GISTIC format where 1 represents a low level gain, 2 a high level gain, -1 a heterozygous loss of a gene and -2 its homozygous loss.
- **crc_plain**  a custom boolean matrix where rows are samples, and columns represent events -- in this case alterations  in a certain gene. Notice with this format one could also custom types of alterations, for instance wider chromosomal aberrations or, in principle, epigenetic states (over-expression, methylated regions, etc.) that are persistent across tumor evolution. 

Whatever is dataset created as explained in the next sections, it can be annotated by adding a mnemonic description of the data, which will be used as plot titles when possible.
Function `annotate.description` raises a warning if the dataset was previously annotated.

```{r, eval=FALSE}
aCML = annotate.description(aCML, 'aCML data (Bioinf.)')
```


## Mutations annotated in a MAF format
We use the function `import.MAF` to import a dataset in MAF format, in this case the following  TCGA dataset

```{r}
head(crc_maf[, 1:10])
```


A default importation is done  without adding parameters to `import.MAF`. In this case, all mutations per gene will be considered equivalent, regardless of the type that is annotated in the MAF.  Also, all genes will be imported, and all samples.


```{r}
dataset_maf = import.MAF(crc_maf)
```


In the above case -- where we see that mutations are annotated as `Missense_Mutation` or `Nonsense_Mutation`, if a gene in a sample has both, these will be merged to a unique `Mutation` type. In this case a pair gene name with `Mutation` will be what we call an *event* in our dataset -- e.g.,  APC Mutation.
If one would like  to have two distinct events in the dataset, i.e., APC `Missense_Mutation` and APC `Nonsense_Mutation`, parameter `merge.mutation.types` should be set to false in the call to `import.MAF`. 


```{r}
dataset_maf = import.MAF(crc_maf, merge.mutation.types = FALSE)
```

Sometimes, we might want to filter out some of the entries in a MAF -- maybe restricting the type of genes, mutations or sample that we want to process. If one defines
`filter.fun` as a function that returns `TRUE` only for those entries which shall be considered, he gets a filter process which is applied to each row of the MAF file prior to transforming that into a TRONCO dataset. In this example we select only mutations annotated to APC -- we access that through the *Hugo_Symbol* flag of a MAF. 

```{r}
dataset_maf = import.MAF(crc_maf, filter.fun = function(x){ x['Hugo_Symbol'] == 'APC'} )
```


It is also sometimes convenient -- especially when working with data collected from a single individual patient -- to distinguish the type of mutations and their position in a gene, or if they are somehow annotated to COSMIC or other databases. For instance, we might want to want to use the `MA.protein.change` annotation in the MAF file to get composite names such as TP53.R175H, TP53.R213, TP53.R267W etc. This can be done by setting {\tt paste.to.Hugo\_Symbol} to have the relevant name of the MAF annotation


```{r}
dataset_maf = import.MAF(crc_maf, 
    merge.mutation.types = FALSE, 
    paste.to.Hugo_Symbol = c('MA.protein.change'))
```


TRONCO supports custom  MAF files, where possibly not all the standard annotations are present, via `irregular = TRUE`.


## Copy Number Variants annotated in the GISTIC format
We use the function `import.GISTIC` to import a dataset in GISTIC format, in this case from

```{r}
crc_gistic
```

In its default execution all the data annotated in the file is imported. But in principle  it is possible to avoid to import some genes or samples; in this case it is sufficient to use parameters `filter.genes` and `filter.samples` for this function.


```{r}
dataset_gistic = import.GISTIC(crc_gistic)
```

## Custom alterations annotated in a boolean matrix
One can annotate its custom type of alterations in a boolean matrix such as `crc_plain`

```{r}
crc_plain
```

In this case,  function `import.genotypes` will convert the matrix to a TRONCO  object where events' names and samples codes will be set from column and row names of the matrix. If this is not possible, these will be generated from templates. By default, the `event.type` is set to `variant` but one can specify a custom name for the alteration that is reported in the matrix

```{r}
dataset_plain = import.genotypes(crc_plain, event.type='myVariant')
```



## Downloading data from the cBio portal for cancer genomics

TRONCO uses the R interface to cBio to query data from the portal. All type of data can be downloaded from the portal, which includes MAF/GISTIC data for a lot of different cancer studies. An example of interaction with the portal is archived at the tool's webpage. 

Here, we  show how to download lung cancer data somatic mutations for genes TP53, KRAS and PIK3CA,  from the lung cancer project run by TCGA, which is archived as  *luad_tcga_pub* at cBio. If some of the parameters to `cbio.query` are missing the function will become interactive by showing a list of possible data available at the portal.

```{r results='hide', eval=FALSE}
data = cbio.query(
    genes=c('TP53', 'KRAS', 'PIK3CA'), 
    cbio.study = 'luad_tcga_pub',
    cbio.dataset = 'luad_tcga_pub_cnaseq', 
    cbio.profile = 'luad_tcga_pub_mutations')
```

---
title: "Data manipulation"
author: 
  - Luca De Sano
  - Daniele Ramazzotti
  - Giulio Caravagna
  - Alex Graudenxi
  - Marco Antoniotti
date: "`r format(Sys.time(), '%B %d, %Y')`"
graphics: yes
package: TRONCO
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Data manipulation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{TRONCO,BiocStyle}
---


```{r, include=FALSE}
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)
```

All examples in this section will be done with the the aCML dataset as reference.

## Modifying events and samples

TRONCO  provides functions for renaming the events that were included in a dataset, or the type associated to a set of events (e.g., a `Mutation` could be renamed to a `Missense Mutation`).


```{r}
dataset = rename.gene(aCML, 'TET2', 'new name')
dataset = rename.type(dataset, 'Ins/Del', 'new type')
as.events(dataset, type = 'new type')
```


and return a modified TRONCO object. More complex operations are also possible. For instance, two events with the same signature -- i.e., appearing in the same samples -- can be joined to a new event (see also Data Consolidation in Model Inference) with the same signature and a new name.


```{r }
dataset = join.events(aCML, 
    'gene 4',
    'gene 88',
    new.event='test',
    new.type='banana',
    event.color='yellow')
```

    
where in this case we also created a new event type, with its own color.

In a similar way we can decide to join all the events of two distinct types, in this case if a gene *x* has signatures for both type of events, he will get a unique signature with an alteration present if it is either of the second *or* the second type 

```{r}
dataset = join.types(dataset, 'Nonsense point', 'Nonsense Ins/Del')
as.types(dataset)
```


TRONCO also provides  functions for deleting specific events, samples or types.


```{r}
dataset = delete.gene(aCML, gene = 'TET2')
dataset = delete.event(dataset, gene = 'ASXL1', type = 'Ins/Del')
dataset = delete.samples(dataset, samples = c('patient 5', 'patient 6'))
dataset = delete.type(dataset, type = 'Missense point')
view(dataset)
```

    
## Modifying patterns

TRONCO  provides functions to edit patterns,  pretty much as for any other type of events. Patterns however have a special denotation and are supported only by CAPRI algorithm -- see Model Reconstruction with CAPRI to see a practical application of that.


## Subsetting a dataset


It is very often the case that we want to subset a dataset by either selecting only some of its samples, or some of its events. Function `samples.selection` returns a dataset with only some selected samples.


```{r}
dataset = samples.selection(aCML, samples = as.samples(aCML)[1:3])
view(dataset)
```


Function `events.selection`, instead, performs selection according to a filter of events. With this function, we can subset data according to a frequency, and we can force inclusion/exclusion of certain events by specifying their name. For instance, here we pick all events with a minimum frequency of 5%, force exclusion of SETBP1 (all events associated), and inclusion of EZH1 and EZH2.

```{r}
dataset = events.selection(aCML,  filter.freq = .05, 
    filter.in.names = c('EZH1','EZH2'), 
    filter.out.names = 'SETBP1')
```
```{r}
as.events(dataset)
```


An example visualization of the data before and after the selection process can be obtained by combining the `gtable` objects returned by \Rfunction{oncoprint}. We here use `gtable = T` to get access to have a GROB table returned, and `silent = T` to avoid that the calls to the function display on the device; the call to `grid.arrange` displays the captured `gtable` objects.


```{r fig.width=7, fig.height=5.5, fig.cap="Multiple output from oncoprint can be captured as a gtable and composed via grid.arrange (package gridExtra). In this case we show  aCML data on top -- displayed after the as.alterations transformation -- versus a selected subdataset of events with a minimum frequency of 5%, force exclusion of SETBP1 (all events associated), and inclusion of EZH1 and EZH2.", results='hide'}
library(gridExtra)
grid.arrange(
    oncoprint(as.alterations(aCML, new.color = 'brown3'), 
        cellheight = 6, cellwidth = 4, gtable = TRUE,
        silent = TRUE, font.row = 6)$gtable,
    oncoprint(dataset, cellheight = 6, cellwidth = 4,
        gtable = TRUE, silent = TRUE, font.row = 6)$gtable, 
    ncol = 1)
```

---
title: "Model inference"
author: 
  - Luca De Sano
  - Daniele Ramazzotti
  - Giulio Caravagna
  - Alex Graudenxi
  - Marco Antoniotti
date: "`r format(Sys.time(), '%B %d, %Y')`"
graphics: yes
package: TRONCO
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Model inference}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{TRONCO,BiocStyle}
---


```{r, include=FALSE}
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)
```

We make use of the most of the  functions described above to show how to perform  inference with various algorithms; the reader should read first those sections  of the vignette to have an explanation of how those functions work. The aCML dataset is used as a test-case for all algorithms, regardless it should be precessed by algorithms to infer ensemble-level progression models.

To replicate the plots of the original  paper were the aCML dataset was first analyzed with CAPRI, we can change the colors assigned to each type of event with the function `change.color`. 


```{r}
dataset = change.color(aCML, 'Ins/Del', 'dodgerblue4')
dataset = change.color(dataset, 'Missense point', '#7FC97F')
as.colors(dataset)
```


#### Data consolidation.

All TRONCO algorithms require an input dataset were events have non-zero/non-one probability, and are all distinguishable. The tool provides a function to  return lists of events which do not satisfy these constraint.

```{r}
consolidate.data(dataset)
```


The aCML data has none of the above issues (the call returns empty lists); if this were not the case data manipulation functions can be used to edit a TRONCO object.


## CAPRI

 In what follows, we show CAPRI's functioning by replicating the aCML case study presented in CAPRI's original paper. Regardless from which types of mutations we include, we select only the genes  mutated at least in the 5% of the patients -- thus   we first use `as.alterations` to have gene-level frequencies, and then we apply there a frequency filter (R's output is omitted). 


```{r}
alterations = events.selection(as.alterations(aCML), filter.freq = .05)
```


To proceed further with the example we  create the *dataset* to be used for the inference of the  model. From the original dataset  we select all the genes whose mutations are occurring at least the 5% of the times, and we get that by the alterations profiles; also we force inclusion of all the events for the  genes involved in an hypothesis (those included in variable `gene.hypotheses`, this list is based on the support  found in the literature of potential aCML patterns).


```{r}
gene.hypotheses = c('KRAS', 'NRAS', 'IDH1', 'IDH2', 'TET2', 'SF3B1', 'ASXL1')
aCML.clean = events.selection(aCML,
    filter.in.names=c(as.genes(alterations), gene.hypotheses))
aCML.clean = annotate.description(aCML.clean, 
    'CAPRI - Bionformatics aCML data (selected events)')
```


We show a new oncoprint of this latest dataset where we annotate the genes in `gene.hypotheses` in order to identify them. The sample names are also shown. 


```{r fig.width=8, fig.height=5.5, fig.cap="Data selected for aCML reconstruction annotated with the events which are part of a pattern that we will input to CAPRI."}
oncoprint(aCML.clean, gene.annot = list(priors = gene.hypotheses), sample.id = TRUE)
```


### Testable hypotheses via logical formulas (i.e., patterns)

CAPRI is the only algorithm in TRONCO that supports hypotheses-testing of  causal structures expressed as logical formulas with AND, OR and XOR operators.  An example invented formula could be

(APC:*Mutation* **XOR** APC:*Deletion*) **OR** CTNNB1:*Mutation*

where APC mutations and deletions are in disjunctive relation with CTNNB1 mutations; this is done to  test if those events could confer equivalent fitness in terms of ensemble-level progression -- see the original CAPRI paper and the PiCnIc pipeline for detailed explanations.

Every formula is transformed into a CAPRI  *pattern*. For every hypothesis it is possible  to specify against which possible target event it should be tested, e.g., one might test the above formula against PIK3CA mutations, but not ATM ones. If this is not done, a pattern is tested against all other events in the dataset but those which constitute itself. A pattern tested against one other event is called an hypothesis.

#### Adding custom hypotheses.

We add the hypotheses that are described in CAPRI's manuscript; we start with  hard exclusivity (XOR) for NRAS/KRAS mutation, 


NRAS:*Missense point* **XOR** KRAS:*Missense point*


tested against all the events in the dataset (default `pattern.effect = *`)
 

```{r }
aCML.hypo = hypothesis.add(aCML.clean, 'NRAS xor KRAS', XOR('NRAS', 'KRAS'))
```

When a pattern is included, a new column in the dataset is created -- whose signature is given by the evaluation of the formula constituting the pattern. We call this operation *lifting of a pattern*, and this shall create not inconsistency in the data -- i.e., it shall not duplicate any of the other columns. TRONCO check this; for instance when we try to include  a soft exclusivity (OR) pattern for the above genes we get an error (not shown).

```{r eval=FALSE}
aCML.hypo = hypothesis.add(aCML.hypo, 'NRAS or KRAS',  OR('NRAS', 'KRAS'))
```


Notice that TRONCO functions can be used to look at their alterations and understand why the OR signature is equivalent to the XOR one -- this happens as no samples harbour both mutations.


```{r fig.width=6, fig.height=1, fig.cap="Oncoprint output to show the perfect (hard) exclusivity among NRAS/KRAS mutations in aCML"}
oncoprint(events.selection(aCML.hypo,
    filter.in.names = c('KRAS', 'NRAS')),
    font.row = 8,
    ann.hits = FALSE)
```


We repeated the same analysis as before for other hypotheses and for the same reasons, we will include only the hard exclusivity pattern. In this case we add a two-levels pattern


SF3B1:*Missense point* **XOR** (ASXL1:*Ins/Del* **XOR** ASXL1:*Nonsense point*)


since ASXL1 is mutated in two different ways, and no samples harbour both mutation types. 


```{r }
aCML.hypo = hypothesis.add(aCML.hypo, 'SF3B1 xor ASXL1', XOR('SF3B1', XOR('ASXL1')),
    '*')
```



Finally, we now do the same for genes TET2 and IDH2. In this case 3 events for the gene TET2 are present, that is `Ins/Del`, `Missense point` and `Nonsense point`. For this reason, since we are not specifying any subset of such events to be considered, all TET2 alterations are used. Since the events present a perfect hard exclusivity, their patters will be included as a XOR. 


```{r }
as.events(aCML.hypo, genes = 'TET2') 
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 xor IDH2',
    XOR('TET2', 'IDH2'),
    '*')
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 or IDH2',
    OR('TET2', 'IDH2'),
    '*')
```


Which is the following pattern


(TET2:*Ins/Del*) **XOR** (TET2:*Missense point*) **XOR** (TET2:*Nonsense point*) **XOR** (IDH2:*Missense point*)


which we can visualize via an `oncoprint`.


```{r fig.width=7, fig.height=2, fig.cap="{oncoprint} output to show the soft exclusivity among NRAS/KRAS mutations in aCML"}
oncoprint(events.selection(aCML.hypo,
    filter.in.names = c('TET2', 'IDH2')),
    font.row = 8,
    ann.hits = FALSE)
```



#### Adding (automatically) hypotheses for homologous events.

We consider homologous events those having the same mnemonic name -- as of function `as.genes` -- but events of different type. For instance, mutations and deletions of the same gene would be considered such (e.g., in the aCML dataset ASXL1 Ins/Del and Nonsense point).
It could be a good idea to test such events, in terms of progression fitness, to test is they might be equivalent; we can do that by building a pattern of exclusivity among them. TRONCO has a function to make this automatically which, by default, adds a soft exclusivity OR pattern among them. 



```{r }
aCML.hypo = hypothesis.add.homologous(aCML.hypo)
```


This function added one  pattern for each of TET2, EZH2, CBL, ASXL1, CSF3R (unless they created duplicated columns in the dataset), with a connective OR/XOR which is appropriate for the events considered; for instance the TET2 homologous pattern 


(TET2:*Ins/Del*) **XOR** (TET2:*Missense point*) **XOR** (TET2:*Nonsense point*) 


was created with a XOR function, as TET2 appears in perfect exclusivity.


#### Adding (automatically) hypotheses for a group of genes.

The idea behind the previous function is generalized by `hypothesis.add.group`, that add a set of hypotheses that can be combinatorially created out of a  group of genes. As such, this function can create an exponential number of hypotheses and should be used with caution as too many hypotheses, with respect to sample size, should not be included. 

This function takes, among its inputs, the top-level logical connective, AND/OR/XOR, a minimum/maximum pattern size -- to restrict the combinatorial sampling of subgroups --, plus a parameter that can be used to constrain the minimum event frequency. If, among the events included some of them have homologous, these are put automatically nested with the same logic of the `hypothesis.add.group` function.


```{r }
dataset = hypothesis.add.group(aCML.clean, OR, group = c('SETBP1', 'ASXL1', 'CBL'))
```

The final dataset that will be given as input to CAPRI is  finally shown. Notice the signatures of  all the lifted patterns.

```{r fig.width=8, fig.height=6.5, fig.cap="oncoprint} output of the a dataset that has patterns that could be given as input to CAPRI to retrieve a progression model."}
oncoprint(aCML.hypo, gene.annot = list(priors = gene.hypotheses), sample.id = TRUE, 
    font.row=10, font.column=5, cellheight=15, cellwidth=4)
```


#### Querying, visualizing and manipulating CAPRI's patterns.


We also provide functions to get the number of hypotheses and patterns present in the data.

```{r}
npatterns(dataset)
nhypotheses(dataset)
```


We can visualize any pattern or the elements involved in them with the following functions.

```{r}
as.patterns(dataset)
as.events.in.patterns(dataset)
as.genes.in.patterns(dataset)
as.types.in.patterns(dataset)
```


Similarily, we can enumerate the  hypotheses with the function `as.hypotheses`, and delete certain patterns and hypotheses. Deleting a pattern consists in deleting all of its hypotheses.


```{r}
head(as.hypotheses(dataset))
dataset = delete.hypothesis(dataset, event = 'TET2')
dataset = delete.pattern(dataset, pattern = 'OR_ASXL1_CBL')
```


#### How to build a pattern.

It is sometimes of help to plot some information about a certain combination of events, and a target -- especially to disentangle the proper logical connectives to use when building a pattern. Here, we test genes SETBP1 and ASXL1 versus Missense point mutations of  CSF3R, and observe that the majority of observations are mutually exclusive, but almost half of the  CSF3R mutated samples with Missense point mutations do not harbout any mutation in SETBP1 and ASXL1.


```{r pattern-plot,fig.show='hide', fig.width=4, fig.height=2.2, fig.cap="Barplot to show an hypothesis: here we test genes SETBP1 and ASXL1 versus Missense point mutations of  CSF3R, which suggests that  that pattern does not 'capture' all the samples with  CSF3R mutations."}
tronco.pattern.plot(aCML,
    group = as.events(aCML, genes=c('SETBP1', 'ASXL1')),
    to = c('CSF3R', 'Missense point'),
    legend.cex=0.8,
    label.cex=1.0)
```

It is also possible to create a circle plot where we can observe the contribution of genes SETBP1 and ASXL1 in every match with a Missense point mutations of  CSF3R.


```{r pattern-plot-circos, fig.width=6, fig.height=6, fig.cap="Circos to show an hypothesis: here we test genes SETBP1 and ASXL1 versus Missense point mutations of  CSF3R. The combination of this and the previous  plots should allow to understand which pattern we shall write in an attempt to capture a potential causality relation between the pattern and the event."}
tronco.pattern.plot(aCML,
    group = as.events(aCML, genes=c('TET2', 'ASXL1')),
    to = c('CSF3R', 'Missense point'),
    legend = 1.0,
    label.cex = 0.8,
    mode='circos')
```




### Model reconstruction

We run the inference of the model by CAPRI algorithm with its default parameter: we use both AIC and BIC as regularizators, Hill-climbing as heuristic search of the solutions and exhaustive bootstrap (`nboot` replicates or more for Wilcoxon testing, i.e., more iterations can be performed if samples are rejected), p-value are set at 0.05. We set the seed for the sake of reproducibility. 

```{r results='hide', include=FALSE}=
aCML.hypo = annotate.description(aCML.hypo, '')
aCML.clean = annotate.description(aCML.clean, '')
```

```{r}
model.capri = tronco.capri(aCML.hypo, boot.seed = 12345, nboot = 5)
model.capri = annotate.description(model.capri, 'CAPRI - aCML')
```


## CAPRESE

The CAPRESE algorithm is one of a set of algorithms to reconstruct progression models from data of an individual patient. This algorithm uses a shrinkage-alike estimator combining correlation and probability raising among pair of events. This algorithm shall return a forest of trees, a special case of a Suppes-Bayes Causal Network.

Despite this is derived to infer progression models from individual level data, we use it here to process aCML data (without patterns and with its default parameters). This algorithm has no bootstrap and, as such, is the quickest available in TRONCO.


```{r}
model.caprese = tronco.caprese(aCML.clean)
model.caprese = annotate.description(model.caprese, 'CAPRESE - aCML')
```




## Directed Minimum Spanning Tree with Mutual Information

This algorithm is meant to  extract a forest of trees of progression from data of an individual patient. This algorithm is based on a formulation of the  problem in terms of minimum spamming trees and exploits results from Edmonds. We test it to infer a model from aCML data as we did with CAPRESE.


```{r}
model.edmonds = tronco.edmonds(aCML.clean, nboot = 5, boot.seed = 12345)
model.edmonds = annotate.description(model.edmonds, 'MST Edmonds - aCML')
```


## Partially Directed Minimum Spanning Tree with Mutual Information

This algorithm extends the previous one in situations where it is not possible to fully assess a confident time ordering among the nodes, hence leading to a partially directed input. This algorithm adopts Gabow search strategy to evaluate the best directed minimum spanning tree among such undirected components. We test it to infer a model from aCML data as all the other algorithms.


```{r}
model.gabow = tronco.gabow(aCML.clean, nboot = 5, boot.seed = 12345)
model.gabow = annotate.description(model.gabow, 'MST Gabow - aCML')
```


## Undirected Minimum Spanning Tree with Likelihood-Fit 

This algorithm is meant   to extract a progression  from data of an individual patient, but it is not constrained to retrieve a tree/forest -- i.e., it could retrieve a direct acyclic graph -- according to the level of noise and heterogeneity of the input data. This algorithm is based on a 
formulation of the  problem in terms of minimum spamming trees and exploits results from Chow Liu and other variants for likelihood-fit. Thus, this algorithm is executed with potentially multiple regularizator as CAPRI -- here we use BIC/AIC.

 We test it to aCML data as all the other algorithms.


```{r}
model.chowliu = tronco.chowliu(aCML.clean, nboot = 5, boot.seed = 12345)
model.chowliu = annotate.description(model.chowliu, 'MST Chow Liu - aCML')
```



## Undirected Minimum Spanning Tree with Mutual Information

This algorithm is meant   to extract a progression  from data of an individual patient. As the Chow Liu algorithm, this  could retrieve a direct acyclic graph  according to the level of noise and heterogeneity of the input data. This algorithm formulatesf the  problem in terms of undirected minimum spamming trees and exploits results from Prim, which are a generalization of Edomonds' ones.  We test it to aCML data as all the other algorithms.



```{r}
model.prim = tronco.prim(aCML.clean, nboot = 5, boot.seed = 12345)
model.prim = annotate.description(model.prim, 'MST Prim - aCML data')
```

---
title: "Post reconstruction"
author: 
  - Luca De Sano
  - Daniele Ramazzotti
  - Giulio Caravagna
  - Alex Graudenxi
  - Marco Antoniotti
date: "`r format(Sys.time(), '%B %d, %Y')`"
graphics: yes
package: TRONCO
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Post reconstruction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{TRONCO,BiocStyle}
---


```{r, include=FALSE}
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)
gene.hypotheses = c('KRAS', 'NRAS', 'IDH1', 'IDH2', 'TET2', 'SF3B1', 'ASXL1')
alterations = events.selection(as.alterations(aCML), filter.freq = .05)
aCML.clean = events.selection(aCML,
    filter.in.names=c(as.genes(alterations), gene.hypotheses))
aCML.clean = annotate.description(aCML.clean, 
    'CAPRI - Bionformatics aCML data (selected events)')
aCML.hypo = hypothesis.add(aCML.clean, 'NRAS xor KRAS', XOR('NRAS', 'KRAS'))
aCML.hypo = hypothesis.add(aCML.hypo, 'SF3B1 xor ASXL1', XOR('SF3B1', XOR('ASXL1')),
    '*')
as.events(aCML.hypo, genes = 'TET2') 
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 xor IDH2',
    XOR('TET2', 'IDH2'),
    '*')
aCML.hypo = hypothesis.add(aCML.hypo,
    'TET2 or IDH2',
    OR('TET2', 'IDH2'),
    '*')
aCML.hypo = hypothesis.add.homologous(aCML.hypo)
aCML.hypo = annotate.description(aCML.hypo, '')
aCML.clean = annotate.description(aCML.clean, '')
model.capri = tronco.capri(aCML.hypo, boot.seed = 12345, nboot = 5)
model.capri = annotate.description(model.capri, 'CAPRI - aCML')
model.caprese = tronco.caprese(aCML.clean)
model.caprese = annotate.description(model.caprese, 'CAPRESE - aCML')
model.edmonds = tronco.edmonds(aCML.clean, nboot = 5, boot.seed = 12345)
model.edmonds = annotate.description(model.edmonds, 'MST Edmonds - aCML')
model.gabow = tronco.gabow(aCML.clean, nboot = 5, boot.seed = 12345)
model.gabow = annotate.description(model.gabow, 'MST Gabow - aCML')
model.chowliu = tronco.chowliu(aCML.clean, nboot = 5, boot.seed = 12345)
model.chowliu = annotate.description(model.chowliu, 'MST Chow Liu - aCML')
model.prim = tronco.prim(aCML.clean, nboot = 5, boot.seed = 12345)
model.prim = annotate.description(model.prim, 'MST Prim - aCML data')
```


TRONCO provides functions to plot a model, access information about the probabilities used to extract it from data, and two types of confidence measures: those used to infer the model, and those computed a posteriori from it.

Function `view` provides updated information about a model if this is available. 

```{r }
view(model.capri)
```


## Visualizing a reconstructed model

We can plot a model by using function \Rfunction{tronco.plot}. Here, we plot the aCML model inferred by CAPRI with BIC and AIC as a regolarizator. We set some parameters to get a nice plot (scaling etc.), and distinguish the edges detected by the two regularization techniques. The confidence of each edge is shown  in terms of temporal priority and probability raising (selective advantage scores) and hypergeometric testing (statistical relevance of the dataset of input). Events are annotated as in the oncoprint, edge p-values above a minium threshold (default 0.05) are red.


```{r,fig.width=4,fig.height=4,warning=FALSE}
tronco.plot(model.capri, 
    fontsize = 12, 
    scale.nodes = 0.6, 
    confidence = c('tp', 'pr', 'hg'), 
    height.logic = 0.25, 
    legend.cex = 0.35, 
    pathways = list(priors = gene.hypotheses), 
    label.edge.size = 10)
```


We can also make a multiplot with this function,  which in this case we do by showing the models inferred by the other algorithms based on Minimum Spanning Trees.


```{r fig.width=7,fig.height=7,warning=FALSE, fig.cap="aCML data processed model by algorithms to extract models from individual patients, we show the otput of  CAPRESE,  and all algorithms based on Minimum Spanning Trees (Edmonds, Chow Liu and Prim). Only the  model retrieved by Chow Liu has two different edge colors as it was regularized with two different strategies: AIC and BIC."}
par(mfrow = c(2,2))
tronco.plot(model.caprese, fontsize = 22, scale.nodes = 0.6, legend = FALSE)
tronco.plot(model.edmonds, fontsize = 22, scale.nodes = 0.6, legend = FALSE)
tronco.plot(model.chowliu, fontsize = 22, scale.nodes = 0.6, legend.cex = .7)
tronco.plot(model.prim, fontsize = 22, scale.nodes = 0.6, legend = FALSE)
```


## Accessing information within a model (e.g., confidence)

We can visualize a summary of the parameters   used for the reconstruction, test if an object has a model or delete it (which shall be done to retrieve the original dataset).

```{r }
as.data.frame(as.parameters(model.capri))
has.model(model.capri)
dataset = delete.model(model.capri)
```


### Model structure
A set of functions can be used to visualize the content of object which contains the reconstructed model. For instance, we can access the adjacency matrix of a model by using `as.adj.matrix` which will return a matrix for each one of the regularizators used -- in this case because CAPRI was run with both BIC/AIC.


```{r}
str(as.adj.matrix(model.capri))
```

### Empirical probabilities

Every model is inferred by estimating the empirical marginal, joint and conditional probabilities for all the events, from input data. These in some cases are estimated by a bootstrap procedure (see the algorithms implemented). TRONCO has functions to extract such table, that could be in turn printed by using external  functions for, e.g., heatmap visualization (see below for an example via the `pheatmap` package). We show these functions working with the CAPRI model; in this case the tables are the same for both BIC/AIC structures as they are computed before performing penalized likelihood-fit. The marginal *P(x)* for *x* an event in the dataset are obtained by `as.marginal.probs`. 

```{r}
marginal.prob = as.marginal.probs(model.capri)
head(marginal.prob$capri_bic)
```

Similarly, the joint *P(x,y)* for every pair of events in the dataset is given by `as.joint.probs`.

```{r}
joint.prob = as.joint.probs(model.capri, models='capri_bic')
joint.prob$capri_bic[1:3, 1:3]
```

And `as.conditional.probs` finally gives the conditional *P(x|y)* for every edge in the dataset.

```{r}
conditional.prob = as.conditional.probs(model.capri, models='capri_bic')
head(conditional.prob$capri_bic)
```


### Confidence measures

Confidence scores  can be accessed by function \Rfunction{as.confidence}, which takes as parameter the type of confidence measure that one wants to access to. This will work for either confidence measures assessed before reconstructing the model -- if available --, or afterwards.

```{r }
str(as.confidence(model.capri, conf = c('tp', 'pr', 'hg')))
```

Other functions  visualize  tables summarizing the  statistics for each edge in the model, For instance, if one uses function `as.selective.advantage.relations`  the  p-values for temporal priority, probability raising and hypergeometric testing, as well as other information about each edge  can be accessed, e.g., the number of observations for the upstream and the downstream events.


```{r selective-advantage}
as.selective.advantage.relations(model.capri)
```


## Confidence via non-parametric and statistical bootstrap

TRONCO provides three different strategies to perform  bootstrap and assess confidence of each edge in terms of a score in the range [0, 100], where 100 is the highest confidence). Non-parametric (default) and statistical bootstrap strategies are available, and can be executed by calling function `tronco.bootstrap` with `type` parameter set appropriately. This function is parallel, and parameter `cores.ratio` (default 1) can be used to percentage of available cores that shall be used to compute the scores.
Parameter `nboot` controls the number of bootstrap iterations.


```{r }
model.boot = tronco.bootstrap(model.capri, nboot = 3, cores.ratio = 0)
model.boot = tronco.bootstrap(model.boot, nboot = 3, cores.ratio = 0, type = 'statistical')
```



Bootstrap scores can be annotated to the `tronco.plot` output by setting them via the confidence parameter `confidence=c('npb', 'sb')`. In this case edge thickness will be proportional to the non-parametric `npb`) scores -- the last to appear in the `confidence` parameter.

```{r fig.width=4, fig.height=4, warning=FALSE, fig.cap="aCML model reconstructed by CAPRI with AIC / BIC as regolarizators and annotated with both non-parametric and statistical bootstrap scores. Edge thickness is proportional to the non-parametric scores."}

tronco.plot(model.boot, 
    fontsize = 12, 
    scale.nodes = .6,   
    confidence=c('sb', 'npb'), 
    height.logic = 0.25, 
    legend.cex = .35, 
    pathways = list(priors= gene.hypotheses), 
    label.edge.size=10)
```


Bootstrap scores can extracted or visualized even with other TRONCO functions. For instance, we can accessall scores via `as.bootstrap.scores`, which resembles  function `as.selective.advantage.relations` and will display the scores per edge. Notice that even function `view` gives an update output by mentioning the available bootstrap scores.


```{r}
as.bootstrap.scores(model.boot)
view(model.boot)
```


If we want to access a matrix with the scores and visualize that in a heatmap we can use for instance the `pheatmap` function of TRONCO. In this case we need to use also function `keysToNames` to translate internal TRONCO keys to mnemonic names in the plot 


```{r fig.width=7, fig.height=7, fig.cap="Heatmap of the bootstrap scores for the CAPRI aCML model (via AIC regularization)."}

pheatmap(keysToNames(model.boot, as.confidence(model.boot, conf = 'sb')$sb$capri_aic) * 100, 
           main =  'Statistical bootstrap scores for AIC model',
           fontsize_row = 6,
           fontsize_col = 6,
           display_numbers = TRUE,
           number_format = "%d"
           )
```



## Confidence via cross-validation (entropy loss, prediction and posterior classification errors)

TRONCO implements *k*-fold cross-validation routines (from the *bnlearn* package) to provide estimates of the following statistics:

- the *negative entropy* (via `tronco.kfold.eloss`) of a whole model ? i.e., the negated expected log-likelihood of the test set for the Bayesian network fitted from the training set.
- the *prediction error* (via `tronco.kfold.prederr`) for a single node *x* and its parents set *X* -- i.e.,  how precisely we can predict the values of *x* by using only the information present in its local distribution, via *X*.
- the *posterior classification error* (via `tronco.kfold.posterr`)  for a single node *x* and one of its parent node *y in X* -- i.e., the values of *x* are predicted using only the information present in *y* by likelihood weighting and Bayesian posterior estimates.


By default, a 10 repetitions from  10-fold cross-validation experiment are perfomed, for all the models which are found inside a TRONCO object -- in this case 2, one for CAPRI with BIC and one for CAPRI with AIC.
 

```{r}
model.boot = tronco.kfold.eloss(model.boot)
model.boot = tronco.kfold.prederr(model.boot, runs = 2, cores.ratio = 0)
model.boot = tronco.kfold.posterr(model.boot, runs = 2, cores.ratio = 0)
```


These results can be visualized in terms of summary tables, as for the other confidence scores.


```{r}
as.kfold.eloss(model.boot)
as.kfold.prederr(model.boot)
as.kfold.posterr(model.boot)
```


Notice that these can be combined to create a nice table with all these statistics  -- we make here the example of a table with all the BIC statistics. This format can be readily exported to external spreadsheets for further visualization.

```{r}
tabular = function(obj, M){
    tab = Reduce(
        function(...) merge(..., all = TRUE), 
            list(as.selective.advantage.relations(obj, models = M),
                as.bootstrap.scores(obj, models = M),
                as.kfold.prederr(obj, models = M),
                as.kfold.posterr(obj,models = M)))
  
    # merge reverses first with second column
    tab = tab[, c(2,1,3:ncol(tab))]
    tab = tab[order(tab[, paste(M, '.NONPAR.BOOT', sep='')], na.last = TRUE, decreasing = TRUE), ]
    return(tab)
}

head(tabular(model.boot, 'capri_bic'))
```



We finally show the plot of the model with the confidences by cross-validation.


```{r fig.width=4,fig.height=4, warning=FALSE, fig.cap="aCML model reconstructed by CAPRI with  AIC/BIC as regolarizators and annotated with  non-parametric, as well as with entropy loss, prediction and posterior classification errors computed via cross-validation. Edge thickness is proportional to the non-parametric  scores."}
tronco.plot(model.boot, 
    fontsize = 12, 
    scale.nodes = .6, 
    confidence=c('npb', 'eloss', 'prederr', 'posterr'), 
    height.logic = 0.25, 
    legend.cex = .35, 
    pathways = list(priors= gene.hypotheses), 
    label.edge.size=10)
```

---
title: "Data visualization"
author: 
  - Luca De Sano
  - Daniele Ramazzotti
  - Giulio Caravagna
  - Alex Graudenxi
  - Marco Antoniotti
date: "`r format(Sys.time(), '%B %d, %Y')`"
graphics: yes
package: TRONCO
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Data visualization}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{TRONCO,BiocStyle}
---


```{r, include=FALSE}
library(TRONCO)
data(aCML)
data(crc_maf)
data(crc_gistic)
data(crc_plain)
```

All examples in this section will be done with the the aCML dataset as reference.

## Summary report for a dataset and boolean queries{#sec:view}

We use the function `view` to get a short summary of a dataset that we loaded in TRONCO; this function reports on the number of samples and  events, plus some meta information that could be displayed graphically.  


```{r}
view(aCML)
```

## Creating views with the `as` functions

Several functions are available to create views over a dataset, with a set of parameter which can constraint the view -- as in the SELECT/JOIN approaches in databases. In the following examples we  show their execution with the default parameters, but  shorten their output to make this document readable.

The main `as` functions are here documented.  `as.genotypes`, that we can use to get the matrix of *genotypes* that we imported.


```{r}
as.genotypes(aCML)[1:10,5:10]
```


Differently, `as.events` and `as.events.in.samples`, that show tables with the events that we are processing in  all dataset or in a specific sample that we want to examine.


```{r}
as.events(aCML)[1:5, ]
as.events.in.sample(aCML, sample = 'patient 2')
```


Concerning genes, `as.genes` shows the mnemonic names of the genes (or chromosomes, cytobands, etc.) that we included in our dataset.


```{r}
as.genes(aCML)[1:8]
```


And `as.types` shows the types of alterations (e.g., mutations, amplifications, etc.) that we have  find in our dataset, and  function `as.colors` shows the list of the  colors which are associated to each type.


```{r}
as.types(aCML)
as.colors(aCML)
```



A function `as.gene` can be used to display the alterations of a specific gene  across the samples


```{r}
head(as.gene(aCML, genes='SETBP1'))
```


Views over samples can be created as well. `as.samples` and `which.samples` list all the samples in the data, or return a list of samples that harbour a certain alteration. The former is 


```{r}
as.samples(aCML)[1:10]
```


and the latter is
 

```{r}
which.samples(aCML, gene='TET2', type='Nonsense point')
```


A slightly different function, which manipulates the data,  is `as.alterations`, which transforms a dataset with events of different type  to events of a unique type, labeled *Alteration*.


```{r}
dataset = as.alterations(aCML)
```

```{r}
view(dataset)
```

 
When samples are enriched with stage information function `as.stages` can be used to create a view over such table. Views over patterns can be created as well -- see Model Inference with CAPRI.

## Dataset size

A set of functions allow to get the number of genes, events, samples, types and patterns in a dataset.


```{r}
ngenes(aCML)
nevents(aCML)
nsamples(aCML)
ntypes(aCML)
npatterns(aCML)
```



## Oncoprints

Oncoprints are the most effective  data-visualization functions in TRONCO. These are heatmaps where rows represent variants, and columns samples (*the reverse* of the input format required by TRONCO), and are annotated and displayed/sorted to enhance which samples have which mutations etc. 

By default `oncoprint` will try to  sort samples and events to enhance exclusivity patterns among the events.


```{r, fig.width=6, fig.height=5, fig.cap="This plot gives a graphical visualization of the events that are in the dataset -- with a color per event type. It sorts samples to enhance exclusivity patterns among the events"}
oncoprint(aCML)
```


But the sorting mechanism is bypassed if one wants to cluster samples or events, or if one wants to split samples by cluster (not shown). In the clustering case, the ordering is given by the dendrograms. In this case we also show the annotation of some groups of events via parameter `gene.annot`.


```{r fig.width=5, fig.height=5, fig.cap="This plot gives a graphical visualization of the events that are in the dataset -- with a color per event type. It it clusters samples/events"}
oncoprint(aCML, 
    legend = FALSE, 
    samples.cluster = TRUE, 
    gene.annot = list(one = list('NRAS', 'SETBP1'), two = list('EZH2', 'TET2')),
    gene.annot.color = 'Set2',
    genes.cluster = TRUE)
```

Oncoprints can be annotated; a special type of annotation is given by stage data. As this is not available for the aCML dataset, we create it randomly, just for the sake of showing how the oncoprint is enriched with this information. This is the random stage map that we create -- if some samples had no stage a NA would be added automatically. 


```{r}
stages = c(rep('stage 1', 32), rep('stage 2', 32))
stages = as.matrix(stages)
rownames(stages) = as.samples(aCML)
dataset = annotate.stages(aCML, stages = stages)
has.stages(aCML)
head(as.stages(dataset))
```


The `as.stages` function can now be used to create a view over stages.

```{r}
head(as.stages(dataset))
```
After that the data is annotated via `annotate.stages` function, we can again plot an oncoprint -- which this time will detect that the dataset has also stages associated, and will diplay those 


```{r fig.width=6, fig.height=5}
oncoprint(dataset, legend = FALSE)
```


If one is willing to display samples grouped according to some variable, for instance after a sample clustering task, he can use `group.samples` parameter of `oncoprint` and that will override the mutual exclusivity ordering. Here, we make the trick of using the stages as if they were such clustering result.


```{r fig.width=6, fig.height=5, fig.cap="Example \texttt{oncoprint} output for aCML data with randomly annotated stages, in left, and samples clustered by group assignment in right -- for simplicity the group variable is again the stage annotation."}
oncoprint(dataset, group.samples = as.stages(dataset))
```
    
## Groups visualization (e.g., pathways)

TRONCO provides functions to visualize groups of events, which in this case are called pathways -- though this could be any group that one would like to define. Aggregation happens with the same rational as the `as.alterations` function, namely by merging the events in the group.

We make an example of a pathway called *MyPATHWAY* involving genes SETBP1, EZH2 and WT1; we want it to be colored in red, and we want to have the genotype of each event to be maintened in the dataset. We proceed as follows  (R's output is omitted).

```{r}
pathway = as.pathway(aCML,
    pathway.genes = c('SETBP1', 'EZH2', 'WT1'),
    pathway.name = 'MyPATHWAY',
    pathway.color = 'red',
    aggregate.pathway = FALSE)
```

Which we then visualize with an `oncoprint`

```{r onco-pathway, fig.width=6.5, fig.height=2, fig.cap="Oncoprint output of a custom pathway called MyPATHWAY involving genes SETBP1, EZH2 and WT1; the genotype of each event is shown."}
oncoprint(pathway, title = 'Custom pathway',  font.row = 8, cellheight = 15, cellwidth = 4)
```

In TRONCO there is also a function which creates the pathway view and the corresponding oncoprint to  multiple pathways, when these are given as a list. We make here a simple example of two custom pathways.


```{r fig.width=6.5, fig.height=1.8, fig.cap="Oncoprint output of a custom pair of pathways, with events shown"}
pathway.visualization(aCML, 
    pathways=list(P1 = c('TET2', 'IRAK4'),  P2=c('SETBP1', 'KIT')),        
    aggregate.pathways=FALSE,
    font.row = 8)
```

If we had to visualize just the signature of the pathway, we could set  `aggregate.pathways=T`.

```{r fig.width=6.5, fig.height=1, fig.cap="Oncoprint output of a custom pair of pathways, with events hidden"}
pathway.visualization(aCML, 
    pathways=list(P1 = c('TET2', 'IRAK4'),  P2=c('SETBP1', 'KIT')),
    aggregate.pathways = TRUE,
    font.row = 8)
```

The same operation could have been done using [WikiPathways](https://wikipathways.org). We can query WikiPathways and collect HGNC gene symbols and titles for pathways of interest as follows. (R's output is omitted).

```{r eval=FALSE}
library(rWikiPathways)
# quotes inside query to require both terms
my.pathways <- findPathwaysByText('SETBP1 EZH2 TET2 IRAK4 SETBP1 KIT')
human.filter <- lapply(my.pathways, function(x) x$species == "Homo sapiens")
my.hs.pathways <- my.pathways[unlist(human.filter)] 
# collect pathways idenifiers
my.wpids <- sapply(my.hs.pathways, function(x) x$id)

pw.title<-my.hs.pathways[[1]]$name
pw.genes<-getXrefList(my.wpids[1],"H") 
```

Now `pw.genes` and `pw.title` can be used as input for the function `as.pathway`.
It is also possible to view and edit these pathways at WikiPathways using the following commands to open tabs in your default browser.

```{r wikipathways, eval=FALSE}
browseURL(getPathwayInfo(my.wpids[1])[2])
browseURL(getPathwayInfo(my.wpids[2])[2])
browseURL(getPathwayInfo(my.wpids[3])[2])
```
 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{has.duplicates}
\alias{has.duplicates}
\title{has.duplicates}
\usage{
has.duplicates(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
TRUE if there are duplicated events in \code{x}.
}
\description{
Return true if there are duplicated events in the TRONCO dataset 'x', which should be
a TRONCO compliant dataset - see \code{is.compliant}. Events are identified by a gene 
name, e.g., a HuGO_Symbol, and a type label, e.g., c('SNP', 'KRAS')
}
\examples{
data(test_dataset)
has.duplicates(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.patterns}
\alias{as.patterns}
\title{as.patterns}
\usage{
as.patterns(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
The patterns in the dataset which constitute CAPRI's hypotheses.
}
\description{
Return the patterns in the dataset which constitute CAPRI's hypotheses.
}
\examples{
data(test_dataset)
as.patterns(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external.R
\name{import.mutex.groups}
\alias{import.mutex.groups}
\title{import.mutex.groups}
\usage{
import.mutex.groups(file, fdr = 0.2, display = TRUE)
}
\arguments{
\item{file}{Mutex results ("ranked-groups.txt" file)}

\item{fdr}{cutoff for fdr}

\item{display}{print summary table of extracted groups}
}
\description{
Create a list of unique Mutex groups for a given fdr cutoff
current Mutex version is Jan 8, 2015
(ref: https://code.google.com/p/mutex/ )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{crc_plain}
\alias{crc_plain}
\title{Plain mutation dataset}
\format{
plain data
}
\source{
data from http://www.nature.com/nature/journal/v487/n7407/full/nature11252.html
}
\usage{
data(crc_plain)
}
\value{
A plain input
}
\description{
This dataset contains an example of plain input of a crc cohort of patients
}
\author{
Daniele Ramazzotti
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.alterations}
\alias{as.alterations}
\title{as.alterations}
\usage{
as.alterations(x, new.type = "Alteration", new.color = "khaki", silent = FALSE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{new.type}{The types label of the new event type, 'Alteration' by default.}

\item{new.color}{The color of the event \code{new.type}, default 'khaki'.}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO compliant dataset with alteration profiles.
}
\description{
Return a dataset where all events for a gene are merged in a unique event, i.e., 
a total of gene-level alterations diregarding the event type. Input 'x' is checked
to be a TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(muts)
as.alterations(muts)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{has.model}
\alias{has.model}
\title{has.model}
\usage{
has.model(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
TRUE if there is a reconstructed model in \code{x}.
}
\description{
Return true if there is a reconstructed model in the TRONCO dataset 'x', which should be
a TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
has.model(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{change.color}
\alias{change.color}
\title{change.color}
\usage{
change.color(x, type, new.color)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{type}{An event type}

\item{new.color}{The new color (either HEX or R Color)}
}
\value{
A TRONCO complian dataset.
}
\description{
Change the color of an event type
}
\examples{
data(test_dataset)
dataset = change.color(test_dataset, 'ins_del', 'red')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{annotate.stages}
\alias{annotate.stages}
\title{annotate.stages}
\usage{
annotate.stages(x, stages, match.TCGA.patients = FALSE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{stages}{A list of stages. Rownames must match samples list of x}

\item{match.TCGA.patients}{Match using TCGA notations (only first 12 characters)}
}
\value{
A TRONCO compliant dataset.
}
\description{
Annotate stage information on the selected dataset
}
\examples{
data(test_dataset)
data(stage)
test_dataset = annotate.stages(test_dataset, stage)
as.stages(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{rename.type}
\alias{rename.type}
\title{rename.type}
\usage{
rename.type(x, old.name, new.name)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{old.name}{The type of event to rename.}

\item{new.name}{The new name}
}
\value{
A TRONCO complian dataset.
}
\description{
Rename an event type
}
\examples{
data(test_dataset)
test_dataset = rename.type(test_dataset, 'ins_del', 'deletion')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{consolidate.data}
\alias{consolidate.data}
\title{consolidate.data}
\usage{
consolidate.data(x, print = FALSE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{print}{A boolean value stating whether to print of not the summary}
}
\value{
The list of any 0 probability, 1 probability and indistinguishable.
}
\description{
Verify if the input data are consolidate, i.e., if there are events
with 0 or 1 probability or indistinguishable in terms of
observations
}
\examples{
data(test_dataset)
consolidate.data(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{tronco.kfold.prederr}
\alias{tronco.kfold.prederr}
\title{tronco.kfold.prederr}
\usage{
tronco.kfold.prederr(
  x,
  models = names(as.models(x)),
  events = as.events(x),
  runs = 10,
  k = 10,
  cores.ratio = 1,
  silent = FALSE
)
}
\arguments{
\item{x}{A reconstructed model (the output of tronco.capri)}

\item{models}{The names of the selected regularizers (bic, aic or caprese)}

\item{events}{a list of event}

\item{runs}{a positive integer number, the number of times cross-validation will be run}

\item{k}{a positive integer number, the number of groups into which the data will be split}

\item{cores.ratio}{Percentage of cores to use. coresRate * (numCores - 1)}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\description{
Perform a k-fold cross-validation using the function bn.cv
and scan every node to estimate its prediction error. For details and examples 
regarding the statistical assesment of an inferred model, 
we refer to the Vignette Section 7.
}
\examples{
data(test_model)
tronco.kfold.prederr(test_model, k = 2, runs = 2, cores.ratio = 0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{delete.gene}
\alias{delete.gene}
\title{delete.gene}
\usage{
delete.gene(x, gene)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{gene}{The name of the gene to delete.}
}
\value{
A TRONCO complian dataset.
}
\description{
Delete a gene
}
\examples{
data(test_dataset)
test_dataset = delete.gene(test_dataset, 'TET2')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{duplicates}
\alias{duplicates}
\title{duplicates}
\usage{
duplicates(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A subset of \code{as.events(x)} with duplicated events.
}
\description{
Return the events duplicated in \code{x}, if any. Input 'x' should be
a TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
duplicates(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.bootstrap.scores}
\alias{as.bootstrap.scores}
\title{as.bootstrap.scores}
\usage{
as.bootstrap.scores(x, events = as.events(x), models = names(x$model))
}
\arguments{
\item{x}{A TRONCO model.}

\item{events}{A subset of events as of \code{as.events(x)}, all by default.}

\item{models}{A subset of reconstructed models, all by default.}
}
\value{
All the bootstrap scores in a TRONCO model
}
\description{
Returns a dataframe with all the bootstrap score in a 
TRONCO model. It is possible to specify a subset of events
or models if multiple reconstruction have been performed.
}
\examples{
data(test_model)
as.bootstrap.scores(test_model)
as.bootstrap.scores(test_model, events=as.events(test_model)[5:15,])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.gene}
\alias{as.gene}
\title{as.gene}
\usage{
as.gene(x, genes, types = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{genes}{The genes to consider, if NA all available genes are used.}

\item{types}{The types of events to consider, if NA all available types are used.}
}
\value{
A matrix, subset of \code{as.genotypes(x)} with colnames substituted  with events' types.
}
\description{
Return the genotypes for a certain set of genes and type of events. Input 'x' should be a
TRONCO compliant dataset - see \code{is.compliant}. In this case column names are substituted
with events' types.
}
\examples{
data(test_dataset)
as.gene(test_dataset, genes = c('EZH2', 'ASXL1'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external.R
\name{export.graphml}
\alias{export.graphml}
\title{export.graphml}
\usage{
export.graphml(x, file, ...)
}
\arguments{
\item{x}{A TRONCO compliant dataset}

\item{file}{Where to save the output}

\item{...}{parameters for tronco.plot}
}
\description{
Create a graphML object which can be imported in cytoscape
This function is based on the tronco.plot fuction
}
\examples{
data(test_model)
export.graphml(test_model, file='text.xml', scale.nodes=0.3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{intersect.datasets}
\alias{intersect.datasets}
\title{intersect.datasets}
\usage{
intersect.datasets(x, y, intersect.genomes = TRUE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{y}{A TRONCO compliant dataset.}

\item{intersect.genomes}{If False -> just samples}
}
\value{
A TRONCO complian dataset.
}
\description{
Intersect samples and events of two dataset
}
\examples{
data(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.marginal.probs}
\alias{as.marginal.probs}
\title{as.marginal.probs}
\usage{
as.marginal.probs(
  x,
  events = as.events(x),
  models = names(x$model),
  type = "observed"
)
}
\arguments{
\item{x}{A TRONCO model.}

\item{events}{A subset of events as of \code{as.events(x)}, all by default.}

\item{models}{A subset of reconstructed models, all by default.}

\item{type}{observed.}
}
\value{
The marginal probabilities in a TRONCO model.
}
\description{
Extract the marginal probabilities from a TRONCO model. The return matrix is indexed with rownames which 
represent genotype keys - these can be resolved with function \code{keysToNames}. It is possible to
specify a subset of events to build the matrix, a subset of models if multiple reconstruction have
been performed. Also, either the observed or fit probabilities can be extracted.
}
\examples{
data(test_model)
as.marginal.probs(test_model)
as.marginal.probs(test_model, events=as.events(test_model)[5:15,])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external.R
\name{TCGA.map.clinical.data}
\alias{TCGA.map.clinical.data}
\title{TCGA.map.clinical.data}
\usage{
TCGA.map.clinical.data(file, sep = "\\t", column.samples, column.map)
}
\arguments{
\item{file}{A file with the clinical data}

\item{sep}{file delimiter}

\item{column.samples}{Required columns}

\item{column.map}{Map to the required columns}
}
\value{
a map
}
\description{
Map clinical data from the TCGA format
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{maf}
\alias{maf}
\title{MAF example data}
\format{
Manual Annotated Format
}
\source{
fake data
}
\usage{
data(maf)
}
\value{
A standard TRONCO object
}
\description{
This dataset contains a standard MAF input for TRONCO
}
\author{
Luca De Sano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tronco.R
\name{tronco.bootstrap}
\alias{tronco.bootstrap}
\title{tronco bootstrap}
\usage{
tronco.bootstrap(
  reconstruction,
  type = "non-parametric",
  nboot = 100,
  cores.ratio = 1,
  silent = FALSE
)
}
\arguments{
\item{reconstruction}{The output of tronco.capri or
tronco.caprese}

\item{type}{Parameter to define the type of sampling
to be performed, e.g., non-parametric for uniform sampling.}

\item{nboot}{Number of bootstrap sampling to be performed
when estimating the model confidence.}

\item{cores.ratio}{Percentage of cores to use
coresRate * (numCores - 1)}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO compliant object with reconstructed model
}
\description{
Bootstrap a reconstructed progression model. For details and examples
regarding the statistical assesment of an inferred model,
we refer to the Vignette Section 7.
}
\examples{
data(test_model)
boot = tronco.bootstrap(test_model, nboot = 1, cores.ratio = 0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.kfold.posterr}
\alias{as.kfold.posterr}
\title{as.kfold.posterr}
\usage{
as.kfold.posterr(
  x,
  events = as.events(x),
  models = names(x$model),
  values = FALSE,
  table = FALSE
)
}
\arguments{
\item{x}{A TRONCO model.}

\item{events}{A subset of events as of \code{as.events(x)}, all by default.}

\item{models}{A subset of reconstructed models, all by default.}

\item{values}{If you want to see also the values}

\item{table}{Keep the original table (defaul false)}
}
\value{
All the posterior classification error scores in a TRONCO model
}
\description{
Returns a dataframe with all the posterior classification error
score in a TRONCO model. It is possible to specify a subset of events
or models if multiple reconstruction have been performed.
}
\examples{
data(test_model_kfold)
data(test_model)
as.kfold.posterr(test_model_kfold)
as.kfold.posterr(test_model_kfold, events=as.events(test_model)[5:15,])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{has.stages}
\alias{has.stages}
\title{has stages}
\usage{
has.stages(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
TRUE if the TRONCO dataset has stage annotations for samples.
}
\description{
Return true if the TRONCO dataset 'x', which should be a TRONCO compliant dataset 
- see \code{is.compliant} - has stage annotations for samples. Some sample stages 
might be annotated as NA, but not all.
}
\examples{
data(test_dataset)
has.stages(test_dataset)
data(stage)
test_dataset = annotate.stages(test_dataset, stage)
has.stages(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{nameToKey}
\alias{nameToKey}
\title{nameToKey}
\usage{
nameToKey(x, name)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{name}{A intelligible event name}
}
\value{
A TRONCO dataset key name
}
\description{
Convert to key an intelligible event names, e.g., change 'Mutation KRAS' in G23.
If a name is not found, an error is raised!
}
\examples{
data(test_model)
adj_matrix = as.adj.matrix(test_model, events=as.events(test_model)[5:15,])$bic

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{nsamples}
\alias{nsamples}
\title{nsamples}
\usage{
nsamples(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
The number of samples in the dataset.
}
\description{
Return the number of samples in the dataset.
}
\examples{
data(test_dataset)
nsamples(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loading.R
\name{import.GISTIC}
\alias{import.GISTIC}
\title{import.GISTIC}
\usage{
import.GISTIC(
  x,
  filter.genes = NULL,
  filter.samples = NULL,
  silent = FALSE,
  trim = TRUE,
  rna.seq.data = NULL,
  rna.seq.up = NULL,
  rna.seq.down = NULL
)
}
\arguments{
\item{x}{Either a dataframe or a filename}

\item{filter.genes}{A list of genes}

\item{filter.samples}{A list of samples}

\item{silent}{A parameter to disable/enable verbose messages.}

\item{trim}{Remove the events without occurrence}

\item{rna.seq.data}{Either a dataframe or a filename}

\item{rna.seq.up}{TODO}

\item{rna.seq.down}{TODO}
}
\value{
A TRONCO compliant representation of the input CNAs.
}
\description{
Transform GISTIC scores for CNAs in a TRONCO compliant object. Input can be either a matrix, with columns
for each altered gene and rows for each sample; in this case colnames/rownames mut be provided. If input
is a character an attempt to load a table from file is performed. In this case the input table format
should be constitent with TCGA data for focal CNA; there should hence be: one column for each sample,
one row for each gene, a column Hugo_Symbol with every gene name and a column Entrez_Gene_Id with every
 gene\'s Entrez ID. A valid GISTIC score should be any value of: "Homozygous Loss" (-2), "Heterozygous
 Loss" (-1), "Low-level Gain" (+1), "High-level Gain" (+2). For details and examples 
regarding the loading functions provided by the package we refer to the Vignette Section 3.
}
\examples{
data(crc_gistic)
gistic = import.GISTIC(crc_gistic)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tronco.R
\name{tronco.chowliu}
\alias{tronco.chowliu}
\title{Tronco Chow Liu}
\usage{
tronco.chowliu(
  data,
  regularization = c("bic", "aic"),
  do.boot = TRUE,
  nboot = 100,
  pvalue = 0.05,
  min.boot = 3,
  min.stat = TRUE,
  boot.seed = NULL,
  silent = FALSE,
  epos = 0,
  eneg = 0
)
}
\arguments{
\item{data}{A TRONCO compliant dataset.}

\item{regularization}{Select the regularization for the
likelihood estimation, e.g., BIC, AIC.}

\item{do.boot}{A parameter to disable/enable the estimation
of the error rates give the reconstructed model.}

\item{nboot}{Number of bootstrap sampling (with rejection)
to be performed when estimating the selective advantage scores.}

\item{pvalue}{Pvalue to accept/reject the valid selective
advantage relations.}

\item{min.boot}{Minimum number of bootstrap sampling
to be performed.}

\item{min.stat}{A parameter to disable/enable the minimum
number of bootstrap sampling required besides nboot if
any sampling is rejected.}

\item{boot.seed}{Initial seed for the bootstrap random
sampling.}

\item{silent}{A parameter to disable/enable verbose
messages.}

\item{epos}{Error rate of false positive errors.}

\item{eneg}{Error rate of false negative errors.}
}
\value{
A TRONCO compliant object with reconstructed
model
}
\description{
Reconstruct a progression model using Chow Liu
algorithm combined with probabilistic causation. For details and examples
regarding the inference process and on the algorithm implemented in the package,
we refer to the Vignette Section 6.
}
\examples{
data(test_dataset_no_hypos)
recon = tronco.chowliu(test_dataset_no_hypos, nboot = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.samples}
\alias{as.samples}
\title{as.samples}
\usage{
as.samples(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A vector of sample IDs
}
\description{
Return all sample IDs for input 'x', which should be a TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
as.samples(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.pathway}
\alias{as.pathway}
\title{as.pathway}
\usage{
as.pathway(
  x,
  pathway.genes,
  pathway.name,
  pathway.color = "yellow",
  aggregate.pathway = TRUE,
  silent = FALSE
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{pathway.genes}{Gene (symbols) involved in the pathway.}

\item{pathway.name}{Pathway name for visualization.}

\item{pathway.color}{Pathway color for visualization.}

\item{aggregate.pathway}{If TRUE drop the events for the genes in the pathway.}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
Extract the subset of events for genes which are part of a pathway.
}
\description{
Given a cohort and a pathway, return the cohort with events restricted to genes 
involved in the pathway. This might contain a new 'pathway' genotype with an alteration mark if
any of the involved genes are altered.
}
\examples{
data(test_dataset)
p = as.pathway(test_dataset, c('ASXL1', 'TET2'), 'test_pathway')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{ngenes}
\alias{ngenes}
\title{ngenes}
\usage{
ngenes(x, types = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{types}{The types of events to consider, if NA all available types are used.}
}
\value{
The number of genes in the dataset involving a certain type of event.
}
\description{
Return the number of genes in the dataset involving a certain type of event.
}
\examples{
data(test_dataset)
ngenes(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capri.hypotheses.R
\name{hypothesis.add.homologous}
\alias{hypothesis.add.homologous}
\title{hypothesis.add.homologous}
\usage{
hypothesis.add.homologous(
  x,
  pattern.cause = "*",
  pattern.effect = "*",
  genes = as.genes(x),
  silent = FALSE
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{pattern.cause}{Possibile causes for the pattern.}

\item{pattern.effect}{Possibile effects for the pattern.}

\item{genes}{List of genes to be considered as possible homologous. For these genes, all the types of mutations will be considered functionally equivalent.}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO compliant object with the added hypotheses
}
\description{
Add all the hypotheses related to homologou events
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{delete.pattern}
\alias{delete.pattern}
\title{delete.pattern}
\usage{
delete.pattern(x, pattern)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{pattern}{A pattern name}
}
\value{
A TRONCO complian dataset.
}
\description{
Delete a pattern and every associated hypotheses from the dataset
}
\examples{
data(test_dataset)
delete.pattern(test_dataset, pattern='XOR_EZH2')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external.R
\name{TCGA.multiple.samples}
\alias{TCGA.multiple.samples}
\title{TCGA.multiple.samples}
\usage{
TCGA.multiple.samples(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A list of barcodes. NA if no duplicated barcode is found
}
\description{
Check if there are multiple sample in x, according to TCGA barcodes naming
}
\examples{
data(test_dataset)
TCGA.multiple.samples(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.genes}
\alias{as.genes}
\title{as.genes}
\usage{
as.genes(x, types = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{types}{The types of events to consider, if NA all available types are used.}
}
\value{
A vector of gene symbols for which a certain type of event exists
}
\description{
Return all gene symbols for which a certain type of event exists in 'x', which should be a
TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
as.genes(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{ntypes}
\alias{ntypes}
\title{ntypes}
\usage{
ntypes(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
The number of types in the dataset.
}
\description{
Return the number of types in the dataset.
}
\examples{
data(test_dataset)
ntypes(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.parameters}
\alias{as.parameters}
\title{as.parameters}
\usage{
as.parameters(x)
}
\arguments{
\item{x}{A TRONCO model.}
}
\value{
A list of parameters
}
\description{
Get parameters of a model
}
\examples{
data(test_model)
as.parameters(test_model)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selection.R
\name{samples.selection}
\alias{samples.selection}
\title{samples.selection}
\usage{
samples.selection(x, samples)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{samples}{A list of samples}
}
\value{
A TRONCO compliant dataset.
}
\description{
Filter a dataset based on selected samples id
}
\examples{
data(test_dataset)
dataset = samples.selection(test_dataset, c('patient 1', 'patient 2'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{delete.samples}
\alias{delete.samples}
\title{delete.samples}
\usage{
delete.samples(x, samples)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{samples}{An array of samples name}
}
\value{
A TRONCO complian dataset.
}
\description{
Delete samples from selected dataset
}
\examples{
data(test_dataset)
dataset = delete.samples(test_dataset, c('patient 1', 'patient 4'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.events.in.patterns}
\alias{as.events.in.patterns}
\title{as.events.in.patterns}
\usage{
as.events.in.patterns(x, patterns = NULL)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{patterns}{A list of patterns for which the list will be returned}
}
\value{
A list of events present in patterns which consitute CAPRI's hypotheses
}
\description{
Return the list of events present in selected patterns
}
\examples{
data(test_dataset)
as.events.in.patterns(test_dataset)
as.events.in.patterns(test_dataset, patterns='XOR_EZH2')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.events.in.sample}
\alias{as.events.in.sample}
\title{as.events.in.sample}
\usage{
as.events.in.sample(x, sample)
}
\arguments{
\item{x}{A TRONCO compliant dataset}

\item{sample}{Vector of sample names}
}
\value{
A list of events which are observed in the input samples list
}
\description{
Return a list of events which are observed in the input samples list
}
\examples{
data(test_dataset)
as.events.in.sample(test_dataset, c('patient 1', 'patient 7'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capri.hypotheses.R
\name{hypothesis.add}
\alias{hypothesis.add}
\title{hypothesis add}
\usage{
hypothesis.add(
  data,
  pattern.label,
  lifted.pattern,
  pattern.effect = "*",
  pattern.cause = "*"
)
}
\arguments{
\item{data}{A TRONCO compliant dataset.}

\item{pattern.label}{Label of the new hypothesis.}

\item{lifted.pattern}{Vector to be added to the lifted genotype resolving the pattern related to the new hypothesis}

\item{pattern.effect}{Possibile effects for the pattern.}

\item{pattern.cause}{Possibile causes for the pattern.}
}
\value{
A TRONCO compliant object with the added hypothesis
}
\description{
Add a new hypothesis by creating a new event and adding it to the compliant genotypes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.stages}
\alias{as.stages}
\title{as.stages}
\usage{
as.stages(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A matrix with 1 column annotating stages and rownames as sample IDs.
}
\description{
Return the association sample -> stage, if any. Input 'x' should be a
TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
data(stage)
test_dataset = annotate.stages(test_dataset, stage)
as.stages(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capri.hypotheses.R
\name{hypothesis.add.group}
\alias{hypothesis.add.group}
\title{hypothesis add group}
\usage{
hypothesis.add.group(
  x,
  FUN,
  group,
  pattern.cause = "*",
  pattern.effect = "*",
  dim.min = 2,
  dim.max = length(group),
  min.prob = 0,
  silent = FALSE
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{FUN}{Type of pattern to be added, e.g., co-occurance, soft or hard exclusivity.}

\item{group}{Group of events to be considered.}

\item{pattern.cause}{Possibile causes for the pattern.}

\item{pattern.effect}{Possibile effects for the pattern.}

\item{dim.min}{Minimum cardinality of the subgroups to be considered.}

\item{dim.max}{Maximum cardinality of the subgroups to be considered.}

\item{min.prob}{Minimum probability associated to each valid group.}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO compliant object with the added hypotheses
}
\description{
Add all the hypotheses related to a group of events
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{ssplit}
\alias{ssplit}
\title{ssplit}
\usage{
ssplit(x, clusters, idx = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{clusters}{A list of clusters. Rownames must match samples list of x}

\item{idx}{ID of a specific group present in stages. If NA all groups will be extracted}
}
\value{
A TRONCO compliant dataset.
}
\description{
Split cohort (samples) into groups, return either all groups or a specific group.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.models}
\alias{as.models}
\title{as.models}
\usage{
as.models(x, models = names(x$model))
}
\arguments{
\item{x}{A TRONCO model.}

\item{models}{The name of the models to extract, e.g. 'bic', 'aic', 'caprese', all by default.}
}
\value{
The models in a reconstructed object.
}
\description{
Extract the models from a reconstructed object.
}
\examples{
data(test_model)
as.models(test_model)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{ebind}
\alias{ebind}
\title{ebind}
\usage{
ebind(..., silent = FALSE)
}
\arguments{
\item{...}{the input datasets}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO complian dataset.
}
\description{
Binds events from one or more datasets, which must be defined over the same set of samples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tronco.R
\name{tronco.capri}
\alias{tronco.capri}
\title{tronco capri}
\usage{
tronco.capri(
  data,
  command = "hc",
  regularization = c("bic", "aic"),
  do.boot = TRUE,
  nboot = 100,
  pvalue = 0.05,
  min.boot = 3,
  min.stat = TRUE,
  boot.seed = NULL,
  silent = FALSE,
  epos = 0,
  eneg = 0,
  restart = 100
)
}
\arguments{
\item{data}{A TRONCO compliant dataset.}

\item{command}{Parameter to define to heuristic search to be performed. Hill Climbing and Tabu search are currently available.}

\item{regularization}{Select the regularization for the likelihood estimation, e.g., BIC, AIC.}

\item{do.boot}{A parameter to disable/enable the estimation of the error rates give the reconstructed model.}

\item{nboot}{Number of bootstrap sampling (with rejection) to be performed when estimating the selective advantage scores.}

\item{pvalue}{Pvalue to accept/reject the valid selective advantage relations.}

\item{min.boot}{Minimum number of bootstrap sampling to be performed.}

\item{min.stat}{A parameter to disable/enable the minimum number of bootstrap sampling required besides nboot if any sampling is rejected.}

\item{boot.seed}{Initial seed for the bootstrap random sampling.}

\item{silent}{A parameter to disable/enable verbose messages.}

\item{epos}{Error rate of false positive errors.}

\item{eneg}{Error rate of false negative errors.}

\item{restart}{An integer, the number of random restarts.}
}
\value{
A TRONCO compliant object with reconstructed model
}
\description{
Reconstruct a progression model using CAPRI algorithm. For details and examples
regarding the inference process and on the algorithm implemented in the package,
we refer to the Vignette Section 6.
}
\examples{
data(test_dataset)
recon = tronco.capri(test_dataset, nboot = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{delete.model}
\alias{delete.model}
\title{delete.model}
\usage{
delete.model(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A TRONCO complian dataset.
}
\description{
Delete a reconstructed model from the dataset
}
\examples{
data(test_model)
model = delete.model(test_model)
has.model(model)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.types}
\alias{as.types}
\title{as.types}
\usage{
as.types(x, genes = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{genes}{A list of genes to consider, if NA all genes are used.}
}
\value{
A matrix with 1 column annotating stages and rownames as sample IDs.
}
\description{
Return the types of events for a set of genes which are in 'x', which should be a
TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
as.types(test_dataset)
as.types(test_dataset, genes='TET2')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external.R
\name{TCGA.shorten.barcodes}
\alias{TCGA.shorten.barcodes}
\title{TCGA.shorten.barcodes}
\usage{
TCGA.shorten.barcodes(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A TRONCO compliant dataset
}
\description{
Keep only the first 12 character of samples barcode if there are no duplicates
}
\examples{
data(test_dataset)
TCGA.shorten.barcodes(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{view}
\alias{view}
\title{view}
\usage{
view(x, view = 5)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{view}{The firse \code{view} events are shown via \code{head}.}
}
\description{
Print to console a short report of a dataset 'x', which should be
a TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
view(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{oncoprint.cbio}
\alias{oncoprint.cbio}
\title{oncoprint.cbio}
\usage{
oncoprint.cbio(
  x,
  file = "oncoprint-cbio.txt",
  hom.del = "Homozygous Loss",
  het.loss = "Heterozygous Loss",
  gain = "Low-level Gain",
  amp = "High-level Gain"
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{file}{name of the file where to save the output}

\item{hom.del}{type of Homozygous Deletion}

\item{het.loss}{type of Heterozygous Loss}

\item{gain}{type of Gain}

\item{amp}{type of Amplification}
}
\value{
A file containing instruction for the CBio visualization Tool
}
\description{
export input for cbio visualization at http://www.cbioportal.org/public-portal/oncoprinter.jsp
}
\examples{
data(crc_gistic)
gistic = import.GISTIC(crc_gistic)
oncoprint.cbio(gistic)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{nevents}
\alias{nevents}
\title{nevents}
\usage{
nevents(x, genes = NA, types = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{genes}{The genes to consider, if NA all available genes are used.}

\item{types}{The types of events to consider, if NA all available types are used.}
}
\value{
The number of events in the dataset involving a certain gene or type of event.
}
\description{
Return the number of events in the dataset involving a certain gene or type of event.
}
\examples{
data(test_dataset)
nevents(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.hypotheses}
\alias{as.hypotheses}
\title{as.hypotheses}
\usage{
as.hypotheses(x, cause = NA, effect = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{cause}{A list of genes to use as causes}

\item{effect}{A list of genes to use as effects}
}
\value{
The hypotheses in the dataset which constitute CAPRI's hypotheses.
}
\description{
Return the hypotheses in the dataset which constitute CAPRI's hypotheses.
}
\examples{
data(test_dataset)
as.hypotheses(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loading.R
\name{import.model}
\alias{import.model}
\title{import.model}
\usage{
import.model(tronco_object, model, model.name = "imported_model")
}
\arguments{
\item{tronco_object}{A TRONCO compliant object}

\item{model}{Either a dataframe or a filename}

\item{model.name}{Name of the imported model}
}
\value{
A TRONCO compliant object
}
\description{
Add an adjacency matrix as a model to a TRONCO compliant object.
Input model can be either a dataframe or a file name.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{order.frequency}
\alias{order.frequency}
\title{order.frequency}
\usage{
order.frequency(x, decreasing = TRUE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{decreasing}{Inverse order. Default TRUE}
}
\value{
A TRONCO compliant dataset with the internal genotypes sorted according to event frequency.
}
\description{
Sort the internal genotypes according to event frequency.
}
\examples{
data(test_dataset)
order.frequency(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{sbind}
\alias{sbind}
\title{sbind}
\usage{
sbind(...)
}
\arguments{
\item{...}{the input datasets}
}
\value{
A TRONCO complian dataset.
}
\description{
Binds samples from one or more datasets, which must be defined over the same set of events
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/which.functions.R
\name{which.samples}
\alias{which.samples}
\title{which.samples}
\usage{
which.samples(x, gene, type, neg = FALSE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{gene}{A list of gene names}

\item{type}{A list of types}

\item{neg}{If FALSE return the list, if TRUE return as.samples() - list}
}
\value{
A list of sample
}
\description{
Return a list of samples with specified alteration
}
\examples{
data(test_dataset)
which.samples(test_dataset, 'TET2', 'ins_del')
which.samples(test_dataset, 'TET2', 'ins_del', neg=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{keysToNames}
\alias{keysToNames}
\title{keysToNames}
\usage{
keysToNames(x, matrix)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{matrix}{A matrix with colnames/rownames which represent genotypes keys.}
}
\value{
The matrix with intelligible colnames/rownames.
}
\description{
Convert colnames/rownames of a matrix into intelligible event names, e.g., change a key G23 in 'Mutation KRAS'.
If a name is not found, the original name is left unchanged.
}
\examples{
data(test_model)
adj_matrix = as.adj.matrix(test_model, events=as.events(test_model)[5:15,])$capri_bic
keysToNames(test_model, adj_matrix)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.selective.advantage.relations}
\alias{as.selective.advantage.relations}
\title{as.selective.advantage.relations}
\usage{
as.selective.advantage.relations(
  x,
  events = as.events(x),
  models = names(x$model),
  type = "fit"
)
}
\arguments{
\item{x}{A TRONCO model.}

\item{events}{A subset of events as of \code{as.events(x)}, all by default.}

\item{models}{A subset of reconstructed models, all by default.}

\item{type}{Either Prima Facie ('pf') or fit ('fit') probabilities, 'fit' by default.}
}
\value{
All the selective advantage relations in a TRONCO model
}
\description{
Returns a dataframe with all the selective advantage relations in a 
TRONCO model. Confidence is also shown - see \code{as.confidence}. It is possible to
specify a subset of events or models if multiple reconstruction have
been performed.
}
\examples{
data(test_model)
as.selective.advantage.relations(test_model)
as.selective.advantage.relations(test_model, events=as.events(test_model)[5:15,])
as.selective.advantage.relations(test_model, events=as.events(test_model)[5:15,], type='pf')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{oncoprint}
\alias{oncoprint}
\title{oncoprint}
\usage{
oncoprint(
  x,
  excl.sort = TRUE,
  samples.cluster = FALSE,
  genes.cluster = FALSE,
  file = NA,
  ann.stage = has.stages(x),
  ann.hits = TRUE,
  stage.color = "YlOrRd",
  hits.color = "Purples",
  null.color = "lightgray",
  border.color = "white",
  text.cex = 1,
  font.column = NA,
  font.row = NA,
  title = as.description(x),
  sample.id = FALSE,
  hide.zeroes = FALSE,
  legend = TRUE,
  legend.cex = 0.5,
  cellwidth = NA,
  cellheight = NA,
  group.by.label = FALSE,
  group.by.stage = FALSE,
  group.samples = NA,
  gene.annot = NA,
  gene.annot.color = "Set1",
  show.patterns = FALSE,
  annotate.consolidate.events = FALSE,
  txt.stats = paste(nsamples(x), " samples\\n", nevents(x), " events\\n", ngenes(x),
    " genes\\n", npatterns(x), " patterns", sep = ""),
  gtable = FALSE,
  ...
)
}
\arguments{
\item{x}{A TRONCO compliant dataset}

\item{excl.sort}{Boolean value, if TRUE sorts samples to enhance exclusivity of alterations}

\item{samples.cluster}{Boolean value, if TRUE clusters samples (columns). Default FALSE}

\item{genes.cluster}{Boolean value, if TRUE clusters genes (rows). Default FALSE}

\item{file}{If not NA write to \code{file} the Oncoprint, default is NA (just visualization).}

\item{ann.stage}{Boolean value to annotate stage classification, default depends on \code{x}}

\item{ann.hits}{Boolean value to annotate the number of events in each sample, default is TRUE}

\item{stage.color}{RColorbrewer palette to color stage annotations. Default is 'YlOrRd'}

\item{hits.color}{RColorbrewer palette to color hits annotations. Default is 'Purples'}

\item{null.color}{Color for the Oncoprint cells with 0s, default is 'lightgray'}

\item{border.color}{Border color for the Oncoprint, default is white' (no border)}

\item{text.cex}{Title and annotations cex, multiplied by font size 7}

\item{font.column}{If NA, half of font.row is used}

\item{font.row}{If NA, max(c(15 * exp(-0.02 * nrow(data)), 2)) is used, where data is the data 
visualized in the Oncoprint}

\item{title}{Oncoprint title, default is as.name(x) - see \code{as.name}}

\item{sample.id}{If TRUE shows samples name (columns). Default is FALSE}

\item{hide.zeroes}{If TRUE trims data - see \code{trim} - before plot. Default is FALSE}

\item{legend}{If TRUE shows a legend for the types of events visualized. Defualt is TRUE}

\item{legend.cex}{Default 0.5; determines legend size if \code{legend = TRUE}}

\item{cellwidth}{Default NA, sets autoscale cell width}

\item{cellheight}{Default NA, sets autoscale cell height}

\item{group.by.label}{Sort samples (rows) by event label - usefull when multiple events per gene are
available}

\item{group.by.stage}{Default FALSE; sort samples by stage.}

\item{group.samples}{If this samples -> group map is provided, samples are grouped as of groups
and sorted according to the number of mutations per sample - usefull when \code{data} was clustered}

\item{gene.annot}{Genes'groups, e.g. list(RAF=c('KRAS','NRAS'), Wnt=c('APC', 'CTNNB1')). Default is NA.}

\item{gene.annot.color}{Either a RColorColorbrewer palette name or a set of custom colors matching names(gene.annot)}

\item{show.patterns}{If TRUE shows also a separate oncoprint for each pattern. Default is FALSE}

\item{annotate.consolidate.events}{Default is FALSE. If TRUE an annotation for events to consolidate is shown.}

\item{txt.stats}{By default, shows a summary statistics for shown data (n,m, |G| and |P|)}

\item{gtable}{If TRUE return the gtable object}

\item{...}{other arguments to pass to pheatmap}
}
\description{
oncoPrint : plot a genotype. For details and examples 
regarding the visualization through oncoprints, we refer to the Vignette Section 4.4.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tronco.R
\name{tronco.gabow}
\alias{tronco.gabow}
\title{Tronco Gabow}
\usage{
tronco.gabow(
  data,
  regularization = "no_reg",
  score = "pmi",
  do.boot = TRUE,
  nboot = 100,
  pvalue = 0.05,
  min.boot = 3,
  min.stat = TRUE,
  boot.seed = NULL,
  silent = FALSE,
  epos = 0,
  eneg = 0,
  do.raising = TRUE
)
}
\arguments{
\item{data}{A TRONCO compliant dataset.}

\item{regularization}{Select the regularization for the
likelihood estimation, e.g., BIC, AIC.}

\item{score}{Select the score for the estimation of
the best tree, e.g., pointwise mutual information (pmi), conditional entropy (entropy).}

\item{do.boot}{A parameter to disable/enable the estimation
of the error rates give the reconstructed model.}

\item{nboot}{Number of bootstrap sampling (with rejection)
to be performed when estimating the selective advantage scores.}

\item{pvalue}{Pvalue to accept/reject the valid selective
advantage relations.}

\item{min.boot}{Minimum number of bootstrap sampling to be
performed.}

\item{min.stat}{A parameter to disable/enable the minimum number
of bootstrap sampling required besides nboot if any sampling
is rejected.}

\item{boot.seed}{Initial seed for the bootstrap random sampling.}

\item{silent}{A parameter to disable/enable verbose messages.}

\item{epos}{Error rate of false positive errors.}

\item{eneg}{Error rate of false negative errors.}

\item{do.raising}{Whether to use or not the raising condition as a prior.}
}
\value{
A TRONCO compliant object with reconstructed model
}
\description{
Reconstruct a progression model using Gabow algorithm combined
with probabilistic causation. For details and examples
regarding the inference process and on the algorithm implemented in the package,
we refer to the Vignette Section 6.
}
\examples{
data(test_dataset_no_hypos)
recon = tronco.gabow(test_dataset_no_hypos, nboot = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{enforce.numeric}
\alias{enforce.numeric}
\title{enforce.numeric}
\usage{
enforce.numeric(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
Convert the internal reprensentation of genotypes to numeric, if not.
}
\description{
Convert the internal reprensentation of genotypes to numeric, if not.
}
\examples{
data(test_dataset)
test_dataset = enforce.numeric(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{annotate.description}
\alias{annotate.description}
\title{annotate.description}
\usage{
annotate.description(x, label)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{label}{A string}
}
\value{
A TRONCO compliant dataset.
}
\description{
Annotate a description on the selected dataset
}
\examples{
data(test_dataset)
annotate.description(test_dataset, 'new description')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.genotypes}
\alias{as.genotypes}
\title{as.genotypes}
\usage{
as.genotypes(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A TRONCO genotypes matrix.
}
\description{
Return all genotypes for input 'x', which should be a TRONCO compliant dataset
see \code{is.compliant}. 
Function \code{keysToNames} can be used to translate colnames to events.
}
\examples{
data(test_dataset)
as.genotypes(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{stage}
\alias{stage}
\title{Stage information for test_dataset}
\format{
Vector of stages
}
\source{
fake data
}
\usage{
data(stage)
}
\value{
A list of stages
}
\description{
This dataset contains stage information for patient in test_dataset
}
\author{
Luca De Sano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{pathway.visualization}
\alias{pathway.visualization}
\title{pathway.visualization}
\usage{
pathway.visualization(
  x,
  title = paste("Pathways:", paste(names(pathways), collapse = ", ", sep = "")),
  file = NA,
  pathways.color = "Set2",
  aggregate.pathways,
  pathways,
  ...
)
}
\arguments{
\item{x}{A TRONCO complian dataset}

\item{title}{Plot title}

\item{file}{To generate a PDF a filename have to be given}

\item{pathways.color}{A RColorBrewer color palette}

\item{aggregate.pathways}{Boolean parameter}

\item{pathways}{Pathways}

\item{...}{Additional parameters}
}
\value{
plot information
}
\description{
Visualise pathways informations
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loading.R
\name{import.MAF}
\alias{import.MAF}
\title{import.MAF}
\usage{
import.MAF(
  file,
  sep = "\\t",
  is.TCGA = TRUE,
  filter.fun = NULL,
  to.TRONCO = TRUE,
  irregular = FALSE,
  paste.to.Hugo_Symbol = NULL,
  merge.mutation.types = TRUE,
  silent = FALSE
)
}
\arguments{
\item{file}{MAF filename}

\item{sep}{MAF separator, default \'\\t\'}

\item{is.TCGA}{TRUE if this MAF is from TCGA; thus its sample codenames can be interpreted}

\item{filter.fun}{A filter function applied to each row. This is expected to return TRUE/FALSE.}

\item{to.TRONCO}{If FALSE returns a dataframe with MAF data, not a TRONCO object}

\item{irregular}{If TRUE seeks only for columns Hugo_Symbol, Tumor_Sample_Barcode and Variant_Classification}

\item{paste.to.Hugo_Symbol}{If a list of column names, this will be pasted each Hugo_Symbol to yield names such as PHC2.chr1.33116215.33116215}

\item{merge.mutation.types}{If TRUE, all mutations are considered equivalent, regardless of their Variant_Classification value. Otherwise no.}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO compliant representation of the input MAF
}
\description{
Import mutation profiles from a Manual Annotation Format (MAF) file. All mutations are aggregated as a
unique event type labeled "Mutation" and assigned a color according to the default of function
\code{import.genotypes}. If this is a TCGA MAF file check for multiple samples per patient is performed
and a warning is raised if these occurr. Customized MAF files can be imported as well provided that 
they have columns Hugo_Symbol, Tumor_Sample_Barcode and Variant_Classification. 
Custom filters are possible (via filter.fun) to avoid loading the full MAF data. For details and examples 
regarding the loading functions provided by the package we refer to the Vignette Section 3.
}
\examples{
data(maf)
mutations = import.MAF(maf)
mutations = annotate.description(mutations, 'Example MAF')
mutations = TCGA.shorten.barcodes(mutations)
oncoprint(mutations)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{aCML}
\alias{aCML}
\title{Atypical chronic myeloid leukemia dataset}
\format{
TRONCO compliant dataset
}
\source{
data from http://www.nature.com/ng/journal/v45/n1/full/ng.2495.html
}
\usage{
data(aCML)
}
\value{
A standard TRONCO object
}
\description{
This file contains a TRONCO compliant dataset
}
\author{
Luca De Sano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{test_model_kfold}
\alias{test_model_kfold}
\title{A complete dataset with a reconstructed model and crossvalidation informations}
\format{
TRONCO compliant dataset
}
\source{
fake data
}
\usage{
data(test_model_kfold)
}
\value{
A standard TRONCO object
}
\description{
This dataset contains a model reconstructed with CAPRI
}
\author{
Luca De Sano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selection.R
\name{events.selection}
\alias{events.selection}
\title{events.selection}
\usage{
events.selection(
  x,
  filter.freq = NA,
  filter.in.names = NA,
  filter.out.names = NA,
  silent = FALSE
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{filter.freq}{[0,1] value which constriants the minimum frequence of selected events}

\item{filter.in.names}{gene symbols which will be included}

\item{filter.out.names}{gene symbols which will NOT be included}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO compliant dataset.
}
\description{
select a subset of the input genotypes 'x'. Selection can be done 
by frequency and gene symbols.
}
\examples{
data(test_dataset)
dataset = events.selection(test_dataset, 0.3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tronco.R
\name{tronco.caprese}
\alias{tronco.caprese}
\title{tronco caprese}
\usage{
tronco.caprese(data, lambda = 0.5, silent = FALSE, epos = 0, eneg = 0)
}
\arguments{
\item{data}{A TRONCO compliant dataset.}

\item{lambda}{Coefficient to combine the raw estimate with a correction factor into a shrinkage estimator.}

\item{silent}{A parameter to disable/enable verbose messages.}

\item{epos}{Error rate of false positive errors.}

\item{eneg}{Error rate of false negative errors.}
}
\value{
A TRONCO compliant object with reconstructed model
}
\description{
Reconstruct a progression model using CAPRESE algorithm. For details and examples
regarding the inference process and on the algorithm implemented in the package,
we refer to the Vignette Section 6.
}
\examples{
data(test_dataset_no_hypos)
recon = tronco.caprese(test_dataset_no_hypos)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loading.R
\name{import.genotypes}
\alias{import.genotypes}
\title{import.genotypes}
\usage{
import.genotypes(geno, event.type = "variant", color = "Darkgreen")
}
\arguments{
\item{geno}{Either a dataframe or a filename}

\item{event.type}{Any 1 in "geno" will be interpreted as a an observed alteration labeled with type "event.type"}

\item{color}{This is the color used for visualization of events labeled as of "event.type"}
}
\value{
A TRONCO compliant dataset
}
\description{
Import a matrix of 0/1 alterations as a TRONCO compliant
dataset. Input "geno" can be either a dataframe or a file name. In
any case the dataframe or the table stored in the file must have a
column for each altered gene and a rows for each sample. Colnames
will be used to determine gene names, if data is loaded from file
the first column will be assigned as rownames. For details and
examples regarding the loading functions provided by the package we
refer to the Vignette Section 3.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{genes.table.report}
\alias{genes.table.report}
\title{genes.table.report}
\usage{
genes.table.report(
  x,
  name,
  dir = getwd(),
  maxrow = 33,
  font = 10,
  height = 11,
  width = 8.5,
  fill = "lightblue",
  silent = FALSE
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{name}{filename}

\item{dir}{working directory}

\item{maxrow}{maximum number of row per page}

\item{font}{document fontsize}

\item{height}{table height}

\item{width}{table width}

\item{fill}{fill color}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
LaTEX code
}
\description{
Generate PDF and laex tables
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{pheatmap}
\alias{pheatmap}
\title{A function to draw clustered heatmaps.}
\usage{
pheatmap(
  mat,
  color = colorRampPalette(rev(brewer.pal(n = 7, name = "RdYlBu")))(100),
  kmeans_k = NA,
  breaks = NA,
  border_color = "grey60",
  cellwidth = NA,
  cellheight = NA,
  scale = "none",
  cluster_rows = TRUE,
  cluster_cols = TRUE,
  clustering_distance_rows = "euclidean",
  clustering_distance_cols = "euclidean",
  clustering_method = "complete",
  cutree_rows = NA,
  cutree_cols = NA,
  treeheight_row = ifelse(cluster_rows, 50, 0),
  treeheight_col = ifelse(cluster_cols, 50, 0),
  legend = TRUE,
  legend_breaks = NA,
  legend_labels = NA,
  annotation_row = NA,
  annotation_col = NA,
  annotation = NA,
  annotation_colors = NA,
  annotation_legend = TRUE,
  drop_levels = TRUE,
  show_rownames = TRUE,
  show_colnames = TRUE,
  main = NA,
  fontsize = 10,
  fontsize_row = fontsize,
  fontsize_col = fontsize,
  display_numbers = FALSE,
  number_format = "\%.2f",
  number_color = "grey30",
  fontsize_number = 0.8 * fontsize,
  gaps_row = NULL,
  gaps_col = NULL,
  labels_row = NULL,
  labels_col = NULL,
  filename = NA,
  width = NA,
  height = NA,
  silent = FALSE,
  legend.cex = 1,
  txt.stats = NA,
  ...
)
}
\arguments{
\item{mat}{numeric matrix of the values to be plotted.}

\item{color}{vector of colors used in heatmap.}

\item{kmeans_k}{the number of kmeans clusters to make, if we want to agggregate the 
rows before drawing heatmap. If NA then the rows are not aggregated.}

\item{breaks}{a sequence of numbers that covers the range of values in mat and is one 
element longer than color vector. Used for mapping values to colors. Useful, if needed 
to map certain values to certain colors, to certain values. If value is NA then the 
breaks are calculated automatically.}

\item{border_color}{color of cell borders on heatmap, use NA if no border should be 
drawn.}

\item{cellwidth}{individual cell width in points. If left as NA, then the values 
depend on the size of plotting window.}

\item{cellheight}{individual cell height in points. If left as NA, 
then the values depend on the size of plotting window.}

\item{scale}{character indicating if the values should be centered and scaled in 
either the row direction or the column direction, or none. Corresponding values are 
\code{"row"}, \code{"column"} and \code{"none"}}

\item{cluster_rows}{boolean values determining if rows should be clustered,}

\item{cluster_cols}{boolean values determining if columns should be clustered.}

\item{clustering_distance_rows}{distance measure used in clustering rows. Possible 
values are \code{"correlation"} for Pearson correlation and all the distances 
supported by \code{\link{dist}}, such as \code{"euclidean"}, etc. If the value is none 
of the above it is assumed that a distance matrix is provided.}

\item{clustering_distance_cols}{distance measure used in clustering columns. Possible 
values the same as for clustering_distance_rows.}

\item{clustering_method}{clustering method used. Accepts the same values as 
\code{\link{hclust}}.}

\item{cutree_rows}{number of clusters the rows are divided into, based on the
 hierarchical clustering (using cutree), if rows are not clustered, the 
argument is ignored}

\item{cutree_cols}{similar to \code{cutree_rows}, but for columns}

\item{treeheight_row}{the height of a tree for rows, if these are clustered. 
Default value 50 points.}

\item{treeheight_col}{the height of a tree for columns, if these are clustered. 
Default value 50 points.}

\item{legend}{logical to determine if legend should be drawn or not.}

\item{legend_breaks}{vector of breakpoints for the legend.}

\item{legend_labels}{vector of labels for the \code{legend_breaks}.}

\item{annotation_row}{data frame that specifies the annotations shown on left
 side of the heatmap. Each row defines the features for a specific row. The 
rows in the data and in the annotation are matched using corresponding row
 names. Note that color schemes takes into account if variable is continuous
 or discrete.}

\item{annotation_col}{similar to annotation_row, but for columns.}

\item{annotation}{deprecated parameter that currently sets the annotation_col if it is missing}

\item{annotation_colors}{list for specifying annotation_row and 
annotation_col track colors manually. It is  possible to define the colors 
for only some of the features. Check examples for  details.}

\item{annotation_legend}{boolean value showing if the legend for annotation 
tracks should be drawn.}

\item{drop_levels}{logical to determine if unused levels are also shown in 
the legend}

\item{show_rownames}{boolean specifying if column names are be shown.}

\item{show_colnames}{boolean specifying if column names are be shown.}

\item{main}{the title of the plot}

\item{fontsize}{base fontsize for the plot}

\item{fontsize_row}{fontsize for rownames (Default: fontsize)}

\item{fontsize_col}{fontsize for colnames (Default: fontsize)}

\item{display_numbers}{logical determining if the numeric values are also printed to 
the cells. If this is a matrix (with same dimensions as original matrix), the contents
of the matrix are shown instead of original values.}

\item{number_format}{format strings (C printf style) of the numbers shown in cells. 
For example "\code{\%.2f}" shows 2 decimal places and "\code{\%.1e}" shows exponential 
notation (see more in \code{\link{sprintf}}).}

\item{number_color}{color of the text}

\item{fontsize_number}{fontsize of the numbers displayed in cells}

\item{gaps_row}{vector of row indices that show shere to put gaps into
heatmap. Used only if the rows are not clustered. See \code{cutree_row}
to see how to introduce gaps to clustered rows.}

\item{gaps_col}{similar to gaps_row, but for columns.}

\item{labels_row}{custom labels for rows that are used instead of rownames.}

\item{labels_col}{similar to labels_row, but for columns.}

\item{filename}{file path where to save the picture. Filetype is decided by 
the extension in the path. Currently following formats are supported: png, pdf, tiff,
 bmp, jpeg. Even if the plot does not fit into the plotting window, the file size is 
calculated so that the plot would fit there, unless specified otherwise.}

\item{width}{manual option for determining the output file width in inches.}

\item{height}{manual option for determining the output file height in inches.}

\item{silent}{do not draw the plot (useful when using the gtable output)}

\item{legend.cex}{Default 0.5; determines legend size if \code{legend = TRUE}}

\item{txt.stats}{By default, shows a summary statistics for shown data (n,m, |G| and |P|)}

\item{\dots}{graphical parameters for the text used in plot. Parameters passed to 
\code{\link{grid.text}}, see \code{\link{gpar}}.}
}
\value{
Invisibly a list of components 
\itemize{
    \item \code{tree_row} the clustering of rows as \code{\link{hclust}} object 
    \item \code{tree_col} the clustering of columns as \code{\link{hclust}} object
    \item \code{kmeans} the kmeans clustering of rows if parameter \code{kmeans_k} was 
specified 
}
}
\description{
A function to draw clustered heatmaps where one has better control over some graphical 
parameters such as cell size, etc.
}
\details{
The function also allows to aggregate the rows using kmeans clustering. This is 
advisable if number of rows is so big that R cannot handle their hierarchical 
clustering anymore, roughly more than 1000. Instead of showing all the rows 
separately one can cluster the rows in advance and show only the cluster centers. 
The number of clusters can be tuned with parameter kmeans_k.

This is a modified version of the original pheatmap 
(https://cran.r-project.org/web/packages/pheatmap/index.html)
edited in accordance with GPL-2.
}
\examples{
# Create test matrix
test = matrix(rnorm(200), 20, 10)
test[1:10, seq(1, 10, 2)] = test[1:10, seq(1, 10, 2)] + 3
test[11:20, seq(2, 10, 2)] = test[11:20, seq(2, 10, 2)] + 2
test[15:20, seq(2, 10, 2)] = test[15:20, seq(2, 10, 2)] + 4
colnames(test) = paste("Test", 1:10, sep = "")
rownames(test) = paste("Gene", 1:20, sep = "")

# Draw heatmaps
pheatmap(test)
}
\author{
Raivo Kolde <rkolde@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capri.hypotheses.R
\name{AND}
\alias{AND}
\title{AND}
\usage{
AND(...)
}
\arguments{
\item{...}{Atoms of the co-occurance pattern given either as labels or as partielly lifted vectors.}
}
\value{
Vector to be added to the lifted genotype resolving the co-occurance pattern
}
\description{
AND hypothesis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loading.R
\name{extract.MAF.HuGO.Entrez.map}
\alias{extract.MAF.HuGO.Entrez.map}
\title{extract.MAF.HuGO.Entrez.map}
\usage{
extract.MAF.HuGO.Entrez.map(file, sep = "\\t")
}
\arguments{
\item{file}{MAF filename}

\item{sep}{MAF separator, default \'\\t\'}
}
\value{
A mapHugo_Symbol -> Entrez_Gene_Id.
}
\description{
Extract a map Hugo_Symbol -> Entrez_Gene_Id from a MAF input file. If some genes map to ID 0
a warning is raised.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{crc_maf}
\alias{crc_maf}
\title{MAF example data}
\format{
Manual Annotated Format
}
\source{
data from http://www.nature.com/nature/journal/v487/n7407/full/nature11252.html
}
\usage{
data(crc_maf)
}
\value{
A MAF file
}
\description{
This dataset contains an example of MAF input of a crc cohort of patients
}
\author{
Daniele Ramazzotti
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{join.events}
\alias{join.events}
\title{join.events}
\usage{
join.events(x, ..., new.event, new.type, event.color)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{...}{A list of events to merge}

\item{new.event}{The name of the resultant event}

\item{new.type}{The type of the new event}

\item{event.color}{The color of the new event}
}
\value{
A TRONCO compliant dataset.
}
\description{
Merge a list of events in an unique event
}
\examples{
data(muts)
dataset = join.events(muts, 'G1', 'G2', new.event='test', new.type='banana', event.color='yellow')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.confidence}
\alias{as.confidence}
\title{as.confidence}
\usage{
as.confidence(x, conf, models = names(x$model))
}
\arguments{
\item{x}{A TRONCO model.}

\item{conf}{A vector with any of 'tp', 'pr', 'hg', 'npb', 'pb', 'sb', 'eloss', 'prederr' or 'posterr'.}

\item{models}{The name of the models to extract, all by default.}
}
\value{
A list of matrices with the event-to-event confidence.
}
\description{
Return confidence information for a TRONCO model. Available information are: temporal priority (tp), 
probability raising (pr), hypergeometric test (hg), parametric (pb), non parametric (npb) or 
statistical (sb) bootstrap, entropy loss (eloss), prediction error (prederr).
Confidence is available only once a model has been reconstructed with any of the algorithms implemented
in TRONCO. If more than one model has been reconstructed - for instance via multiple regularizations - 
confidence information is appropriately nested. The requested confidence is specified via 
vector parameter \code{conf}.
}
\examples{
data(test_model)
as.confidence(test_model, conf='tp')
as.confidence(test_model, conf=c('tp', 'hg'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.colors}
\alias{as.colors}
\title{as.colors}
\usage{
as.colors(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A named vector of colors.
}
\description{
Return the colors associated to each type of event in 'x', which should be a
TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
as.colors(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{delete.hypothesis}
\alias{delete.hypothesis}
\title{delete.hypothesis}
\usage{
delete.hypothesis(x, event = NA, cause = NA, effect = NA)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{event}{Can be an event or pattern name}

\item{cause}{Can be an event or pattern name}

\item{effect}{Can be an event or pattern name}
}
\value{
A TRONCO complian dataset.
}
\description{
Delete an hypothesis from the dataset based on a selected event.
Check if the selected event exist in the dataset and delete his associated hypothesis
}
\examples{
data(test_dataset)
delete.hypothesis(test_dataset, event='TET2')
delete.hypothesis(test_dataset, cause='EZH2')
delete.hypothesis(test_dataset, event='XOR_EZH2')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{trim}
\alias{trim}
\title{trim}
\usage{
trim(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
A TRONCO compliant dataset.
}
\description{
Deletes all events which have frequency 0 in the dataset.
}
\examples{
data(test_dataset)
test_dataset = trim(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.adj.matrix}
\alias{as.adj.matrix}
\title{as.adj.matrix}
\usage{
as.adj.matrix(x, events = as.events(x), models = names(x$model), type = "fit")
}
\arguments{
\item{x}{A TRONCO model.}

\item{events}{A subset of events as of \code{as.events(x)}, all by default.}

\item{models}{A subset of reconstructed models, all by default.}

\item{type}{Either the prima facie ('pf') or the post-regularization ('fit') matrix, 'fit' by default.}
}
\value{
The adjacency matrix of a TRONCO model.
}
\description{
Extract the adjacency matrix of a TRONCO model. The matrix is indexed with colnames/rownames which 
represent genotype keys - these can be resolved with function \code{keysToNames}. It is possible to
specify a subset of events to build the matrix, a subset of models if multiple reconstruction have
been performed. Also, either the prima facie matrix or the post-regularization matrix can be extracted.
}
\examples{
data(test_model)
as.adj.matrix(test_model)
as.adj.matrix(test_model, events=as.events(test_model)[5:15,])
as.adj.matrix(test_model, events=as.events(test_model)[5:15,], type='pf')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selection.R
\name{rank.recurrents}
\alias{rank.recurrents}
\title{rank.recurrents}
\usage{
rank.recurrents(x, n)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{n}{The number of events to rank}
}
\value{
the first n recurrent events
}
\description{
Return the first n recurrent events
}
\examples{
data(test_dataset)
dataset = rank.recurrents(test_dataset, 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external.R
\name{export.mutex}
\alias{export.mutex}
\title{export,mutex}
\usage{
export.mutex(
  x,
  filename = "tronco_to_mutex",
  filepath = "./",
  label.mutation = "SNV",
  label.amplification = list("High-level Gain"),
  label.deletion = list("Homozygous Loss")
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{filename}{The name of the file}

\item{filepath}{The path where to save the file}

\item{label.mutation}{The event type to use as mutation}

\item{label.amplification}{The event type to use as amplification (can be a list)}

\item{label.deletion}{The event type to use as amplification (can be a list)}
}
\value{
A MUTEX example matrix
}
\description{
Create an input file for MUTEX
(ref: https://code.google.com/p/mutex/ )
}
\examples{
data(crc_gistic)
dataset = import.GISTIC(crc_gistic)
export.mutex(dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{test_dataset_no_hypos}
\alias{test_dataset_no_hypos}
\title{A complete dataset}
\format{
TRONCO compliant dataset
}
\source{
fake data
}
\usage{
data(test_dataset_no_hypos)
}
\value{
A standard TRONCO object
}
\description{
This dataset contains a complete test dataset
}
\author{
Luca De Sano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tronco.plot.R
\name{tronco.plot}
\alias{tronco.plot}
\title{tronco.plot}
\usage{
tronco.plot(
  x,
  models = names(x$model),
  fontsize = NA,
  height = 2,
  width = 3,
  height.logic = 1,
  pf = FALSE,
  disconnected = FALSE,
  scale.nodes = NA,
  title = as.description(x),
  confidence = NA,
  p.min = 0.05,
  legend = TRUE,
  legend.cex = 1,
  edge.cex = 1,
  label.edge.size = NA,
  expand = TRUE,
  genes = NULL,
  relations.filter = NA,
  edge.color = "black",
  pathways.color = "Set1",
  file = NA,
  legend.pos = "bottom",
  pathways = NULL,
  lwd = 3,
  samples.annotation = NA,
  export.igraph = FALSE,
  create.new.dev = TRUE,
  ...
)
}
\arguments{
\item{x}{A reconstructed model (the output of the inference
by a tronco function)}

\item{models}{A vector containing the names of the
algorithms used (caprese, capri_bic, etc)}

\item{fontsize}{For node names. Default NA for
automatic rescaling}

\item{height}{Proportion node height - node width.
Default height 2}

\item{width}{Proportion node height - node width.
Default width 2}

\item{height.logic}{Height of logical nodes.
Defaul 1}

\item{pf}{Should I print Prima Facie?
Default False}

\item{disconnected}{Should I print disconnected
nodes? Default False}

\item{scale.nodes}{Node scaling coefficient
(based on node frequency). Default NA (autoscale)}

\item{title}{Title of the plot. Default as.description(x)}

\item{confidence}{Should I add confidence
informations? No if NA}

\item{p.min}{p-value cutoff. Default automatic}

\item{legend}{Should I visualise the legend?}

\item{legend.cex}{CEX value for legend. Default 1.0}

\item{edge.cex}{CEX value for edge labels. Default 1.0}

\item{label.edge.size}{Size of edge labels.
Default NA for automatic rescaling}

\item{expand}{Should I expand hypotheses? Default TRUE}

\item{genes}{Visualise only genes in this list.
Default NULL, visualise all.}

\item{relations.filter}{Filter relations to dispaly
according to this functions. Default NA}

\item{edge.color}{Edge color. Default 'black'}

\item{pathways.color}{RColorBrewer colorser
for patways. Default 'Set1'.}

\item{file}{String containing filename for PDF output.
If NA no PDF output will be provided}

\item{legend.pos}{Legend position. Default 'bottom',}

\item{pathways}{A vector containing pathways information
as described in as.patterns()}

\item{lwd}{Edge base lwd. Default 3}

\item{samples.annotation}{= List of samples to search
for events in model}

\item{export.igraph}{If TRUE export the generated igraph
object}

\item{create.new.dev}{If TRUE create a new graphical device 
when calling trono.plot. Set this to FALSE, e.g., if you do not 
wish to create a new device when executing the command with 
export.igraph = TRUE}

\item{...}{Additional arguments for RGraphviz
plot function}
}
\value{
Information about the reconstructed model
}
\description{
Plots a progression model from a recostructed dataset. For details and examples
regarding the visualization of an inferred model,
we refer to the Vignette Section 7.
}
\examples{
data(test_model)
tronco.plot(test_model)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external.R
\name{export.nbs.input}
\alias{export.nbs.input}
\title{export.nbs.input}
\usage{
export.nbs.input(x, map_hugo_entrez, file = "tronco_to_nbs.mat")
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{map_hugo_entrez}{Hugo_Symbol-Entrez_Gene_Id map}

\item{file}{output file name}
}
\description{
Create a .mat file which can be used with NBS clustering
(ref: http://chianti.ucsd.edu/~mhofree/wordpress/?page_id=26)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capri.hypotheses.R
\name{OR}
\alias{OR}
\title{OR}
\usage{
OR(...)
}
\arguments{
\item{...}{Atoms of the soft exclusive pattern given either as labels or as partielly lifted vectors.}
}
\value{
Vector to be added to the lifted genotype resolving the soft exclusive pattern
}
\description{
OR hypothesis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correctness.R
\name{is.compliant}
\alias{is.compliant}
\title{is.compliant}
\usage{
is.compliant(
  x,
  err.fun = "[ERR]",
  stage = !(all(is.null(x$stages)) || all(is.na(x$stages)))
)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{err.fun}{string which identifies the function which called is.compliant}

\item{stage}{boolean flag to check x$stage datagframe}
}
\value{
on error stops the computation
}
\description{
Check if 'x' is compliant with TRONCO's input: that is if it has dataframes 
x$genotypes, x$annotations, x$types and x$stage (optional)
}
\examples{
data(test_dataset)
is.compliant(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{tronco.pattern.plot}
\alias{tronco.pattern.plot}
\title{tronco.pattern.plot}
\usage{
tronco.pattern.plot(
  x,
  group = as.events(x),
  to,
  gap.cex = 1,
  legend.cex = 1,
  label.cex = 1,
  title = paste(to[1], to[2]),
  mode = "barplot"
)
}
\arguments{
\item{x}{A TRONCO compliant dataset}

\item{group}{A list of events (see as.events() for details)}

\item{to}{A target event}

\item{gap.cex}{cex parameter for gap}

\item{legend.cex}{cex parameter for legend}

\item{label.cex}{cex parameter for label}

\item{title}{title}

\item{mode}{can be 'circos' or 'barplot'}
}
\description{
tronco.pattern.plot : plot a genotype
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.description}
\alias{as.description}
\title{as.description}
\usage{
as.description(x)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}
}
\value{
The description annotating the dataset, if any.
}
\description{
Return the description annotating the dataset, if any. Input 'x' should be
a TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
as.description(test_dataset)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{tronco.kfold.posterr}
\alias{tronco.kfold.posterr}
\title{tronco.kfold.posterr. For details and examples 
regarding the statistical assesment of an inferred model, 
we refer to the Vignette Section 7.}
\usage{
tronco.kfold.posterr(
  x,
  models = names(as.models(x)),
  events = as.events(x),
  runs = 10,
  k = 10,
  cores.ratio = 1,
  silent = FALSE
)
}
\arguments{
\item{x}{A reconstructed model (the output of tronco.capri)}

\item{models}{The names of the selected regularizers (bic, aic or caprese)}

\item{events}{a list of event}

\item{runs}{a positive integer number, the number of times cross-validation will be run}

\item{k}{a positive integer number, the number of groups into which the data will be split}

\item{cores.ratio}{Percentage of cores to use. coresRate * (numCores - 1)}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\description{
Perform a k-fold cross-validation using the function bn.cv
and scan every node to estimate its posterior classification error.
}
\examples{
data(test_model)
tronco.kfold.posterr(test_model, k = 2, runs = 2, cores.ratio = 0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.kfold.eloss}
\alias{as.kfold.eloss}
\title{as.kfold.eloss}
\usage{
as.kfold.eloss(x, models = names(x$model), values = FALSE)
}
\arguments{
\item{x}{A TRONCO model.}

\item{models}{A subset of reconstructed models, all by default.}

\item{values}{If you want to see also the values}
}
\value{
All the bootstrap scores in a TRONCO model
}
\description{
Returns a dataframe with all the average/stdev entropy loss score of a 
TRONCO model. It is possible to specify models if multiple
reconstruction have been performed.
}
\examples{
data(test_model_kfold)
as.kfold.eloss(test_model_kfold)
as.kfold.eloss(test_model_kfold, models='capri_aic')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editing.functions.R
\name{join.types}
\alias{join.types}
\title{join.types}
\usage{
join.types(x, ..., new.type = "new.type", new.color = "khaki", silent = FALSE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{...}{type to merge}

\item{new.type}{label for the new type to create}

\item{new.color}{color for the new type to create}

\item{silent}{A parameter to disable/enable verbose messages.}
}
\value{
A TRONCO compliant dataset.
}
\description{
For an input dataset merge all the events of two or more distincit types
(e.g., say that missense and indel mutations are events
of a unique "mutation" type)
}
\examples{
data(test_dataset_no_hypos)
join.types(test_dataset_no_hypos, 'ins_del', 'missense_point_mutations')
join.types(test_dataset_no_hypos, 'ins_del',
     'missense_point_mutations', new.type='mut', new.color='green')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{as.events}
\alias{as.events}
\title{as.events}
\usage{
as.events(x, genes = NA, types = NA, keysToNames = FALSE)
}
\arguments{
\item{x}{A TRONCO compliant dataset.}

\item{genes}{The genes to consider, if NA all available genes are used.}

\item{types}{The types of events to consider, if NA all available types are used.}

\item{keysToNames}{If TRUE return a list of mnemonic name composed by type + gene}
}
\value{
A matrix with 2 columns (event type, gene name) for the events found.
}
\description{
Return all events involving certain genes and of a certain type in 'x', which should be a
TRONCO compliant dataset - see \code{is.compliant}.
}
\examples{
data(test_dataset)
as.events(test_dataset)
as.events(test_dataset, types='ins_del')
as.events(test_dataset, genes = 'TET2')
as.events(test_dataset, types='Missing')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.functions.R
\name{nhypotheses}
\alias{nhypotheses}
\title{Return the number of hypotheses in the dataset}
\usage{
nhypotheses(x)
}
\arguments{
\item{x}{the dataset.}
}
\description{
Return the number of hypotheses in the dataset
}
\examples{
data(test_dataset)
nhypotheses(test_dataset)

}
